Package: mpath
Title: Regularized Linear Models
Version: 0.4-2.27
Date: 2026-07-12
Authors@R: c(
      person(given = "Zhu", family = "Wang", role = c("aut", "cre"), email = "zwang145@uthsc.edu"),
      person(given = "Achim", family = "Zeileis", role = "aut"),
      person(given = "Simon", family = "Jackman", role = "aut"),
      person(given = "Brian", family = "Ripley", role = "aut"),
      person(given = "Patrick", family = "Breheny", role = "aut")
      )
Maintainer: Zhu Wang <zwang145@uthsc.edu>
Description: Algorithms compute robust estimators for loss functions in the concave convex (CC) family by the iteratively reweighted convex optimization (IRCO), an extension of the iteratively reweighted least squares (IRLS). The IRCO reduces the weight of the observation that leads to a large loss; it also provides weights to help identify outliers. Applications include robust (penalized) generalized linear models and robust support vector machines. The package also contains penalized Poisson, negative binomial, zero-inflated Poisson, zero-inflated negative binomial regression models and robust models with non-convex loss functions. Wang et al. (2014) <doi:10.1002/sim.6314>,
      Wang et al. (2015) <doi:10.1002/bimj.201400143>,
      Wang et al. (2016) <doi:10.1177/0962280214530608>,
      Wang (2021) <doi:10.1007/s11749-021-00770-2>,
      Wang (2024) <doi:10.1111/anzs.12409>.
Depends: R (>= 3.5.0), methods, glmnet
Imports: MASS, pscl, numDeriv, foreach, doParallel, bst, WeightSVM
Suggests: R.rsp
VignetteBuilder: R.rsp
License: GPL-2
URL: https://github.com/zhuwang46/mpath
BugReports: https://github.com/zhuwang46/mpath/issues
Packaged: 2026-07-12 20:40:57 UTC; zwang145
NeedsCompilation: yes
RoxygenNote: 7.1.1
Author: Zhu Wang [aut, cre],
  Achim Zeileis [aut],
  Simon Jackman [aut],
  Brian Ripley [aut],
  Patrick Breheny [aut]
Repository: CRAN
Date/Publication: 2026-07-12 21:50:02 UTC

useDynLib(mpath)

export(glmreg, glmreg_fit, glmregNB, zipath, llfun, predictzeroinfl1, cv.folds, cv.glmreg, cv.glmreg_fit, cv.glmregNB, cv.zipath, tuning.zipath, pval.zipath, conv2glmreg, conv2zipath, estfunReg, sandwichReg, breadReg, meatReg, hessianReg, se, be.zeroinfl, rzi, ncl, nclreg, ncl_fit, nclreg_fit, cv.nclreg, cv.nclreg_fit, stan, zipath_fit, irglm, irglmreg, irglmreg_fit, irsvm, irsvm_fit, cv.irsvm, cv.irsvm_fit, loss2, loss2_irsvm, loss3, cv.irglmreg, cv.irglmreg_fit, compute_wt, update_wt, compute_g,
gfunc, cfun2num, y2num, y2num4glm, check_s)

import("methods")
import("stats")
import("glmnet")
importFrom("MASS", "theta.ml")
importFrom("MASS", "glm.nb")
importFrom("MASS", "rnegbin")
importFrom("MASS", "negative.binomial")
importFrom("pscl", "zeroinfl")
importFrom("pscl", "predprob")
importFrom("numDeriv", "hessian")
importFrom("graphics", "plot", "segments")
importFrom("graphics", "axis", "matplot", "text")
importFrom("utils", "capture.output")
importFrom("foreach", "foreach")
importFrom("foreach", "%dopar%")
importFrom("foreach", "%:%")
importFrom("doParallel", "registerDoParallel")
importFrom("doParallel", "stopImplicitCluster")
importFrom("bst", "loss")
importFrom("bst", "bfunc")
importFrom("bst", "gradient")
importFrom("bst", "bst_control")
importFrom("bst", "bst")
importFrom("bst", "evalerr")
importFrom("WeightSVM", "wsvm")

S3method(glmreg, formula)
S3method(glmreg, matrix)
S3method(glmreg, default)
S3method(coef, glmreg)
S3method(deviance, glmreg)
S3method(plot, glmreg)
S3method(predict, glmreg)
S3method(logLik, glmreg)
S3method(AIC, glmreg)
S3method(BIC, glmreg)
S3method(predprob, glmreg)
S3method(cv.glmreg, formula)
S3method(cv.glmreg, matrix)
S3method(cv.glmreg, default)
S3method(plot, cv.glmreg)
S3method(predict, cv.glmreg)
S3method(coef, cv.glmreg)
S3method(summary, glmregNB)
S3method(print, summary.glmregNB)

S3method(zipath, formula)
S3method(zipath, matrix)
S3method(zipath, default)
S3method(coef, zipath)
S3method(plot, zipath)
S3method(predict, zipath)
S3method(summary, zipath)
S3method("residuals", "zipath")
S3method("fitted", "zipath")
S3method(logLik, zipath)
S3method(AIC, zipath)
S3method(BIC, zipath)
S3method(estfunReg, zipath)
S3method(hessianReg, zipath)
S3method(breadReg, zipath)
S3method(meatReg, zipath)
S3method(se, zipath)
S3method(predprob, zipath)
S3method(cv.zipath, formula)
S3method(cv.zipath, matrix)
S3method(cv.zipath, default)
S3method(predict, cv.zipath)
S3method(coef, cv.zipath)
S3method(plot, cv.zipath)
S3method(print, summary.zipath)

S3method(ncl, formula)
S3method(ncl, matrix)
S3method(ncl, default)
S3method(predict, ncl)
S3method(nclreg, formula)
S3method(nclreg, matrix)
S3method(nclreg, default)
S3method(predict, nclreg)
S3method(coef, nclreg)
S3method(cv.nclreg, formula)
S3method(cv.nclreg, matrix)
S3method(cv.nclreg, default)
S3method(plot, cv.nclreg)
S3method(coef, cv.nclreg)

S3method(irglm, formula)
#S3method(irglm, matrix)
#S3method(irglm, default)
S3method(predict, irglm)
S3method(irglmreg, formula)
S3method(irglmreg, matrix)
S3method(irglmreg, default)
S3method(predict, irglmreg)
S3method(coef, irglmreg)
S3method(cv.irglmreg, formula)
S3method(cv.irglmreg, matrix)
S3method(cv.irglmreg, default)
S3method(plot, cv.irglmreg)
S3method(coef, cv.irglmreg)
S3method(irsvm, formula)
S3method(irsvm, matrix)
S3method(irsvm, default)
S3method(cv.irsvm, formula)
S3method(cv.irsvm, matrix)
S3method(cv.irsvm, default)

## ----echo=FALSE,results='hide'----------------------------
library("knitr")
options(prompt = "R> ", continue = " ", width = 60, useFancyQuotes = FALSE)
opts_chunk$set(tidy=TRUE, highlight=FALSE, background="white")
opts_chunk$set(dev=c('pdf','postscript'), fig.path='cancerfigure/')

## ----message=FALSE, eval=FALSE----------------------------
#  #The data files below were downloaded on June 1, 2016
#  require("openxlsx")
#  library("mpath")
#  bc <- t(read.delim("GSE20194_series_matrix.txt.gz",sep="",header=FALSE,skip=80))
#  colnames(bc) <- bc[1,]
#  bc <- bc[-1, -c(1, 2)]
#  ###The last column is empty with variable name !series_matrix_table_end, thus omitted
#  bc <- bc[, -22284]
#  mode(bc) <- "numeric" ### convert character to numeric
#  dat1 <- openxlsx::openxlsx("GSE20194_MDACC_Sample_Info.xls", sheet=1, header=TRUE)
#  y <- dat1$characteristics..ER_status
#  y <- ifelse(y=="P", 1, -1)
#  table(y)
#  res <- rep(NA, dim(bc)[2])
#  for(i in 1:dim(bc)[2])
#  res[i] <- abs(t.test(bc[,i]~y)$statistic)
#  ### find 3000 largest absolute value of t-statistic
#  tmp <- order(res, decreasing=TRUE)[1:3000]
#  dat <- bc[, tmp]
#  ### standardize variables
#  dat <- scale(dat)

## ----message=FALSE----------------------------------------
### number of replicates
nrun <- 100
### penalty type
penalty <- c("enet", "snet", "mnet")
### Smallest value for lambda, as a fraction of lambda.max, the smallest value for which all coefficients are zero except the intercept
ratio <- 0.25
type.path <- "nonactive"
nlam <- ifelse(type.path!="onestep", 30, 100)
### The training data is contaminated by randomly switching response variable labels at varying pre-specified proportions
per <- c(0, 0.05, 0.10, 0.15)
### what quantity is minimized for tuning parameter selection
tuning <- "error"
n.cores <- 5
### robust nonconvex loss function, rfamily type and logistic
type <- c("closs", "gloss", "qloss", "binomial")
### and corresponding labels
type1 <- c("Closs", "Gloss", "Qloss", "Logistic")
### and corresponding tuning parameter 
s <- c(0.9, 1.01, 0.5)
mstop <- 50
plot.it <- TRUE

## ----brcancer, fig.show='hold', eval=FALSE----------------
#  summary7 <- function(x) c(summary(x), sd=sd(x))
#  ptm <- proc.time()
#  for(k in (1:4)){   ### k controls family argument rfamily type (see above)
#    if(type[k]=="gloss")
#    nu <- 0.1
#    else
#    nu <- 0.01
#    for(j in (1:3)){    ### j controls argument penalty type (see above)
#      gam <- ifelse(penalty[j]=="snet", 3.7, 12)
#      err.m1 <- nvar.m1 <- errbest.m1 <- lambest.m1 <- matrix(NA, ncol=4, nrow=nrun)
#      nvarbest.m1 <- mstopcv.m1 <- matrix(NA, ncol=4, nrow=nrun)
#      colnames(err.m1) <- c("cont-0%", "cont-5%", "cont-10%", "cont-15%")
#      colnames(mstopcv.m1) <- colnames(nvarbest.m1) <- colnames(err.m1)
#      colnames(nvar.m1) <- colnames(err.m1)
#      colnames(errbest.m1) <- colnames(err.m1)
#      colnames(lambest.m1) <- colnames(err.m1)
#      for (ii in 1:nrun){
#        set.seed(1000+ii)
#        trid <- c(sample(which(y==1))[1:50], sample(which(y==-1))[1:50])
#        dtr <- dat[trid, ]
#        dte <- dat[-trid, ]
#        ytrold <- y[trid]
#        yte <- y[-trid]
#        ### number of patients/no. variables in training and test data
#        dim(dtr)
#        dim(dte)
#        ### randomly contaminate data
#        ntr <- length(trid)
#        set.seed(1000+ii)
#        con <- sample(ntr)
#        for(i in (1:4)){    ### i controls how many percentage of data contaminated, see argument per above
#          ytr <- ytrold
#          percon <- per[i]
#          ### randomly flip labels of the samples in training set according to pre-defined contamination level
#          if(percon > 0){
#            ji <- con[1:(percon*ntr)]
#            ytr[ji] <- -ytrold[ji]
#          }
#          ### fit a model with nclreg for nonconvex loss or glmreg for logistic loss, and use cross-validation to select best penalization parameter
#  	if(type[k] %in% c("closs", "gloss", "qloss")){
#          dat.m1 <- nclreg(x=dtr, y=ytr, s=s[k], iter=100, rfamily = type[k], penalty=penalty[j], lambda.min.ratio=ratio, gamma=gam, mstop.init=mstop, nu.init=nu, type.path=type.path, decreasing=FALSE, type.init="bst")
#          lambda <- dat.m1$lambda[1:nlam]
#          set.seed(1000+ii)
#  	cvm1 <- cv.nclreg(x=dtr, y=ytr, nfolds=5, n.cores=n.cores, parallel=TRUE, s=s[k], lambda=lambda, rfamily = type[k], penalty=penalty[j], gamma=gam, type=tuning, plot.it=FALSE, type.init=dat.m1$type.init, mstop.init=dat.m1$mstop.init, nu.init=dat.m1$nu.init, type.path=type.path, decreasing=dat.m1$decreasing)
#          err1 <- predict(dat.m1, newdata=dte, newy=yte, type="error")
#        }
#        else{
#          dat.m1 <- glmreg(x=dtr, y=(ytr+1)/2, family = type[k], penalty=penalty[j], lambda.min.ratio=ratio, gamma=gam)
#          set.seed(1000+ii)
#  	cvm1 <- cv.glmreg(x=dtr, y=(ytr+1)/2, nfolds=5, n.cores=n.cores, parallel=TRUE, lambda=dat.m1$lambda,
#          family = type[k], penalty=penalty[j], gamma=gam, plot.it=FALSE)
#          err1 <- apply((yte > -1)!=predict(dat.m1, newx=dte, type="class"), 2, mean)
#        }
#        optmstop <- cvm1$lambda.which
#        err.m1[ii, i] <- err1[optmstop]
#        if(ii==1) varid <- names(predict(dat.m1, which=optmstop, type="nonzero")) else
#         varid <- intersect(varid, names(predict(dat.m1, which=optmstop, type="nonzero")))
#        nvar.m1[ii, i] <- length(predict(dat.m1, which=optmstop, type="nonzero"))
#        errbest.m1[ii, i] <- min(err1, na.rm=TRUE)
#        lambest.m1[ii, i] <- which.min(err1)
#        nvarbest.m1[ii, i] <- length(predict(dat.m1, which=which.min(err1), type="nonzero"))
#      }
#      if(ii %% nrun ==0){
#      if(type[k]%in% c("closs", "gloss", "qloss"))
#      cat(paste("\nfamily ", type1[k], ", s=", s[k], sep=""), "\n")
#      else cat(paste("\nfamily ", type1[k], sep=""), "\n")
#      pentype <- switch(penalty[j],
#      "enet"="LASSO",
#      "mnet"="MCP",
#      "snet"="SCAD")
#      cat("penalty=", pentype, "\n")
#      if(penalty[j] %in% c("snet", "mnet")) cat("gamma=", gam, "\n")
#      cat("common variables selected:", varid, "\n")
#      cat("best misclassification error\n")
#      print(round(apply(errbest.m1, 2, summary7),4))
#      cat("which lambda has best error\n")
#      print(round(apply(lambest.m1, 2, summary7),1))
#      cat("number of variables selected with best error\n")
#      print(round(apply(nvarbest.m1, 2, summary7),1))
#      cat("CV based misclassification error\n")
#      print(round(apply(err.m1, 2, summary7),4))
#      cat("number of variables selected by CV\n")
#      print(round(apply(nvar.m1, 2, summary7),1))
#      if(plot.it){
#        par(mfrow=c(2, 1))
#        boxplot(err.m1, main="Misclassification error", subset="", sub=paste(type1[k], "-", pentype, sep=""))
#        boxplot(nvar.m1, main="No. variables", subset="", sub=paste(type1[k], "-", pentype, sep=""))
#      }
#    }
#  }
#  }
#    }
#      print(proc.time()-ptm)

## ----sessionInfo------------------------------------------
sessionInfo();

CCC   Different from midloop, internal subroutine, used only for
C     family!=1;
C     Middle loop: Update the quadratic approximation likelihood
C     input:
C     n
C     m
C     x
C     y
C     mu
C     offset
C     family
C     nlambda
C     lamk
C     alpha
C     gam
C     maxit
C     del
C     thresh
C     wt              used only for family=1
C     trace
C     maxit: not used (and maxit=1) for family=1      
C     output:
C     beta
C     b0
C     yhat
C     dev
C     eta, mu

      subroutine midloopGLM(n,m,x,y,yold,weights, mu, eta, offset,
     +     family, penalty,lamk, 
     +     alpha, gam, theta, rescale, standardize, intercept, eps,
     +     maxit, thresh, nulldev, wt, beta, b0,yhat,dev,trace,convmid, 
     +     satu, ep, pll, activeset, jk)
      
      implicit none
      integer standardize, trace, penalty, maxit, i, j, jj, nmid, n, 
     +     family, m,converged,convmid, satu, intercept,
     +     rescale,activeset(m), jk, ii
      double precision x(n,m),y(n), mu(n), z(n), eta(n), wt(n), w(n), 
     +     del,olddev,weights(n),yold(n), 
     +     thresh, nulldev, dev, theta, thetaold, wtw(n),lamk(m),alpha, 
     +     gam, eps, beta(m), betaold(m), b0, b0old, yhat(n), ep, 
     +     pll(maxit), offset(n)

      do 5 jj=1, maxit
         pll(jj)=0
 5    enddo
      jj = 1
      dev = nulldev
      del = nulldev * 10
      convmid = 0
      satu = 0
 1000 if (jj .LE. maxit .AND. convmid .EQ. 0 .AND. satu .EQ. 0) then
         if(trace.EQ.1)then 
            call intpr("family=", -1, family, 1)
            call intpr("Middle loop: Update the quadratic approximation 
     +likelihood function", -1, 1, 0)
            call intpr(" middle loop iteration jj=", -1, jj, 1)
            call dblepr("convergency criterion at beginning", -1, del,1)
            call intpr("convmid", -1, convmid,1)
         endif
         call DCOPY(m, beta, 1, betaold, 1)
         
         b0old = b0
         thetaold = theta
         call glmlink(n,mu,family,theta,w, ep)
         call zeval(n, y, eta, mu, w, family, z)

         do 10 i=1, n
            wtw(i)=wt(i) * w(i)
            z(i)=z(i) - offset(i)
 10      continue
         call loop_glm(x, y, z, n, m, w, mu, penalty, thresh, eps, 
     +        standardize, intercept,family,beta, b0, lamk, alpha, gam, 
     +        weights,trace,nmid, rescale, converged, theta,
     +        pll(jj),activeset, jk)
         do 220 i = 1, n
            yhat(i) = b0
            do 230 ii = 1, jk
            j=activeset(ii)
                  yhat(i) = yhat(i) + x(i,j) * beta(j)
 230        continue
 220     continue
C        call DCOPY(n, yhat, 1, eta, 1)
         do 350 i = 1, n
            yhat(i) = yhat(i) + offset(i)
            eta(i) = yhat(i)
 350     continue
         call linkinv(n, eta, family, mu)
         olddev = dev
C     compute deviance dev
         call deveval(n, yold, mu, theta, weights, family, dev)
         if(family .EQ. 2)then
           if(dev/nulldev .LT.0.01)then
C           if(dev/nulldev .LT.0.01 .OR. nulldev .LT. 0.01)then
C          because dev in deveval depends on weights, if we scale
C          weights/n, then dev can be very small, but maybe not that
C          important      
C           if(dev/nulldev .LT.0.01 .OR. dev.LT.0.0001)then
C               call dblepr("saturated model, residual deviance = ", -1, 
C     +              dev,1)
C               call dblepr("saturated model, null deviance = ", -1, 
C     +              nulldev, 1)
C               call intpr("family", -1, family, 1)
C                call rwarn("saturated model for family='binomial', exiti
C     +ng coordinate descent iteration. Consider decreasing lambda.zero.m
C     +in.ratio if zipath is called. This may be caused by larger maxit.em
C     in zipath!") 
                satu = 1
C               call DCOPY(m, betaold, 1, beta, 1)
C               b0 = b0old
C               theta = thetaold
            endif
         endif
         del = dabs(dev - olddev)
         convmid = converged
         olddev = dev
         jj=jj + 1
         goto 1000
      endif
      if(trace.EQ.1)then
         call intpr("  Iterations used in the middle loop:", -1, jj-1,1)
         call dblepr("deviance difference at the end of middle loop "
     +        , -1, del, 1)
      endif
      
      return
      end
C     link function for glm model
C     input:
C     n is the length of mu
C     mu(n) is a vector of length n
C     family is an integer: 1 for gaussian, 2 for binomial, 
C     3 for poisson and 4 for negative binomial 
C     theta: a parameter for negative binomial family

C     output:
C     mu
C     w       
      subroutine glmlink(n, mu, family, theta, w, ep)
      
      implicit none
      integer i, n, family
      double precision mu(n), theta, ep, w(n)
C     parameter (ep=1e-5)

      do 10 i=1, n
         if(family.EQ.1)then
            w(i) = 1
         else if(family.EQ.2)then
            if(1-mu(i) .LT. ep)then
               mu(i) = 1
               w(i) = ep
            else if(mu(i) .LT. ep) then
               mu(i) = 0
               w(i) = ep
            else
               w(i)=mu(i)*(1-mu(i))
            endif
         else if(family.EQ.3)then
            w(i) = mu(i)
         else if(family.EQ.4)then
            if(mu(i) .LT. ep)then
               mu(i) = ep
C     else if(mu(i) .GT. 1000)then  ### changed 5/29/15
C     mu(i) = 1000
            endif
            w(i) = mu(i)/(1 + 1/theta * mu(i))
         endif
 10   continue
      
      return 
      end

C     link inverse function
C     input:
C     n
C     eta
C     family is an integer: 1 for gaussian, 2 for binomial, 
C     3 for poisson and 4 for negative binomial 
C     output:
C     mu      
      subroutine linkinv(n, eta, family, mu)

      implicit none
      integer n, family, i
      double precision eta(n), mu(n)
      
      do 10 i= 1, n
         if(family.EQ.1)then
            mu(i) = eta(i)
         else if(family.EQ.2)then
            mu(i) = 1.0D0/(1+exp(-eta(i)))
            if(mu(i) .LT. 1e-5)then
               mu(i) = 1e-5
            else if(mu(i) .GT. 1-1e-5)then
               mu(i) = 1-1e-5
            endif
C         else if(family.EQ.3)then
C            mu(i) = exp(eta(i))
         else if(family.EQ.3 .OR. family.EQ.4)then
            if(eta(i) .LT. log(1e-5))then
               mu(i) = 1e-5
            else
C     else if(eta(i) .LT. log(1000D0))then  ### changed 5/29/15
               mu(i) = exp(eta(i))
C     else 
C     mu(i) = 1000
            endif
         endif
 10   continue
      return
      end

C     evaluate z value
C     input:
C     n
C     y(n)
C     eta(n)
C     mu(n)
C     family is an integer: 1 for gaussian, 2 for binomial, 
C     3 for poisson and 4 for negative binomial 
C     output:
C     z(n)      
      subroutine zeval(n, y, eta, mu, w, family, z)
      
      implicit none
      integer n, i, family
      double precision y(n), eta(n), mu(n), w(n), z(n)

      do 10 i= 1, n
         if(family.EQ.1)then
            z(i) = y(i)
         else if(family.EQ.2)then
            z(i) = eta(i) + (y(i) - mu(i))/w(i)
         else if(family.EQ.3 .OR. family.EQ.4)then
            z(i) = eta(i) + (y(i)-mu(i))/mu(i)
         endif
 10   continue
      return
      end

C     deviance evaluation
C     input:
C     n
C     y(n)
C     mu(n)
C     theta
C     weights(n)
C     family is an integer: 1 for gaussian, 2 for binomial, 
C     3 for poisson and 4 for negative binomial 

C     output
C     dev

      subroutine deveval(n, y, mu, theta, weights, family, dev)
      implicit none

      integer n, family, i
      double precision dev, y(n), mu(n), theta, weights(n),tmp
      integer cisnan
      external cisnan
      
      dev = 0.0D0
      do 10 i=1, n
         if(family.EQ.1)then
            dev = dev + weights(i) * (y(i) - mu(i))**2 
         else if(family.EQ.2)then
            tmp = 0
            if(y(i) .GT. 0)then
               tmp = tmp + y(i)*dlog(y(i))
            endif
            if(mu(i) .GT. 0)then
               tmp = tmp - y(i)*dlog(mu(i))
            endif
            if(1-y(i) .GT. 0)then
               tmp = tmp + (1-y(i))*dlog(1-y(i))
            endif
            if(1-mu(i) .GT. 0)then
               tmp = tmp - (1-y(i))*dlog(1-mu(i))
            endif
            dev = dev + 2*weights(i)*tmp
         else if(family.EQ.3)then
            dev=dev+2*(weights(i)*(-y(i)+mu(i)+y(i)*dlog(max(1.0D0,y(i))
     +           /mu(i))))
         else 
            dev=dev+2*(weights(i)*(y(i)*dlog(max(1.0D0,y(i))/mu(i))-
     +           (y(i)+theta)*dlog((y(i)+theta)/(mu(i) + theta)))) 
         endif
         if(cisnan(dev).NE.0) then
             call intpr("dev is NA in Fortran src/deveval, check (1)if s
     +ome columns of x have the same values;(2)y is large, hence linear 
     +predictor estimate eta is large, then mu=exp(eta) as output of lin
     +kinv call is Inf if family=3 (poisson) or 4 (negbin)", -1, 1, 1)
            call intpr("i=", -1, i, 1)
            call dblepr("y(i)=", -1, y(i), 1)
            call dblepr("mu(i)=", -1, mu(i), 1)
            call dblepr("theta=", -1, theta, 1)
            call dblepr("dev=", -1, dev, 1)
            call rexit("TODO: set upper limit of mu in linkinv")
            exit
         endif
 10   continue
      return
      end

CCC   ref R/loglik.R
      subroutine loglikFor(n, y, mu, theta, w, family, ll)
      implicit none

      integer n, family, i, y0
      double precision ll, y(n), mu(n), theta, w(n)
      double precision rlgamma
      external rlgamma

      ll = 0
      do 10 i=1,n
         if(family .EQ. 4)then  !negbin
            if(y(i) .EQ. 0)then
               y0 = 1
            else 
               y0=0
            endif
            ll=ll + w(i) * (rlgamma(theta+y(i))-rlgamma(theta)-
     +           rlgamma(y(i)+1)+
     +           theta*log(theta) + y(i)*log(mu(i)+y0) - (theta + y(i))*
     +           log(theta +  mu(i)))
         else if(family .EQ. 1)then !gaussian
            ll=ll - w(i)*(y(i)-mu(i))**2 !changed 11/21/15
         else if(family .EQ. 2) then !binomial
            if(mu(i) .GT. 0 .AND. mu(i) .LT. 1)then
               ll=ll + w(i)*(y(i)*log(mu(i)/(1-mu(i)))+log(1-mu(i)))
            endif
         else if(family .EQ. 3) then !poisson
            ll=ll + w(i)*(-mu(i) + y(i)*log(mu(i))-rlgamma(y(i)+1))
         endif
 10   continue
      return
      end
      

C     linear predict value eta = a0 + x * b + offset
C input: n, m, nlambda, x, b, a0, offset, family
C output: eta, mu
      subroutine pred(n,m, nlambda,x,b,a0,offset, family, eta,mu)
      implicit none
      integer n,m,nlambda,family,k,i,j
      double precision b(m,nlambda),a0(nlambda),eta(n,nlambda)
      double precision mu(n,nlambda),x(n,m), offset(n)
      external linkinv

      do 30 k=1,nlambda
         do 20 i=1,n
            eta(i,k) = a0(k) + offset(i)
            do 10 j=1,m
               eta(i,k) = eta(i,k) + x(i,j) * b(j,k)
 10         continue
            call linkinv(1, eta(i,k), family, mu(i,k))
 20      continue
 30   continue
      return
      end

      subroutine penEval(theta, lone, ltwo, gam, penalty, res)
      implicit none
      double precision theta, lone, ltwo, gam, res
      integer penalty
      if(penalty .EQ. 2)then    !mnet
         if(abs(theta) .LE. gam * lone)then 
            res = lone * abs(theta) - theta**2/(2*gam)+0.5*ltwo*theta**2
         else 
            res = 0.5*gam*lone**2 + 0.5*ltwo*theta**2
         endif
      else if(penalty .EQ. 3)then !scad
         if(abs(theta) .LE. lone)then 
            res= lone*abs(theta) + 0.5*ltwo*theta**2
         else if(abs(theta) .LE. gam * lone)then 
            res = ((gam * lone * abs(theta) - 
     +           0.5*(theta**2+lone**2))/(gam-1) + 0.5*ltwo*theta**2)
         else 
            res = lone**2*(gam**2-1)/(2*(gam-1))+0.5*ltwo*theta**2
         endif
      else if(penalty .EQ. 1)then !enet
         res = abs(theta)*lone + 0.5*ltwo*theta**2
      endif

      return
      end

      subroutine penGLM(start, m,lambda, alpha, gam, penalty, pen)
      implicit none
      integer m, j, penalty
      double precision start(m), lambda(m), alpha, gam, res, pen

      pen = 0
      do 10 j=1, m
         call penEval(start(j), lambda(j)*alpha, 
     +        lambda(j)*(1-alpha), gam, penalty, res)
         pen = pen + res
 10   enddo
      
      return
      end

C     used in irglmreg.R ((adpated from nclreg_fortran.f)
C     output: nlambdacal: number of lambda actually computed including
C     repeated ones
      subroutine irglmreg_fortran(x, y, weights, n,m,start, etastart,
     +     mustart, offset, iter, nlambda, lambda, alpha, gam, rescale, 
     +     standardize, intercept, penaltyfactor, maxit, eps, theta, 
     +     penalty, trace,del,cfun, dfun, s,thresh,decreasing, 
     +     active, beta, b0, yhat, los, pll, nlambdacal, delta,
     +     weights_cc)
      implicit none
      integer n,m,i,ii,j,jj,penalty,nlambda, standardize, maxit,
     +     trace, iter, jk, active, activeset(m), intercept,rescale, 
     +     m_act, nlambdacal, uturn, decreasing, cutpoint, cfun,
     +     dfun,dfunnew, 
     +     AllocateStatus, DeAllocateStatus, varsel(m), varsel_old(m)
      double precision x(n, m), y(n), weights(n), weights_update(n), 
     +     weights_cc(n, nlambda), start(m+1),etastart(n),
     +     mustart(n), offset(n), lambda(nlambda), alpha, gam, eps, 
     +     penaltyfactor(m), thresh, beta(m, nlambda),
     +     b0(nlambda), b0_1, yhat(n), del, lambda_i, s,sumw,wt(n),  
     +     fk(n), los(nlambda), pll(nlambda), penval, delta, theta
      double precision, dimension(:, :), allocatable :: x_act
      double precision, dimension(:), allocatable :: start_act, beta_1,
     +     penaltyfactor_act

      if(dfun .EQ. 8)then
         dfunnew= 3
         else if(dfun .EQ. 9)then
          dfunnew= 4
       endif
      m_act = m
      jk = m
      sumw = 0.0D0
       do i=1, n
          sumw=sumw+weights(i)
         enddo
         do i=1, n
          wt(i)=weights(i)/sumw
       enddo

      allocate(start_act(m_act+1), stat=AllocateStatus)
      if(AllocateStatus .NE. 0)then
         return
      endif
      allocate(penaltyfactor_act(m_act), stat=AllocateStatus)
      allocate(x_act(n, m), stat=AllocateStatus)
      if(AllocateStatus .NE. 0)then
         return
      endif
      call copymatrix(n, m, x, x_act)
      allocate(beta_1(m_act), stat=AllocateStatus)
      if(AllocateStatus .NE. 0)then
         return
      endif
      do 5 j=1, m
         beta_1(j)=0
         activeset(j)=j
 5    continue
      call dcopy(m+1, start, 1, start_act, 1)
      call dcopy(m, penaltyfactor, 1, penaltyfactor_act, 1)
      do 101 j=1, m
         varsel_old(j)=j
         varsel(j)=j
 101  continue
      i=1
      nlambdacal=0
      uturn=0
      cutpoint=1
 10   if(i .LE. nlambda)then
         if(cutpoint > 1 .AND. uturn==0 .AND. i < cutpoint)then
            i = cutpoint + 1
         endif
         if(trace .EQ. 1)then
            call intpr("irglmreg_fortran lambda iteration", -1, i, 1)
            if(uturn==1)then
               call intpr("uturn=1", -1, 1, 1)
            endif
         endif
C unlike in nclreg_fortran, no need B: lambda_i=lambda(i)/B
         if(i .EQ. 101)then
             call intpr("i=", -1, i, 1)
         endif
         lambda_i=lambda(i)
         call irglmreg_onelambda(x_act, y,weights, n,m_act,start_act,
     +        etastart, mustart, yhat, offset, lambda_i, alpha, gam, 
     +        rescale, standardize, intercept, penaltyfactor_act, 
     +        maxit, eps, theta, penalty, trace, iter, del, cfun, dfun,
     +        s, thresh, beta_1, b0_1, fk, delta, weights_update)
         nlambdacal=nlambdacal+1
          if(dfun .EQ. 1 .OR. dfun .EQ. 4 .OR. dfun .EQ. 5)then
              call loss2(n, y, fk, wt, cfun, dfun, s, delta, los(i))
          else
          call loss3(n,y,mustart,theta,wt,cfun,dfunnew,s,delta,
     +        los(i))
          endif
         call penGLM(beta_1, m_act, lambda_i*penaltyfactor_act, 
     +        alpha, gam, penalty, penval)
         if(standardize .EQ. 1)then
            pll(i)=los(i) + n*penval
         else 
            pll(i)=los(i) + penval
         endif
         b0(i) = b0_1
         do ii=1, n
            weights_cc(ii,i)=weights_update(ii)
         enddo
         if(jk .GT. 0)then
            do 200 ii = 1, m_act
               beta(varsel(ii), i) = beta_1(ii)
 200        continue
         endif
         if(decreasing==0 .AND. uturn==0 .AND. active .EQ. 1)then
            call find_activeset(m_act, beta_1, eps, activeset, jk)
C     this activeset is relative to the current x_act, but how about
C     relative to x instead? compute varsel for true index in x
            if(jk .NE. m_act .AND. jk .GT. 0)then
               deallocate(start_act, stat=DeAllocateStatus)
               allocate(start_act(jk+1), stat=AllocateStatus)
               deallocate(penaltyfactor_act, stat=DeAllocateStatus)
               allocate(penaltyfactor_act(jk), stat=AllocateStatus)
               start_act(1) = b0_1
               do 35 ii=1, jk
                  start_act(ii+1)=beta_1(activeset(ii))
                  varsel(ii)=varsel_old(activeset(ii))
                  varsel_old(ii)=varsel(ii)
                  penaltyfactor_act(ii)=penaltyfactor(varsel(ii))
 35            continue
               deallocate(beta_1, stat=DeAllocateStatus) 
               allocate(beta_1(jk), stat=AllocateStatus)
               do 37 ii=1, jk
               beta_1(ii)=0
 37            continue
               deallocate(x_act, stat=DeAllocateStatus)
               allocate(x_act(n, jk), stat=AllocateStatus)
C     update x_act matrix
               do 55 jj=1, n
                  do 45 ii=1, jk
                     x_act(jj, ii) = x(jj, varsel(ii))
 45               continue
 55            continue
               m_act = jk
            endif
         endif
C     redo (i-1)-lambda estimates with the current start_act for the
C     i-th lambda. Note, i=i-2 not i-1 will do this since i=i+1 is
C     computed before the end of the loop.
C      if(i > 1 .AND. uturn==0 .AND. cutpoint==1)then
C          if(abs(los(i)-los(i-1))/los(i) > epscycle)then
C            do j=1, m_act
C            start_act(j)=0
C            enddo
C            cutpoint = i
C            uturn=1
C          endif
C     endif
C      if(uturn==1)then
C         i = i - 2
C      endif
C      if(i==0 .AND. uturn==1)then
C         uturn=0
C      endif
      i = i + 1
      goto 10
      endif
      deallocate(beta_1, start_act, x_act, penaltyfactor_act)

      return
      end

### SVM based on composite operator
irsvm <- function(x, ...) UseMethod("irsvm")

irsvm.default <- function(x, ...) {
    if (extends(class(x), "Matrix"))
        return(irsvm.matrix(x = x, ...))
    if (inherits(x, "data.frame"))
        return(irsvm.matrix(x = x, ...))
    if (inherits(x, "numeric") && is.null(dim(x)))
        return(irsvm.matrix(x = as.matrix(x), ...))
    stop("no method for objects of class ", sQuote(class(x)),
         " implemented")
}

irsvm.formula <- function(formula, data, weights, contrasts=NULL, ...){
    ## extract x, y, etc from the model formula and frame
    if(missing(data)) data <- environment(formula)
    mf <- match.call(expand.dots = FALSE)
    m <- match(c("formula", "data", "weights"), names(mf), 0L)
    mf <- mf[c(1L, m)]
    mf$drop.unused.levels <- TRUE
    mf[[1L]] <- as.name("model.frame")
    mf <- eval(mf, parent.frame())

    mt <- attr(mf, "terms") # allow model.frame to have updated it

    Y <- model.response(mf, "any") # e.g. factors are allowed
    ## avoid problems with 1D arrays, but keep names
    if(length(dim(Y)) == 1L) {
        nm <- rownames(Y)
        dim(Y) <- NULL
        if(!is.null(nm)) names(Y) <- nm
    }
    ## null model support
    X <- if (!is.empty.model(mt)) model.matrix(mt, mf, contrasts) else matrix(,NROW(Y), 0L)
    ## avoid any problems with 1D or nx1 arrays by as.vector.
    weights <- as.vector(model.weights(mf))
    if(!length(weights)) weights <- rep(1, nrow(mf))
    else if(any(weights < 0)) stop("negative weights not allowed")
    if(!is.null(weights) && !is.numeric(weights))
        stop("'weights' must be a numeric vector")
    if(length(weights) != length(Y))
        stop("'weights' must be the same length as response variable")	 
    RET <- irsvm_fit(X, Y, weights,...)
    RET$call <- match.call()
    RET <- c(RET, list(formula=formula, terms = mt, data=data,
                       contrasts = attr(X, "contrasts"),
                       xlevels = .getXlevels(mt, mf)))
    class(RET) <- "wsvm"
    RET
}
irsvm.matrix <- function(x, y, weights, ...){
    RET <- irsvm_fit(x, y, weights,...)
    RET$call <- match.call()
    return(RET)
}

irsvm_fit <- function(x,y, weights, cfun="ccave", s=NULL, delta=0.0001, type=NULL, 
               kernel      = "radial",
               #degree      = 3,
               #gamma       = if (is.vector(x)) 1 else 1 / ncol(x),
               #coef0       = 0,
               cost        = 1,
               #class.weights = NULL,
               #cachesize   = 100,
               #tolerance   = 0.001,
               epsilon     = 0.1,
               #shrinking   = TRUE,
               #probability = FALSE,
               #fitted      = TRUE,
               #subset,
               #na.action,
               iter=10, reltol=1e-5, trace=FALSE,
               ...){
    call <- match.call()
     ## determine model type
         if (is.null(type)) type <-
           if (is.factor(y)) "C-classification"
         else "eps-regression"
    cfunold <- cfun
    cfun <- cfun2num(cfun)
    dfun <- switch(type,
                   "C-classification"=6,
                   "nu-classification"=6,
                   "eps-regression"=2,
                   "nu-regression"=2)
    if(dfun==6){
           if (is.factor(y)) {
                 y <- as.integer(y)
             } else {
                     if(any(as.integer(y) != y))
                         stop("y variable has to be of factor or         integer type for classification\n")
                     y <- as.factor(y)
                     y <- as.integer(y)
             }
    if(length(unique(y))!=2) stop("y must be a binary variable for classification\n")
    else if(!all(names(table(y)) %in% c(1, -1))){
        y[y==1] <- -1
        y[y==2] <- 1
        } 
    }
    if(is.null(s)) s <- assign_s(cfun, y) else check_s(cfun, dfun=6, s)
    if(cfun==6)
           if(s > 1) delta <- (s-1)/2 else
           if(s==1) delta <- 0 else{
             if(is.null(delta)) stop("delta must be provided")
             if(delta <= 0) stop("delta must be positive")
           }
    ## avoid any problems with 1D or nx1 arrays by as.vector.
    if(missing(weights)) weights=rep(1,length(y))
    weights <- as.vector(weights)
    if(!is.null(weights) && !is.numeric(weights))
        stop("'weights' must be a numeric vector")
    ## check weights 
    if( !is.null(weights) && any(weights < 0) ){
        stop("negative weights not allowed")
    }
    if(length(weights) != length(y))
        stop("'weights' must be the same length as response variable")
    n <- length(y)
    if(all(x[,1]==1)) xtmp <- x[,-1] else xtmp <- x
    d1 <- 10 
    k <- 1
    if(trace) {
        cat("\niterations ...\n")
    }
    los <- rep(NA, iter)
    weights_update <- weights
    subset <- 1:n
    while(d1 > reltol && k <= iter){
        RET <- wsvm(xtmp, y, weights*weights_update, type=type, kernel=kernel, cost=cost, epsilon=epsilon, cross=0, subset=subset, ...)
        pred <- predict(RET, xtmp, decision.values=TRUE)
        if(dfun==6){
        fitted.values <- attr(pred, "decision.values")
         if(y[subset[1]]==-1)
            fitted.values <- -fitted.values
        }else fitted.values <- pred
        weights_update <- .Fortran("update_wt_ccsvm",
                                   n=as.integer(n),
                                   weights=as.double(weights),
                                   y=as.double(y),
                                   f=as.double(fitted.values),
                                   cfun=as.integer(cfun),
                                   dfun=as.integer(dfun),
                                   s=as.double(s),
                                   eps=as.double(epsilon),
                                   delta=as.double(delta),
                                   weights_update=as.double(rep(0, n)),
                                   PACKAGE="mpath")$weights_update
        subset <- which(weights_update > 0)
        if(trace && kernel=="linear"){
            beta <- t(RET$coefs) %*% as.matrix(xtmp[RET$index,]) #kernel="linear" only: convert dual coef to RET$coefs to primal coef
                                        #RET$b0 <- -RET$rho #intercept
###for some reason, depending on the sign of the first element of y, in wsvm, version 1.7-5 or svm version 1.7.3, beta and b0 are minus corresponding values coef_ and intercetp_ in Python scikit.learn.svm.SVC version xxx. Likewise, fitted values have inverse sign
        ### furthermore, it is questionable on the beta value from wsvm sometimes, but fitted values seem ok.
            if(dfun==6 && y[1]==-1)
                beta <- -beta
            los[k] <- .Fortran("loss2_ccsvm",
                               n=as.integer(n),
                               y=as.double(y),
                               f=as.double(fitted.values),
                               weights=as.double(weights),
                               cfun=as.integer(cfun),
                               dfun=as.integer(dfun),
                               s=as.double(s),
                               eps=as.double(epsilon),
                               delta=as.double(delta),
                               los=as.double(0.0),
                               PACKAGE="mpath")$los
            los[k] <- los[k]+0.5/cost*sum(beta^2) ### only for kernel="linear"
            cat("\niteration", k, ", robust loss value", los[k], "\n") 
            if(k > 1){
                d1 <- abs((los[k]-los[k-1])/los[k-1])
                if(los[k] > los[k-1]){
                    k <- iter
                }
            }
        }else{
            if(k > 1){
                d1 <- sum((fitold-fitted.values)^2)/sum(fitted.values^2)
            }
            fitold <- fitted.values
        }
        k <- k + 1
	if(trace) cat("d1=", d1, ", k=", k, ", d1 > reltol && k <= iter: ", (d1 > reltol && k <= iter), "\n")
    }
    RET$cfun <- cfunold
    RET$s <- s
    RET$delta <- delta
    RET$weights_update <- weights_update
    RET
}

C update weights
C output: weights_update
      subroutine update_wt(n, weights, y, f, cfun, dfun, s, delta,
     +     weights_update)
       implicit none
       integer cfun, dfun, i, n
       double precision weights(n),weights_update(n), y(n),f(n),
     + u(n),z(n), v(n), s, delta
       external compute_v, compute_z, compute_u 
 
C     compute u
             call compute_u(dfun, n, y, f, u)
C     compute z=s(u), s is not used for the selected dfun
             call compute_z(dfun, n, u, z, s)
C     compute derivative of -g(z)
             call compute_v(cfun, n, z, s, delta, v)
C     update weights
             do 10 i=1, n
              weights_update(i)=-weights(i)*v(i)
10         continue
      return
      end

C compute density function of Poisson distribution with parameter lambda
C if log_true=1, log density value is computed
      function dpois(x, lambda, log_true)
      integer :: x, j, log_true, Factorial
      double precision :: lambda, dpois, res
      external :: Factorial

      if(lambda .LT. 0)then
              call dblepr("lambda should be nonnegative", -1, lambda, 1)
      endif
      if(log_true==1)then
              res=0
          if(x > 0)then 
              do j=1, x
               res=res+dlog(j*1.0D0)
              enddo
          endif
        dpois = -lambda+x*1.0D0*dlog(lambda)- res
      else 
              dpois = dexp(-lambda)*lambda**x/Factorial(x)
      endif
      return
      end function

C     see vignettes\kkt.pdf for how to find the lambda_max for both the
C     original loss and in EM algorithm 
C     For the EM algorith, uncomment related lines with respect to derz1 and
C     derx1
C     inputs: B with # of columns kx, G with # of columns kz are
C     count-model and zero-model design matrix with the first columns 1s
C     for intercept, this makes the subroutine valid even if there is an
C     intercept-only component in count-model or zero-model.
C     outputs: lmax_count, lmax_zero
      subroutine lmax_zipath(B, G, y, y1, weights, n, kx, kz, family, 
     +     coefc, coefz, alpha_count, alpha_zero, penaltyfactor_count, 
     +     penaltyfactor_zero, theta, lmax_count, lmax_zero)
      implicit none
      integer i, j, k, n, kx, kz, family, y1(n)
      double precision B(n, kx), G(n, kz), y(n), weights(n), coefc,
     +     coefz, alpha_count, alpha_zero, penaltyfactor_zero(kz), 
     +     penaltyfactor_count(kx), lmax_count, lmax_zero, derz(kz),
     +     derx(kx), penfac_zero(kz), penfac_count(kx), sumw, sumx, 
     +     sumz, p, derip, mu, a, c, theta, z(n)
C     +     , derz1(kz), derx1(kx)
      
      if(kz<1)then
           call rexit("zero model design matrix needs at least one
     +          column including intercept")
      endif
      if(kx<1)then
           call rexit("count model design matrix needs at least one
     +          column including intercept")
      endif
      do j=1, kz
         derz(j)=0
C     derz1(j)=0
      enddo
      do k=1, kx
         derx(k)=0
C     derx1(k)=0
      enddo
      sumw = sum(weights)
      do i=1, n
         weights(i)=weights(i)/sumw
      enddo
      sumx=sum(penaltyfactor_count)
      sumz=sum(penaltyfactor_zero)
      p=dexp(coefz)/(1+dexp(coefz))
      mu=dexp(coefc)
      if(family==3)then
         a=  dexp(coefz)/(dexp(coefz)+dexp(-dexp(coefc)))
         c=  dexp(-dexp(coefc))*
     +        (-dexp(coefc))/(dexp(coefz)+dexp(-dexp(coefc)))
         if(kz > 1)then
            do 10 j=2, kz
               do 20 i=1, n
                  if(y1(i)==0)then
                     derz(j)=derz(j)+weights(i)*a*G(i,j)
                  endif
                  derz(j)=derz(j)-weights(i)*p*G(i,j)
 20            enddo
C               if(penaltyfactor_zero(j) > 0.0d0)then
                 derz(j)=dabs(derz(j))
                 penfac_zero(j)=penaltyfactor_zero(j)/sumz*kz
                 derz(j)=derz(j)/(penfac_zero(j)*alpha_zero)
C               endif
 10         enddo
         endif
         if(kx > 1)then
            do 30 j=2, kx
               do 40 i=1, n
                  if(y1(i)==0)then
                     derx(j)=derx(j)+weights(i)*c*B(i,j)
                  else
                     derx(j)=derx(j)+weights(i)*(y(i)*B(i,j)
     +                    -mu*B(i,j))
                  endif
 40            enddo
C               if(penaltyfactor_count(j) > 0.0d0)then
                derx(j)=dabs(derx(j))
                penfac_count(j)=penaltyfactor_count(j)/sumx*kx
                derx(j)=derx(j)/(penfac_count(j)*alpha_count)
C               endif
 30         enddo
         endif
      else if(family==4)then
         a=(theta/(mu+theta))**theta
         do i=1, n
            if(y1(i)==0)then
               z(i)=1/(1+dexp(-coefz)*a)
            else 
               z(i)=0
            endif
         enddo
         if(kz > 1)then
            do 110 j=2, kz
               do 120 i=1, n
C     derivative of p
                  derip=G(i,j)*p**2/dexp(coefz)
                  if(y1(i)==0)then
                     derz(j)=derz(j)+weights(i)*derip*(1-a)/(p+(1-p)*a)
                  else
                     derz(j)=derz(j)-weights(i)*derip/(1-p)
                  endif
C     derz1(j)=derz1(j)+weights(i)*(z(i)*G(i,j)-p*G(i,j))
 120           enddo
C               if(penaltyfactor_zero(j) > 0.0d0)then
                derz(j)=dabs(derz(j))
                penfac_zero(j)=penaltyfactor_zero(j)/sumz*kz
                derz(j)=derz(j)/(penfac_zero(j)*alpha_zero)
C               endif
               
C     derz1(j)=dabs(derz1(j))
C     derz1(j)=derz1(j)/(penfac_zero(j)*alpha_zero)
 110        enddo
         endif
         if(kx > 1)then
            do 130 j=2, kx
               do 140 i=1, n
C     derivative of mu
                  derip=B(i,j)*mu
                  if(y1(i)==0)then
                     derx(j)=derx(j)-weights(i)*(1-p)*derip*theta*
     +                    a/(mu+theta)/(p+(1-p)*a)
                  else
                     derx(j)=derx(j)+weights(i)*derip*
     +                    (y(i)/mu-(y(i)+theta)/(mu+theta))
                  endif
C     derx1(j)=derx1(j)+weights(i)*(1-z(i))*derip*
C     +       (y(i)/mu-(y(i)+theta)/(mu+theta))
 140           enddo
C               if(penaltyfactor_count(j) > 0.0d0)then
                derx(j)=dabs(derx(j))
                penfac_count(j)=penaltyfactor_count(j)/sumx*kx
                derx(j)=derx(j)/(penfac_count(j)*alpha_count)
C               endif
C     derx1(j)=dabs(derx1(j))
C     derx1(j)=derx1(j)/(penfac_count(j)*alpha_count)
 130        enddo
         endif
      endif

      lmax_count = maxval(derx)
      lmax_zero = maxval(derz)
C      call dblepr("lmax_count", -1, lmax_count, 1)
C     call dblepr("lmax_count with EM", -1, maxval(derx1), 1)
C      call dblepr("lmax_zero", -1, lmax_zero, 1)
C     call dblepr("lmax_zero with EM", -1, maxval(derz1), 1)
      return
      end

#compute updated weights
#input: z value from convex function dfun
#outpu: updated weights
### this is different from R/update_wt in that the inputs are different 
compute_wt <- function(z, weights, cfun, s, delta=0.0001){
    if(is.null(s)) s <- assign_s(cfun, z) else check_s(cfun, dfun=1, s) #for now forget about dfun
    if(missing(weights)) weights <- rep(1, length(z))
    if(cfun==6)
           if(s > 1) delta <- (s-1)/2 else
           if(s==1) delta <- 0 else{
             if(is.null(delta)) stop("delta must be provided")
             if(delta <= 0) stop("delta must be positive")
           }
    n <- length(z)
    v <- .Fortran("compute_v",
             cfun=as.integer(cfun),
             n=as.integer(n),
             z=as.double(z),
             s=as.double(s),
             delta=as.double(delta),
             v=as.double(rep(0, n)),
             PACKAGE="mpath")$v
    -v*weights
}

cv.zipath <- function(X, ...) UseMethod("cv.zipath", X)
 
cv.zipath.default <- function(X, ...) {
     if (extends(class(X), "Matrix"))
         return(zipath.matrix(X=X, ...))
     stop("no method for objects of class ", sQuote(class(X)),
          " implemented")
 }
 
cv.zipath.formula <- function(formula, data, weights, offset=NULL,                 contrasts=NULL, ...){
     ## call and formula
     cl <- match.call()
     if(!attr(terms(formula), "intercept"))
         stop("non-intercept model is not implemented")
     if(missing(data)) data <- environment(formula)
     mf <- match.call(expand.dots = FALSE)
     m <- match(c("formula", "data", "na.action", "weights", "offset"), names(mf), 0)
     mf <- mf[c(1, m)]
     mf$drop.unused.levels <- TRUE
 
     ## extended formula processing
     if(length(formula[[3]]) > 1 && identical(formula[[3]][[1]], as.name("|")))
     {
         ff <- formula
         formula[[3]][1] <- call("+")
         mf$formula <- formula
         ffc <- . ~ .
         ffz <- ~ .
         ffc[[2]] <- ff[[2]]
         ffc[[3]] <- ff[[3]][[2]]
         ffz[[3]] <- ff[[3]][[3]]
         ffz[[2]] <- NULL
     } else {
         ffz <- ffc <- ff <- formula
         ffz[[2]] <- NULL
     }
     if(inherits(try(terms(ffz), silent = TRUE), "try-error")) {
         ffz <- eval(parse(text = sprintf( paste("%s -", deparse(ffc[[2]])), deparse(ffz) )))
     }
 
     ## call model.frame()
     mf[[1]] <- as.name("model.frame")
     mf <- eval(mf, parent.frame())
     ## extract terms, model matrices, response
     mt <- attr(mf, "terms")
     mtX <- terms(ffc, data = data)
     X <- model.matrix(mtX, mf)
     Xold <- X
 ### This means X possibly has intercept (1st) column depending on the specifications 
mtZ <- terms(ffz, data = data)
     mtZ <- terms(update(mtZ, ~ .), data = data)
     Z <- model.matrix(mtZ, mf)
     Zold <- Z
     Y <- model.response(mf, "numeric")
     ## weights and offset
     n <- length(Y)
     weights <- model.weights(mf)
     if(is.null(weights)) weights <- rep.int(1, n)
     offsetx <- model_offset_2(mf, terms = mtX, offset = TRUE)
     if(is.null(offsetx)) offsetx <- 0
   if(length(offsetx) == 1) offsetx <- rep.int(offsetx, n)
   offsetx <- as.vector(offsetx)
   offsetz <- model_offset_2(mf, terms = mtZ, offset = FALSE)
   if(is.null(offsetz)) offsetz <- 0
   if(length(offsetz) == 1) offsetz <- rep.int(offsetz, n)
   offsetz <- as.vector(offsetz)
     RET <- cv.zipath_fit(X, Z, Y, weights=weights, offsetx=offsetx, offsetz=offsetz, ...)
     RET$call <- match.call()
     RET$terms = list(count = mtX, zero = mtZ, full = mt)
     RET$call = cl
     RET$formula = ff
     RET$levels = .getXlevels(mt, mf)
     RET$contrasts = list(count = attr(X, "contrasts"), zero = attr(Z,            "contrasts"))
     RET$model <- mf
     RET

}
cv.zipath.matrix <- function(X, Z, Y, weights, offsetx=NULL, offsetz=NULL, ...){
     RET <- cv.zipath_fit(X, Z, Y, weights, offsetx=offsetx, offsetz=offsetz, ...)
     RET$call <- match.call()
     return(RET)
 }

cv.zipath_fit <- function(X, Z, Y, weights, offsetx, offsetz, nlambda=100, lambda.count=NULL, 
		      lambda.zero=NULL, nfolds=10, foldid, plot.it=TRUE, se=TRUE, n.cores=2, 
              trace=FALSE, parallel=FALSE, ...){
    call <- match.call()
    if(missing(foldid) && nfolds < 3)
        stop("smallest nfolds should be 3\n")
    n <- length(Y)
    K <- nfolds
    ### only to obtain lambda values
    zipath.obj <- zipath_fit(X, Z, Y, weights, offsetx, offsetz, nlambda=nlambda, lambda.count=lambda.count, lambda.zero=lambda.zero, maxit=1, ...)
    #zipath.obj <- do.call("zipath", list(formula, data, weights, nlambda=nlambda, lambda.count=lambda.count, lambda.zero=lambda.zero, ...))
    if(is.null(lambda.count) || is.null(lambda.zero)){
    lambda.count <- zipath.obj$lambda.count
    lambda.zero <- zipath.obj$lambda.zero
    nlambda <- zipath.obj$nlambda
    }
    else nlambda <- length(lambda.count)
    if(missing(foldid))
        all.folds <- cv.folds(n, K)
    else {
        all.folds <- foldid
        K <- nfolds <- length(foldid)
    }
    bic <- matrix(NA, nlambda, K)
    if(parallel){
    cl <- eval(parse(text="parallel:::makeCluster(n.cores)"))
    registerDoParallel(cl)
    i <- 1  ###needed to pass R CMD check with parallel code below
    residmat <- foreach(i=seq(K), .combine=cbind, .packages='mpath') %dopar% {
        omit <- all.folds[[i]]
    #	fitcv <- do.call("zipath", list(formula, data[-omit,], weights[-omit], lambda.count=lambda.count, lambda.zero=lambda.zero, nlambda=nlambda, ...))
        fitcv <- zipath_fit(X[-omit,,drop=FALSE], Z[-omit,,drop=FALSE], Y[-omit], weights[-omit], offsetx[-omit], offsetz[-omit], nlambda=nlambda, lambda.count=lambda.count, lambda.zero=lambda.zero, ...)
        logLik(fitcv, X=X[omit,, drop=FALSE], Z=Z[omit,, drop=FALSE], y=Y[omit], weights=weights[omit], offsetx=offsetx[omit], offsetz=offsetz[omit])
    }
    eval(parse(text="parallel:::stopCluster(cl)"))
    }
    else{
     residmat <- matrix(NA, nlambda, K)
     for(i in seq(K)) {
       if(trace)
         cat("\n CV Fold", i, "\n\n")
       omit <- all.folds[[i]]
       fitcv <- zipath_fit(X[-omit,,drop=FALSE], Z[-omit,,drop=FALSE], Y[-omit], weights[-omit], offsetx[-omit], offsetz[-omit], nlambda=nlambda, lambda.count=lambda.count, lambda.zero=lambda.zero, ...)
       #fitcv <- do.call("zipath", list(formula, data[-omit,], weights[-omit], lambda.count=lambda.count, lambda.zero=lambda.zero, nlambda=nlambda, ...))
       residmat[, i] <- logLik(fitcv, X=X[omit,, drop=FALSE], Z=Z[omit,, drop=FALSE], y=Y[omit], weights=weights[omit], offsetx=offsetx[omit], offsetz=offsetz[omit])
     }
    }
    cv <- apply(residmat, 1, mean)
    cv.error <- sqrt(apply(residmat, 1, var)/K)
    lambda.which <- which.max(cv)
    obj<-list(residmat=residmat, bic=bic, cv = cv, cv.error = cv.error, foldid=all.folds, nlambda=nlambda, lambda.which= lambda.which, lambda.optim = list(count=lambda.count[lambda.which], zero=lambda.zero[lambda.which]))
    #obj<-list(fit=zipath.obj, residmat=residmat, bic=bic, cv = cv, cv.error = cv.error, foldid=all.folds, nlambda=nlambda, lambda.which= lambda.which, lambda.optim = list(count=lambda.count[lambda.which], zero=lambda.zero[lambda.which]))
    class(obj) <- c("cv.zipath")
    if(plot.it) plot(obj,se=se)
    obj
}

"plot.cv.zipath" <-
    function(x,se=TRUE,ylab=NULL, main=NULL, width=0.02, col="darkgrey", ...){
        nlambda <- x$nlambda
        cv <- x$cv
        cv.error <- x$cv.error
        if(is.null(ylab))
            ylab <- "log-likelihood"
        plot(1:nlambda, cv, type = "b", xlab = "Index of lambda pair", ylab= ylab, ylim = range(cv, cv + cv.error, cv - cv.error), main=main)
if(se)
    error.bars(1:nlambda, cv + cv.error, cv - cv.error,
               width = width, col=col)

invisible()
}

predict.cv.zipath <- function(object, newdata, ...)
predict(object$fit, newdata, which=object$lambda.which, ...)

coef.cv.zipath <- function(object, which=object$lambda.which, model = c("full", "count", "zero"), ...) {
    model <- match.arg(model)
    rval <- object$fit$coefficients
    rval <- switch(model,
                   "full" = list(count=rval$count[, which], zero=rval$zero[, which]),
                   "count" = rval$count[,which],
                   "zero" = rval$zero[,which])
    rval
}

### penalized optimization for regression and classification
irglmreg <- function(x, ...) UseMethod("irglmreg")

irglmreg.default <- function(x, ...) {
    if (extends(class(x), "Matrix"))
        return(irglmreg.matrix(x = x, ...))
    stop("no method for objects of class ", sQuote(class(x)),
         " implemented")
}

irglmreg.formula <- function(formula, data, weights, offset=NULL, contrasts=NULL, ...){
    ## extract x, y, etc from the model formula and frame
    if(!attr(terms(formula, data=data), "intercept"))
        stop("non-intercept model is not implemented")
    if(missing(data)) data <- environment(formula)
    mf <- match.call(expand.dots = FALSE)
    m <- match(c("formula", "data", "weights",
                 "offset"), names(mf), 0L)
    mf <- mf[c(1L, m)]
    mf$drop.unused.levels <- TRUE
    mf[[1L]] <- as.name("model.frame")
    mf <- eval(mf, parent.frame())

    mt <- attr(mf, "terms") # allow model.frame to have updated it

    Y <- model.response(mf, "any") # e.g. factors are allowed
    ## avoid problems with 1D arrays, but keep names
    if(length(dim(Y)) == 1L) {
        nm <- rownames(Y)
        dim(Y) <- NULL
        if(!is.null(nm)) names(Y) <- nm
    }
    ## null model support
    X <- if (!is.empty.model(mt)) model.matrix(mt, mf, contrasts) else matrix(,NROW(Y), 0L)
    ## avoid any problems with 1D or nx1 arrays by as.vector.
    weights <- as.vector(model.weights(mf))
    if(!length(weights)) weights <- rep(1, nrow(mf))
    else if(any(weights < 0)) stop("negative weights not allowed")
    if(!is.null(weights) && !is.numeric(weights))
        stop("'weights' must be a numeric vector")
    if(length(weights) != length(Y))
        stop("'weights' must be the same length as response variable")

    offset <- as.vector(model.offset(mf))
    if(!is.null(offset)) {
        if(length(offset) != NROW(Y))
            stop(gettextf("number of offsets is %d should equal %d (number of observations)", length(offset), NROW(Y)), domain = NA)
    }

    RET <- irglmreg_fit(X[,-1], Y, weights=weights, offset=offset, ...)
    RET$call <- match.call()
    RET <- c(RET, list(formula=formula, terms = mt, data=data,
                       contrasts = attr(X, "contrasts"),
                       xlevels = .getXlevels(mt, mf)))
    class(RET) <- "irglmreg"
    RET
}
irglmreg.matrix <- function(x, y, weights, offset=NULL, ...){
    RET <- irglmreg_fit(x, y, weights, offset=offset, ...)
    RET$call <- match.call()
    return(RET)
}

irglmreg_fit <- function(x,y, weights, offset, cfun="ccave", dfun="gaussian", 
                      s=NULL, delta=0.1, fk=NULL, iter=10, reltol=1e-5, penalty=c("enet","mnet","snet"), nlambda=100, lambda=NULL, type.path=c("active", "nonactive"), decreasing=TRUE, lambda.min.ratio=ifelse(nobs<nvars,.05, .001),alpha=1, gamma=3, rescale=TRUE, standardize=TRUE, intercept=TRUE, penalty.factor = NULL, maxit=1000, type.init=c("bst", "co", "heu"), init.family=NULL, mstop.init=10, nu.init=0.1, eps=.Machine$double.eps, epscycle=10, thresh=1e-6, parallel=FALSE, n.cores=2, theta, trace=FALSE, tracelevel=1){
### compute h value
    compute.h <- function(rfamily, y, fk_old, s, B){
        if(rfamily=="clossR")
            h <- gradient(family=rfamily, u=y-fk_old, s=s)/B+fk_old
        else if(rfamily %in% c("closs", "gloss", "qloss"))
            h <- -y*gradient(family=rfamily, u=y*fk_old, s=s)/B+fk_old
        h
    }
    dfunold2 <- dfun
    dfunold <- dfun[[1]]
    ### there will be no gaussianC to distinguish
    if(dfunold %in%c("gaussian","gaussianC") || dfunold %in%c(1,4)) rescale <- FALSE
    cfunold <- cfun
    cfun <- cfun2num(cfun)
    dfun <- dfun2num(dfunold)
    if(!(cfun %in% 1:8)) stop("cfun is not implemented\n")
    if(!(dfun %in% c(1, 4, 5, 8, 9))) stop("dfun is not implemented\n")
    if(dfun==1 | dfun==4) dfunnew <- "gaussian"
    else dfunnew <- dfunold
     if(dfunold=="poisson"){
         theta <- 1 ### not used anyway
         family <- 3
     }else if(dfunold=="negbin"){
         if(missing(theta)) stop("theta has to be provided for family=negbin()")
         family <- 4
     }else theta <- 0 # not used
    #else if(dfun==5) dfunnew <- "binomial"
    #else stop("not implemented yet for dfun=", dfun, "\n")
    call <- match.call()
    penalty <- match.arg(penalty)
    type.path <- match.arg(type.path)
    type.init <- match.arg(type.init)
    if(type.path=="active")
        active <- 1
    else active <- 0
    if (!is.null(lambda) && length(lambda) > 1 && all(lambda == cummin(lambda))){
        decreasing <- TRUE
    }
    else if(!is.null(lambda) && length(lambda) > 1 && all(lambda == cummax(lambda)))
        decreasing <- FALSE
    if(!is.null(init.family)) rfamily <- init.family else{
                                                         if(cfun==4 && dfun==1) rfamily <- "clossR"
                                                         else if(cfun==4 &&     dfun==4) rfamily <- "closs"
                                                         else rfamily <- "gaussian"
                                                     }
    if(dfun %in% 4:6)
        y <- y2num(y)
    if(!is.matrix(x)) x <- matrix(x)
    nm <- dim(x)
    nobs <- n <- nm[1]
    if(length(y)!=n) stop("length of y is different from row of x\n")
    nvars <- m <- nm[2]
    if(missing(weights)) weights=rep(1,nobs)
    weights <- as.vector(weights)
    w <- weights/sum(weights)
    if(!is.null(weights) && !is.numeric(weights))
        stop("'weights' must be a numeric vector")
    ## check weights and offset
    if( !is.null(weights) && any(weights < 0) ){
        stop("negative weights not allowed")
    }
    if (is.null(offset)){
        is.offset <- FALSE
        offset <- rep.int(0, nobs)
    }else is.offset <- TRUE
    pentype <- switch(penalty,
                      "enet"=1,
                      "mnet"=2,
                      "snet"=3)

    if(is.null(penalty.factor))
        penalty.factor <- rep(1, nvars)
    if(all(penalty.factor==0)){	
        lambda <- rep(0, nvars) 
        penalty.factor <- rep(1, nvars)
    }
    xold <- x
    if(is.null(s) || is.na(s)) s <- assign_s(cfun, y) else check_s(cfun, dfunold, s)
    if(cfun==6)
        if(s > 1) delta <- (s-1)/2 else
                                       if(s==1) delta <- 0 else{
                                                               if(is.null(delta)) stop("delta must be provided")
                                                               if(delta <= 0) stop("delta must be positive")
                                                           }
    penfac <- penalty.factor/sum(penalty.factor) * nvars
    zscore <- function(rfamily, RET, s){
        if(rfamily=="clossR"){
            z <- gradient(family=rfamily, u=y-RET$fitted.values, s=s)
            scores <- abs(crossprod(x, w*z))/(penfac*alpha)
        } 
        else if(rfamily=="closs"){ 
            z <- gradient(family=rfamily, u=y*RET$fitted.values, s=s)
            scores <- abs(crossprod(x, w*(y*z)))/(penfac*alpha) ### note y=1/-1, thus can be further simplified without y
        }
    }
### initiate predictive value
    start <- NULL
    if(is.null(fk) || is.null(lambda)){
        if(type.init %in% c("co", "heu")){ ### use co function to generate intercept-only model
            RET <- irglm(y~1, data=data.frame(cbind(y, rep(1, n))), iter=10000, reltol=1e-20, weights=weights, s=s, cfun=cfun, dfun=dfunold2, init.family=init.family, trace=FALSE)
            if(type.init=="co") start <- c(coef(RET), rep(0, nvars))
### end of intercept computing
            else if(type.init=="heu"){ ### heuristic for choosing starting values
                v <- zscore(rfamily, RET, s)
                ix <- which(v >= quantile(v, 0.9))
                b0.1 <- coef(RET)
                beta.1 <- rep(0, nvars)
                beta.1[ix] <- 1
                start <- c(b0.1, beta.1)
                RET$fitted.values <- x %*% beta.1 + b0.1
            }
        }
        else if(type.init=="bst") ### use bst function to generate results
        {
            RET <- bst(x, y, family=rfamily, ctrl = bst_control(mstop=mstop.init, nu=nu.init, s=s, intercept=TRUE))
            RET$fitted.values <- RET$yhat
            RET$weights_update <- weights_update <- weights
            #.Fortran("update_wt",
            #                                                 n=as.integer(n),
            #                                                 weights=as.double(weights),
            #                                                 y=as.double(y),
            #                                                 f=as.double(RET$yhat),
            #                                                 cfun=as.integer(cfun),
            #                                                 dfun=as.integer(dfun),
            #                                                 s=as.double(s),
            #                                                 delta=as.double(delta),
            #                                                 weights_update=as.double(rep(0, n)),
            #                                                 PACKAGE="mpath")$weights_update
            start <- c(attributes(coef(RET))$intercept, coef(RET))
        }
    }
    else {
        RET <- NULL
        RET$fitted.values <- fk
    }
    #los <- .Fortran("loss2",
    #                n=as.integer(n),
    #                y=as.double(y),
    #                f=as.double(RET$fitted.values),
    #                weights=as.double(weights/n),
    #                cfun=as.integer(cfun),
    #                dfun=as.integer(dfun),
    #                s=as.double(s),
    #                delta=as.double(delta),
    #                los=as.double(0.0),
    #                PACKAGE="mpath")$los
    if(dfun==5) ytmp <- (y+1)/2 else ytmp <- y
    if(is.null(lambda)){
### method A, to obtain lambda values from fitting the penalized regression. 
        lambda <- try(glmreg_fit(x, ytmp, weights=RET$weights_update, offset=offset, lambda.min.ratio=lambda.min.ratio, nlambda=nlambda, alpha=alpha,gamma=gamma, rescale=FALSE, standardize=standardize, intercept=intercept, penalty.factor = penalty.factor, maxit=1, eps=eps, family=dfunnew, penalty=penalty)$lambda)  ### changed 8/17/2020
        if(inherits(lambda, "try-error"))
          stop("Initial value can't compute penalty lambda values. Possible reason: initial weights are all zero. Try to enlarge s value, or change type.init/init.family\n")
### with type.init, add two different lambda sequences and make lambda values flexible to have different solution paths
        if(!decreasing)#{### solution path backward direction
            lambda <- rev(lambda)
    }
    nlambda <- length(lambda)
    beta <- matrix(0, ncol=nlambda, nrow=m)
    fitted <- matrix(NA, ncol=nlambda, nrow=n)
    b0 <- rep(0, nlambda)
    weights_cc <- matrix(0, nrow=n, ncol=nlambda)
### initialize start values -begin
### Todo: initial values?
    mustart <- rep(0, n)
    etastart <- rep(0, n)
    fk_old <- RET$fitted.values
### check next line if update is needed
    if(type.init %in% c("bst", "co") && dfun==5)
        tmp <- init(RET$weights_update/sum(RET$weights_update), ytmp, offset, family=dfunnew)
    else if(type.init%in%c("bst","co")) ##it seems we don't need tmp next line
        tmp <- init(RET$weights_update, ytmp, offset, family=dfunnew)
    mustart <- tmp$mu
    etastart <- tmp$eta     
### initialize the start values -end
    stopit <- FALSE
    ### create a function for parallel computing can lead to object-not-found problem
    #typeP <- function(x, y, weights, start, etastart, mustart, offset,lambda,alpha,gamma,intercept,penalty.factor,maxit,eps, pentype, trace, iter, reltol, cfun, dfun, s, thresh, delta, m){
    if(isTRUE(parallel)){
    i <- 1
    cl <- parallel::makeCluster(n.cores, outfile="")
         registerDoParallel(cl)
         fitall <- foreach(i=1:nlambda, .packages=c("mpath")) %dopar%{
            RET <- .Fortran("irglmreg_onelambda",
                            x_act=as.double(x), 
                            y=as.double(y), 
                            weights=as.double(weights),
                            n=as.integer(n),
                            m_act=as.integer(m),	
                            start_act=as.double(start), 
                            etastart=as.double(etastart),
                            mustart=as.double(mustart),
                            yhat=as.double(rep(0, n)),
                            offset=as.double(offset),
                            lambda_i=as.double(lambda[i]), 
                            alpha=as.double(alpha),        
                            gam=as.double(gamma), 
			                rescale=as.integer(rescale),
                            #standardize=as.integer(0),
                            standardize=as.integer(standardize),
                            intercept=as.integer(intercept),
                            penaltyfactor_act=as.double(penalty.factor),
                            maxit=as.integer(maxit),
                            eps=as.double(eps), 
                            theta=as.double(theta),
                            penalty=as.integer(pentype), 
                            trace=as.integer(trace),
                            iter=as.integer(iter),
                            del=as.double(reltol),
                            cfun=as.integer(cfun),
                            dfun=as.integer(dfun),
                            s=as.double(s),
                            thresh=as.double(thresh), 
                            beta_1=as.double(rep(0, m)),
                            b0_1=as.double(0),
                            fk=as.double(rep(0, n)),
                            delta=as.double(delta),
                            weights_update=as.double(rep(0, n)),
                            PACKAGE="mpath")
            list(beta=RET$beta, b0=RET$b0, yhat=RET$yhat, weights_update=RET$weights_update)   
        }
    parallel::stopCluster(cl)
    RET <- fitall[[nlambda]]    
    for(k in 1:nlambda){
             beta[,k] <- fitall[[k]]$beta
             b0[k] <- fitall[[k]]$b0
             fitted[,k] <- fitall[[k]]$yhat
             weights_cc[,k] <- fitall[[k]]$weights_update
         }
        tmp <- list(beta=beta, b0=b0, RET=RET, fitted=fitted, weights_cc=weights_cc)
    }
    typeA <- function(beta, b0){
        if(dfun %in% c(1, 4)) dfuntmp <- 1 else if(dfun==5) dfuntmp <- 2
        else if(dfun==8) dfuntmp <- 3 else if(dfun==9) dfuntmp <- 4
        else if(dfun==6)
                                        if(all(x[,1]==1)) xtmp <- x[,-1] else xtmp <- x
        i <- 1
        los <- pll <- matrix(NA, nrow=iter, ncol=nlambda)
        weights_cc <- matrix(NA, nrow=n, ncol=nlambda)
        if(trace && tracelevel==2) tracel <- 1 else tracel <- 0
        while(i <= nlambda){
            if(trace) message("\nloop in lambda:", i, ", lambda=", lambda[i], "\n")
             if(trace) {
                 cat(" IRCO iterations ...\n")
             }
            k <- 1
            d1 <- 10
            weights_update <- weights
            satu <- 0
            while(d1 > reltol && k <= iter && satu==0){
                fitted.values <- fk
                RET <- .Fortran("glmreg_fit_fortran",
	                        x=as.double(x), 
				y=as.double(ytmp), 
				weights=as.double(weights_update),
			        n=as.integer(n),
			        m=as.integer(m),	
				start=as.double(start), 
				etastart=as.double(etastart),
	                        mustart=as.double(mustart),
			       	offset=as.double(offset),
			       	nlambda=as.integer(1),
			       	lambda=as.double(lambda[i]), 
				alpha=as.double(alpha),        
			        gam=as.double(gamma), 
				rescale=as.integer(rescale),
			    #standardize=as.integer(0), ### changed 08/17/2020
			   	standardize=as.integer(standardize),
			       	intercept=as.integer(intercept),
				penaltyfactor=as.double(penalty.factor),
				thresh=as.double(thresh), 
				epsbino=as.double(0), 
				maxit=as.integer(maxit),
			        eps=as.double(eps), 
				theta=as.double(theta),
			       	family=as.integer(dfuntmp),
				penalty=as.integer(pentype), 
				trace=as.integer(tracel),
				beta=as.double(matrix(0, ncol=1, nrow=m)),
				b0=as.double(0),
				yhat=as.double(rep(0, n)),
                                satu=as.integer(0),
				PACKAGE="mpath")
                satu <- RET$satu
                if(satu==1) warnings("saturated binomial model")
                fk <- RET$yhat
                etastart <- RET$etastart
                mustart <- RET$mustart
                if(dfun%in%c(1,4,5)){
                weights_update <- .Fortran("update_wt",
                                           n=as.integer(n),
                                           weights=as.double(weights),
                                           y=as.double(y),
                                           f=as.double(etastart),
                                           cfun=as.integer(cfun),
                                           dfun=as.integer(dfun),
                                           s=as.double(s),
                                           delta=as.double(delta),
                                           weights_update=as.double(rep(0, n)),
                                           PACKAGE="mpath")$weights_update
                los[k, i] <- .Fortran("loss2",
                                            n=as.integer(n),
                                            y=as.double(y),
                                            f=as.double(etastart),
                                            weights=as.double(weights),
                                            cfun=as.integer(cfun),
                                            dfun=as.integer(dfun),
                                            s=as.double(s),
                                            delta=as.double(delta),
                                            los=as.double(0.0),
                                            PACKAGE="mpath")$los
                }
                else{
                weights_update <- compute_wt3(y,mustart,theta,weights,cfun,family,s,delta) #for poisson/negbin
                los[k, i] <- sum(loss3(y,mustart,theta,weights,cfun,family,s,delta)$tmp)
                } 
                ### changed the following line 08/17/2020
                if(dfun!=5) start <- c(RET$b0, RET$beta)
                #if(!dfun %in% c(5, 8, 9)) start <- c(RET$b0, RET$beta)
 ###penalized loss value for beta
                 penval <- .Fortran("penGLM",
                                    start=as.double(RET$beta),
                                    m=as.integer(m),
                                    lambda=as.double(lambda[i]*penalty.factor),
                                    alpha=as.double(alpha),
                                    gam=as.double(gamma),
                                    penalty=as.integer(pentype),
                                    pen=as.double(0.0),
                                    PACKAGE="mpath")$pen
                 if(standardize)  ### lambda value, hence penval, depends on whether standardize is TRUE/FALSE - to be confirmed
                     pll[k, i] <- los[k, i] + n*penval
                 else pll[k, i] <- los[k, i] + penval
                 if(k > 1)
                 d1 <- abs((pll[k, i] - pll[k-1, i])/pll[k-1, i])
                 if(trace) cat("\n  iteration", k, ": relative change of fk", d1, ", robust loss value", los[k, i], ", penalized loss value", pll[k, i], "\n")
                 if(trace) cat("  d1=", d1, ", k=", k, ", d1 > reltol && k <= iter: ", (d1 > reltol && k <= iter), "\n")
                k <- k + 1
            }
                beta[,i] <- RET$beta
                b0[i] <- RET$b0
                weights_cc[,i] <- weights_update
                i <- i + 1
        }
        list(beta=beta, b0=b0, RET=RET, risk=los, pll=pll, weights_cc=weights_cc)
    }
    typeBB <- function(beta, b0){
        if(trace && tracelevel==2) tracel <- 1 else tracel <- 0
        if(type.path=="active" && decreasing)
            RET <- .Fortran("irglmreg_ad",
			    x=as.double(x), 
			    y=as.double(y),
			    weights=as.double(weights),
			    n=as.integer(n),
			    m=as.integer(m),
			    start=as.double(start), 
			    etastart=as.double(etastart),
			    mustart=as.double(mustart),
			    offset=as.double(offset),
			    iter=as.integer(iter),
			    nlambda=as.integer(nlambda), 
			    lambda=as.double(lambda), 
			    alpha=as.double(alpha),
                            gam=as.double(gamma), 
			    rescale=as.integer(rescale),
			    standardize=as.integer(standardize), 
			    #standardize=as.integer(0), ### changed 8/17/2020
			    intercept=as.integer(intercept), 
			    penaltyfactor=as.double(penalty.factor),
			    maxit=as.integer(maxit), 
			    eps=as.double(eps), 
			    theta=as.double(theta), 
			    epscycle=as.double(epscycle), 
			    penalty=as.integer(pentype), 
			    trace=as.integer(tracel), 
			    del=as.double(reltol), 
			    cfun=as.integer(cfun),
			    dfun=as.integer(dfun),
			    s=as.double(s),
			    thresh=as.double(thresh),
			    decreasing=as.integer(decreasing),
			    beta=as.double(beta),
                            b0=as.double(b0), 
			    yhat=as.double(RET$fitted.values), 
			    los=as.double(rep(0, nlambda)),
			    pll=as.double(rep(0, nlambda)),
			    nlambdacal=as.integer(0),
			    delta=as.double(delta),
			    weights_cc=as.double(matrix(0, nrow=n, ncol=nlambda)),
			    PACKAGE="mpath")
        else
	    RET <- .Fortran("irglmreg_fortran",
			    x=as.double(x), 
			    y=as.double(y),
			    weights=as.double(weights),
			    n=as.integer(n),
			    m=as.integer(m),
			    start=as.double(start), 
			    etastart=as.double(etastart),
			    mustart=as.double(mustart),
			    offset=as.double(offset),
			    iter=as.integer(iter),
			    nlambda=as.integer(nlambda), 
			    lambda=as.double(lambda), 
			    alpha=as.double(alpha),
                            gam=as.double(gamma), 
			    rescale=as.integer(rescale),
			    standardize=as.integer(standardize), 
			    #standardize=as.integer(0), #changed 8/17/2020
			    intercept=as.integer(intercept), 
			    penaltyfactor=as.double(penalty.factor),
			    maxit=as.integer(maxit), 
			    eps=as.double(eps), 
			    theta=as.double(theta), 
			    penalty=as.integer(pentype), 
			    trace=as.integer(tracel), 
			    del=as.double(reltol), 
			    cfun=as.integer(cfun),
			    dfun=as.integer(dfun),
			    s=as.double(s),
			    thresh=as.double(thresh),
			    decreasing=as.integer(decreasing),
			    active=as.integer(active),
			    beta=as.double(beta),
                            b0=as.double(b0), 
			    yhat=as.double(RET$fitted.values), 
			    los=as.double(rep(0, nlambda)),
			    pll=as.double(rep(0, nlambda)),
			    nlambdacal=as.integer(0),
			    delta=as.double(delta),
			    weights_cc=as.double(matrix(0, nrow=n, ncol=nlambda)),
			    PACKAGE="mpath")
        list(beta=matrix(RET$beta, ncol=nlambda), b0=RET$b0, RET=RET, risk=RET$los, pll=RET$pll, nlambdacal=RET$nlambdacal, weights_cc=RET$weights_cc)
    }
        #tmp <- typeP(x, y, weights, start, etastart, mustart, offset,lambda[i],alpha,gamma,intercept,penalty.factor,maxit,eps, pentype, trace, iter, reltol, cfun, dfun, s, thresh, delta, m) 
    if(is.null(parallel)) tmp <- typeA(beta, b0) else 
    if(!parallel) tmp <- typeBB(beta, b0)
    beta <- tmp$beta
    b0 <- tmp$b0
    RET <- tmp$RET
    RET$weights_update <- tmp$weights_cc
    if(!is.null(parallel) && standardize && dfun%in%c(5,8,9)){
        tmp1 <- stan(x, weights)
        meanx <- tmp1$meanx
        normx <- tmp1$normx
        beta <- beta/normx
        b0 <- b0 - crossprod(meanx, beta)
      }
    if(nlambda == 1){
         RET$df <- sum(abs(beta) > 0) ### df= number of nonzero coefficients     (intercept excluded)
     }
    else{
         RET$df <- apply(abs(beta) > 0, 2, sum) ##number of nonzero              coefficients for each lambda
     }
    if(intercept) npar <- 1+RET$df else npar <- RET$df
    if(is.null(colnames(x))) varnames <- paste("V", 1:ncol(x), sep="")
    else varnames <- colnames(x)
    dimnames(beta) <- list(varnames, round(lambda, digits=4))
    RET$beta <- beta
    RET$b0 <- matrix(b0, nrow=1)
    RET$x <- xold
    RET$y <- y
    RET$call <- call
    RET$lambda <- lambda
    RET$nlambda <- nlambda
    RET$penalty <- penalty
    RET$s <- s
    #RET$risk <- tmp$risk
    #RET$pll <- tmp$pll
    RET$nlambdacal <- tmp$nlambdacal
    RET$cfun <- cfunold
    RET$dfun <- dfunold
    RET$type.init <- type.init
    RET$mstop.init <- mstop.init
    RET$nu.init <- nu.init
    RET$decreasing <- decreasing
    RET$type.path <- type.path
    RET$is.offset <- is.offset
    RET$fitted.values <- fitted
    RET$family <- dfunold
    RET$twologlik <- 2*logLik.glmreg(RET, newx=x, y=y, weights=weights_update, offset=offset)
    RET$aic <- -RET$twologlik + 2*npar
    RET$bic <- -RET$twologlik + log(n)*npar
    class(RET) <- "irglmreg"
    RET
}

### compute classification/prediction error based on predictive values
co_evalerr <- function(dfun, y, yhat){
    if(dfun %in% c(1:3,8,9))
        mean((y - yhat)^2)
    else if(dfun %in% 4:7)
        (mean(y != sign(yhat)))
}

predict.irglmreg <- function(object, newdata=NULL, weights=NULL, newy=NULL, newoffset=NULL, which=1:object$nlambda, type=c("link", "response","class","loss", "error", "coefficients", "nonzero"), na.action=na.pass, ...){
    type=match.arg(type)
    if(is.null(newdata)){
        if(!match(type,c("coefficients", "nonzero"),FALSE))stop("You need to supply a value for 'newdata'")
        ynow <- object$y
    }
    else{
        if(!is.null(object$terms)){
            mf <- model.frame(delete.response(object$terms), newdata, na.action = na.action, xlev = object$xlevels)
            ynow <- model.frame(object$terms, newdata)[,1] ### extract response variable
            newdata <- model.matrix(delete.response(object$terms), mf, contrasts = object$contrasts)
            if(!is.null(ynow) && !is.null(newy))
                warnings("response y is used from newdata, but newy is also provided. Check if newdata contains the same y as newy\n")
        }
        else ynow <- newy
    }
    if(type=="coefficients") return(coef.glmreg(object)[,which])
    if(type=="nonzero"){
        nbeta <- object$beta[,which]
        if(length(which)>1) return(eval(parse(text="glmnet:::nonzeroCoef(nbeta[,,drop=FALSE],bystep=TRUE)")))
        #if(length(which)>1) return(nonzeroCoef(nbeta[,,drop=FALSE],bystep=TRUE))
        else return(which(abs(nbeta) > 0))
    }
    if(is.null(newdata))
        newx <- as.data.frame(object$x)
    else newx <- as.data.frame(newdata)
    if(dim(newx)[2]==dim(object$beta)[1]) ### add intercept
        newx <- cbind(1, newx)
    if(object$is.offset)
        if(is.null(newoffset))
            stop("offset is used in the estimation but not provided in prediction\n")
        else offset <- newoffset
    else offset <- rep(0, length(ynow))
    res <- offset + as.matrix(newx) %*% rbind(object$b0, object$beta)
    if(type=="link") return(res[, which])
    if(type %in% c("link", "response")) return(predict(object, newx=newx, newoffset=newoffset, which=which, type=type))
    if(type %in% c("loss", "error") && is.null(ynow)) stop("response variable y missing\n")
    if(type=="loss"){
        object$cfun <- cfun2num(object$cfun)
        object$dfun <- dfun2num(object$dfun)
        if(object$dfun %in% 4:5) ynow <- y2num(ynow)
        tmp <- rep(NA, length(which))
        n <- length(ynow)
        if(missing(weights)) weights <- rep(1/n, n)
        for(i in 1:length(which)){
            if(object$dfun %in% c(1,4,5)) 
                tmp[i] <- .Fortran("loss2",
                               n=as.integer(n),
                               y=as.double(ynow),
                               f=as.double(res[,which[i]]),
                               weights=as.double(weights),
                               cfun=as.integer(object$cfun),
                               dfun=as.integer(object$dfun),
                               s=as.double(object$s),
                               delta=as.double(object$delta),
                               los=as.double(0.0),
                               PACKAGE="mpath")$los
        else{
          mu <- predict(object, newx=newx, newoffset=newoffset,which=which[i], type="response")
          tmp[i] <- sum(loss3(object$y,mu,object$theta,weights,object$cfun,family,object$s,object$delta)$tmp)
            }
        }
        return(tmp)
    }
    if(type=="error"){
        object$dfun <- dfun2num(object$dfun)
        if(object$dfun %in% 4:5) ynow <- y2num(ynow)
        tmp <- rep(NA, length(which))
        for(i in 1:length(which))
            tmp[i] <- co_evalerr(object$dfun, ynow, res[,which[i]])
        return(tmp)
    }
}


coef.irglmreg <- function(object, ...)
    coef.glmreg(object)

CCC   ONLY used for family=1; compare with midloopGLM for other family
C     Middle loop: Update the quadratic approximation likelihood
C     input:
C     n
C     m
C     x
C     y
C     mu
C     family
C     nlambda
C     lamk
C     alpha
C     gam
C     maxit
C     del
C     thresh
C     wt              used only for family=1
C     trace
C     innermaxit: not used except for family=1      
C     maxit: not used (and maxit=1) for family=1      
C     output:
C     beta
C     b0
C     yhat
C     dev

      subroutine midloop(n,m,x,y,xold,yold,weights, mu, eta, offset,
     +     family, penalty,lamk, alpha, gam, theta, rescale, 
     +     standardize, intercept, eps,innermaxit,
     +     maxit, thresh, nulldev, wt, beta, b0,yhat,dev,trace,convmid, 
     +     ep, normx, xd, avg, activeset, jk, fullset)
      
      implicit none
      integer standardize, intercept, trace, penalty, maxit, i, j, 
     +     nmid, n, family, innermaxit, m,converged,convmid, rescale,
     +     fullset(m),activeset(m), jk
      double precision x(n,m),y(n), mu(n), z(n), eta(n), wt(n), w(n), 
     +     del,olddev,weights(n),xold(n,m), yold(n),normx(m),xd(m), 
     +     thresh, nulldev, dev, theta, wtw(n),lamk(m),alpha, 
     +     gam, eps, beta(m), b0, yhat(n),avg, ep, offset(n)

C      innermaxit = maxit
      maxit = 1
C     innermaxit serves as maxit here

      dev = nulldev
      call glmlink(n,mu,family,theta,w, ep)
      call zeval(n, y, eta, mu, w, family, z)
      do 10 i=1, n
         wtw(i)=wt(i) * w(i)
         z(i)=z(i) - offset(i)
 10   continue
      call preprocess(x, z, n, m, wtw, family, standardize,
     +     normx, xd, avg)
      call lmnetGaus(x, z, n, m, wtw, lamk, alpha, gam, thresh, 
     +     innermaxit, eps, standardize, intercept, penalty, xd, 
     +     beta, b0, avg, nmid,rescale, converged, activeset, jk,
     +     fullset)
      do 220 i = 1, n
         yhat(i) = b0
         do 230 j = 1, m
            if(family .EQ. 1)then
               yhat(i) = yhat(i) + xold(i,j) * beta(j)
            else
               yhat(i) = yhat(i) + x(i,j) * beta(j)
            endif
 230     continue
 220  continue
C     call DCOPY(n, yhat, 1, eta, 1)
      do 350 i = 1, n
         yhat(i) = yhat(i) + offset(i)
         eta(i) = yhat(i)
 350  continue
      call linkinv(n, eta, family, mu)
      olddev = dev
C     compute deviance dev
      call deveval(n, yold, mu, theta, weights, family, dev)
      del = dabs(dev - olddev)
      convmid = converged
      if(trace.EQ.1)then
         call dblepr("deviance difference at the end of middle loop "
     +        , -1, del, 1)
      endif
      
      return
      end

C input: mu, family, epsbino
C output: eta
      subroutine gfunc(mu, n, family, epsbino, eta)
      implicit none
      integer :: family, n, i
      double precision :: mu(n), epsbino, eta(n)

      do i=1, n
       if(family==1)then
         eta(i) = mu(i)
       else if(family==2)then
         if(1-mu(i) > epsbino .AND. mu(i) > epsbino)then
          eta(i)=dlog(mu(i)/(1-mu(i)))
         else if(mu(i) .LE. epsbino)then
           eta(i)=dlog(epsbino/(1-epsbino))
         else if(mu(i) .GE. 1-epsbino)then
          eta(i)=dlog(1-epsbino/epsbino)
         endif
       else if(family==3 .OR. family==4)then
          eta(i)=dlog(mu(i))
       endif
      enddo

      return
      end

C converted from R mpath/R/compute.h function
C input rfamily, y, fk_old, s, B -- no changes
C       clossR: 11, closs: 12, gloss: 13, qloss: 14
C output h
      subroutine compute_h(rfamily, n, y, fk_old, s, B, h)
              implicit none
              integer rfamily, i, n
              double precision y(n), fk_old(n), s, B, u(n), g, h(n)

      if(rfamily .EQ. 11)then
              do 10 i=1, n
              u(i)=y(i)-fk_old(i)
              call gradient(rfamily, 1, u(i), s, g)
              h(i) = g/B + fk_old(i)
 10           continue
      else if (rfamily .EQ. 12 .OR. rfamily .EQ. 13 
     +         .OR. rfamily .EQ. 14) then
              do 30 i=1, n
              u(i)=y(i)*fk_old(i)
              call gradient(rfamily, 1, u(i), s, g)
              h(i) = -y(i)*g/B + fk_old(i)
  30          continue
      end if
             
            return
            end 

C     outer loop: sequence of lambda
C     input:
C     start
C     mu: a vector
C     eta: a vector
C     output
C     b: beta
C     bz: b0
C     resdev: residual deviance
C     ypre: yhat
C     mu, eta: uppdated values

      subroutine outloop(x,y,weights, wt, n,m, penalty, nlambda, lam, 
     +     alpha,gam,theta,rescale,mu,eta, offset,
     #family,standardize, intercept, nulldev, 
     +     thresh, maxit, innermaxit, eps, trace, start, startv,b, bz,
     +     resdev,ypre, convout, satu, good, ep, outpll)

      implicit none
      integer n,m,i,j,k,kk,penalty, nlambda, family, standardize, maxit,
     +     innermaxit, trace,convmid,convout(nlambda), startv, rescale, 
     +     satu,good,convact,fullset(m),activeset(m),activesetold(m),
     +     jk,jkold,intercept
      double precision x(n,m), y(n), wt(n), lam(m, nlambda),alpha,
     +     gam, theta,mu(n),eta(n), nulldev,thresh, eps, b(m,nlambda),
     +     bz(nlambda),xold(n,m), yold(n), start(m+1), resdev(nlambda), 
     +     ypre(n,nlambda), lamk(m), beta(m), b0,dev,
     +     weights(n),yhat(n),ep, pll(maxit), outpll(maxit, nlambda),
     +     normx(m),xd(m),avg, offset(n)

      if(family .NE. 1)then
         call preprocess(x, y, n, m, weights, family, standardize,
     +        normx, xd, avg)
      endif
C     keep a copy of x and y in case x and y will be changed in subroutine lmnet if standardize = 1
      do 100 j=1, m
         do 110 i=1, n
            xold(i,j)=x(i,j)
 110     continue
 100  continue
      call DCOPY(n, y, 1, yold, 1)
      if(startv .EQ. 0)then
         b0 = eta(1)
         do j=1, m
            beta(j) = 0
         enddo
      else 
         b0 = start(1)
         do j=1, m
            beta(j) = start(j+1)
         enddo
      endif
      k = 1
      satu = 0
C     good = nlambda
      do 401 j=1, m
         activeset(j)=j
         fullset(j)=j
 401  continue
      jk = m
 1000 if(k .LE. nlambda .AND. satu .EQ. 0)then
         if(trace.EQ.1)then
            call intpr("Outer loop: sequence of lambda", -1, 1, 1)
            call intpr("  lambda iteration", -1, k, 1)
            call dblepr("  lambda value", -1, lam(1,k), 1)
         endif
         do 10 j=1,m
            lamk(j) = lam(j,k)
 10      continue
C     if jk=0, it means an intercept-only model, thus, we update
C     activeset
C     if(jk .EQ. 0)then
C     do 18 j=1, m
C     activeset(j)=j
C     18         continue
C     jk = m
C     endif
C     Active set: begin with the 1st element of the sequence of lambda
C     values, i.e., k=1 in this subroutine. Now, the active set
C     contains all variables, cycle through all coefficients in the
C     active set with coordinate descent algorithm until convergence,
C     then cycle through the full set with all variables, but only ONCE.
C     This generates a new active set. Compare with the previous (old) 
C     active set. If no changes, then we are done with this 1st lambda. 
C     Otherwise, we repeat the above process until the updated active
C     set remains the same, or the number of iteration (convact) is met. 
C     Next, move to the 2nd element of the sequence of lambda values. We 
C     repeat the above process with the current active set.   
C     
C     70: if block --begin
         if(family .EQ. 1)then
C     For family=1, active set is implemented in midloop -> lmnetGaus 
C     -> loop_gaussian
            call midloop(n,m,x,y, xold,yold,weights,mu, eta, offset,
     +           family, penalty,lamk,alpha,gam,theta,rescale,
     +           standardize, intercept, eps,innermaxit, maxit, thresh,
     +           nulldev, wt, beta, b0, yhat, dev, trace, convmid, ep,
     +           normx,xd,avg, activeset, jk, fullset)
         else 
C     active set applies for family!=1. 
            convact=0
C     some loop, if no change to the active set, stop
            kk = 1
C     max.cycle: in case of nonconvergence the program looks for a cycle
C     that repeats itself, default is 2 (kk <= 2)
            do 2000 while (kk <= 2 .AND. convact == 0)
C     set maxit=1, and have a complete cycle through all the variables
               call midloopGLM(n,m,x,y,yold,weights,mu,eta,offset,
     +              family, penalty,lamk,alpha,gam,theta,rescale,
     +              standardize, intercept,
     +              eps,1,thresh,nulldev,wt,beta,b0,yhat, 
     +              dev,trace,convmid,satu,ep,pll,fullset,m)
               call find_activeset(m, beta, eps, activesetold, jkold)
C     it is possible, jk=0 if beta=0, like intercept-only model
               if(jkold .EQ. 0)then
                  convact=1
                  exit
               endif
C     now cycle through only the active set
               call midloopGLM(n,m,x,y,yold,weights,mu,eta,offset,
     +              family, penalty,lamk,alpha,gam,theta,rescale,
     +              standardize, intercept,
     +              eps,maxit,thresh,nulldev,wt,beta,b0,yhat,
     +              dev,trace,convmid,satu,ep,pll,
     +              activesetold, jkold)
C     set maxit=1, and have a complete cycle through all the variables
               call midloopGLM(n,m,x,y,yold,weights,mu,eta,offset,
     +              family, penalty,lamk,alpha,gam,theta,rescale,
     +              standardize, intercept,
     +              eps,1,thresh,nulldev,wt,beta,b0,yhat, 
     +              dev,trace,convmid,satu,ep,pll,fullset,m)
C     update the active set with only non-zero coefficients 
               call find_activeset(m, beta, eps, activeset, jk)
C     check if the active set was changed--begin
C     it is possible, jk=0 if beta=0, like intercept-only model
               if(jk .EQ. 0)then
                  convact=1
                  exit
               endif
               if(jkold .NE. jk)then
                  convact=0
               else
                  do 901 j=1, jk
                     if(activesetold(j) .NE. activeset(j))then
                        exit
                     endif
                     if(j .EQ. jk)then
                        convact = 1
                        exit
                     endif
 901              continue
               endif
C     check if the active set was changed--end
               kk=kk+1
 2000       continue
         endif
C     70: if block --end
         if(satu .EQ. 1)then
            good = k - 1
         endif
         convout(k)=convmid
         if(family .NE. 1)then
            do 15 i=1, maxit
               outpll(i,k) = pll(i)
 15         continue
         endif
         do 20 j=1, m
            b(j, k) = beta(j)
 20      continue
         bz(k) = b0
         resdev(k) = dev
         call linkinv(n, yhat, family, ypre(1:n,k))
C     call linkinv(n, yhat, family, v)
C     do 30 i=1, n
C     ypre(i,k) = v(i)
C     30      continue
         k = k + 1
         if(k .LE. nlambda .AND. satu .EQ. 0)then
            do 40 j=1,m
               b(j, k) = b(j, k-1)
 40         continue
         endif
         goto 1000
      endif
      
      return
      end  

C used in irglmreg_onelambda, step 3 in the IRCO Algorithm
C input dfun, if dfun=1-3, for regression; otherwise, if dfun=4-7 for
C classification, if dfun=8 or 9, Poisson or negative binomial
C output u
      subroutine compute_u(dfun, n, y, f, u)
              implicit none
              integer dfun, i, n
              double precision y(n), f(n), u(n)

      do 10 i=1, n
      if(dfun .LE. 3)then
              u(i)=y(i)-f(i)
          else if(dfun .GE. 4 .AND. dfun .LE. 7)then
              u(i)=y(i)*f(i)
          else 
              u(i)=f(i)
      end if
 10           continue
             
      return
      end 

### nonconvex linear (NCL) models for regression and classification
ncl <- function(x, ...) UseMethod("ncl")

ncl.default <- function(x, ...) {
    if (extends(class(x), "Matrix"))
        return(ncl.matrix(x = x, ...))
    stop("no method for objects of class ", sQuote(class(x)),
         " implemented")
}

ncl.formula <- function(formula, data, weights, offset=NULL, contrasts=NULL, 
                        x.keep=FALSE, y.keep=TRUE, ...){
    ## extract x, y, etc from the model formula and frame
                                        #if(!attr(terms(formula, data=data), "intercept"))
                                        #    stop("non-intercept model is not implemented")
    if(missing(data)) data <- environment(formula)
    mf <- match.call(expand.dots = FALSE)
    m <- match(c("formula", "data", "weights",
                 "offset"), names(mf), 0L)
    mf <- mf[c(1L, m)]
    mf$drop.unused.levels <- TRUE
    mf[[1L]] <- as.name("model.frame")
    mf <- eval(mf, parent.frame())

    mt <- attr(mf, "terms") # allow model.frame to have updated it

    Y <- model.response(mf, "any") # e.g. factors are allowed
    ## avoid problems with 1D arrays, but keep names
    if(length(dim(Y)) == 1L) {
        nm <- rownames(Y)
        dim(Y) <- NULL
        if(!is.null(nm)) names(Y) <- nm
    }
    ## null model support
    X <- if (!is.empty.model(mt)) model.matrix(mt, mf, contrasts) else matrix(,NROW(Y), 0L)
    ## avoid any problems with 1D or nx1 arrays by as.vector.
    weights <- as.vector(model.weights(mf))
    if(!length(weights)) weights <- rep(1, nrow(mf))
    else if(any(weights < 0)) stop("negative weights not allowed")
    if(!is.null(weights) && !is.numeric(weights))
        stop("'weights' must be a numeric vector")
    if(length(weights) != length(Y))
        stop("'weights' must be the same length as response variable")	 
    offset <- as.vector(model.offset(mf))
    if(!is.null(offset)) {
        if(length(offset) != NROW(Y))
            stop(gettextf("number of offsets is %d should equal %d (number of observations)", length(offset), NROW(Y)), domain = NA)
    }
                                        #if(!attr(terms(formula, data=data), "intercept"))
                                        #RET <- ncl_fit(X[,-1], Y, weights,...)
                                        #else
    RET <- ncl_fit(X, Y, weights,...)
    RET$call <- match.call()
    RET <- c(RET, list(formula=formula, terms = mt, data=data,
                       contrasts = attr(X, "contrasts"),
                       xlevels = .getXlevels(mt, mf)))
    if(x.keep) RET$x <- data[,colnames(data)%in%attr(mt, "term.labels")]
    if(y.keep) RET$y <- Y
    class(RET) <- "ncl"
    RET
}
ncl.matrix <- function(x, y, weights, offset=NULL, ...){
    RET <- ncl_fit(x, y, weights, offset=offset, ...)
    RET$call <- match.call()
    return(RET)
}

ncl_fit <- function(x,y, weights, offset=NULL, rfamily=c("clossR", "closs", "gloss", "qloss"), s=NULL, fk=NULL, iter=10, reltol=1e-5, trace=FALSE){
    call <- match.call()
    rfamily <- match.arg(rfamily)
    if(is.null(s)) stop("s must be a number\n")
     if(rfamily %in% c("closs", "qloss", "clossR") &&  s <= 0) stop("s must be positive\n")
         else if(rfamily == "gloss" && s <=1) stop("s must > 1 for gloss\n")
    if(rfamily %in% c("closs", "gloss", "qloss"))
        if(!all(names(table(y)) %in% c(1, -1)))
            stop("response variable must be 1/-1 for family ", rfamily, "\n")
    ## avoid any problems with 1D or nx1 arrays by as.vector.
    if(missing(weights)) weights=rep(1,length(y))
    weights <- as.vector(weights)
    if(!is.null(weights) && !is.numeric(weights))
        stop("'weights' must be a numeric vector")
    ## check weights and offset
    if( !is.null(weights) && any(weights < 0) ){
        stop("negative weights not allowed")
    }
    if(length(weights) != length(y))
        stop("'weights' must be the same length as response variable")
    if (is.null(offset)){
         is.offset <- FALSE
         offset <- rep.int(0, length(y))
     }else is.offset <- TRUE
    famtype <- switch(rfamily,
                      "closs"="clossMM",
                      "gloss"="glossMM",
                      "qloss"="qlossMM",
                      "clossR"="clossRMM"
                      )
    if(is.null(fk)){
        bsttype <- switch(rfamily,
                          "closs"="closs",
                          "gloss"="gloss",
                          "qloss"="qloss",
                          "clossR"="clossR"
                          )
### initiate predictive value
        RET <- bst(x, y, family=bsttype, ctrl = bst_control(mstop=1, s=s))
        RET$fitted.values <- RET$yhat
    }
    else {
        RET <- NULL
        RET$fitted.values <- fk
    }
    cost <- 0.5
    ### the optimization problem doesn't use cost, unlike bst
    los <- 2*loss(y, f=RET$fitted.values, cost, family = rfamily, s=s, fk=RET$fitted.values)
    d1 <- 10 
    k <- 1
    if(trace) {
        cat("\nQuadratic majorization iterations ...\n")
        cat("\ninitial loss", mean(los), "\n")
    }
    los <- rep(NA, iter)
    B <- bfunc(family=rfamily, s=s)
    while(d1 > reltol && k <= iter){
        fk_old <- RET$fitted.values
                                        #if(rfamily=="closs")
                                        #fk_old<- pmin(1, pmax(-1, fk_old)) ### truncated at -1 or 1
	if(rfamily=="clossR"){
            h <- gradient(family=rfamily, u=y-fk_old, s=s)/B+fk_old
        }
        else{ 
	    z <- y*fk_old
	    h <- -y*(gradient(family=rfamily, u=z, s=s)/B-z)
        }
    RET <- lm.wfit(x, h, weights, offset)
	fk <- RET$fitted.values
                                        #if(rfamily=="closs")
                                        #fk <- pmin(1, pmax(-1, fk)) ### truncated at -1 or 1
	los[k] <- 2*mean(loss(y, f=fk, cost, family = rfamily, s=s, fk=NULL))
	if(trace){
        }
#	d1 <- sum((fk_old - fk)^2)/max(1, sum(fk_old^2))
	if(trace) cat("\niteration", k, ": relative change of fk", d1, ", robust loss value", los[k], "\n") 
	if(k > 1){
            d1 <- abs((los[k]-los[k-1])/los[k-1])
            if(los[k] > los[k-1]){
		      k <- iter
            }
        }
        k <- k + 1
	if(trace) cat("d1=", d1, ", k=", k, ", d1 > reltol && k <= iter: ", (d1 > reltol && k <= iter), "\n")
    }
    RET$h <- h
    RET$x <- x
    RET$y <- y
    RET$s <- s
    RET$call <- call
    RET$rfamily <- RET$family <- rfamily
    RET$is.offset <- is.offset
    RET
}

predict.ncl=function(object, newdata=NULL, newoffset=NULL, type=c("response","class","loss", "error", "coefficients"), na.action=na.pass, ...){
    type=match.arg(type)
    if(is.null(newdata)){
        if(!match(type,c("coefficients"),FALSE))stop("You need to supply a value for 'newdata'")
        ynow <- object$y
    }
    else{
        if(!is.null(object$terms)){
            mf <- model.frame(delete.response(object$terms), newdata, na.action = na.action, xlev = object$xlevels)
            ynow <- model.frame(object$terms, newdata)[,1] ### extract response variable
	    newdata <- model.matrix(delete.response(object$terms), mf, contrasts = object$contrasts)
        }
    }
     if(object$is.offset)
      if(is.null(newoffset))
        stop("offset is used in the estimation but not provided in prediction\n")
      else offset <- newoffset
    else offset <- rep(0, length(ynow))
    if(type=="coefficients") return(object$coef)
    if(is.null(newdata))
        res <- object$fitted.values
    else
        res <- offset + newdata %*% object$coef
    if(type=="response") return(res)
    if(type=="loss") return(mean(loss(ynow, res, family = object$family, s=object$s)))
    if(type=="error") return(evalerr(object$family, ynow, res))
}

### from glmnet version 1.9-1 package
plot.glmreg=function(x, xvar=c("norm","lambda","dev"),label=FALSE,shade=TRUE, ...){
xvar=match.arg(xvar)
which <- x$lambda
if(length(which)>1) 
eval(parse(text="glmnet:::plotCoef(x$beta,lambda=x$lambda,df=x$df,dev=x$dev.ratio,label=label,xvar=xvar,...)"))
}

###Adapted from file MASS/R/negbin.R
glmregNB <- function(formula, data, weights, offset=NULL, nlambda=100, lambda=NULL, lambda.min.ratio=ifelse(nobs<nvars,.05, .001),alpha=1, gamma=3, rescale=TRUE, standardize=TRUE, penalty.factor = rep(1, nvars), thresh=1e-3, maxit.theta=10, maxit=1000, eps=.Machine$double.eps, trace=FALSE, start = NULL, etastart = NULL, mustart = NULL, theta.fixed=FALSE, theta0=NULL, init.theta=NULL, link=log, penalty=c("enet","mnet","snet"), method="glmreg_fit", model=TRUE, x.keep=FALSE, y.keep=TRUE, contrasts=NULL, convex=FALSE, parallel=TRUE, n.cores=2, ...)
{
    if(!theta.fixed) init.theta <- theta0[1]
    if(theta.fixed)
        if(length(theta0)!=nlambda)
            stop("length of theta0 must be the same as nlambda if theta.fixed=TRUE\n")
        else if(any(theta0 <= 0))
            stop("theta0 must be positive\n") 
    penalty <- match.arg(penalty)
    if(penalty=="enet") convex=FALSE
    loglik <- function(n, th, mu, y, w)
        sum(w*(lgamma(th + y) - lgamma(th) - lgamma(y + 1) + th * log(th) +
               y * log(mu + (y == 0)) - (th + y) * log(th + mu)))
    link <- substitute(link)
    fam0 <- if(missing(init.theta) || is.null(init.theta))
                do.call("poisson", list(link = link))
            else
                do.call("negative.binomial", list(theta = init.theta, link = link))
    dots <- list(...)
    mf <- Call <- match.call()
    m <- match(c("formula", "data", "subset", "weights", "na.action",
                 "etastart", "mustart", "offset"), names(mf), 0)
    mf <- mf[c(1, m)]
    mf$drop.unused.levels <- TRUE
    mf[[1L]] <- as.name("model.frame")
    mf <- eval.parent(mf)
    Terms <- attr(mf, "terms")
    if(method == "model.frame") return(mf)
    Y <- model.response(mf, "numeric")
    ## null model support
    X <- if (!is.empty.model(Terms)) model.matrix(Terms, mf, contrasts) else matrix(,NROW(Y),0)
    w <- model.weights(mf)
    nm <- dim(X[,-1])
    nobs <- n <- nm[1]
    nvars <- m <- nm[2]
    if(is.null(penalty.factor))
        penalty.factor <- rep(1, nvars)
                                        #  if(missing(weights)) w <- weights <- rep(1, length(y))
    if(!length(w)) w <- weights <- rep(1, nrow(mf))
    else w <- weights
    if(any(w < 0)) stop("negative weights not allowed")
    offset <- model.offset(mf)
    ## these allow starting values to be expressed in terms of other vars.
    mustart <- model.extract(mf, "mustart")
    etastart <- model.extract(mf, "etastart")
### thoughts: use glm.nb to estimate intercept-only model which can take care of theta estimate 
    if(is.null(lambda)){
        lmax <- findlam(x=X[,-1], y=Y, weights=weights, family="negbin", theta=NULL, w=NULL, z=NULL, alpha=alpha, penalty.factor=penalty.factor, standardize=standardize)
        lpath <- seq(log(lmax), log(lambda.min.ratio * lmax), length.out=nlambda)
        lambda <- exp(lpath)
    }
    else 
        nlambda <- length(lambda)
    if(standardize){
        wt <- weights/sum(weights)
        one <- rep(1, length(Y))
        Xnew <- X[,-1]
        if(dim(Xnew)[2] > 0){
            xx <- Xnew
            meanx <- drop(wt %*% Xnew)
            Xnew <- scale(Xnew, meanx, FALSE) # centers x
            Xnew <- sqrt(wt) * Xnew
            normx <- sqrt(drop(one %*% (Xnew^2)))
            X[,-1] <- Xnew <- scale(xx, meanx, normx)
        }
    }
    beta <- matrix(NA, ncol=nlambda, nrow=m)
    fitted <- matrix(NA, ncol=nlambda, nrow=n)
    pll <- matrix(NA, ncol=nlambda, nrow=maxit)
    b0 <- tht <- nulldev <- resdev <- rep(NA, nlambda)                           
    convout <- twologlik <- rep(NA, nlambda)
    if(parallel){
        warning("trace argument is ignored since parallel=TRUE")
        cl <- parallel::makeCluster(n.cores)
        registerDoParallel(cl)
        fitall <- foreach(k=1:nlambda) %dopar%{
            if(!missing(method)) {
                if(!exists(method, mode = "function"))
                    stop("unimplemented method: ", sQuote(method))
                glm.fitter <- get(method)
            } else {
                method <- "glm.fit"
                glm.fitter <- stats::glm.fit
            }
            if(is.null(mustart) || missing(init.theta)){
                fit <- glm.nb(Y ~ 1, weights=weights)
                mu <- fit$fitted.values
                th <- fit$theta
            }
            else{
                mu <- mustart
                th <- init.theta
            }
            if(theta.fixed) th <- theta0[k]
            iter <- 0
                                        #d1 <- sqrt(2 * max(1, fit$df.residual))
            d2 <- del <- 1
            Lm <- loglik(n, th, mu, Y, w)
            converged <- FALSE                                    
            while((iter <- iter + 1) <= maxit.theta && !converged){
                eta <- log(mu)
                fit <- glmreg_fit(x=X[,-1], y=Y, weights=w, start = start, etastart=eta, mustart = mu, offset=offset, lambda=lambda[k],alpha=alpha,gamma=gamma,rescale=rescale, standardize=standardize, penalty.factor = penalty.factor, thresh=thresh, maxit=maxit, eps=eps, family="negbin", theta=th, trace=trace, penalty=penalty)
                t0 <- th
                mu <- fit$fitted.values
                if(!theta.fixed){
                                        #      th <- theta.ml(Y, mu, sum(w), w, limit=maxit.theta,
                    th <- theta.ml(Y, mu, sum(w), w,
                                   trace = trace)
                }
                else th <- theta0[k]
                start <- c(fit$b0, fit$beta)
                del <- t0 - th
                Lm0 <- Lm
                #penval <- ifelse(standardize, n*fit$penval, fit$penval)
                #Lm <- loglik(n, th, mu, Y, w) - penval
                Lm <- loglik(n, th, mu, Y, w)
                fit$df.residual <- n - fit$df - 1
                d1 <- sqrt(2 * max(1, fit$df.residual))
                converged <- abs((Lm0 - Lm)/d1) + abs(del/d2) < 1e-8
            }
            Lm <- loglik(n, th, mu, Y, w)
            fit$converged <- converged
            fit$twologlik <- 2 * Lm
            fit$th <- th
            fit
        }
                                        #stopImplicitCluster()
        parallel::stopCluster(cl)
        for(k in 1:nlambda){
            tht[k] <- fitall[[k]]$th
            beta[,k] <- fitall[[k]]$beta
            b0[k] <- fitall[[k]]$b0
            fitted[,k] <- fitall[[k]]$fitted
            convout[k] <- fitall[[k]]$converged
            twologlik[k] <- fitall[[k]]$twologlik
            nulldev[k] <- fitall[[k]]$nulldev
            resdev[k] <- fitall[[k]]$resdev
            if(trace)
                pll[,k] <- fitall[[k]]$pll
        }
        fit <- fitall[[nlambda]] ### Why the last model?
    }else{
        k <- 1
        while(k <= nlambda){
            if(trace) message("loop in lambda:", k, "\n")
            if(k==1){
                if(trace){ cat("Initial fit family is:")
                    print(fam0)
                }
                if(trace) message("Initial fit:")
                if(!missing(method)) {
                    if(!exists(method, mode = "function"))
                        stop("unimplemented method: ", sQuote(method))
                    glm.fitter <- get(method)
                } else {
                    method <- "glm.fit"
                    glm.fitter <- stats::glm.fit
                }
                if(is.null(mustart) || missing(init.theta)){
                    fit <- glm.nb(Y ~ 1, weights=weights)
                    mu <- fit$fitted.values
                    th <- fit$theta
                }
                else{
                    mu <- mustart
                    th <- init.theta
                }
                if(trace)
                    message("Initial value for theta:", signif(th))
            }
            if(theta.fixed) th <- theta0[k]
            iter <- 0
                                        #d1 <- sqrt(2 * max(1, fit$df.residual))
            d2 <- del <- 1
            Lm <- loglik(n, th, mu, Y, w)
            converged <- FALSE
            while((iter <- iter + 1) <= maxit.theta && !converged){
                eta <- log(mu)
                fit <- glmreg_fit(x=X[,-1], y=Y, weights=w, start = start, etastart=eta, mustart = mu, offset=offset, lambda=lambda[k],alpha=alpha,gamma=gamma,        rescale=rescale, standardize=standardize, penalty.factor = penalty.factor, thresh=thresh, maxit=maxit, eps=eps, family="negbin", theta=th, trace=trace,        penalty=penalty)
                t0 <- th
                mu <- fit$fitted.values
                if(!theta.fixed){
                    th <- theta.ml(Y, mu, sum(w), w,
                                   trace = trace)
                }
                else th <- theta0[k]
                start <- c(fit$b0, fit$beta)
                del <- t0 - th
                Lm0 <- Lm
                #penval <- ifelse(standardize, n*fit$penval, fit$penval)
                #Lm <- loglik(n, th, mu, Y, w) - penval # is penval needed?
                Lm <- loglik(n, th, mu, Y, w)
                fit$df.residual <- n - fit$df - 1
                d1 <- sqrt(2 * max(1, fit$df.residual))
                converged <- abs((Lm0 - Lm)/d1) + abs(del/d2) < 1e-8
                if(trace) {
                    Ls <- loglik(n, th, Y, Y, w)
                    Dev <- 2 * (Ls - Lm)
                    message("Theta(", iter, ") =", signif(th),
                            ", 2(Ls - Lm) =", signif(Dev))
                }
            }
            if(!is.null(attr(th, "warn"))) fit$th.warn <- attr(th, "warn")
            if(trace && iter > maxit.theta) {
                warning("alternation limit reached")
                fit$th.warn <- gettext("alternation limit reached")
            }
            tht[k] <- th
            beta[,k] <- as.vector(fit$beta)
            b0[k] <- fit$b0
            Lm <- loglik(n, th, mu, Y, w)
            twologlik[k] <- as.vector(2 * Lm)
            nulldev[k] <- fit$nulldev
            resdev[k] <- fit$resdev
            if(trace) pll[,k] <- fit$pll
            convout[k] <- converged
            fitted[,k] <- fit$fitted.values
            k <- k + 1
        }
    }
    class(fit) <- c("glmregNB", "glmreg", "lm")
    fit$terms <- Terms
    fit$formula <- as.vector(attr(Terms, "formula"))
    ## make result somewhat reproducible, but not sure what does init.theta mean here!
    Call$init.theta <- signif(as.vector(fit$theta), 10)
    Call$link <- link
    fit$call <- Call
    if(model) fit$model <- mf
    fit$na.action <- attr(mf, "na.action")
    if(x.keep) fit$x <- data[,colnames(data)%in%attr(Terms, "term.labels")]
    if(!y.keep) fit$y <- NULL
    if(standardize) {
        beta <- beta/normx
        b0 <- b0 - crossprod(meanx, beta)
    }
    fit$twologlik <- twologlik
    fit$df <- apply(abs(beta) > 0, 2, sum) ### excluding intercept and theta
    fit$aic <- -fit$twologlik + 2*(2+fit$df)
    fit$bic <- -fit$twologlik + log(n)*(2+fit$df)
    fit$fitted.values <- fitted
    fit$contrasts <- attr(X, "contrasts")
    fit$xlevels <- .getXlevels(Terms, mf)
    fit$method <- method
    fit$offset <- fit$offset
    if(is.null(colnames(X[,-1]))) varnames <- paste("V", 1:ncol(X[,-1]), sep="")
    else varnames <- colnames(X[,-1])
    dimnames(beta) <- list(varnames, round(lambda, digits=4))
    fit$beta <- beta
    fit$b0 <- matrix(b0, nrow=1)

    fit$lambda <- lambda
    fit$nlambda <- nlambda
    fit$nulldev <- nulldev
    fit$resdev <- resdev
    if(trace)
        fit$pll <- pll
    else fit$pll <- NULL
    fit$theta <- as.vector(tht)
    fit$converged <- convout
    fit
}


summary.glmregNB <- function(object, ...)
{
    if(!is.null(object$th.warn))
        summ <- object[c("theta", "twologlik", "th.warn")]
    else summ <- object[c("theta", "twologlik")]
    class(summ) <- "summary.glmregNB"
    summ
}

print.summary.glmregNB <- function(x, ...)
{
    NextMethod()
    cat("\n              Theta: ", format(round(x$theta, 3)),
        "\n")
    if(!is.null(x$th.warn))
        cat("Warning while fitting theta:", x$th.warn,"\n")
    cat("\n 2 x log-likelihood: ", format(round(x$twologlik, 3)), "\n")
    invisible(x)
}
pen_sum <- function(b, n, lambda, alpha, gamma, penalty){
    lc <- length(b)
    pen1 <- rep(0, lc)
    if(lc > 0){
        for(j in 1:lc)
            pen1[j] <- pen_eval(abs(b[j]), lone=lambda*alpha, ltwo=lambda*(1-alpha), gamma=gamma, penalty=penalty)
    }
    return(n*(sum(pen1)))
}

C loss value in subroutine irglmreg, cf: loss.f
C dfun=1,4,5
C input: f is linear predictors, not probability if dfun=5
C output: los is the average loss

      subroutine loss2(n, y, f, weights, cfun, dfun, s, delta, los)
       implicit none
       integer cfun, dfun, i, n
       double precision y(n),f(n),u(n),z(n),gval(n),s,los,weights(n),
     +   delta
       external compute_g, compute_z, compute_u 

      if(dfun .NE. 1 .AND. dfun .NE. 4 .AND. dfun .NE. 5)then
          call rexit("dfun not implmented in loss2")
      endif
      call compute_u(dfun, n, y, f, u)
C     s is not used in compute_z for dfun:1,4-7
      call compute_z(dfun, n, u, z, s)
      call compute_g(cfun, n, z, s, delta, gval)
      los = 0.d0
      do 20 i=1, n
      los = los + weights(i)*gval(i)
20    continue
C      los=los/n

      return
      end

### multi-split p-value for zeroinfl function (ZIP)
pval.zipath <- function(formula, data, weights, subset, na.action, offset, standardize=TRUE,
                     family = c("poisson", "negbin", "geometric"),
                     penalty = c("enet", "mnet", "snet"), gamma.count=3, gamma.zero=3, prop=0.5, trace=TRUE, B=10, ...){
    fam <- match.arg(family)
    pen <- match.arg(penalty)
    j <- 1
    count.pval <- zero.pval <- NULL

    while(j <= B){
### 1. randomly split the original data into two disjoint groups, din and dout, of equal size
        n <- dim(data)[1]
        nhalf <- round(n*prop,0)
        if(trace) cat("\nmulti-split", j, "\n")
        set.seed(j)
        s <- sample(n)[1:nhalf]
        din <- data[s,]
        dout <- data[-s,]
### 2. using only din to estimate the set of active predictors S 
        fitpen <- tuning.zipath(formula, data=din, family=fam, penalty=pen, gamma.count=gamma.count, gamma.zero=gamma.zero, ...)
        rhs1.0 <- attr(fitpen$terms$count, "term.labels")
        rhs2.0 <- attr(fitpen$terms$zero, "term.labels")
        xsel <- coef(fitpen, which=which.min(fitpen$bic))
        if(trace) print(xsel)
        rhs1c <- which(abs(xsel$count)[-1] > 0)
        rhs2z <- which(abs(xsel$zero)[-1] > 0)
        rhs1 <- rhs1.0[rhs1c]
        rhs2 <- rhs2.0[rhs2z]
### 3(a) using only dout to fit the selected variables in S with ordinary ZIP and calculate the corresponding p-values Q_j, for j \in S 
### 3(b) set the remaining p-values to 1
        if(length(rhs1)==0) rhs1tmp <- 1
        else {
            rhs1tmp <- rhs1[1]
            if(length(rhs1) > 1)
                for(i in 2:length(rhs1))
                    rhs1tmp <- paste(rhs1tmp, "+", rhs1[i])
        }
        if(length(rhs2)==0) rhs2tmp <- 1
        else {
            rhs2tmp <- rhs2[1]
            if(length(rhs2) > 1)
                for(i in 2:length(rhs2))
                    rhs2tmp <- paste(rhs2tmp, "+", rhs2[i])
        }
        res <- deparse(terms(fitpen$terms$count)[[2]])      ### response variable
                                        #rhs1 <- attr(fit$terms$count, "term.labels")
                                        #rhs2 <- attr(fit$terms$zero, "term.labels")
        out <- paste(res, "~", rhs1tmp, "|", rhs2tmp)
                                        # set the environment of the formula (i.e. where should
                                        # R look for variables when data aren't specified?)
	environment(out) <- parent.frame()
                                        #fit <- zipath(eval(parse(text=out)), data=data, penalty=penalty)
        fit <- try(zeroinfl(eval(parse(text=out)), data=dout, dist=fam))
        if(trace) print(summary(fit))
        if(inherits(fit, "try-error")){
        j <- j+1
        B <- B+1
        next
        }
        coef <- summary(fit)$coef
### excluding intercept
        if(length(rhs1c) > 0){
        pc <- rep(1, length(rhs1.0))
        pc[rhs1c] <- coef$count[2:(1+length(rhs1c)),4] 
        ### 4. Define the adjusted p-values as 
        pc <- pc*(length(rhs1c)+length(rhs2z)) 
        count.pval <- rbind(count.pval, pmin(1, pc)) 
        }
        if(length(rhs2z) > 0){
        pz <- rep(1, length(rhs2.0))
        pz[rhs2z] <- coef$zero[2:(1+length(rhs2z)),4] 
        pz <- pz*(length(rhs1c)+length(rhs2z)) 
        zero.pval <- rbind(zero.pval, pmin(1, pz))
        }
        j <- j + 1
    }
    colnames(count.pval) <- rhs1.0
    colnames(zero.pval) <- rhs2.0
    Q <- function(x, gamma) min(1, quantile(x, gamma, na.rm=TRUE)/gamma)
    P <- function(x){
    gam <- seq(0.05, 0.99, by=0.01)
    res <- rep(NA, length(gam))
    for(i in 1:length(gam))
    res[i] <- Q(x, gam[i])
    res <- min(1, (1-log(min(gam)))*min(res))
}
    count.pval.q <- apply(count.pval, 2, P)
    zero.pval.q <- apply(zero.pval, 2, P)
    return(list(count.pval=count.pval, zero.pval=zero.pval, count.pval.q=count.pval.q, zero.pval.q=zero.pval.q))
}

C adapted from MASS/negbin theta.ml function but
C input n is different from theta.ml there
      subroutine theta_ml(y, mu, n, weights, limit, eps, t0, trace)

      implicit none
      integer i, n, it, limit, trace
      double precision a, t0, y(n), mu(n), weights(n), 
     +       eps, del, score, info
      external score, info
      
C     eps = .Machine$double.eps**0.25 
      eps = 0.0001220703 
      a=0
      do i=1, n
       a=a+weights(i)*(y(i)/mu(i) - 1.0D0)**2
      enddo
      t0 = sum(weights)/a
      it = 1
      del = 1
      if(trace .EQ. 1)then 
              call dblepr("initial theta=", -1, t0, 1)
      endif

30    if(it .LE. limit .AND. dabs(del) > eps)then
        t0 = dabs(t0)
        del=score(n, t0, mu, y, weights)/info(n,t0, mu,y,weights)
        t0 = t0 + del
        if(trace .EQ. 1)then 
              call intpr("theta iteration", -1, it, 1)
              call dblepr("        theta=", -1, t0, 1)
        endif
        it = it + 1
        goto 30
      endif
      if(t0 < 0)then
              t0 = 0
              call intpr("estimate truncated at zero", -1, 1, 1)
      endif
      if(it .EQ. limit .AND. trace .EQ. 1)then
              call intpr("iteration limit reached", -1, 1, 1)
      endif
      
      return
      end

      function score(n, th, mu, y, w)
      implicit none
      integer i, n
      double precision y(n), mu(n), th, w(n), rdigamma, 
     + score
      external :: rdigamma

      score = 0
      do i=1, n
       score=score+w(i)*(rdigamma(th + y(i)) - rdigamma(th) + dlog(th) +
     +          1.0D0 - dlog(th + mu(i)) - (y(i) + th)/(mu(i) + th))
      enddo
      
      return
      end function 

      function info(n, th, mu, y, w)
      implicit none
      integer i, n
      double precision y(n), mu(n), th, w(n), info, 
     + rtrigamma
      external :: rtrigamma 
      
      info = 0
      do i=1, n
       info=info+w(i)*(-rtrigamma(th + y(i)) + rtrigamma(th) - 
     +      1.0D0/th + 2.0D0/(mu(i) + th) - 
     +      (y(i) + th)/(mu(i) + th)**2.0D0)
      enddo
      
      return
      end function 

C     used in nclreg_fortran subroutine for one single lambda penalty
C     parameter
C     input: start_act, etastart, mustart
C     output: start_act, etastart, mustart, beta_1, b0_1, fk
      subroutine nclreg_onelambda(x_act, y, weights, n,m_act,start_act, 
     +     etastart, mustart, yhat, offset, lambda_i, alpha, gam, 
     +     intercept,penaltyfactor_act,maxit, eps, penalty, trace, iter,
     +     del, rfamily, B, s, thresh, beta_1, b0_1, fk)
      implicit none
      integer n,k,j,intercept,penalty,maxit,trace,iter,rfamily,m_act,
     +        satu
      double precision y(n), weights(n),etastart(n), mustart(n), 
     +     offset(n),lambda_i,alpha,gam,eps,los,penval,pll,pll_old,
     +     thresh, b0_1, yhat(n), d, del, fk_old(n), s, B, 
     +     h(n), fk(n), x_act(n, m_act), start_act(m_act+1),
     +     beta_1(m_act), penaltyfactor_act(m_act)

         k = 1
         d = 10
         call loss(n, y, yhat, rfamily, s, los)
         call penGLM(beta_1, m_act, lambda_i*penaltyfactor_act,
     +        alpha, gam, penalty, penval)
         pll_old=los + penval
 500     if(d .GT. del .AND. k .LE. iter)then
            if(trace .EQ. 1)then
               call intpr("  nclreg_onelambda iteration k=", -1, k, 1)
               call dblepr("     start_act", -1, start_act, m_act+1)
            endif
            call dcopy(n, yhat, 1, fk_old, 1)
            call compute_h(rfamily, n, y, fk_old, s, B, h)
C     check if h has NAN value, can be useful to debug
C            if(trace .EQ. 1)then
C            do 30 ii=1, n
C               if(h(ii) .NE. h(ii))then
C                  call intpr("  nclreg_onelambda iteration k=",-1,k,1)
C                  call intpr("    ii=", -1, ii, 1)
C                  call dblepr("     h(ii)", -1, h(ii), 1)
C                  call dblepr("     fk_old(ii)", -1, fk_old(ii), 1)
C                  exit
C               endif
C 30         continue
C            endif
            call glmreg_fit_fortran(x_act, h,weights,n,m_act,start_act, 
     +           etastart, mustart, offset, 1, lambda_i, alpha,
     +           gam, 0, 0, intercept, penaltyfactor_act, thresh,
     +           0.0D0, maxit, eps, 0.0D0, 1,
     +           penalty, trace, beta_1, b0_1, yhat, satu)
            call dcopy(n, yhat, 1, fk, 1)
            call dcopy(n, yhat, 1, mustart, 1)
C     call dcopy(n, yhat, 1, etastart, 1): this is not needed for
C     the above glmref_fit call (that calls zeval) since family=1.
C            d = 0
C            d=d+(start_act(1)-b0_1)**2
            start_act(1) = b0_1
            if(m_act .GT. 0)then
               do 100 j=1, m_act
C                  d=d+(start_act(j+1)-beta_1(j))**2
                  start_act(j+1)=beta_1(j)
 100           continue
            endif
         call loss(n, y, yhat, rfamily, s, los)
         call penGLM(beta_1, m_act, lambda_i*penaltyfactor_act,
     +        alpha, gam, penalty, penval)
         pll=los + penval
         d=dabs((pll-pll_old)/pll_old)
         pll_old = pll
C           d = 0
C            do 120 ii=1, n
C               d=d+(fk_old(ii) - fk(ii))**2
C  120       continue
            if(trace .EQ. 1)then
C               call dblepr("b0_1", -1, b0_1, 1)
               call dblepr("beta_1", -1, beta_1, m_act)
               call dblepr("updated start_act", -1, start_act, m_act+1)
C              d is not an array
C              call dblepr("     d=", -1, d, 1) 
            endif
            k = k + 1
            goto 500
         endif

      return
      end

### random number generation for zero-inflated response variable
### x is the design matrix of count model
### z is the design matrix of zero model
### a is the coefficient for x
### b is the coefficient for z
rzi <- function(n, x, z, a, b, theta=1, family=c("poisson", "negbin", "geometric"), infl=TRUE){
    family <- match.arg(family)
    if(family=="geometric" && theta!=1) stop("theta should be 1 for family='geometric'")
    mu <- exp(a[1] + x %*% a[-1])
    p <- exp(b[1] + z %*% b[-1])
    w <- 1/(1+1/p)  ### logit link
    if(family=="poisson")
        y <- rpois(n, lambda=mu)
    else y <- rnegbin(n, mu=mu, theta=theta) ###check
    if(infl){
        y[runif(n) < w] <- 0
        cat("zero inflation", mean(w), "\n")
        cat("proportion of y=0", (length(y[y==0])/length(y)), "\n")
    }
    return(y)
}

C compute density function of negative biomial distribution with
C parameter size and mu, in the same parameterization as in R dnbinom.
C rlgamma calls C function lgammafn: log gamma function, C Mathlib
C function
      function dnbinom(x, size_n, mu, log_true)
      integer :: x, j, Factorial, log_true
      double precision :: res, p, size_n, mu, dnbinom, rlgamma
      external :: Factorial, rlgamma

      if(size_n .LE. 0)then
        call dblepr("size should be strictly positive", -1, size_n, 1)
      endif

      p = size_n/(size_n+mu)
      if(log_true==0)then
      dnbinom=dexp(rlgamma(x*1.0D0+size_n)-rlgamma(size_n))/Factorial(x)
     +        *p**size_n*(1-p)**x
      else
        res=0 
        if(x > 0)then
            do j=1, x
            res=res+dlog(j*1.0D0)
            enddo
        endif
        dnbinom = rlgamma(x*1.0D0+size_n)-rlgamma(size_n) - res
     +  +size_n*dlog(p)+x*1.0D0*dlog(1-p) 
      endif

      return
      end function

C###Adapted from file MASS/R/glmregNB.R
C input: theta0 is useful if theta_fixed=1, in which case theta0 is a 
C  vector of length nlambda
      subroutine glmregnb_fortran(x, y, weights, n, m, offset, nlambda,
     +    lambda, penalty, alpha, gam, rescale, standardize, intercept, 
     +    penaltyfactor, thresh, maxit_theta, maxit, eps,epsbino,start, 
     +    etastart, mustart, thetastart, theta_fixed, theta0, trace,
     +    beta, b0, tht, yhat)
      implicit none
      integer k, i, j, n, m, trace, nlambda, penalty, maxit_theta, 
     +  maxit, rescale, standardize, iter, theta_fixed, satu, intercept
      double precision x(n, m), y(n), lambda(nlambda), weights(n),
     +  offset(n), start(m+1), etastart(n), mustart(n), thetastart, 
     +  del, epsbino, theta0(nlambda), d, tht(nlambda), eps, thresh, 
     +  alpha, gam, penaltyfactor, beta_1(m, 1), b0_1, 
     +  beta(m, nlambda), b0(nlambda), yhat(n)
      
      k = 1
C      convout <- twologlik <- rep(NA, nlambda)
      del = eps**0.25
10    if(k <= nlambda)then
        if(trace .EQ.1 )then
            call intpr("loop in lambda:", -1,  k, 1)
        endif
        if(theta_fixed .EQ. 1)then
            thetastart = theta0(k)
        endif
        iter = 0
        d = 10
30        if(d .GT. del .AND. iter <= maxit_theta)then
            call glmreg_fit_fortran(x, y, weights, n, m,start,
     +       etastart, mustart, offset, 1, lambda(k), alpha, gam, 
     +       rescale, standardize, intercept, penaltyfactor, thresh, 
     +       epsbino, maxit,eps, thetastart, 4, penalty, trace, beta_1, 
     +       b0_1, yhat, satu)
            call dcopy(n, yhat, 1, mustart, 1)
            do i=1, n
            etastart(i) = dlog(mustart(i))
            enddo
            if(theta_fixed .EQ. 0)then
                call theta_ml(y, mustart, n, weights, 10,
     +           eps**0.25, thetastart, trace)
            endif
            d=0
            d=d+(start(1)-b0_1)**2
            start(1)=b0_1
            do j=1, m
            d=d+(start(j+1)-beta_1(j,1))**2
            start(j+1)=beta_1(j,1)
            enddo
C            del <- thetastart - th
C            Lm0 <- Lm
C            penval <- ifelse(standardize, n*fit$penval, fit$penval)
C            Lm <- loglik(n, th, mu, Y, w) - penval
C            fit$df.residual <- n - fit$df - 1
C            d1 <- sqrt(2 * max(1, fit$df.residual))
C            converged <- abs((Lm0 - Lm)/d1) < 1e-8
C            if(trace) {
C                Ls <- loglik(n, th, Y, Y, w)
C                Dev <- 2 * (Ls - Lm)
C                message("Theta(", iter, ") =", signif(th),
C                        ", 2(Ls - Lm) =", signif(Dev))
C            }
            iter=iter + 1
            goto 30
        endif
        tht(k) = thetastart
        call dcopy(m, beta_1, 1, beta(1:m, k), 1)
        b0(k) = b0_1
C        call loglikFor(n, y, mustart, thetastart, weights, 4, loglik(k))
C        nulldev(k) = nulldev0
C        resdev(k) = resdev0
C        if(trace) pll[,k] <- fit$pll
C        convout[k] <- converged
C        fitted[,k] <- fit$fitted.values
        k = k + 1
        goto 10
      endif
 
      return
      end

C     used in irglmreg.R (adapted from nclreg_ad) for active=TRUE and decreasing=TRUE
C     output: nlambdacal: number of lambda actually computed including
C     repeated ones
C     
      subroutine irglmreg_ad(x, y, weights, n,m,start, etastart,
     +     mustart, offset, iter, nlambda, lambda, alpha, gam,rescale, 
     +     standardize, intercept, penaltyfactor, maxit, eps, theta, 
     +     epscycle, penalty, trace, del, cfun, dfun, s, thresh, 
     +     decreasing, beta, b0, yhat, los, pll, nlambdacal, delta,
     +     weights_cc)
      implicit none
      integer n,m,i,ii,j,jj,penalty,nlambda, standardize, maxit,
     +     trace, iter, jk, activeset(m), intercept, rescale,
     +     activeset_old(m), m_act, nlambdacal, uturn, decreasing, 
     +     cutpoint, nact, conv, jc, fakejk, AllocateStatus, cfun, dfun,
     +     dfunnew
      double precision x(n,m),y(n),weights(n),weights_update(n),
     +     weights_cc(n, nlambda), sumw, wt(n),start(m+1),
     +     etastart(n),mustart(n),offset(n),lambda(nlambda),alpha,gam,
     +     eps, penaltyfactor(m), thresh, beta(m, nlambda), epscycle,
     +     betaall(m), b0all, b0(nlambda), b0_1, yhat(n), del, theta, 
     +     lambda_i, s, fk(n), los(nlambda),delta, pll(nlambda), penval
      double precision, dimension(:, :), allocatable :: x_act
      double precision, dimension(:), allocatable :: start_act, beta_1,
     +     penaltyfactor_act

      if(dfun .EQ. 8)then
        dfunnew= 3
        else if(dfun .EQ. 9)then
         dfunnew= 4
      endif
      b0all=start(1)
      sumw = 0.0D0
      do i=1, n
       sumw=sumw+weights(i)
      enddo
      do i=1, n
       wt(i)=weights(i)/sumw
      enddo

      do ii=1, m
         activeset(ii)=ii
         activeset_old(ii)=ii
         betaall(ii)=start(ii+1)
      enddo
      call find_activeset(m, start(2:(m+1)), eps, activeset, jk)
      fakejk=0
      if(jk==0)then
         jk=1
         activeset(1)=1
         fakejk=1
      endif
      do ii=1, jk
         activeset_old(ii)=activeset(ii)
      enddo
      m_act = jk
      allocate(start_act(m_act+1), stat=AllocateStatus)
      if(AllocateStatus .NE. 0)then
         return
      endif
      allocate(penaltyfactor_act(m_act), stat=AllocateStatus)
      allocate(x_act(n, m_act), stat=AllocateStatus)
      if(AllocateStatus .NE. 0)then
         return
      endif
      do jj=1, n
         do ii=1, m_act
            x_act(jj, ii)=x(jj, activeset(ii))
         enddo
      enddo

      allocate(beta_1(m_act), stat=AllocateStatus)
      if(AllocateStatus .NE. 0)then
         return
      endif
      start_act(1)=start(1)
      do j=1, m_act
         beta_1(j)=0
         start_act(j+1)=start(1+activeset(j))
         penaltyfactor_act(j)=penaltyfactor(activeset(j))
      enddo
      i=1
      nlambdacal=0
      uturn=0
      cutpoint=1
 10   if(i .LE. nlambda)then
         if(trace .EQ. 1)then
            call intpr("irglmreg_ad lambda iteration i=", -1, i, 1)
         endif
C unlike in nclreg_ad, no need B: lambda_i=lambda(i)/B
         lambda_i=lambda(i)
         j=1
         nact=2
         conv=0
13000    if(j <= nact .AND. conv==0)then
            if(trace .EQ. 1)then
               call intpr("begin activeset irglmreg_onelambda",-1,1,1)
            endif
            call irglmreg_onelambda(x_act, y,weights, n,m_act,start_act,
     +           etastart, mustart, yhat, offset, lambda_i, alpha, gam, 
     +           rescale, standardize, intercept, penaltyfactor_act, 
     +           maxit, eps, theta, penalty,trace, iter,del, cfun, dfun,
     +           s, thresh, beta_1, b0_1, fk, delta, weights_update)
            start(1)=b0_1
            do ii=1, m_act
               start(activeset(ii)+1)=beta_1(ii)
            enddo
            if(j .NE. nact)then
               if(trace .EQ. 1)then
C                  call dblepr("b0_1=", -1, b0_1, 1)
                  call dblepr("beta_1=", -1, beta_1, m_act)
                  call intpr("begin fullset irglmreg_onelambda",1,1,1)
                  call dblepr("start=", -1, start, m+1)
               endif
               call irglmreg_onelambda(x,y,weights,n,m,start, etastart,
     +              mustart, yhat, offset, lambda_i, alpha, gam, 
     +              rescale, standardize, intercept, penaltyfactor, 
     +              maxit, eps, theta, penalty, trace, 1, del, cfun, 
     +              dfun, s, thresh, betaall, b0all,fk, delta,
     +              weights_update)
               call find_activeset(m, betaall, eps, activeset, jk)
               if(jk==0)then
                  jk=1
                  activeset(1)=1
                  fakejk=1
               endif
            endif
         if(jk .NE. m_act)then
             conv=0
         else
            jc=0
            do ii=1, max(jk, m_act)
               if(activeset(ii)==activeset_old(ii))then
                  jc=jc+1
               endif
            enddo
            if(jk==jc)then
               conv=1
            endif
        endif
            if(jk .NE. m_act .OR. jk .NE. jc)then
               deallocate(beta_1, start_act, penaltyfactor_act, x_act)
               allocate(beta_1(jk), stat=AllocateStatus)
               allocate(start_act(jk+1), stat=AllocateStatus)
               allocate(penaltyfactor_act(jk), stat=AllocateStatus)
               allocate(x_act(n, jk), stat=AllocateStatus)
               start_act(1)=b0all
               do 11135 ii=1, jk
                  beta_1(ii)=0
                  start_act(ii+1)=betaall(activeset(ii))
                  penaltyfactor_act(ii)=penaltyfactor(activeset(ii))
                  activeset_old(ii)=activeset(ii)
11135          continue
               do 11155 jj=1, n
                  do 11145 ii=1, jk
                     x_act(jj, ii)=x(jj, activeset(ii))
11145             continue
11155          continue
               m_act = jk
            else 
               if(fakejk==1)then
                  start_act(1)=b0all
                  start_act(2)=betaall(1)
               endif
            endif
            j=j+1
            goto 13000
         endif
         nlambdacal=nlambdacal+1
         b0(i)=b0_1
         do ii=1, n
         weights_cc(ii,i)=weights_update(ii)
         enddo
         if(jk .GT. 0)then
            do ii=1, jk
               beta(activeset(ii), i)=beta_1(ii)
            enddo
         endif
         if(dfun .EQ. 1 .OR. dfun .EQ. 4 .OR. dfun .EQ. 5)then
             call loss2(n, y, fk, wt, cfun, dfun, s, delta, los(i))
         else 
         call loss3(n,y,mustart,theta,wt,cfun,dfunnew,s,delta,
     +        los(i))
         endif
         call penGLM(beta_1, m_act, lambda_i*penaltyfactor_act, 
     +        alpha, gam, penalty, penval)
         if(standardize .EQ. 1)then
            pll(i)=los(i) + n*penval
         else 
            pll(i)=los(i) + penval
         endif
C     redo (i-1)-lambda estimates with the current start_act for the
C     i-th lambda. Note, i=i-2 not i-1 will do this since i=i+1 is
C     computed before the end of the loop.
         if(decreasing==1)then
            if(i > 1)then
               if(abs(pll(i)-pll(i-1))/pll(i) > epscycle)then
                  if(cutpoint==1)then
                     cutpoint = i
                  endif
                  i = i - 2
                  uturn=1
               else 
                  uturn=0
               endif
            endif
         endif
         if(cutpoint > 1 .AND. uturn==0)then
            i = cutpoint
            cutpoint = 1
         endif
         i = i + 1
         goto 10
      endif
      deallocate(beta_1, start_act, x_act, penaltyfactor_act)
      return
      end

C loss subroutine converted from R package bst in bst.R function
C family = c("gaussian", "binom", "poisson"), s=-1, sh, fk=NULL
C      gausssian: 1, bino: 2, poisson:3, negative binomial: 4, clossR: 11, closs: 12, gloss:
C      13, qloss: 14
C sh and fk are not used below, but may needed in the future release
C cost: but not implemented in optimization problem, unlike in bst!
C output: los is the average loss

      subroutine loss(n, y, f, family, s, los)
       implicit none
       integer family, i, n
       double precision x, y(n),f(n),u, s, los
       external nonconvexloss

C      if(family .EQ. 2)then
C           do 10 i=1, n
C               if(y(i) .EQ. 1.0)then
C                 ly(i)=1-cost
C               else  
C                 ly(i)=1.0D0
C                 ly(i)=cost
C               endif
C10         continue 
C      endif
      los = 0.d0
      do 20 i=1, n
        if(family .EQ. 1)then
          los = los + 0.5D0*(y(i) - f(i))**2
        else if(family .EQ. 2)then
          los =  los + dlog(1+dexp(-y(i)*f(i))) 
C          los =  los + ly(i)*dlog(1+dexp(-y(i)*f(i))) 
        else if(family .EQ. 11)then
                u=y(i)-f(i)
          call nonconvexloss(family, u, s, x)
          los = los + x
        else if(family .EQ. 12 .OR. family .EQ. 13 .OR. 
     +          family .EQ. 14)then
                u=y(i)*f(i)
                 call nonconvexloss(family, u, s, x)
                 los = los + x
        endif
20    continue
        los=los/n

      return
      end

C ###clossR for C-loss regression with u= f - y
C ###closs  for C-loss classification with y={1, 1} and u=y * f
C family
C   clossR: 11, closs: 12, gloss: 13, qloss: 14
      subroutine nonconvexloss(family, u, s, los)
       implicit none
       integer family
       double precision cval, u, s, los
       external pnorm_fortran

       if(family .EQ. 11)then
        los=1-1/dexp(u**2/(2*s**2))
         else if(family .EQ. 12)then
           cval = 1/(1 - dexp(-1/(2*s**2)))
           los=cval*(1-1/dexp((1-u)**2/(2*s**2)))
         else if(family .EQ. 13)then
           los=2**s/(dexp(u)+1)**s 
         else if(family .EQ. 14)then
                 u=u/s
                 call pnorm_fortran(u)
                 los=2*(1-u)
       endif

        return
        end

C compute pnorm as in R, but we have the erf() function available in
C Fortran; it's a linear transformation of pnorm().  That is,
C erf(x) = 2*pnorm(x*sqrt(2)) - 1

        subroutine pnorm_fortran(x)
                implicit none
                double precision x
                
                  x=(erf(x/sqrt(2.0))+1)/2
                return
        end

### backward elimination for zeroinfl function
be.zeroinfl <- function(object, data, dist=c("poisson", "negbin", "geometric"), alpha=0.05, trace=FALSE){
    if(!inherits(object, "zeroinfl")) stop("object must be zeroinfl\n")
    dist <- match.arg(dist)
    fit <- object
    rhs1 <- attr(fit$terms$count, "term.labels")
    rhs2 <- attr(fit$terms$zero, "term.labels")
    nj <- length(rhs1)*length(rhs2)
    j <- 1
    if(trace) {
        cat("Initial model\n")
        print(summary(fit))
    }
    RET <- matrix(NA, nrow=nj, ncol=3)
    colnames(RET) <- c("loglik", "BIC", "AIC")
    while(T){
        if(trace) cat("\nstep", j, "\n")
        coef <- summary(fit)$coef
### excluding intercept
        d <- dim(coef$count)[1]
        if(dist!="negbin")
        count.pval <- coef$count[-1,4] ### find maximum p-value from count
        else count.pval <- coef$count[-c(1,d),4] ### find maximum p-value from count model 
        zero.pval <- coef$zero[-1,4] ### find maximum p-value from count model 
        nc <- length(count.pval)
        nz <- length(zero.pval)
        if(dist!="negbin")
        count.order <- order(coef$count[-1,4], decreasing=TRUE) ### which variable has maximum p-value from count model 
        else count.order <- order(coef$count[-c(1,d),4], decreasing=TRUE) ### which variable has maximum p-value from count model 
        zero.order <- order(coef$zero[-1,4], decreasing=TRUE) ### which variable has maximum p-value from zero model 
        rhs1 <- attr(fit$terms$count, "term.labels")
        rhs2 <- attr(fit$terms$zero, "term.labels")

        kc <- 1
        kz <- 1
            count.max <- count.pval[count.order[kc]] 
            zero.max  <- zero.pval[zero.order[kz]]
            if(is.na(count.max) && is.na(zero.max)) break
            else if(is.na(zero.max)) zero.max <- 0
                 else if(is.na(count.max)) count.max <- 0
            if(count.max > zero.max)
                if(count.max > alpha){
                    newid <- count.order[kc]
                    if(dist!="negbin")
                    dropvar <- rownames(coef$count)[-1][newid]
                    else dropvar <- rownames(coef$count)[-c(1,d)][newid]
                    if(trace) cat("drop variable in count component: ", rhs1[newid],"\n")
                    rhs1 <- rhs1[-newid]
                    kc <- kc + 1
                }
                else break
            else if(zero.max > alpha){
                newid <- zero.order[kc]
                dropvar <- rownames(coef$zero)[-1][newid]
                if(trace) cat("drop variable in zero component: ", rhs2[newid],"\n")
                rhs2 <- rhs2[-newid]
            }
            else break
        if(length(rhs1)==0) rhs1tmp <- 1
        else {
         rhs1tmp <- rhs1[1]
         if(length(rhs1) > 1)
            for(i in 2:length(rhs1))
                rhs1tmp <- paste(rhs1tmp, "+", rhs1[i])
        }
        if(length(rhs2)==0) rhs2tmp <- 1
        else {
        rhs2tmp <- rhs2[1]
        if(length(rhs2) > 1)
            for(i in 2:length(rhs2))
                rhs2tmp <- paste(rhs2tmp, "+", rhs2[i])
}
        res <- deparse(terms(fit$terms$count)[[2]])      ### response variable
        out <- paste(res, "~", rhs1tmp, "|", rhs2tmp)
                                        # set the environment of the formula (i.e. where should
                                        # R look for variables when data aren't specified?)
	environment(out) <- parent.frame()
        fit <- try(zeroinfl(eval(parse(text=out)), data=data, dist=dist))
        if(inherits(fit, "try-error"))
        break
        if(trace){
        print(summary(fit))
        cat("\nloglik of zero-inflated model", logLik(fit), "\n")
        cat("\nBIC of zero-inflated model", AIC(fit, k=log(dim(data)[1])))
        cat("\nAIC of zero-inflated model", AIC(fit))
        }
        RET[j,1] <- logLik(fit)
        RET[j,2] <- AIC(fit, k=log(dim(data)[1]))
        RET[j,3] <- AIC(fit)
        j <- j + 1
    }
    if(trace) print(RET[complete.cases(RET),])                                    #}
    return(fit)
}

### from pscl package, may be useful
## convenience helper function
model_offset_2 <- function(x, terms = NULL, offset = TRUE)
## allow optionally different terms
## potentially exclude "(offset)"
{
  if(is.null(terms)) terms <- attr(x, "terms")
  offsets <- attr(terms, "offset")
  if(length(offsets) > 0) {
    ans <- if(offset) x$"(offset)" else NULL
    if(is.null(ans)) ans <- 0
    for(i in offsets) ans <- ans + x[[deparse(attr(terms, "variables")[[i + 1]])]]
    ans
  }
  else {
    ans <- if(offset) x$"(offset)" else NULL
  }
  if(!is.null(ans) && !is.numeric(ans)) stop("'offset' must be numeric")
  ans
}

"cv.folds" <-
    function(n, folds = 10)
{
    split(sample(1:n), rep(1:folds, length = n))
}

cv.glmreg <- function(x, ...) UseMethod("cv.glmreg", x)

cv.glmreg.default <- function(x, ...) {
    if (extends(class(x), "Matrix"))
        return(cv.glmreg.matrix(x = x, ...))
    stop("no method for objects of class ", sQuote(class(x)),
         " implemented")
}

cv.glmreg.formula <- function(formula, data, weights, offset=NULL, contrasts=NULL, ...){
    ## extract x, y, etc from the model formula and frame
    if(!attr(terms(formula, data=data), "intercept"))
        stop("non-intercept model is not implemented")
    if(missing(data)) data <- environment(formula)
    mf <- match.call(expand.dots = FALSE)
    m <- match(c("formula", "data", "weights",
                 "offset"), names(mf), 0L)
    mf <- mf[c(1L, m)]
    mf$drop.unused.levels <- TRUE
    mf[[1L]] <- as.name("model.frame")
    mf <- eval(mf, parent.frame())

    mt <- attr(mf, "terms") # allow model.frame to have updated it

    Y <- model.response(mf, "any") # e.g. factors are allowed
    ## avoid problems with 1D arrays, but keep names
    if(length(dim(Y)) == 1L) {
        nm <- rownames(Y)
        dim(Y) <- NULL
        if(!is.null(nm)) names(Y) <- nm
        if(!is.null(nm)) names(Y) <- nm
    }
    ## null model support
    X <- if (!is.empty.model(mt)) model.matrix(mt, mf, contrasts) else matrix(,NROW(Y), 0L)
    ## avoid any problems with 1D or nx1 arrays by as.vector.
    weights <- as.vector(model.weights(mf))
    if(!length(weights)) weights <- rep(1, nrow(mf))
    else if(any(weights < 0)) stop("negative weights not allowed")
    if(!is.null(weights) && !is.numeric(weights))
        stop("'weights' must be a numeric vector")
    if(length(weights) != length(Y))
        stop("'weights' must be the same length as response variable")

    offset <- as.vector(model.offset(mf))
    if(!is.null(offset)) {
        if(length(offset) != NROW(Y))
            stop(gettextf("number of offsets is %d should equal %d (number of observations)", length(offset), NROW(Y)), domain = NA)
    }
### End of addition 08/07/2012 

    RET <- cv.glmreg_fit(X[,-1], Y, weights, offset=offset, ...)
    RET$call <- match.call()
    return(RET)
}
cv.glmreg.matrix <- function(x, y, weights, offset=NULL, ...){
    RET <- cv.glmreg_fit(x, y, weights, offset=offset, ...)
    RET$call <- match.call()
    return(RET)
}

cv.glmreg_fit <- function(x, y, weights, offset, lambda=NULL, balance=TRUE, 
                          family=c("gaussian", "binomial", "poisson", "negbin"), type=c("loss", "error"), 
                          nfolds=10, foldid, plot.it=TRUE, se=TRUE, n.cores=2, trace=FALSE, parallel=FALSE,  
                          ...){
    call <- match.call()
    if(missing(foldid) && nfolds < 3)
        stop("smallest nfolds should be 3\n")
    family <- match.arg(family)
    type <- match.arg(type)
    nm <- dim(x)
    nobs <- n <- nm[1]
    nvars <- m <- nm[2]
    if(missing(weights)) weights <- rep(1, nobs)
                                        #    if(is.null(offset)) offset <- rep(0, nobs)
    K <- nfolds
    glmreg.obj <- glmreg_fit(x, y, weights, offset=offset, lambda=lambda, family=family, ...)
    lambda <- glmreg.obj$lambda
    nlambda <- length(lambda)
    if(missing(foldid)){
        if(family=="binomial" && balance)  
            invisible(capture.output(all.folds <- eval(parse(text="balanced.folds(y, K)"))))
        else all.folds <- cv.folds(length(y), K)
    }
    else all.folds <- foldid
    if(parallel){
        cl <- eval(parse(text="parallel:::makeCluster(n.cores)"))
        registerDoParallel(cl)
        i <- 1  ###needed to pass R CMD check with parallel code below
                                        #residmat <- foreach(i=seq(K), .combine=cbind) %dopar% {
                                        #residmat should be a list since it is possible to generate different length of loss values, thus, cbind may fail
        residmat <- foreach(i=seq(K), .packages=c("mpath")) %dopar% {
            omit <- all.folds[[i]]
            if(!is.null(offset)){
                offsetnow <- offset[- omit] 
                newoffset <- offset[omit] 
            }else offsetnow <- newoffset <- NULL 
            fitcv <- glmreg_fit(x[ - omit,,drop=FALSE ], y[ -omit], weights=weights[- omit], offset=offsetnow, lambda=lambda, family=family, ...)
            if(type=="loss") logLik(fitcv, newx=x[omit,, drop=FALSE], y[omit], weights=weights[omit], offset=newoffset)
            else if(family=="binomial" && type=="error"){
                tmp <- predict(fitcv, newx=x[omit,, drop=FALSE], weights=weights[omit], offset=newoffset, type="class")!=y[omit]
                apply(tmp, 2, mean)
            }
        }
        eval(parse(text="parallel:::stopCluster(cl)"))
        residmat <- sapply(residmat, '[', seq(max(sapply(residmat, length))))
        }
        else{
            residmat <- matrix(NA, nlambda, K)
            for(i in seq(K)) {
                if(trace)
                    cat("\n CV Fold", i, "\n\n")
                omit <- all.folds[[i]]
                if(!is.null(offset)){
                    offsetnow <- offset[- omit] 
                    newoffset <- offset[omit] 
                }else offsetnow <- newoffset <- NULL 
                fitcv <- glmreg_fit(x[ - omit,,drop=FALSE ], y[ -omit], weights=weights[- omit], offset=offsetnow, lambda=lambda, family=family, ...)
                if(type=="loss") residmat[1:fitcv$nlambda, i] <- logLik(fitcv, newx=x[omit,, drop=FALSE], y[omit], weights=weights[omit], offset=newoffset)
                else if(family=="binomial" && type=="error"){
                tmp <- predict(fitcv, newx=x[omit,, drop=FALSE], weights=weights[omit], offset=newoffset, type="class")!=y[omit]
                residmat[1:fitcv$nlambda, i] <- apply(tmp, 2, mean)
                }
            }
        }
        cv <- apply(residmat, 1, mean)
        cv.error <- sqrt(apply(residmat, 1, var)/K)
        if(type=="loss") lambda.which <- which.max(cv) else
            if(type=="error" && family=="binomial") lambda.which <- which.min(cv)
        if(family=="binomial")
            good <- which(!is.na(cv))
        else
            good <- 1:nlambda
        obj<-list(fit=glmreg.obj, residmat=residmat[good,], lambda = lambda[good], cv = cv[good], cv.error = cv.error[good], foldid=all.folds, lambda.which= lambda.which, lambda.optim = lambda[lambda.which], type=type)
        if(plot.it) plot.cv.glmreg(obj,se=se)
        class(obj) <- "cv.glmreg"
        obj
    }

    "plot.cv.glmreg" <-
        function(x,se=TRUE,ylab=NULL, main=NULL, width=0.02, col="darkgrey", ...){
            lambda <- x$lambda
            cv <- x$cv
            cv.error <- x$cv.error
            if(is.null(ylab) && x$fit$family=="binomial" && x$type=="error")
                ylab <- "misclassification error"
            else if(is.null(ylab))
                ylab <- "log-likelihood"
            if(all(lambda > 0)){
            plot(log(lambda), cv, type = "b", xlab = expression(log(lambda)), ylab= ylab, ylim = range(cv, cv + cv.error, cv - cv.error), main=main)
            if(se)
                error.bars(log(lambda), cv + cv.error, cv - cv.error,
                           width = width, col=col)
            }else{
            plot(lambda, cv, type = "b", xlab = expression(lambda), ylab= ylab, ylim = range(cv, cv + cv.error, cv - cv.error), main=main)
            if(se)
                error.bars(lambda, cv + cv.error, cv - cv.error,
                           width = width, col=col)
            }

            invisible()
        }

    predict.cv.glmreg=function(object, newx, ...){
      	predict(object$fit,newx,which=object$lambda.which,...)
    }

    coef.cv.glmreg=function(object,which=object$lambda.which,...){
        coef(object$fit,which=which,...)
    }

### tuning parameter selection for function zipath
### output: return of the best model 
tuning.zipath <- function(formula, data, weights, subset, na.action, offset, standardize=TRUE,
                   family = c("poisson", "negbin", "geometric"),penalty = c("enet", "mnet", "snet"), 
lambdaCountRatio = .0001, lambdaZeroRatio = c(.1, .01, .001), maxit.theta=1, gamma.count=3, gamma.zero=3, ...){
    fam <- match.arg(family)
    pen <- match.arg(penalty)
    ratio <- lambdaZeroRatio
        fm.ratio <- vector("list", length=length(ratio))
        reskk <- rep(NA, length=length(ratio))
        for(kk in 1:length(ratio)){
            fm.ratio[[kk]] <- zipath(formula, data = data, family = fam, gamma.count=gamma.count, gamma.zero=gamma.zero, nlambda=100, lambda.count.min.ratio=lambdaCountRatio, lambda.count=NULL, lambda.zero= NULL, lambda.zero.min.ratio=ratio[kk], maxit.theta=maxit.theta, theta.fixed=FALSE, trace=FALSE, penalty=pen, rescale=FALSE)
            reskk[kk] <- min(fm.ratio[[kk]]$bic)
        }
      fm <- fm.ratio[[which.min(reskk)]]
    return(fm)
}

C compute z=s(u), step 3 in the IRCO Algorithm
C used in irglmreg_onelambda, 
C input dfun, n, u(n), s -- no changes
C       1: u^2/2
C output z(n)
      subroutine compute_z(dfun, n, u, z, eps)
              implicit none
              integer dfun, i, n
              double precision u(n), z(n), eps, s1

      if (dfun .EQ. 3)then
          s1=eps**2/2
      end if
      do 10 i=1, n
      if(dfun .EQ. 1)then
              z(i)=u(i)**2/2
      else if (dfun .EQ. 2)then
          if (dabs(u(i)) .LE. eps)then
              z(i)=0.0D0
          else
              z(i)=dabs(u(i))-eps
          endif
      else if (dfun .EQ. 3)then
          if (dabs(u(i)) .LE. eps)then
              z(i)=u(i)**2/2
          else
              z(i)=eps*dabs(u(i))-s1
          endif
      else if (dfun .EQ. 4)then
              z(i)=(1-u(i))**2/2
      else if (dfun .EQ. 5)then
          if(u(i) .GT. -10.0D0)then
              z(i)=dlog(1+dexp(-u(i)))
          else
              z(i)=-u(i)
          endif
      else if (dfun .EQ. 6)then
              z(i)=max(0.D0, 1-u(i))
      else if (dfun .EQ. 7)then
              z(i)=dexp(-u(i))
      end if
 10           continue
             
            return
            end 

C     update the active set with only non-zero coefficients
C     input: m, beta, eps
C     output: activeset, jk
      subroutine find_activeset(m, beta, eps, activeset, jk)

      implicit none
      integer j, jk, m, activeset(m)
      double precision beta(m), eps


      do 90 j=1, m
         activeset(j)=0
90    continue
      
C     jk: number of variables in active set
      jk = 0
C     it is possible, jk=0 if beta=0, like intercept-only
C     model
      do 60 j=1, m
      if(dabs(beta(j)) .GT. eps)then
C     used jk not j, to be indexed in loop_gaussian subroutine
         jk=jk+1
         activeset(jk)=j
      endif
60    continue
      
      return
      end

C***********matrix and vector product
C A(N) = B(N,M) * C(M)   
C input:
C N
C M
C B(N,M)
C C(M) is vector with length M

C output:
C vector A(N)
      SUBROUTINE MVECPROD(N,M,B,C,A)
      INTEGER N,M,I
      DOUBLE PRECISION B(N,M),C(M),A(N),U
      DO 10 I=1,N
         U=0.D0
         DO 5 J=1, M
         U= U+ B(I,J)*C(J)
    5 CONTINUE
         A(I)=U
   10 CONTINUE
      END

C copy a matrix
C input: x is an n x m matrix
C      y is the output and y=x
      subroutine copymatrix(n, m, x, y)
              implicit none
              integer i, j, n, m
              double precision x(n, m), y(n, m)

      do 100 j=1, m
       do 110 i=1, n
              y(i, j)=x(i, j)
 110   continue
 100  continue

              return
              end

C similar to and derived from R/glmreg_fit
C input: maxit, start, etastart, mustart - are these changed in outloop
C subroutine?
c output: yhat is the updated mustart as output
      subroutine glmreg_fit_fortran(x, y, weights, n,m,start, etastart,
     +         mustart, offset, nlambda, lambda, alpha, gam, rescale, 
     +         standardize, intercept, penaltyfactor, thresh, epsbino, 
     +         maxit,eps,theta,family,penalty,trace,beta,b0, yhat, satu)
      implicit none
      integer n,m, i,j, penalty, nlambda, family, standardize, maxit,
     +     innermaxit, trace, rescale, good, intercept,
     +     satu, convout(nlambda), startv, tmpit
C      double precision, intent(in) :: etastart(n),mustart(n),start(m+1)
      double precision :: etastart(n),mustart(n),start(m+1)
      double precision x(n, m), xold(n,m), y(n), weights(n), 
Cstart(m+1),etastart(n), mustart(n), 
     +     offset(n), lambda(nlambda), lam(m, nlambda), 
     +     alpha, gam, thresh, epsbino, eps, theta, penaltyfactor(m),
     +     beta(m, nlambda), b0(nlambda), sumpen, outpll(maxit,nlambda),
     +     wt(n), meanx(m), normx(m),xd(m), nulldev,
     +     penfac(m), resdev(nlambda), yhat(n), mu(n), sumwt,
     +     crossprod_beta(nlambda), meany, meanoffset

      if(family.EQ.2)then
          do i=1, n
          if(y(i) < 0 .OR. y(i) > 1)then
          call rexit("y value should be between 0 and 1 in
     +     src/glmreg_fit_fortran")
      endif
          enddo
      endif
      call  deveval(n, y, mustart, theta, weights, family, nulldev)
      startv = 1
      sumpen = sum(penaltyfactor)
      do 110 i=1, m
      penfac(i) = penaltyfactor(i)/sumpen * m
110   continue 
C generate outer product of two vectors penfac and lambda, return lam
      call outprod(m, penfac, nlambda, lambda, lam)
      if(family.EQ.1)then 
        innermaxit = maxit
        tmpit = 1
      else 
              innermaxit = 1
              tmpit = maxit
      endif
                    
      sumwt = sum(weights)
      do 130 i=1, n
      wt(i) = weights(i)/sumwt
130   continue
C    wtnew <- weights/sum(weights)
C    meanx <- drop(wtnew %*% x)
C     compute weighted column averages meanx = x^(transpose) * wt
      call DGEMV('T',n, m, 1.0D0, x, n, wt, 1, 0.0D0, meanx, 1)

C    if(standardize){
C        xx <- scale(x, meanx, FALSE) # centers x
C        xx <- sqrt(wtnew) * xx
C        one <- rep(1,n)  
C        normx <- sqrt(one %*% xx^2)
C        xx <- scale(x, meanx, normx)
C    }
C choose family=2 (any interger except 1 to avoid centering/scaling y) below
      if(family .EQ. 1 .AND. standardize .EQ. 1)then
           do 1100 j=1, m
          do 1110 i=1, n
             xold(i,j)=x(i,j)
 1110     continue
 1100  continue
      endif
      if(standardize .EQ. 1)then
              call preprocess(x,y,n,m,weights, 2,standardize,normx,xd,
     +     mu)
      endif
C    need to check if satu is input or output
          satu = 0
          good = nlambda
      call outloop(x,y,weights,wt,n,m,penalty,nlambda,lam,alpha,gam,
     +            theta,rescale,mustart,etastart,offset,family,
     +            standardize,intercept, nulldev,thresh,tmpit,
     +            innermaxit,eps,trace,start,startv,beta,b0,resdev,yhat,
     +            convout, satu, good, epsbino,outpll)
      if (standardize .EQ. 1)then
         do 200 j=1, nlambda
         do 250 i=1, m
                beta(i, j)=beta(i, j)/normx(i)
 250      continue
 200      continue
C     compute crossproduct, like crossprod(meanx, beta) in R
C http://www.tat.physik.uni-tuebingen.de/~kley/lehre/ftn77/tutorial/blas.html
         call DGEMV('T',m, nlambda, 1.0D0, beta, m, meanx, 1, 0.0D0, 
     +             crossprod_beta, 1)
        if(intercept .EQ. 1)then
         do 300 j=1, nlambda
                b0(j)=b0(j) - crossprod_beta(j)
 300      continue
         if (family .EQ. 1)then
                meany=sum(y)/n
                meanoffset=sum(offset)/n
            do 400 j=1, nlambda
                b0(j)=b0(j) + meany - meanoffset
 400        continue
         endif
        endif
      endif
      if(family .EQ. 1 .AND. standardize .EQ. 1)then
           do 1200 j=1, m
          do 1210 i=1, n
             x(i,j)=xold(i,j)
 1210     continue
 1200  continue
         endif

      return
      end

### SVM based on composite operator
cv.irsvm <- function(x, ...) UseMethod("cv.irsvm")

cv.irsvm.default <- function(x, ...) {
    if (extends(class(x), "Matrix"))
        return(cv.irsvm.matrix(x = x, ...))
    if (inherits(x, "data.frame"))
        return(cv.irsvm.matrix(x = x, ...))
    if (inherits(x, "numeric") && is.null(dim(x)))
        return(cv.irsvm.matrix(x = as.matrix(x), ...))
    stop("no method for objects of class ", sQuote(class(x)),
         " implemented")
}

cv.irsvm.formula <- function(formula, data, weights, contrasts=NULL, ...){
    ## extract x, y, etc from the model formula and frame
    if(!attr(terms(formula, data=data), "intercept"))
        stop("non-intercept model is not implemented with model formula, try model matrix instead")
    if(missing(data)) data <- environment(formula)
    mf <- match.call(expand.dots = FALSE)
    m <- match(c("formula", "data", "weights"), names(mf), 0L)
    mf <- mf[c(1L, m)]
    mf$drop.unused.levels <- TRUE
    mf[[1L]] <- as.name("model.frame")
    mf <- eval(mf, parent.frame())

    mt <- attr(mf, "terms") # allow model.frame to have updated it

    Y <- model.response(mf, "any") # e.g. factors are allowed
    ## avoid problems with 1D arrays, but keep names
    if(length(dim(Y)) == 1L) {
        nm <- rownames(Y)
        dim(Y) <- NULL
        if(!is.null(nm)) names(Y) <- nm
    }
    ## null model support
    X <- if (!is.empty.model(mt)) model.matrix(mt, mf, contrasts) else matrix(,NROW(Y), 0L)
    ## avoid any problems with 1D or nx1 arrays by as.vector.
    weights <- as.vector(model.weights(mf))
    if(!length(weights)) weights <- rep(1, nrow(mf))
    else if(any(weights < 0)) stop("negative weights not allowed")
    if(!is.null(weights) && !is.numeric(weights))
        stop("'weights' must be a numeric vector")
    if(length(weights) != length(Y))
        stop("'weights' must be the same length as response variable")	 
    RET <- cv.irsvm_fit(X[,-1], Y, weights,...)
    RET$call <- match.call()
    RET <- c(RET, list(formula=formula, terms = mt, data=data,
                       contrasts = attr(X, "contrasts"),
                       xlevels = .getXlevels(mt, mf)))
    RET
}
cv.irsvm.matrix <- function(x, y, weights, ...){
    RET <- cv.irsvm_fit(x, y, weights,...)
    RET$call <- match.call()
    return(RET)
}

cv.irsvm_fit <- function(x,y, weights, cfun="ccave", s=c(1, 5), type=NULL, 
                         kernel      = "radial",
                                        #degree      = 3,
                         gamma       = 2^(-4:10),
                                        #coef0       = 0,
                         cost = 2^(-4:4),
                                        #class.weights = NULL,
                                        #cachesize   = 100,
                                        #tolerance   = 0.001,
                         epsilon     = 0.1,
                                        #shrinking   = TRUE,
                                        #probability = FALSE,
                                        #fitted      = TRUE,
                                        #subset,
                                        #na.action,
                         balance=TRUE,
                         nfolds=10, foldid, trim_ratio=0.9, n.cores=2,
                         ...){
    call <- match.call()
    ## determine model type
          if (is.null(type)) type <-
            if (is.factor(y)) "C-classification"
          else "eps-regression"
    sval <- s
    costseq <- cost
    if(kernel=="linear") gammaseq <- 0 else gammaseq <- gamma
    if(is.null(dim(x))) x <- as.matrix(x) 
    nm <- dim(x)
    nobs <- n <- nm[1]
    nvars <- m <- nm[2]
    ## avoid any problems with 1D or nx1 arrays by as.vector.
     if(missing(weights)) weights=rep(1,n)
     weights <- as.vector(weights)
     if(!is.null(weights) && !is.numeric(weights))
         stop("'weights' must be a numeric vector")
     ## check weights 
     if( !is.null(weights) && any(weights < 0) ){
         stop("negative weights not allowed")
     }
     if(length(weights) != length(y))
         stop("'weights' must be the same length as response variable")
    K <- nfolds
    if(missing(foldid)){
        if(type %in% c("C-classification", "nu-classification") && balance)
         invisible(capture.output(all.folds <- eval(parse(text="balanced.folds(y, K)"))))
        else all.folds <- cv.folds(n, K)
    }
    else all.folds <- foldid
    cl <- eval(parse(text="parallel:::makeCluster(n.cores)")) 
    registerDoParallel(cl)
    i <- ii <- jj <- l <- 1
    residmat <- foreach(i=seq(K), .combine=cbind)%:% 
        foreach(l=sval, .combine='rbind')%:%
        foreach(ii=gammaseq, .combine='rbind')%:%
        foreach(jj=costseq, .combine='rbind', .packages="mpath")   %dopar%{
    #residmat <- NULL; for(i in seq(K)) for(l in sval) for(ii in gammaseq) for(jj in costseq){
            omit <- all.folds[[i]]
            m <- try(irsvm(x[-omit,,drop=FALSE], y[-omit], weights=weights[-omit], cfun=cfun, cost=jj, gamma=ii, s=l, type=type, ...), silent=TRUE)
            if(!inherits(m, "try-error")){
                new <- predict(m, x[omit,,drop=FALSE], weights=weights[omit])
                if(type%in%c("C-classification", "nu-classification")) errorTU <- mean(new!=y[omit])
                else {
                    mloss <- (y[omit]-new)^2
                    mloss1 <- order(mloss, decreasing=FALSE)
                    mloss2 <- mloss1[1:(length(omit)*trim_ratio)]
                    errorTU <- mean(mloss[mloss2]) #trimmed MSE
                }
                c(l, ii, jj, errorTU)
                #residmat <- rbind(residmat, c(l, ii, jj, errorTU))
            }
        }
    eval(parse(text="parallel:::stopCluster(cl)")) 
    kseq <- 4*(1:K)
    dl <- length(dim(residmat))
    if (!dl)
        stop("no parameters are valid for SVM")
    err <- apply(residmat[,kseq], 1, mean) #average error cross K folds
    opt <- residmat[which.min(err), 1:3]
    if(kernel=="linear"){
        residmat <- residmat[, -2*(1:K)]
        colnames(residmat) <- rep(c("s", "cost", "error"), K)
    }else                                                                       colnames(residmat) <- rep(c("s", "gamma", "cost", "error"), K)
    RET <- list(residmat=residmat, cost=opt[3], gamma=opt[2], s=opt[1])
    RET
}

C     
C     #coordinate descent algorithm
C     #ref: Regularization paths for generalized linear models via coordinate descent, Friedman et al., JSS, 2010, 33(1)
C     output: updated
C     beta
C     b0
C     yhat
C     jj
C     input: others including following
C     beta
C     b0
C     mu
C     w
C     z
C     X
C     y

      subroutine loop_glm(x,y,z,n,m,w,mu, penalty,thresh, eps, 
     +     standardize, intercept, family, beta, b0, lambda, alpha, gam,
     +     wtold,trace,jj,rescale,converged,theta,pll,activeset, jk)
      implicit none
      integer trace, n, m, standardize, intercept, family, jj, i,j,
     +     penalty, converged, rescale,activeset(m), jk, ii
      double precision x(n, m), y(n), thresh, eps,beta(m),beta_old(m) 
      double precision lambda(m), alpha, gam, wtold(n)
      double precision z(n), b0, b0_old, xd(m), yhat(n), w(n),
     +     r(n),xwx,xwr, wtnew(n), wsum,mu(n), theta, pll_old, pll
         
      if(intercept .EQ. 0)then
           b0 = 0
      endif
      wsum = 0
      do i=1, n
         wsum = wsum + wtold(i)
      enddo
      do i=1, n
         if(standardize .EQ. 1)then
            wtnew(i) = wtold(i)/wsum
         else 
            wtnew(i) = wtold(i)
         endif
      enddo
C     current penalized log-likehood value
      if(trace .EQ. 1) then
         call evalpll(y,x,n,m,beta,b0,family, theta, wtnew, alpha,
     +        gam,lambda, penalty, pll)
      else 
         pll=0
      endif     
      pll_old = pll
      jj = 1
      do 100 j = 1, m
         beta_old(j) = beta(j)
 100  continue
      b0_old = b0
      do 10 i = 1, n
         yhat(i) = b0
         do 20 ii = 1, jk
            j = activeset(ii)
            yhat(i) = yhat(i) + x(i,j) * beta(j)
 20      continue         
 10   continue
C     for binomial and poisson distributions
C     family=1 is not really used here
      if(family .EQ. 1)then
         do i = 1, n
            r(i) = y(i) - mu(i)
         enddo
      else 
         if(family .EQ. 2 .OR. family .EQ. 3)then
            do i = 1, n
               r(i) = (y(i) - mu(i))/w(i)
            enddo
         else 
            if(family .EQ. 4)then
C     for negative binomila distribution
               do i = 1, n
                  r(i) = (y(i) - mu(i))/mu(i)
               enddo
            endif
         endif
      endif 
C     intercept 
      xwr =0 
      xwx =0
      do 200 i=1, n
         xwr = xwr + wtnew(i) * w(i) * r(i)
         xwx = xwx + wtnew(i) * w(i)
 200  continue
      if(intercept .EQ. 1)then
         b0 = xwr/xwx + b0_old
         do 210 i=1, n
            r(i) = r(i) - (b0 - b0_old)
 210     continue
      endif
C     calculate response z
C     cycle through in the active set (begin with if statement)
      do 40 ii = 1, jk
         j = activeset(ii)
         z = 0.0D0
         xwr = 0
         xwx = 0
         do 50 i = 1, n
            xwr = xwr + wtnew(i) * x(i,j) * w(i) * r(i)
            xwx = xwx + wtnew(i) * x(i,j) * w(i) * x(i,j)
 50      continue 
         z = xwr+ xwx * beta_old(j)
         xd(j) = xwx   
         if(penalty.EQ.1) then
            call enet(z,xd(j), lambda(j)*alpha, lambda(j)*(1-alpha),
     +           beta(j))
         else if(penalty.EQ.2) then
            call mcp(z, xd(j), lambda(j)*alpha, lambda(j)*(1-alpha), 
     +           gam, rescale, beta(j))
         else if(penalty.EQ.3) then
            call scad(z, xd(j), lambda(j)*alpha, 
     +           lambda(j)*(1-alpha), gam, rescale, beta(j))
         endif

C     check if beta(j) strictly increases penalized loglikehood function 
         if(trace .EQ. 1)then
            call evalpll(y,x,n,m,beta,b0,family, theta, wtnew, alpha,
     +           gam,lambda, penalty, pll) 
         else 
            pll=0
         endif    
         if(dabs(beta(j)-beta_old(j)) .GT. eps)then
            do 65 i = 1, n
               r(i) = r(i) - x(i, j) * (beta(j) - beta_old(j))
 65         continue
         endif
C     70: cycle through in the active set (end with endif statement) 
 40   continue
      call checkConvergence(m, beta, beta_old, eps, thresh, 
     +        converged, activeset, jk)

      return
      end

C     compute penalized log-likelihood value, this is only practically useful if rescale=FALSE
      subroutine evalpll(y,x,n,m,beta,b0,family, theta, weights, alpha,
     +     gam,lambda, penalty, pll)     
      implicit none
      integer penalty, n,m,i,j, family 
      double precision x(n, m), y(n), weights(n), lambda(m),eta(n),
     +     yhat(n), alpha, gam, b0, beta(m), mu(n), theta, pll, ll

      do 220 i = 1, n
         yhat(i) = b0
         do 230 j = 1, m
            yhat(i) = yhat(i) + x(i,j) * beta(j)
 230     continue
 220  continue
      call DCOPY(n, yhat, 1, eta, 1)
      call linkinv(n, eta, family, mu)
      call loglikFor(n, y, mu, theta, weights, family, ll)
C     compute penalty value pll
      call penGLM(beta, m, lambda, alpha, gam,
     +     penalty, pll)
      pll = ll - pll

      return
      end

predict.glmreg <- function(object,newx,newoffset, which=1:length(object$lambda), type=c("link","response","class","coefficients","nonzero"), na.action=na.pass, ...){
 type=match.arg(type)
  if(missing(newx)){
    if(!match(type,c("coefficients","nonzero"),FALSE))stop("You need to supply a value for 'newx'")
     }
  else{
if(!is.null(object$terms)){
 mf <- model.frame(delete.response(object$terms), newx, na.action = na.action, xlev = object$xlevels)
 newx <- model.matrix(delete.response(object$terms), mf, contrasts = object$contrasts)
 newx <- newx[, -1, drop = FALSE] ### remove the intercept
}
}
  b0=as.matrix(object$b0)
  rownames(b0)="(Intercept)"
  nbeta=rbind2(b0,as.matrix(object$beta))[,which]
    vnames=dimnames(nbeta)[[1]]
    lambda=object$lambda[which]
  nlambda <- length(lambda)
  if(type=="coefficients")return(nbeta)
  if(type=="nonzero"){
  #if(nlambda>1) return(nonzeroCoef(nbeta[-1,,drop=FALSE],bystep=TRUE))
  if(nlambda>1) return(eval(parse(text="glmnet:::nonzeroCoef(nbeta[-1,,drop=FALSE],bystep=TRUE)")))
  else return(which(abs(nbeta[-1]) > 0))
}
  famtype <- switch(object$family,
					"gaussian"=1,
					"binomial"=2,
					"poisson"=3,
					"negbin"=4)
  n <- dim(newx)[1]
  m <- dim(newx)[2]
  if(is.list(newx))
      newx <- matrix(unlist(newx), ncol=m)
      #newx <- as.matrix(as.data.frame(lapply(newx, as.numeric)))
  if(dim(object$beta)[1] != m) stop("number of columns in newx should be the same as number of rows in beta\n")
  if(object$is.offset)
    if(is.null(newoffset))
      stop("offset is used in the estimation but not provided in prediction\n")
    else offset <- newoffset
  else offset <- rep(0, n)
  res <- .Fortran("pred",
				  n=as.integer(n),
				  m=as.integer(m),
				  nlambda=as.integer(nlambda),
				  x=as.double(newx),
				  b=as.double(object$beta[,which]),
				  a0=as.double(object$b0[which]),
				  offset=as.double(offset),
                  family=as.integer(famtype),
				  eta = as.double(matrix(0,n,nlambda)),
				  mu = as.double(matrix(0,n,nlambda)),
				  PACKAGE="mpath")
  eta <- matrix(res$eta, ncol=nlambda)
  mu <- matrix(res$mu, ncol=nlambda)
#  to be considered below: 02/28/2020
#  if(object$is.offset){
#	  if(is.null(newoffset)) stop("newoffset is required\n")
#          else 
#     {
#      eta <- eta + newoffset
#      mu <- eta
#     }
#  }
  colnames(eta) <- colnames(mu) <- colnames(object$beta[,which])
  #pihat <- exp(eta)/(1+exp(eta))
    if (object$family=="gaussian" | type=="link") return(eta)
    if (type=="response") return(mu)
	if (object$family=="binomial" & type=="class") return(eta>0)
  }
coef.glmreg <- function(object,which=1:length(object$lambda),...)
  {
  b0=matrix(object$b0, nrow=1)
  rownames(b0)="(Intercept)"
  nbeta=rbind2(b0,as.matrix(object$beta))
    return(nbeta[,which])
  }

C compute subdifferential of -g(z), step 4 in the IRCO Algorithm
C used in irglmreg_onelambda 
C output v
      subroutine compute_v(cfun, n, z, s, delta, v)
              implicit none
              integer cfun, i, n
              double precision PI, s, delta, s1, s2, z(n), v(n)

       if(cfun .EQ. 1)then
           s1 = s**2/2
       else if(cfun .EQ. 2)then
           PI=4.D0*DATAN(1.D0)
           s1 = s**2*PI**2/2
       else if(cfun .EQ. 3)then
           s1 = s**2
           s2 = 1.0/s1**2
       else if(cfun .EQ. 4)then
           s1 = s**2
       else if(cfun .EQ. 5)then
           s1 = dexp(-s)
       else if(cfun .EQ. 6)then
           s1=-delta**(s-1)/(delta+1)**(s+1)
       else if(cfun .EQ. 8)then
           PI=4.D0*DATAN(1.D0)
           s1 = 2*dexp(-delta/s)/dsqrt(PI*s*delta)
       endif

      do 10 i=1, n
      if(cfun .EQ. 1)then
          if(z(i) .LE. s1)then
              v(i)=-1
          else
              v(i)=-s*(2*z(i))**(-0.5)
          endif
      else if(cfun .EQ. 2)then
          if(z(i)==0)then
              v(i)=-1
          else if(z(i) > s1)then
              v(i)=0
          else
              v(i)=-s*dsin((2*z(i))**(0.5)/s)/(2*z(i))**(0.5)
          endif
      else if(cfun .EQ. 3)then
          if(z(i) .LE. s1/2)then
              v(i)=-s2*(2*z(i)-s1)**2
          else
              v(i)=0.D0
          endif
      else if(cfun .EQ. 4)then
              v(i)=-dexp(-z(i)/s1)
      else if(cfun .EQ. 5)then
          v(i)=-1/((z(i)+1)*(z(i)*s1+1))
      else if(cfun .EQ. 6)then
          if(z(i) .LE. delta)then
           v(i)=s1
          else
           v(i)=-z(i)**(s-1)/(z(i)+1)**(s+1)
          endif
      else if(cfun .EQ. 7)then
          if(z(i) .LE. s)then
              v(i)=-1
          else
              v(i)=0
          endif
      else if(cfun .EQ. 8)then
          if(z(i) .LE. delta)then
           v(i)=-s1
          else
           v(i)=-2*dexp(-z(i)/s)/dsqrt(PI*s*z(i))
          endif
C              if(v(i) .gt. 0)then
C               call   rexit("the v value should be less than 0")
      end if
 10   continue
             
            return
            end 

### composite optimization based glm
irglm <- function(x, ...) UseMethod("irglm")

irglm.formula <- function(formula, data, weights, offset=NULL, contrasts=NULL, 
cfun="ccave", dfun=gaussian(), s=NULL, delta=0.1, fk=NULL, init.family=NULL, iter=10, reltol=1e-5, theta, x.keep=FALSE, y.keep=TRUE, trace=FALSE, ...){
    if(missing(data)) data <- environment(formula)
    mf <- match.call(expand.dots = FALSE)
    m <- match(c("formula", "data", "weights",
                 "offset"), names(mf), 0L)
    mf <- mf[c(1L, m)]
    mf$drop.unused.levels <- TRUE
    mf[[1L]] <- as.name("model.frame")
    mf <- eval(mf, parent.frame())

    mt <- attr(mf, "terms") # allow model.frame to have updated it

    Y <- model.response(mf, "any") # e.g. factors are allowed
    ## avoid problems with 1D arrays, but keep names
    if(length(dim(Y)) == 1L) {
        nm <- rownames(Y)
        dim(Y) <- NULL
        if(!is.null(nm)) names(Y) <- nm
    }
    ## null model support
    X <- if (!is.empty.model(mt)) model.matrix(mt, mf, contrasts) else matrix(,NROW(Y), 0L)
    ## avoid any problems with 1D or nx1 arrays by as.vector.
    weights <- as.vector(model.weights(mf))
    if(!length(weights)) weights <- rep(1, nrow(mf))
    else if(any(weights < 0)) stop("negative weights not allowed")
    if(!is.null(weights) && !is.numeric(weights))
        stop("'weights' must be a numeric vector")
    if(length(weights) != length(Y))
        stop("'weights' must be the same length as response variable")	 
    offset <- as.vector(model.offset(mf))
    if(!is.null(offset)) {
        if(length(offset) != NROW(Y))
            stop(gettextf("number of offsets is %d should equal %d (number of observations)", length(offset), NROW(Y)), domain = NA)
    }
    x <- X
    y <- Y
    call <- match.call()
    cfunold <- cfun
    dfunold <- dfun[[1]]
    if(dfunold=="poisson"){
        theta <- 1 ### not used anyway
        family <- 3 
    }else if(dfunold=="negbin"){
        if(missing(theta)) stop("theta has to be provided for family=negbin()")
        family <- 4
    } 
    cfun <- cfun2num(cfun)
    if(dfunold%in%c("gaussian","gaussianC","binomial","poisson","negbin")) dfunold <- dfun2num(dfunold)
                                        #if(dfun==7 && !is.null(offset))
                                        #    warning("dfun=", dfun, ", offset not implmented\n")
    if(is.null(s) || is.na(s)) s <- assign_s(cfun, y) else check_s(cfun, dfunold, s)
    if(cfun==6)
        if(s > 1) delta <- (s-1)/2 else
                                       if(s==1) delta <- 0 else{
                                                               if(is.null(delta)) stop("delta must be provided")
                                                               if(delta <= 0) stop("delta must be positive")
                                                           }
                                        #for dfunold=4, y must be 1 or -1
                                        #for dfunold=5, y must be 0 or 1
    if(dfunold==4)
        y <- y2num(y)
    if(dfunold==5)
        y <- y2num4glm(y)
    ## avoid any problems with 1D or nx1 arrays by as.vector.
    if(missing(weights)) weights=rep(1,length(y))
    weights <- as.vector(weights)
    if(!is.null(weights) && !is.numeric(weights))
        stop("'weights' must be a numeric vector")
    ## check weights and offset
    if( !is.null(weights) && any(weights < 0) ){
        stop("negative weights not allowed")
    }
    if(length(weights) != length(y))
        stop("'weights' must be the same length as response variable")
    n <- length(y)
    if (is.null(offset)){
        is.offset <- FALSE
        offset <- rep.int(0, n)
    }else is.offset <- TRUE
     if(!is.null(init.family)) rfamily <- init.family else{
                                                          if(cfun==4 && dfunold==1)  rfamily <- "clossR"
                                                          else if(cfun==4 &&         dfunold==4) rfamily <- "closs"
                                                          else rfamily <- "gaussian"
                                                      }
     if(is.null(fk)){
 ### initiate predictive value
         RET <- bst(x, y, family=rfamily, ctrl = bst_control(mstop=1, s=s))
         RET$linear.predictors <- RET$yhat
     }
     else {
         RET <- NULL
         RET$linear.predictors <- fk
     }
    d1 <- 10 
    k <- 1
    if(trace) {
        cat("\niterations ...\n")
    }
    los <- rep(NA, iter)
    ytmp <- y
    if(dfunold==5)
        ytmp[ytmp==0] <- -1
    weights_update <- weights
###initial values
    if(!dfunold %in% c(1,4,5,8,9)){
        envir <- list2env(list(weights_update=weights_update, offset=offset), parent=environment(formula))
        environment(formula) <- envir
        RET <- suppressWarnings(glm(formula, data, family=dfun, weights=weights_update, offset=offset))
    }
    while(d1 > reltol && k <= iter){
        if(dfunold %in% c(1,4,5)){
            weights_update <- .Fortran("update_wt",
                                       n=as.integer(n),
                                       weights=as.double(weights),
                                       y=as.double(ytmp),
                                       f=as.double(RET$linear.predictors),
                                       cfun=as.integer(cfun),
                                       dfun=as.integer(dfunold),
                                       s=as.double(s),
                                       delta=as.double(delta),
                                       weights_update=as.double(rep(0, n)),
                                       PACKAGE="mpath")$weights_update
            los[k] <- .Fortran("loss2",
                               n=as.integer(n),
                               y=as.double(ytmp),
                               f=as.double(RET$linear.predictors),
                               weights=as.double(weights/n),
                               cfun=as.integer(cfun),
                               dfun=as.integer(dfunold),
                               s=as.double(s),
                               delta=as.double(delta),
                               los=as.double(0.0),
                               PACKAGE="mpath")$los
        }
        else{
            tmp1 <- loss3(y,mu=RET$fitted.values,theta,weights,cfun,family,s,delta)
            weights_update <- compute_wt(tmp1$z, weights, cfun, s, delta)
            los[k] <- sum(tmp1$tmp)
        }
        envir <- list2env(list(weights_update=weights_update, offset=offset), parent=environment(formula))
        environment(formula) <- envir
        RET <- suppressWarnings(glm(formula, data, family=dfun, weights=weights_update, offset=offset))
	if(trace) cat("\niteration", k, ", robust loss value", los[k], "\n") 
	if(k > 1){
            d1 <- abs((los[k]-los[k-1])/los[k-1])
            if(los[k] > los[k-1]){
                k <- iter
            }
        }
        k <- k + 1
	if(trace) cat("d1=", d1, ", k=", k, ", d1 > reltol && k <= iter: ", (d1 > reltol && k <= iter), "\n")
    }
    RET$s <- s
    RET$call <- call
    RET$cfun <- cfunold
    RET$dfun <- dfunold
    RET$weights <- weights
    RET$weights_update <- weights_update
    RET$is.offset <- is.offset
    RET <- c(RET, list(formula=formula, terms = mt, data=data,
                       contrasts = attr(X, "contrasts"),
                       xlevels = .getXlevels(mt, mf)))
    if(x.keep) RET$x <- data[,colnames(data)%in%attr(mt, "term.labels")]
    if(y.keep) RET$y <- Y
    class(RET) <- c("irglm", "glm")
    RET
}

predict.irglm=function(object, newdata=NULL, weights=NULL, newoffset=NULL, type=c("link","response","class","loss", "error", "coefficients"), na.action=na.pass, ...){
    type=match.arg(type)
    if(is.null(newdata)){
        if(!match(type,c("coefficients"),FALSE))stop("You need to supply a value for 'newdata'")
        ynow <- object$y
    }
    else{
        if(!is.null(object$terms)){
            mf <- model.frame(delete.response(object$terms), newdata, na.action = na.action, xlev = object$xlevels)
            ynow <- model.frame(object$terms, newdata)[,1] ### extract response variable
            newdata <- model.matrix(delete.response(object$terms), mf, contrasts = object$contrasts)
        }
    }
    if(object$is.offset)
        if(is.null(newoffset))
            stop("offset is used in the estimation but not provided in prediction\n")
        else offset <- newoffset
    else offset <- rep(0, length(ynow))
    if(type=="coefficients") return(object$coef)
    if(is.null(newdata))
        res <- object$linear.predictors
    else
        res <- offset + newdata %*% object$coef
    if(type=="link") return(res)
    if(type %in% c("link", "response")) return(predict(object, type=type)) 
    object$dfun <- dfun2num(object$dfun)
    if(object$dfun %in% 4) ynow <- y2num(ynow)
    if(type=="loss"){
        n <- length(ynow)
        if(missing(weights)) weights <- rep(1/n, n)
        object$cfun <- cfun2num(object$cfun)
        if(object$dfun%in%c(1,4)) return(.Fortran("loss2",
                                                  n=as.integer(length(ynow)),
                                                  y=as.double(ynow),
                                                  f=as.double(res),
                                                  weights=as.double(weights),
                                                  cfun=as.integer(object$cfun),
                                                  dfun=as.integer(object$dfun),
                                                  s=as.double(object$s),
                                                  delta=as.double(object$delta),
                                                  los=as.double(0.0),
                                                  PACKAGE="mpath")$los)
        else 
            sum(loss3(object$y,mu=predict(object, data.frame(newdata), offset=offset, type="response"),object$theta,weights,object$cfun,family,object$s,object$delta)$tmp)
    }
    if(type=="error" && object$dfun %in%c(1,4)) 
        return(co_evalerr(object$dfun, ynow, res))
    if(type=="error" && object$dfun!=5) 
        return(co_evalerr(object$dfun, ynow, predict(object, data.frame(newdata), offset=offset, type="response")))
    else if(type=="class" && object$dfun==5)
        return(predict(object, data.frame(newdata), offset=offset, type="response") > 0)
    else if(type=="error" && object$dfun==5)
        return(ynow!=(predict(object, data.frame(newdata), offset=offset, type="response") > 0))
}

check_s <- function(cfun, dfun, s){
    if(cfun %in% c(1:6,8) && s <= 0) stop("s must > 0 for cfun=", cfun, "\n")
    else if(cfun==7 && s < 0) stop("s must be nonnegative for cfun=", cfun, "\n")
    else if(cfun==7 && s < 1 && dfun %in%c("gaussianC", "binomial", "hinge")) stop("s must > or = 1 for cfun=tcave and dfun=gaussianC, binomial, hinge")
    else if(cfun==7 && s < 1 && dfun %in%c(4, 5, 6)) stop("s must > or = 1 for cfun=tcave and dfun=gaussianC, binomial, hinge")
    if(cfun %in% c(2, 3) && s < 1) warnings("The program can crash if s value is too close to 0 for cfun=", cfun, "\n")
}

                                        #for cfun=1, 2, 3, choose s value to achieve 95% efficiency for Huber's method, Andrew's since and Tukey's biweight loss, respectively. For cfun=4 and dfun=1 or 4, s < 1 is nonconvex; For cfun=4 and dfun=4, s=1 is an approximation to the hinge loss. cf: Singh et al 2014, Pattern Recognition47:441-453, page 445. For cfun=5, s=4 (or mu in the paper) was used in Krause and Singer (2004) ICML. For cfun=6, s=2 was used in Li and Bradic (2018) JASA. For cfun=7 and dfun=6, s=-1 works best in Wu and Liu 2007, JASA 102(479):974-983, page 981 
assign_s <- function(cfun, y){
    if(cfun==1) return(1.345)
    if(cfun==2) return(1.339)
    if(cfun==3) return(4.685)
    if(cfun==4) return(0.5)
    if(cfun==5) return(4)
    if(cfun==6) return(1)
    if(cfun==7) return(1)
    if(cfun==8) return(1)
}

cfun2num <- function(cfun){
    switch(cfun,
           "hcave"=1,
           "acave"=2,
           "bcave"=3,
           "ccave"=4,
           "dcave"=5,
           "gcave"=6,
           "tcave"=7,
           "ecave"=8)
}
dfun2num <- function(dfun){
    switch(dfun,
           "gaussian"=1,
           "eps-regression"=2,
           "huber"=3,
           "gaussianC"=4,
           "binomial"=5,
           "hinge"=6,
           "exponential"=7,
           "poisson"=8,
           "negbin"=9)
}
### output: +1/-1
y2num <- function(y){
    if (is.factor(y)) {
        y <- as.integer(y)
    } else {
        if(any(as.integer(y) != y))
            stop("y variable has to be of factor or integer type for classification\n")
        y <- as.factor(y)
        y <- as.integer(y)
    }
    if(length(unique(y))!=2) stop("y must be a binary variable for classification\n")
    else if(!all(names(table(y)) %in% c(1, -1))){
        y[y==1] <- -1
        y[y==2] <- 1
    }
    y
}
### output: 0/1
y2num4glm <- function(y){
    if (is.factor(y)) {
        y <- as.integer(y)
    } else {
        if(any(as.integer(y) != y))
            stop("y variable has to be of factor or integer type for classification\n")
        y <- as.factor(y)
        y <- as.integer(y)
    }
    if(length(unique(y))!=2) stop("y must be a binary variable for classification\n")
    else if(!all(names(table(y)) %in% c(0, 1))){
        y[y==1] <- 0
        y[y==2] <- 1
    }
    y
}

###compute composite loss values for dfun=poisson and negbin
### output: z is a vector of maximum log-likelihood value, i.e., s(u) values 
### output: tmp is a vector of g(s(u)) values
### There is a ../src/loss3.f function, which is different from the same named function for different purposes: the Fortran code is used to check convergency
loss3 <- function(y,mu,theta,weights,cfun,family,s,delta){
    n <- length(y)
    z <- tmp <- rep(NA, n)
    for(i in 1:n){
        z[i] <- .Fortran("loglikFor",
                         n=as.integer(1),
                         y=as.double(y[i]),
                         mu=as.double(mu[i]),
                         theta=as.double(theta),
                         w=as.double(1),
                         family=as.integer(family),
                         ll=as.double(0),
                         PACKAGE="mpath")$ll
        z[i] <- -z[i] #convert MLE of log-likelihood to minimization of negative log-likelihood
        tmp[i] <- weights[i] * .Fortran("compute_g",
                                        cfun=as.integer(cfun),
                                        n=as.integer(1),
                                        z=as.double(z[i]),
                                        s=as.double(s),
                                        delta=as.double(delta),
                                        gval=as.double(0),
                                        PACKAGE="mpath")$gval
    }
    list(z=z, tmp=tmp)
}

###compute updated weights for composite loss function with dfun=poisson and negbin 
compute_wt3 <- function(y,mu,theta,weights,cfun,family,s,delta){
    n <- length(y)
    z <- tmp <- rep(NA, n)
    for(i in 1:n){
        z[i] <- .Fortran("loglikFor",
                         n=as.integer(1),
                         y=as.double(y[i]),
                         mu=as.double(mu[i]),
                         theta=as.double(theta),
                         w=as.double(1),
                         family=as.integer(family),
                         ll=as.double(0),
                         PACKAGE="mpath")$ll
    }
    z <- -z #convert MLE of log-likelihood to minimization of negative log-likelihood
    compute_wt(z, weights, cfun, s, delta)
}

zipath <- function(X, ...) UseMethod("zipath")

zipath.default <- function(X, ...) {
    if (extends(class(X), "Matrix"))
        return(zipath.matrix(X=X, ...))
    stop("no method for objects of class ", sQuote(class(X)),
         " implemented")
}

zipath.formula <- function(formula, data, weights, offset=NULL, contrasts=NULL, ...){
    ## call and formula
    cl <- match.call()
    if(!attr(terms(formula), "intercept"))
        stop("non-intercept model is not implemented")
    if(missing(data)) data <- environment(formula)
    mf <- match.call(expand.dots = FALSE)
    m <- match(c("formula", "data", "na.action", "weights", "offset"), names(mf), 0)
    mf <- mf[c(1, m)]
    mf$drop.unused.levels <- TRUE
    
    ## extended formula processing
    if(length(formula[[3]]) > 1 && identical(formula[[3]][[1]], as.name("|")))
    {
        ff <- formula
        formula[[3]][1] <- call("+")
        mf$formula <- formula
        ffc <- . ~ .
        ffz <- ~ .
        ffc[[2]] <- ff[[2]]
        ffc[[3]] <- ff[[3]][[2]]
        ffz[[3]] <- ff[[3]][[3]]
        ffz[[2]] <- NULL
    } else {
        ffz <- ffc <- ff <- formula
        ffz[[2]] <- NULL
    }
    if(inherits(try(terms(ffz), silent = TRUE), "try-error")) {
        ffz <- eval(parse(text = sprintf( paste("%s -", deparse(ffc[[2]])), deparse(ffz) )))
    }

    ## call model.frame()
    mf[[1]] <- as.name("model.frame")
    mf <- eval(mf, parent.frame())
    
    ## extract terms, model matrices, response
    mt <- attr(mf, "terms")
    mtX <- terms(ffc, data = data)
    X <- model.matrix(mtX, mf)
    Xold <- X
### This means X possibly has intercept (1st) column depending on the specifications 
    mtZ <- terms(ffz, data = data)
    mtZ <- terms(update(mtZ, ~ .), data = data)
    Z <- model.matrix(mtZ, mf)
    Zold <- Z
    Y <- model.response(mf, "numeric")
    ## weights and offset
    weights <- model.weights(mf)
    offsetx <- model_offset_2(mf, terms = mtX, offset = TRUE)
    offsetz <- model_offset_2(mf, terms = mtZ, offset = FALSE)
    RET <- zipath_fit(X, Z, Y, weights=weights, offsetx=offsetx, offsetz=offsetz, ...)
    RET$call <- match.call()
    class(RET) <- "zipath"
    RET$terms = list(count = mtX, zero = mtZ, full = mt)
    RET$call = cl
    RET$formula = ff
    RET$levels = .getXlevels(mt, mf)
    RET$contrasts = list(count = attr(X, "contrasts"), zero = attr(Z, "contrasts"))
    RET$model <- mf
    RET
}
zipath.matrix <- function(X, Z, Y, weights, offsetx=NULL, offsetz=NULL, ...){
    RET <- zipath_fit(X, Z, Y, weights, offsetx=offsetx, offsetz=offsetz, ...)
    RET$call <- match.call()
    return(RET)
}
zipath_fit <- function(X, Z, Y, weights, offsetx, offsetz, standardize=TRUE, intercept=TRUE, family = c("poisson", "negbin", "geometric"), link = c("logit", "probit", "cloglog", "cauchit", "log"), penalty = c("enet", "mnet", "snet"), start = NULL, y = TRUE, x = FALSE, nlambda=100, lambda.count=NULL, lambda.zero=NULL, type.path=c("active", "nonactive"), penalty.factor.count=NULL, penalty.factor.zero=NULL, lambda.count.min.ratio=.0001, lambda.zero.min.ratio=.1, alpha.count=1, alpha.zero=alpha.count, gamma.count=3, gamma.zero=gamma.count, rescale=FALSE, init.theta=NULL, theta.fixed=FALSE, EM=TRUE, maxit.em=200, convtype=c("count", "both"), maxit= 1000, maxit.theta =10, reltol = 1e-5, thresh=1e-6, eps.bino=1e-5, shortlist=FALSE, trace=FALSE, ...)
{
    family <- match.arg(family)
    if(is.null(init.theta) && family=="negbin" && theta.fixed)
        stop("missing argument init.theta while family=='negbin' and theta.fixed is TRUE\n")
    if(length(gamma.count) > 1 || length(gamma.zero) > 1)
        stop("gamma.count or gamma.zero must be a scalar")
    if(family=="geometric"){
        family <- "negbin"
        init.theta <- 1
        theta.fixed <- TRUE
    }
    penalty <- match.arg(penalty)
    link <- match.arg(link)
    type.path <- match.arg(type.path)
    if(type.path=="active")
        active <- 1
    else active <- 0
                                        #lambda.min.ratio <- lambda.count.min.ratio
    ## set up likelihood
    ziPoisson <- function(parms) {
        ## count mean
        mu <- as.vector(exp(X %*% parms[1:kx] + offsetx))
        ## binary mean
        phi <- as.vector(linkinv(Z %*% parms[(kx+1):(kx+kz)] + offsetz))
        
        ## log-likelihood for y = 0 and y >= 1
        loglik0 <- log( phi + exp( log(1-phi) - mu ) ) ## -mu = dpois(0, lambda = mu, log = TRUE)
        loglik1 <- log(1-phi) + dpois(Y, lambda = mu, log = TRUE)
        ## collect and return
        loglik <- sum(weights[Y0] * loglik0[Y0]) + sum(weights[Y1] * loglik1[Y1])
        loglik
    }

    ziNegBin <- function(parms) {
        ## count mean
        mu <- as.vector(exp(X %*% parms[1:kx] + offsetx))
        ## binary mean
        phi <- as.vector(linkinv(Z %*% parms[(kx+1):(kx+kz)] + offsetz))
        ## negbin size
        theta <- exp(parms[(kx+kz)+1])
        ## log-likelihood for y = 0 and y >= 1
        loglik0 <- log( phi + exp( log(1-phi) + suppressWarnings(dnbinom(0, size = theta, mu = mu, log = TRUE)) ) )
        loglik1 <- log(1-phi) + suppressWarnings(dnbinom(Y, size = theta, mu = mu, log = TRUE))

        ## collect and return
        loglik <- sum(weights[Y0] * loglik0[Y0]) + sum(weights[Y1] * loglik1[Y1])
        loglik
    }

                                        #log negative binomial density
    dnbinom1 <- function(x, size, mu){
        n <- size
        p <- size/(size+mu)
        lgamma(x+n) - lgamma(n) - lgamma(x+1) + n*log(p) + x*log(1-p)   
    }        
    ziNegBin1 <- function(parms) {
        ## count mean
        mu <- as.vector(exp(X %*% parms[1:kx] + offsetx))
        ## binary mean
        phi <- as.vector(linkinv(Z %*% parms[(kx+1):(kx+kz)] + offsetz))
        ## negbin size
        theta <- exp(parms[(kx+kz)+1])
        ## log-likelihood for y = 0 and y >= 1
        loglik0 <- log( phi + exp( log(1-phi) + dnbinom1(0, size = theta, mu = mu))  )
        loglik1 <- log(1-phi) + dnbinom1(Y, size = theta, mu = mu)
        ## collect and return
        loglik <- rep(NA, n)
        for(i in 1:n){
            if(Y[i]==0) loglik[i] <- weights[i]*loglik0[i]
            else 
                loglik[i] <- weights[i] * loglik1[i]
        }
        loglik
    }
    ziGeom1 <- function(parms) ziNegBin1(c(parms, 0))
    
    ziGeom <- function(parms) ziNegBin(c(parms, 0))
    ziNegBin2 <- function(parms) ziNegBin(c(parms, log(init.theta)))
    pen_zipath <- function(start){
        countcoef1 <- start$count[-1]
        zerocoef1 <- start$zero[-1]
        lc <- length(countcoef1) ### excluding intercept
        if(lc > 0){
            pen1 <- rep(0, lc)
            for(j in 1:lc){
                pen1[j] <- pen_eval(countcoef1[j], lone=lambda.count[k]*alpha.count, ltwo=lambda.count[k]*(1-alpha.count), gamma=gamma.count, penalty=penalty)
            }
        }
        else pen1 <- 0 
        lz <- length(zerocoef1)  ### excluding intercept
        if(lz > 0){
            pen2 <- rep(0, lz)
            for(j in 1:lz)
                pen2[j] <- pen_eval(zerocoef1[j], lone=lambda.zero[k]*alpha.zero, ltwo=lambda.zero[k]*(1-alpha.zero), gamma=gamma.zero, penalty=penalty)
        }
        else pen2 <- 0
        return(n*(sum(pen1) + sum(pen2)))
    }
###check KKT conditions-begin (only for LASSO penalty at the moment)
    check_kkt <- function(parms){
        for(jj in 1:nlambda){
            cat("lambda iteration, jj=", jj, "\n")
            cat("count model part coefficients\n")
            tmp1 <- gradfun(parms[,jj])
            cat("gradients\n")
            print(tmp1)
            for(j in 1:(kx-1)){
                cat("\nvariable j=", j, "lambda.count[jj]=", lambda.count[jj], "coefc[j+1,jj]=", coefc[j+1, jj], "abs(coefc[j+1],jj)=", abs(coefc[j+1,jj]), "\n")
                if(abs(coefc[j+1, jj]) > 0)
                    cat("tmp1[j+1]-n*lambda.count[jj]*sign(coefc[j+1,jj])",
                        tmp1[j+1]-n*lambda.count[jj]*sign(coefc[j+1,jj]), "\n")
                else
                    cat("abs(tmp1[j+1])-n*lambda.count[jj])",
                        abs(tmp1[j+1])-n*lambda.count[jj], "\n")
            }
            cat("\nzero model part coefficients\n")
            for(j in 1:(kz-1)){
                cat("\nvariable j=", j, "lambda.zero[jj]=", lambda.zero[jj], "coefz[j+1,jj]=", coefz[j+1, jj], "abs(coefz[j+1],jj)=", abs(coefz[j+1,jj]), "\n")
                if(abs(coefz[j+1,jj]) > 0)
                    cat("tmp1[kx+j+1]-n*lambda.zero[jj]*sign(coefz[j+1,jj])",
                        tmp1[kx+j+1]-n*lambda.zero[jj]*sign(coefz[j+1,jj]), "\n")
                else
                    cat("abs(tmp1[kx+j+1])-n*lambda.zero[jj])",
                        abs(tmp1[kx+j+1])-n*lambda.zero[jj], "\n")
            }
        }
    }
###check KKT conditions-end (only for count.alpha=zero.alpha=0, and LASSO penalty at the moment)
    famtype <- switch(family,
                      "gaussian"=1,
                      "binomial"=2,
                      "poisson"=3,
                      "negbin"=4)
    pentype <- switch(penalty,
                      "enet"=1,
                      "mnet"=2,
                      "snet"=3)
    pen_zipath1 <- function(start){
        m=length(start$count[-1])
        pencount <- 0
	if(m > 0)
            pencount <- .Fortran("penGLM",
                                 start=as.double(start$count[-1]),
                                 m=as.integer(length(start$count[-1])),
                                 lambda=as.double(lambda.count[k]*penalty.factor.count),
                                 alpha=as.double(alpha.count),
                                 gam=as.double(gamma.count),
                                 penalty=as.integer(pentype),
                                 pen=as.double(0),
                                 PACKAGE="mpath")$pen
        m=length(start$zero[-1])
        penzero <- 0
        if(m > 0)
            penzero <- .Fortran("penGLM",
                                start=as.double(start$zero[-1]),
                                m=as.integer(length(start$zero[-1])),
                                lambda=as.double(lambda.zero[k]*penalty.factor.zero),
                                alpha=as.double(alpha.zero),
                                gam=as.double(gamma.zero),
                                penalty=as.integer(pentype),
                                pen=as.double(0),
                                PACKAGE="mpath")$pen
        return(n*(pencount + penzero))
    } 

    gradPoisson <- function(parms, total=TRUE) {
        ## count mean
        eta <- as.vector(X %*% parms[1:kx] + offsetx)
        mu <- exp(eta)
        ## binary mean
        etaz <- as.vector(Z %*% parms[(kx+1):(kx+kz)] + offsetz)
        muz <- linkinv(etaz)
        
        ## densities at 0
        clogdens0 <- -mu
        dens0 <- muz * (1 - as.numeric(Y1)) + exp(log(1 - muz) + clogdens0)

        ## working residuals  
        wres_count <- ifelse(Y1, Y - mu, -exp(-log(dens0) + log(1 - muz) + clogdens0 + log(mu)))
        wres_zero <- ifelse(Y1, -1/(1-muz) * linkobj$mu.eta(etaz),
        (linkobj$mu.eta(etaz) - exp(clogdens0) * linkobj$mu.eta(etaz))/dens0)
        
        if(total)
            colSums(cbind(wres_count * weights * X, wres_zero * weights * Z))
        else cbind(wres_count * weights * X, wres_zero * weights * Z)
    }
    
    gradGeom <- function(parms, total=TRUE) {
        ## count mean
        eta <- as.vector(X %*% parms[1:kx] + offsetx)
        mu <- exp(eta)
        ## binary mean
        etaz <- as.vector(Z %*% parms[(kx+1):(kx+kz)] + offsetz)
        muz <- linkinv(etaz)

        ## densities at 0
        clogdens0 <- dnbinom(0, size = 1, mu = mu, log = TRUE)
        dens0 <- muz * (1 - as.numeric(Y1)) + exp(log(1 - muz) + clogdens0)

        ## working residuals  
        wres_count <- ifelse(Y1, Y - mu * (Y + 1)/(mu + 1), -exp(-log(dens0) +
                                                                 log(1 - muz) + clogdens0 - log(mu + 1) + log(mu)))
        wres_zero <- ifelse(Y1, -1/(1-muz) * linkobj$mu.eta(etaz),
        (linkobj$mu.eta(etaz) - exp(clogdens0) * linkobj$mu.eta(etaz))/dens0)
        
        if(total)
            colSums(cbind(wres_count * weights * X, wres_zero * weights * Z))
        else cbind(wres_count * weights * X, wres_zero * weights * Z)
    }

    gradNegBin <- function(parms, total=TRUE) {
        ## count mean
        eta <- as.vector(X %*% parms[1:kx] + offsetx)
        mu <- exp(eta)
        ## binary mean
        etaz <- as.vector(Z %*% parms[(kx+1):(kx+kz)] + offsetz)
        muz <- linkinv(etaz)
        ## negbin size
        theta <- exp(parms[(kx+kz)+1])

        ## densities at 0
        clogdens0 <- dnbinom(0, size = theta, mu = mu, log = TRUE)
        dens0 <- muz * (1 - as.numeric(Y1)) + exp(log(1 - muz) + clogdens0)

        ## working residuals  
        wres_count <- ifelse(Y1, Y - mu * (Y + theta)/(mu + theta), -exp(-log(dens0) +
                                                                         log(1 - muz) + clogdens0 + log(theta) - log(mu + theta) + log(mu)))
        wres_zero <- ifelse(Y1, -1/(1-muz) * linkobj$mu.eta(etaz),
        (linkobj$mu.eta(etaz) - exp(clogdens0) * linkobj$mu.eta(etaz))/dens0)
        wres_theta <- theta * ifelse(Y1, digamma(Y + theta) - digamma(theta) +
                                         log(theta) - log(mu + theta) + 1 - (Y + theta)/(mu + theta),
                                     exp(-log(dens0) + log(1 - muz) + clogdens0) *
                                     (log(theta) - log(mu + theta) + 1 - theta/(mu + theta)))

        if(total)
            colSums(cbind(wres_count * weights * X, wres_zero * weights * Z, wres_theta))
        else cbind(wres_count * weights * X, wres_zero * weights * Z, wres_theta)
    }
    checkconv <- function(start, start_old, type=c("count","both"), eps=.Machine$double.eps, thresh=.001){
        type <- match.arg(type)
        checkcount <- .Fortran("checkConvergence",
                               m = as.integer(length(start$count)),
                               beta = as.double(start$count),
                               beta_old = as.double(start_old$count),
                               eps = as.double(eps),
                               thresh = as.double(thresh),
                               converged = as.integer(0),
                               PACKAGE = "mpath")$converged
        if(type=="count") return(checkcount)
        else
            if(checkcount==0) return(FALSE)
        else {
            checkzero <- .Fortran("checkConvergence",
                                  m = as.integer(length(start$zero)),
                                  beta = as.double(start$zero),
                                  beta_old = as.double(start_old$zero),
                                  eps = as.double(eps),
                                  thresh = as.double(thresh),
                                  converged = as.integer(0),
                                  PACKAGE = "mpath")$converged
            return(checkzero)
        }
    }
    convtype <- match.arg(convtype)
    loglikfun <- switch(family,
                        "poisson" = ziPoisson,
                        "geometric" = ziGeom,
                        "negbin" = ziNegBin)
    gradfun <- switch(family,
                      "poisson" = gradPoisson,
                      "geometric" = gradGeom,
                      "negbin" = gradNegBin)
    ## binary link processing
    linkstr <- match.arg(link)
    linkobj <- make.link(linkstr)
    linkinv <- linkobj$linkinv

    if(trace) cat("Zero-inflated Count Model\n",
                  paste("count model:", family, "with log link\n"),
                  paste("zero-inflation model: binomial with", linkstr, "link\n"), sep = "")
    
    

    ## sanity checks
    if(length(Y) < 1) stop("empty model")
    if(all(Y > 0)) stop("invalid dependent variable, minimum count is not zero")  
    if(!isTRUE(all.equal(as.vector(Y), as.integer(round(Y + 0.001)))))
        stop("invalid dependent variable, non-integer values")
    Y <- as.integer(round(Y + 0.001))
    if(any(Y < 0)) stop("invalid dependent variable, negative counts")

    if(trace) {
        cat("dependent variable:\n")
        tab <- table(factor(Y, levels = 0:max(Y)), exclude = NULL)
        names(dimnames(tab)) <- NULL
        print(tab)
    }
    
    ## convenience variables
    n <- length(Y)
    kx <- NCOL(X)
    kz <- NCOL(Z)
    Xold <- X
    Zold <- Z
    Xnew <- as.matrix(X[,-1])
    Znew <- as.matrix(Z[,-1])
    nobs <- n
    Y0 <- Y <= 0
    Y1 <- Y > 0
    ## weights and offset
                                        # weights <- model.weights(mf)
    if(is.null(weights)) weights <- 1
    if(length(weights) == 1) weights <- rep.int(weights, n)
    sumw <- sum(weights)
    weights <- weights/sumw 
    weights <- as.vector(weights)
    if(is.null(offsetx)) offsetx <- 0
    if(length(offsetx) == 1) offsetx <- rep.int(offsetx, n)
    offsetx <- as.vector(offsetx)
    if(is.null(offsetz)) offsetz <- 0
    if(length(offsetz) == 1) offsetz <- rep.int(offsetz, n)
    offsetz <- as.vector(offsetz)
### standardize variables
    if(standardize){
        wt <- weights
        one <- rep(1, length(Y))
        if(dim(Xnew)[2] > 0){
            xx <- Xnew
            meanx <- drop(wt %*% Xnew)
            Xnew <- scale(Xnew, meanx, FALSE) # centers x
            Xnew <- sqrt(wt) * Xnew
            normx <- sqrt(drop(one %*% (Xnew^2)))
            X[,-1] <- Xnew <- scale(xx, meanx, normx)
        }
        if(dim(Znew)[2] > 0){
            zz <- Znew
            meanz <- drop(wt %*% Znew)
            Znew <- scale(Znew, meanz, FALSE) # centers x
            Znew <- sqrt(wt) * Znew
            normz <- sqrt(drop(one %*% (Znew^2)))
            Z[,-1] <- Znew <- scale(zz, meanz, normz)
        }
    }

    ## starting values
    if(!is.null(start)) {
        valid <- TRUE
        if(!("count" %in% names(start))) {
            valid <- FALSE
            warning("invalid starting values, count model coefficients not specified")
            start$count <- rep.int(0, kx)
        }
        if(!("zero" %in% names(start))) {
            valid <- FALSE
            warning("invalid starting values, zero-inflation model coefficients not specified")
            start$zero <- rep.int(0, kz)
        }
        if(length(start$count) != kx) {
            valid <- FALSE
            warning("invalid starting values, wrong number of count model coefficients")
        }
        if(length(start$zero) != kz) {
            valid <- FALSE
            warning("invalid starting values, wrong number of zero-inflation model coefficients")
        }
        if(family == "negbin") {
            if(!("theta" %in% names(start))) start$theta <- 1
            start <- list(count = start$count, zero = start$zero, theta = as.vector(start$theta[1]))
        } else {
            start <- list(count = start$count, zero = start$zero)
        }
        if(!valid) start <- NULL
    }
    if(!is.null(lambda.count) && !is.null(lambda.zero)){
        if(length(lambda.count) != length(lambda.zero))
            stop("length of lambda.count must be the same as lambda.zero\n")
        else nlambda <- length(lambda.count)
    } 
    if(family == "negbin") theta <- rep(NA, nlambda)
    coefc <- matrix(NA, nrow=dim(X)[2], ncol=nlambda) 
    coefz <- matrix(NA, nrow=dim(Z)[2], ncol=nlambda) 
    se <- matrix(NA, nrow=dim(X)[2]+dim(Z)[2]+(family=="negbin"), ncol=nlambda) 
    ll <- rep(NA, nlambda)
    convout <- rep(NA, nlambda)
### How to incorporate offset here?
    if(all(diff(weights)==0)) weightsnow <- rep.int(1L, n) 
    else weightsnow <- weights
    fit0 <- zeroinfl(Y~1+offset(offsetx)|1+offset(offsetz), weights=weightsnow, dist=family)
    if(family=="negbin" && is.null(init.theta))
            init.theta <- fit0$theta
### find solution of intercept model with family = negbin and fixed theta parameter, and replace the estimates for fit0
    ## ML estimation of theta
    if(family=="negbin" && theta.fixed) {
        #if(is.null(init.theta)){
        #    init.theta <- fit0$theta
        #}
        Xtmp <- X
        Ztmp <- Z
        X <- as.matrix(X[,1])
        Z <- as.matrix(Z[,1])
        kx <- 1
        kz <- 1 
### find the solution of estimates for the intercept-only model with fixed init.theta; what about offset? 
        fit0.nb <- optim(fn = ziNegBin2, gr = gradfun,
                         par = c(fit0$coef$count, fit0$coef$zero), #if(dist == "negbin") log(start$theta) else NULL),
                         method = "Nelder-Mead", hessian = FALSE, control = list(fnscale=-1))
        fit0$coefficients <- list(count=fit0.nb$par[1], zero=fit0.nb$par[2])
        fit0$theta <- init.theta
        X <- Xtmp
        Z <- Ztmp
        kx <- NCOL(X)
        kz <- NCOL(Z)
    }
    if(is.null(penalty.factor.count)) penalty.factor.count <- rep(1, dim(Xnew)[2])
    if(all(penalty.factor.count==0)){
        penalty.factor.count <- rep(1, dim(Xnew)[2])
        lambda.count <- rep(0, nlambda)
    }
    if(is.null(penalty.factor.zero)) penalty.factor.zero <- rep(1, dim(Znew)[2])
    if(all(penalty.factor.zero==0)){
        penalty.factor.zero <- rep(1, dim(Znew)[2])
        lambda.zero <- rep(0, nlambda)
    }
                                        #for poisson family or negbin using KKT conditions to compute lambda_count and lambda_zero values
    if(is.null(lambda.count) || is.null(lambda.zero)){
        if(family=="poisson") thetanow <- 10000
            else thetanow <- init.theta
            #if(family=="negbin")
            #    if(theta.fixed) thetanow <- init.theta
            #    else thetanow <- fit0$theta
	    lmax <- .Fortran("lmax_zipath",
                         B=as.double(X),
                         G=as.double(Z),
                         y=as.double(Y),
                         y1=as.integer(Y1),
                         weights=as.double(weights), 
                         n=as.integer(n), 
                         kx=as.integer(kx),
                         kz=as.integer(kz),
                         family=as.integer(famtype),
                         coefc=as.double(fit0$coefficients$count),
                         coefz=as.double(fit0$coefficients$zero),
                         alpha_count=as.double(alpha.count),
                         alpha_zero=as.double(alpha.zero),
                         penaltyfactor_count=as.double(c(1,penalty.factor.count)),
                         penaltyfactor_zero=as.double(c(1, penalty.factor.zero)),
                         theta=as.double(thetanow),
                         lmax_count=as.double(1),
                         lmax_zero=as.double(1),
                         PACKAGE="mpath")
        if(kx > 1){
            lpath <- seq(log(lmax$lmax_count), log(lambda.count.min.ratio * lmax$lmax_count), length.out=nlambda)
            lambda.count <- exp(lpath)
        }
        else lambda.count <- rep(0, nlambda)
        if(kz > 1){
            lpath <- seq(log(lmax$lmax_zero), log(lambda.zero.min.ratio * lmax$lmax_zero), length.out=nlambda)
            lambda.zero <- exp(lpath)
        }
        else lambda.zero <- rep(0, nlambda)
    }
    model_count <- list(coefficients = c(fit0$coefficients$count, rep(0, dim(Xnew)[2])), fitted.values=predict(fit0, type="count"))
    model_zero <- list(coefficients = c(fit0$coefficients$zero, rep(0, dim(Znew)[2])), fitted.values=predict(fit0, type="zero"))
    start <- list(count = model_count$coefficients, zero = model_zero$coefficients)
    vcov <- vector("list", length=nlambda)
    if(family == "negbin"){
        #if(!theta.fixed)
        #    start$theta <- fit0$theta
        #else start$theta <- init.theta
    }
    if(is.null(start$theta)) start$theta <- 1
    if(active==1 && (kx == 1 || kz == 1))
        warnings("type.path='active' is not implemented when count model or zero model is intercept-only, and the model is fitted with type.path='nonactive'.\n")
    if(active==1 && kx > 1 && kz > 1)
        RET1 <- .Fortran("zipath_active", 
                         x=as.double(Xnew), 
                         z=as.double(Znew), 
                         y=as.double(Y), 
                         y1=as.integer(Y1), 
                         weights=as.double(weights/sum(weights)), 
                         n=as.integer(n), 
                         kx=as.integer(kx-1), 
                         kz=as.integer(kz-1),
                         start_count=as.double(model_count$coefficients),
                         start_zero=as.double(model_zero$coefficients),
                         mustart_count=as.double(model_count$fitted.values), 
                         mustart_zero=as.double(model_zero$fitted.values), 
                         offsetx=as.double(offsetx), 
                         offsetz=as.double(offsetz), 
                         intercept=as.integer(intercept),
                         nlambda=as.integer(nlambda), 
                         lambda_count=as.double(lambda.count),
                         lambda_zero=as.double(lambda.zero),
                         alpha_count=as.double(alpha.count),
                         alpha_zero=as.double(alpha.zero),
                         gam_count=as.double(gamma.count),
                         gam_zero=as.double(gamma.zero),
                         penaltyfactor_count=as.double(penalty.factor.count),
                         penaltyfactor_zero=as.double(penalty.factor.zero),
                         maxit=as.integer(maxit), 
                         eps=as.double(.Machine$double.eps), 
                         family=as.integer(famtype),
                         penalty=as.integer(pentype), 
                         trace=as.integer(trace), 
                         coefc=as.double(matrix(0, kx, nlambda)), 
                         coefz=as.double(matrix(0, kz, nlambda)), 
                         yhat=as.double(rep(0, n)), 
                         iter=as.integer(maxit.em),
                         del=as.double(reltol), 
                         rescale=as.integer(rescale), 
                         thresh=as.double(thresh),
                         epsbino=as.double(eps.bino),
                         theta_fixed=as.integer(theta.fixed), 
                         maxit_theta=as.integer(maxit.theta), 
                         theta=as.double(start$theta), 
                         thetaout=as.double(rep(start$theta, nlambda)), 
                         PACKAGE="mpath")
    else RET1 <- .Fortran("zipath_nonactive", 
                          x=as.double(Xnew), 
                          z=as.double(Znew), 
                          y=as.double(Y), 
                          y1=as.integer(Y1), 
                          weights=as.double(weights/sum(weights)), 
                          n=as.integer(n), 
                          kx=as.integer(kx-1), 
                          kz=as.integer(kz-1),
                          start_count=as.double(model_count$coefficients),
                          start_zero=as.double(model_zero$coefficients),
                          mustart_count=as.double(model_count$fitted.values), 
                          mustart_zero=as.double(model_zero$fitted.values), 
                          offsetx=as.double(offsetx), 
                          offsetz=as.double(offsetz), 
                          intercept=as.integer(intercept),
                          nlambda=as.integer(nlambda), 
                          lambda_count=as.double(lambda.count),
                          lambda_zero=as.double(lambda.zero),
                          alpha_count=as.double(alpha.count),
                          alpha_zero=as.double(alpha.zero),
                          gam_count=as.double(gamma.count),
                          gam_zero=as.double(gamma.zero),
                          penaltyfactor_count=as.double(penalty.factor.count),
                          penaltyfactor_zero=as.double(penalty.factor.zero),
                          maxit=as.integer(maxit), 
                          eps=as.double(.Machine$double.eps), 
                          family=as.integer(famtype),
                          penalty=as.integer(pentype), 
                          trace=as.integer(trace), 
                          coefc=as.double(matrix(0, kx, nlambda)), 
                          coefz=as.double(matrix(0, kz, nlambda)), 
                          yhat=as.double(rep(0, n)), 
                          iter=as.integer(maxit.em),
                          del=as.double(reltol), 
                          rescale=as.integer(rescale), 
                          thresh=as.double(thresh),
                          epsbino=as.double(eps.bino),
                          theta_fixed=as.integer(theta.fixed), 
                          maxit_theta=as.integer(maxit.theta), 
                          theta=as.double(start$theta), 
                          thetaout=as.double(rep(start$theta, nlambda)), 
                          PACKAGE="mpath")
    coefc <- matrix(RET1$coefc, ncol=nlambda)
    coefz <- matrix(RET1$coefz, ncol=nlambda)
    if(trace && penalty=="enet" && alpha.count==1 && alpha.zero==1){
        if(family=="poisson")
            tmp <- rbind(coefc, coefz)
        else
            tmp <- rbind(coefc, coefz, matrix(log(theta),nrow=1))
        check_kkt(tmp)
    }
    theta <- RET1$thetaout
    for(k in 1:nlambda)
        ll[k] <- loglikfun(c(coefc[,k], coefz[,k], log(theta[k])))
    ll <- sumw*ll
    if(standardize) {
        coefc <- as.matrix(coefc)
        if(dim(coefc)[1] > 1){
            coefc[-1,] <- coefc[-1,]/normx 
            coefc[1,] <- coefc[1,] - crossprod(meanx, coefc[-1,])
        }
        coefz <- as.matrix(coefz)
        if(dim(coefz)[1] > 1){
            coefz[-1,] <- coefz[-1,]/normz
            coefz[1,] <- coefz[1,] - crossprod(meanz, coefz[-1,])
        }
    }
    if(family != "negbin")  theta <- NULL
    else names(theta) <- lambda.count
    rownames(coefc) <- names(start$count) <- colnames(X)
    rownames(coefz) <- names(start$zero) <- colnames(Z)
    if(nlambda > 1){
        if(lambda.count[1] > 0.0001)
            colnames(coefc) <- round(lambda.count, digits=4)
        else colnames(coefc) <- round(lambda.count, digits=8)
        if(lambda.zero[1] > 0.0001)
            colnames(coefz) <- round(lambda.zero, digits=4)
        else colnames(coefz) <- round(lambda.zero, digits=8)
    } 
    ## fitted and residuals
    mu <- exp(Xold %*% coefc + offsetx)
    phi <- linkinv(Zold %*% coefz + offsetz)
    Yhat <- (1-phi) * mu
    #res <- sqrt(weights) * (Y - Yhat)
    res <- sqrt(weights*sumw) * (Y - Yhat)
    ## effective observations
    nobs <- sum(weights > 0) ## = n - sum(weights == 0)
    dfc <- apply(abs(coefc) > 0, 2, sum) 
    if(family=="negbin") dfc <- 1+dfc
    dfz <- apply(abs(coefz) > 0, 2, sum) 
    aic <- -2*ll + 2*(dfc+dfz)
    bic <- -2*ll + log(n)*(dfc+dfz)
    if(shortlist)
        rval <- list(coefficients = list(count = coefc, zero = coefz),
                     lambda.count = lambda.count,
                     lambda.zero = lambda.zero,
                     alpha.count = alpha.count,
                     alpha.zero = alpha.zero,
                     gamma.count = gamma.count,
                     gamma.zero = gamma.zero,
                     loglik = ll,
                     aic = aic,
                     bic = bic,                        
                     family = family,
                     nlambda = nlambda,
                     theta = theta)
    else
        rval <- list(coefficients = list(count = coefc, zero = coefz),
                     residuals = res,
                     fitted.values = Yhat,
                                        #satu = model_zero$satu,
                     nlambda=nlambda,
                     lambda.count = lambda.count,
                     lambda.zero = lambda.zero,
                     alpha.count = alpha.count,
                     alpha.zero = alpha.zero,
                     gamma.count = gamma.count,
                     gamma.zero = gamma.zero,
                     start = start,
                                        #weights = if(identical(as.vector(weights), rep.int(1L, n))) NULL else weights,
                     weights = weights*sumw,
                     offset = list(count = if(identical(offsetx, rep.int(0, n))) NULL else offsetx,
                                   zero = if(identical(offsetz, rep.int(0, n))) NULL else offsetz),
                     n = nobs,
                     df.null = nobs - 2, 
                     df.residual = dfc + dfz,
                                        #           terms = list(count = mtX, zero = mtZ, full = mt),
                     theta = theta,
                     theta.fixed = theta.fixed,
                     loglik = ll,
                     aic = aic,
                     bic = bic,                         
                     family = family,
                     theta.fixed=theta.fixed,
                     penalty=penalty,
                     link = linkstr,
                     linkinv = linkinv)
    if(y) rval$y <- Y
    if(x) rval$x <- list(count = Xold, zero = Zold)
    class(rval) <- "zipath"
    return(rval)
}

coef.zipath <- function(object, which=1:object$nlambda, model = c("full", "count", "zero"), ...) {
    model <- match.arg(model)
    rval <- object$coefficients
    rval <- switch(model,
                   "full" = list(count=rval$count[, which], zero=rval$zero[, which]), 
                   "count" = rval$count[,which],
                   "zero" = rval$zero[,which])
    rval
}

print.zipath <- function(x, digits = max(3, getOption("digits") - 3), ...)
{

    cat("\nCall:", deparse(x$call, width.cutoff = floor(getOption("width") * 0.85)), "", sep = "\n")
    
    cat(paste("Count model coefficients (", x$family, " with log link):\n", sep = ""))
    print.default(format(x$coefficients$count, digits = digits), print.gap = 2, quote = FALSE)
    if(x$family == "negbin"){
        cat("\nTheta:", round(x$theta, digits),"\n")} else cat("\n")
    
    cat(paste("\nZero-inflation model coefficients (binomial with ", x$link, " link):\n", sep = ""))
    print.default(format(x$coefficients$zero, digits = digits), print.gap = 2, quote = FALSE)
    cat("\n")
                                        #  }
    
    invisible(x)
}

summary.zipath <- function(object,...)
{
    ## residuals
    object$residuals <- residuals(object, type = "pearson")
    
    if(object$family == "negbin") {
        object$coefficients$count <- rbind(object$coefficients$count, "Log(theta)" = log(object$theta))
    }
    
    object$fitted.values <- object$terms <- object$model <- object$y <-
        object$x <- object$levels <- object$contrasts <- object$start <- NULL

    class(object) <- "summary.zipath"
    object
}

print.summary.zipath <- function(x, digits = max(3, getOption("digits") - 3), ...)
{

    cat("\nCall:", deparse(x$call, width.cutoff = floor(getOption("width") * 0.85)), "", sep = "\n")
    cat(paste("\nCount model coefficients (", x$family, " with log link):\n", sep = ""))
    printCoefmat(x$coefficients$count, digits = digits, signif.legend = FALSE)
    
    cat(paste("\nZero-inflation model coefficients (binomial with ", x$link, " link):\n", sep = ""))
    printCoefmat(x$coefficients$zero, digits = digits, signif.legend = FALSE)
    if(x$family == "negbin"){
        cat("\nTheta:", round(x$theta, digits),"\n")} else cat("\n")
    cat("Log-likelihood:", formatC(x$loglik, digits = digits), "\n")
                                        #cat("Converged:", x$converged, "\n")
    
    invisible(x)
}

### Based on predict.zeroinfl in R package pscl
predict.zipath <- function(object, newdata, which=1:object$nlambda, type = c("response", "prob", "count", "zero", "nonzero"),
                           na.action = na.pass, at = NULL, ...)
{
    type <- match.arg(type)
    if(type=="nonzero"){
        nbetacount=object$coefficients$count[,which]
        nbetazero=object$coefficients$zero[,which]
        if(length(which)>1)
            return(list(nbetacount=eval(parse(text="glmnet:::nonzeroCoef(nbetacount[-1,,drop=FALSE],bystep=TRUE)")),
                        nbetazero=eval(parse(text="glmnet:::nonzeroCoef(nbetazero[-1,,drop=FALSE],bystep=TRUE"))))
        else return(list(nbetacount=which(abs(nbetacount[-1]) > 0), nbetazero=which(abs(nbetazero[-1]) > 0)))
    }
    predictzeroinfl1(object, newdata, which, type,
                     na.action = na.pass, at = at, ...)
}

### based on pscl package predict.zeroinfl function
predictzeroinfl1 <- function(object, newdata, which=1:object$nlambda, type = c("response", "prob", "count", "zero"),
                             na.action = na.pass, at = NULL, ...)
{
    type <- match.arg(type)
    object$coefficients$count <- object$coefficients$count[,which]
    object$coefficients$zero <- object$coefficients$zero[,which] 
    object$nlambda <- length(which)
    ## if no new data supplied
    if(missing(newdata)) {
        rval <- object$fitted.values[,which]
        if(type != "response") {
            if(!is.null(object$x)) {
                X <- object$x$count
                Z <- object$x$zero
            } else if(!is.null(object$model)) {
                X <- model.matrix(object$terms$count, object$model, contrasts = object$contrasts$count)
                Z <- model.matrix(object$terms$zero,  object$model, contrasts = object$contrasts$zero)
            } else {
                stop("predicted probabilities cannot be computed with missing newdata")
            }
            offsetx <- if(is.null(object$offset$count)) rep.int(0, NROW(X)) else object$offset$count
            offsetz <- if(is.null(object$offset$zero))  rep.int(0, NROW(Z)) else object$offset$zero
            mu <- exp(X %*% object$coefficients$count + offsetx)
            phi <- object$linkinv(Z %*% object$coefficients$zero + offsetz)
        }
    } else {
        mf <- model.frame(delete.response(object$terms$full), newdata, na.action = na.action, xlev = object$levels)
        X <- model.matrix(delete.response(object$terms$count), mf, contrasts = object$contrasts$count)
        Z <- model.matrix(delete.response(object$terms$zero),  mf, contrasts = object$contrasts$zero)
        offsetx <- model_offset_2(mf, terms = object$terms$count, offset = FALSE)
        offsetz <- model_offset_2(mf, terms = object$terms$zero,  offset = FALSE)
        if(is.null(offsetx)) offsetx <- rep.int(0, NROW(X))
        if(is.null(offsetz)) offsetz <- rep.int(0, NROW(Z))
        if(!is.null(object$call$offset)) offsetx <- offsetx + eval(object$call$offset, newdata)
        mu <- exp(X %*% object$coefficients$count + offsetx)
        phi <- object$linkinv(Z %*% object$coefficients$zero + offsetz)
        rval <- (1-phi) * mu
    }
    
    ## predicted means for count/zero component
    if(type == "count") rval <- mu
    if(type == "zero") rval <- phi

    ## predicted probabilities
    if(type == "prob") {
        if(!is.null(object$y)) y <- object$y
        else if(!is.null(object$model)) y <- model.response(object$model)
        else stop("predicted probabilities cannot be computed for fits with y = FALSE and model = FALSE")

        yUnique <- if(is.null(at)) 0:max(y) else at
        nUnique <- length(yUnique)
        rval <- vector("list", length=object$nlambda)
        for(j in 1:length(rval)){ 
            rval[[j]] <- matrix(NA, nrow = length(y), ncol = nUnique)
        }

        for(j in 1:object$nlambda){ 
            switch(object$family,
                   "poisson" = {
                       rval[[j]][, 1] <- phi[,j] + (1-phi[,j]) * exp(-mu[,j])
                       for(i in 2:nUnique) rval[[j]][,i] <- (1-phi[,j]) * dpois(yUnique[i], lambda = mu[,j])
                   },
                   "negbin" = {
                       theta <- object$theta[j]
                       rval[[j]][, 1] <- phi[,j] + (1-phi[,j]) * dnbinom(0, mu = mu[,j], size = theta[j])
                       for(i in 2:nUnique) rval[[j]][,i] <- (1-phi[,j]) * dnbinom(yUnique[i], mu = mu[,j], size = theta[j])
                   },
                   "geometric" = {
                       rval[[j]][, 1] <- phi[,j] + (1-phi[,j]) * dnbinom(0, mu = mu[,j], size = 1)
                       for(i in 2:nUnique) rval[[j]][,i] <- (1-phi[,j]) * dnbinom(yUnique[i], mu = mu[,j], size = 1)
                   })
        }
    }
    rval
}


fitted.zipath <- function(object, ...) {
    object$fitted.values
}

residuals.zipath <- function(object, type = c("pearson", "response"), ...) {

    type <- match.arg(type)
    res <- object$residuals

    switch(type,
           
           "response" = {
               return(res)
           },
           
           "pearson" = {
               mu <- predict(object, type = "count")
               phi <- predict(object, type = "zero")
               theta1 <- switch(object$family,
                                "poisson" = 0,
                                "geometric" = 1,
                                "negbin" = 1/object$theta)
               vv <- object$fitted.values * (1 + (phi + theta1) * mu)
               return(res/sqrt(vv))  
           })
}

terms.zipath <- function(x, model = c("count", "zero"), ...) {
    x$terms[[match.arg(model)]]
}

model.matrix.zipath <- function(object, model = c("count", "zero"), ...) {
    model <- match.arg(model)
    if(!is.null(object$x)) rval <- object$x[[model]]
    else if(!is.null(object$model)) rval <- model.matrix(object$terms[[model]], object$model, contrasts = object$contrasts[[model]])
    else stop("not enough information in fitted model to return model.matrix")
    return(rval)
}

### evaluation of penalty function
###consider enet: \lambda_1 |theta| + 1/2 * \lambda_2 theta^2, mnet and snet penalty
pen_eval <- function(theta, lone, ltwo, gamma,
                     penalty = c("enet", "mnet", "snet")){
    if(lone < 0 || ltwo < 0) stop("lone and ltwo should be non-negative values\n")
    penalty <- match.arg(penalty)
    if(!penalty %in% c("enet", "mnet", "snet"))
        stop("penalty type ", penalty, " is not implemented\n")
    if(penalty == "mnet"){
        if(gamma <= 1) stop("gamma should > 1 for mnet penenaly\n")
        if(abs(theta) <= (gamma * lone)) return(lone * abs(theta) - theta^2/(2*gamma) + 0.5*ltwo*theta^2)
        else return(0.5*gamma*lone^2 + 0.5*ltwo*theta^2)
    }
    if(penalty == "snet"){
        if(gamma <= 2) stop("gamma should > 2 for snet penalty\n")
        if(abs(theta) <= lone) return(lone*abs(theta) + 0.5*ltwo*theta^2)
        if(abs(theta) <= (gamma * lone)) return((gamma * lone * abs(theta) - 0.5*(theta^2+lone^2))/(gamma-1) + 0.5*ltwo*theta^2)
        return(lone^2*(gamma^2 - 1)/(2*(gamma-1)) + 0.5*ltwo*theta^2)
    }
    if(penalty == "enet")
        return(abs(theta)*lone + 0.5*ltwo*theta^2)
}
### evaluation of second derivative of penalty function
###consider enet: \lambda_1 |theta| + 1/2 * \lambda_2 theta^2, mnet and snet penalty
pen2_eval <- function(theta, lone, ltwo, gamma,
                      penalty = c("enet", "mnet", "snet")){
    if(lone < 0 || ltwo < 0) stop("lone and ltwo should be non-negative values\n")
    penalty <- match.arg(penalty)
    if(!penalty %in% c("enet", "mnet", "snet"))
        stop("penalty type ", penalty, " is not implemented\n")
    if(penalty == "mnet"){
        if(gamma <= 1) stop("gamma should > 1 for mnet penalty\n")
        if(abs(theta) <= (gamma * lone)) return(-1/gamma+ltwo)
        else return(ltwo)
    }
    if(penalty == "snet"){
        if(gamma <= 2) stop("gamma should > 2 for snet penalty\n")
        if(abs(theta) > lone && abs(theta) < (gamma * lone)) return(-1/(gamma-1) + ltwo)
        else return(ltwo)
    }
    if(penalty == "enet")
        return(ltwo)
}

### convert zeroinfl object to class zipath, thus can be used to predict newdata
conv2zipath <- function(object, family=c("poisson", "negbin", "geometric")){
    family <- match.arg(family)
    if(inherits(object, "zeroinfl")) class(object) <- "zipath"
    object$family <- family
    object$nlambda <- 1
    object$fitted.values <- matrix(object$fitted.values, ncol=1)
    namecount <- names(object$coefficients$count)
    namezero <- names(object$coefficients$zero)
    object$coefficients$count <- matrix(object$coefficients$count, ncol=1)
    object$coefficients$zero <- matrix(object$coefficients$zero, ncol=1)
    rownames(object$coefficients$count) <- namecount
    rownames(object$coefficients$zero) <- namezero
    object
}

predprob.zipath <- function(obj, which, at, ...) {
    if(length(which) > 1)
        stop("length of which should be 1\n")
    predict(obj, type = "prob", which=which, at=at, ...)[[1]]
}

hessianReg <- function(x, which, ...)
    UseMethod("hessianReg")

hessianReg.zipath <- function(x, which, log=FALSE, ...) {
    ## extract data
    Y <- if(is.null(x$y)) model.response(model.frame(x)) else x$y
    X <- model.matrix(x, model = "count")
    Z <- model.matrix(x, model = "zero")
    beta <- coef(x, model = "count", which=which)
    gamma <- coef(x, model = "zero", which=which)
    kx1 <- abs(beta) > 0
    kx2 <- which(kx1)
    kz1 <- abs(gamma) > 0
    kz2 <- which(kz1)
    X <- X[,kx2]
    Z <- Z[,kz2]
    if(length(kx2)==1) X <- matrix(X)
    if(length(kz2)==1) Z <- matrix(Z)
    beta <- beta[kx2]
    gamma <- gamma[kz2]
    if(x$family=="negbin")
        theta <- x$theta[which]

    offset <- x$offset
    if(is.list(offset)) {
        offsetx <- offset$count
        offsetz <- offset$zero
    } else {
        offsetx <- offset
        offsetz <- NULL
    }
    if(is.null(offsetx)) offsetx <- 0
    if(is.null(offsetz)) offsetz <- 0
    linkobj <- make.link(x$link)
                                        #  wts <- weights(x)
    weights <- weights(x)
    if(is.null(weights)) weights <- 1
    Y0 <- Y <= 0
    Y1 <- Y > 0
    eta <- as.vector(X %*% beta + offsetx)
    mu <- exp(eta)             ### \mu_i in paper
    etaz <- as.vector(Z %*% gamma + offsetz)
    muz <- linkobj$linkinv(etaz)
    if(x$family=="negbin"){
        g1 <- theta/(theta+mu) ### r_i in paper
        g1t <- g1^theta        ### s_i in paper
        g2 <- mu/(theta+mu)    ### t_i in paper
### warning: check if this works for other links: muz=exp(etaz)/(1+exp(etaz))?
        v <- muz               ### p_i in paper
        etaz.e <- exp(etaz)    ### \xi_i in paper
        h <- etaz.e + g1t      ### h_i in paper
        p <- dim(X)[2]
        q <- dim(Z)[2]
        rval <- matrix(NA, p+q+1, p+q+1)
### w.r.t (theta,theta)
        rv0 <- g1t*theta^3*(log(g1))^2*etaz.e+2*g1t*log(g1)*mu*etaz.e*(log(g1)+1)*theta^2 + mu^2*etaz.e*g1t*(2*log(g1)+1+(log(g1))^2)*theta+mu^2*etaz.e*g1t+mu^2*g1t^2
        rv0 <- rv0/(theta*(theta+mu)^2*h^2)
        rv1 <- trigamma(Y+theta)-trigamma(theta)+(Y+theta)/((theta+mu)^2)+1/theta*g2-1/(theta+mu)
        rval[p+q+1, p+q+1] <- sum(weights[Y0]*rv0[Y0])+sum(weights[Y1]*rv1[Y1]) 
        if(log) rval[p+q+1, p+q+1] <- theta^2*rval[p+q+1, p+q+1] # for log(theta)
### for X coef partial derivative w.r.t. (beta_j, beta_k)
        for(j in 1:p)
            for(k in j:p){
                rv <- ifelse(Y1, -theta*g2^2/mu*(theta+Y), 
                             -theta^2*mu*(-g1t*mu*etaz.e+etaz.e*g1t+g1t^2)/(((theta+mu)*h)^2)) 
                rval[j,k] <- sum(weights*X[,j]*X[,k]*rv)  
            }
### for X coef and Z coef (beta, gamma)
        for(j in 1:p)
            for(k in (p+1):(p+q)){
                rv0 <- theta*X[,j]*Z[,k-p]*g1^theta*g2*etaz.e/h^2
                rval[j,k] <- sum(weights[Y0]*rv0[Y0])     
            } 
### for (beta, theta)
        for(j in 1:p){
            rv0 <- -(theta^2*g1t*log(g1)*etaz.e+theta*g1t*log(g1)*mu*etaz.e+g1t*theta*mu*etaz.e+g1t*mu*etaz.e+g1t^2*mu)*mu
            rv0 <- rv0/(((theta+mu)*h)^2) 
            rv1 <- -mu*(-Y+mu)/((theta+mu)^2)
            rval[j, p+q+1] <- sum(weights[Y0]*rv0[Y0]*X[Y0,j]) + sum(weights[Y1]*rv1[Y1]*X[Y1,j])
            if(log) rval[j, p+q+1] <- theta*rval[j, p+q+1]  ### if theta was parameterized as log(theta), see gradfun
        }
###w.r.t. (gamma_j, gamma_k)
        for(j in (p+1):(p+q))
            for(k in j:(p+q)){
                rv <- ifelse(Y1, -v*(1-v), -v+v^2+etaz.e/h-(etaz.e/h)^2)
                rval[j,k] <- sum(weights*Z[,j-p]*Z[,k-p]*rv)  
            }
###w.r.t. {theta, gamma}
        for(k in (p+1):(p+q)){
            rv0 <- g1t*(log(g1)+g2)*etaz.e*Z[,k-p]
                                        #rv0 <- g1t*(log(g1)+(1/(theta+mu)-theta/(theta+mu)^2)*(theta+mu))*etaz.e*Z[,k-p]
            rv0 <- -rv0/h^2
            rval[k,p+q+1] <- sum(weights[Y0]*rv0[Y0])
            if(log) rval[k,p+q+1] <- theta * rval[k, p+q+1]  ### if theta was parameterized as log(theta)
        }
        rval[lower.tri(rval)] = t(rval)[lower.tri(rval)]
    }
                                        #    if(log)
                                        #        colnames(rval) <- c(names(coef(x)$count), names(coef(x)$zero), "log(theta)")
                                        #    else colnames(rval) <- c(names(coef(x)$count), names(coef(x)$zero), "theta")
                                        #else colnames(rval) <- c(names(coef(x)), "theta")
                                        #    rownames(rval) <- colnames(rval)
    return(rval)
}

### derived from sandwich function in R package sandwich
sandwichReg <- function(x, breadreg.=breadReg, meatreg.=meatReg, which, log=FALSE, ...){
    if(missing(which)) which <- 1:x$nlambda
    RET <- vector("list", length=x$nlambda)
    for(k in which){
        if(is.function(breadReg)) breadreg. <- breadreg.(x, k, log=log)
        if(is.function(meatReg)) meatreg. <- meatreg.(x, k, log=log)
        n <- x$n
        RET[[k]] <- 1/n*(breadreg. %*% meatreg. %*% breadreg.)   
    }
    RET
}

meatReg <- function(x, which, ...)
    UseMethod("meatReg")

meatReg.zipath <- function(x, which, log=FALSE, ...){
    obj1 <- x
    k <- which
    count <- as.matrix(coef(x)$count)[,k]
    zero <- as.matrix(coef(x)$zero)[,k]
    obj1$coefficients <- list(count=matrix(count, ncol=1), zero=matrix(zero, ncol=1))
    obj1$nlambda <- 1
    if(x$family=="negbin")
        obj1$theta <- x$theta[k]
    obj1$fitted.values <- x$fitted.values[,k]
    grad <- estfunReg(obj1, log=log)
    kx1 <- abs(count) > 0
    kx2 <- which(kx1)
    kz1 <- abs(zero) > 0
    kz2 <- which(kz1)
    if(x$family=="negbin") npar <- c(kx2, length(kz1)+kz2, dim(grad)[2])
    else npar <- c(kx2, length(kz1)+kz2)
    grad <- grad[,npar] 
    n <- NROW(grad)
    tmp <- apply(grad, 2, mean)
    RET <- 1/n*crossprod(grad)  - tmp %o% tmp ### note there is an additional 2nd item for penalized regression. in ordinary regression, the 2nd item=0 
                                        #        rownames(RET) <- colnames(RET)
    return(RET)
}

breadReg <- function(x, which, ...)
    UseMethod("breadReg")

breadReg.zipath <- function(x, which, log=FALSE, ...){
    family <- x$family
    n <- x$n
    kx <- dim(x$coefficients$count)[1]    
    kx1 <- abs(x$coefficients$count[,which]) > 0
    kx2 <- which(kx1)
                                        #    kz <- dim(x$coefficients$zero)[1]    
    kz1 <- abs(x$coefficients$zero[,which]) > 0
    kz2 <- which(kz1)
    sigmax <- rep(0, sum(kx1))
    sigmaz <- rep(0, sum(kz1)+(family=="negbin"))
                                        #sigma <- rep(0, sum(kx1)+sum(kz1)+(family=="negbin"))
                                        #sigma <- rep(0, kx+kz+(family=="negbin"))
    k <- which
    start <- x$coefficient
    if(length(kx2) > 1)
        for(ss in 2:length(kx2))
            sigmax[ss] <- pen2_eval(abs(start$count[kx2[ss],k]), lone=x$lambda.count[k]*x$alpha.count, ltwo=x$lambda.count[k]*(1-x$alpha.count), gamma=x$gamma.count, penalty=x$penalty)
    if(length(kz2) > 1)
        for(ss in 2:length(kz2))
            sigmaz[ss] <- pen2_eval(abs(start$zero[kz2[ss],k]), lone=x$lambda.zero[k]*x$alpha.zero, ltwo=x$lambda.zero[k]*(1-x$alpha.zero), gamma=x$gamma.zero, penalty=x$penalty)
    sigma <- c(sigmax, sigmaz)
    hes <- hessianReg(x, which, log=log)
    s1 <- try(-solve(hes - n*diag(sigma))*n)
    return(s1)
}

estfunReg <- function(x, ...)
    UseMethod("estfunReg")

estfunReg.zipath <- function(x, which=1, log=FALSE, ...) {
    ## extract data
    if(length(which)!=1) stop("argument which should be one integer only\n")
    Y <- if(is.null(x$y)) model.response(model.frame(x)) else x$y
    X <- model.matrix(x, model = "count")
    Z <- model.matrix(x, model = "zero")
    beta <- coef(x, model = "count")
    gamma <- coef(x, model = "zero")
    if(x$nlambda > 1){
        beta <- beta[,which]
        gamma <- gamma[,which]
    }
    if(x$family=="negbin")
        theta <- x$theta[which]
    offset <- x$offset
    if(is.list(offset)) {
        offsetx <- offset$count
        offsetz <- offset$zero
    } else {
        offsetx <- offset
        offsetz <- NULL
    }
    if(is.null(offsetx)) offsetx <- 0
    if(is.null(offsetz)) offsetz <- 0
    linkobj <- make.link(x$link)
    wts <- weights(x)
    if(is.null(wts)) wts <- 1
    Y1 <- Y > 0
    eta <- as.vector(X %*% beta + offsetx)
    mu <- exp(eta)
    etaz <- as.vector(Z %*% gamma + offsetz)
    muz <- linkobj$linkinv(etaz)

    ## density for y = 0
    clogdens0 <- switch(x$family,
                        "poisson" = -mu,
                        "geometric" = dnbinom(0, size = 1, mu = mu, log = TRUE),
                        "negbin" = dnbinom(0, size = theta, mu = mu, log = TRUE))
    dens0 <- muz * (1 - as.numeric(Y1)) + exp(log(1 - muz) + clogdens0)

    ## working residuals  
    wres_count <- switch(x$family,
                         "poisson" = ifelse(Y1, Y - mu, -exp(-log(dens0) + log(1 - muz) + clogdens0 + log(mu))),
                         "geometric" = ifelse(Y1, Y - mu * (Y + 1)/(mu + 1), -exp(-log(dens0) +
                                                                                  log(1 - muz) + clogdens0 - log(mu + 1) + log(mu))),
                         "negbin" = ifelse(Y1, Y - mu * (Y + theta)/(mu + theta), -exp(-log(dens0) +
                                                                                       log(1 - muz) + clogdens0 + log(theta) - log(mu + theta) + log(mu))))
    wres_zero <- ifelse(Y1, -1/(1-muz) * linkobj$mu.eta(etaz),
    (linkobj$mu.eta(etaz) - exp(clogdens0) * linkobj$mu.eta(etaz))/dens0)
    wres_theta <- ifelse(Y1, digamma(Y + theta) - digamma(theta) +
                             log(theta) - log(mu + theta) + 1 - (Y + theta)/(mu + theta),
                         exp(-log(dens0) + log(1 - muz) + clogdens0) *
                         (log(theta) - log(mu + theta) + 1 - theta/(mu + theta)))

    ## compute gradient from data
    if(log)
        rval <- cbind(wres_count * wts * X, wres_zero * wts * Z, theta*wres_theta)
    else rval <- cbind(wres_count * wts * X, wres_zero * wts * Z, wres_theta)
    if(x$family!="negbin")
        colnames(rval) <- c(names(coef(x)$count), names(coef(x)$zero))
    else{
                                        #       	  if(log)
                                        #      colnames(rval) <- c(names(coef(x)$count), names(coef(x)$zero), "log(theta)")
                                        #   else colnames(rval) <- c(names(coef(x)$count), names(coef(x)$zero), "theta")
    }
                                        #  rownames(rval) <- rownames(X)
    return(rval)
}

se <- function(x, which, log=TRUE, ...)
    UseMethod("se")

se.zipath <- function(x, which, log=TRUE, ...){
    if(x$family=="poisson"){
###use a large theta of negbin distribution to approximate poisson distribution
        x$family <- "negbin"
        x$theta.fixed <- TRUE
        x$theta <- rep(10000, x$nlambda)
    }
    if(identical(as.vector(x$weights), rep.int(1L, x$n))) NULL 
    else if(all(diff(x$weights)==0)) x$weights <- rep.int(1L, x$n) 
    else stop("The results may be incorrect if weights are different\n")
    if(is.null(which)) stop("which has no numerical value\n")
    if(length(which) > 1) stop("which is an integer value\n")
    tmp <- sandwichReg(x, which=which, log=log)
    tmp0 <- sqrt(diag(tmp[[which]]))
    varcount <- which(abs(coef(x)$count[,which]) > 0)
    varzero <- which(abs(coef(x)$zero[,which]) > 0)
    tmp1 <- rownames(x$coefficient$count)[varcount]
    kx <- length(tmp1)
    tmp2 <- rownames(x$coefficient$zero)[varzero]
    kz <- length(tmp2)
    res <- list(count=tmp0[1:kx],zero=tmp0[(kx+1):(kx+kz)])
    names(res$count) <- tmp1
    names(res$zero) <- tmp2
    if(x$family=="negbin" & !x$theta.fixed)
        res$theta <- tmp0[length(tmp0)]
    res
}

### from pamr package
permute.rows <-function(x)
{
        dd <- dim(x)
        n <- dd[1]
        p <- dd[2]
        mm <- runif(length(x)) + rep(seq(n) * 10, rep(p, n))
        matrix(t(x)[order(mm)], n, p, byrow = TRUE)
}

loss2 <- function(y, f, weights, cfun, dfun, s, delta=0.0001){
    cfun <- cfun2num(cfun)
    dfun <- dfun2num(dfun)
    #if(!dfun %in% c(1, 4:7)) stop("dfun is not implemented")
    if(!cfun %in% 1:8) stop("cfun is not implemented")
    check_s(cfun, dfun, s)
    if(dfun %in% 4:7 && !setequal(unique(y), c(-1, 1))) stop("y must be -1/1 for binary classification")
    n <- length(y)
    if(n!=length(f)) stop("y and f should have comparable length\n")
    if(missing(weights)) weights <- rep(1, length(y))
    if(n!=length(weights)) stop("y and weights should have comparable length\n")
    if(cfun==6)
           if(s > 1) delta <- (s-1)/2 else
           if(s==1) delta <- 0 else{
             if(is.null(delta)) stop("delta must be provided")
             if(delta <= 0) stop("delta must be positive")
           }
    .Fortran("loss2",
                                    n=as.integer(n),
                                    y=as.double(y),
                                    f=as.double(f),
                                    weights=as.double(weights),
                                    cfun=as.integer(cfun),
                                    dfun=as.integer(dfun),
                                    s=as.double(s),
                                    delta=as.double(delta),
                                    los=as.double(0.0),
                                    PACKAGE="mpath")$los
}

loss2_irsvm <- function(y, f, weights, cfun, dfun, s, eps, delta=0.0001){
    check_s(cfun, dfun=6, s)
    cfun <- cfun2num(cfun)
    if(!cfun %in% 1:8) stop("cfun is not implemented")
    dfun <- switch(dfun,
                    "eps-regression"=2,
                    "nu-regression"=2,
                    "C-classification"=6,
                    "nu-classification"=6)
    if(!dfun %in% c(2, 6)) stop("dfun is not intended for loss2_irsvm")
    if(dfun==2 && eps < 0) stop("eps should be >= 0 for dfun=2 (eps-regression)")
    n <- length(y)
    if(n!=length(f)) stop("y and f should have comparable length\n")
    if(missing(weights)) weights <- rep(1, length(y))
    if(n!=length(weights)) stop("y and weights should have comparable length\n")
      if(cfun==6)
           if(s > 1) delta <- (s-1)/2 else
           if(s==1) delta <- 0 else{
             if(is.null(delta)) stop("delta must be provided")
             if(delta <= 0) stop("delta must be positive")
           }
    .Fortran("loss2_ccsvm",
                                    n=as.integer(n),
                                    y=as.double(y),
                                    f=as.double(f),
                                    weights=as.double(weights),
                                    cfun=as.integer(cfun),
                                    dfun=as.integer(dfun),
                                    s=as.double(s),
                                    eps=as.double(eps),
                                    delta=as.double(delta),
                                    los=as.double(0.0),
                                    PACKAGE="mpath")$los
}

C loss value in subroutine irglmreg, cf: loss.f
C cost: Not implemented in the optimization problem!
C output: los is the average loss
C compare to loss2 subroutine, there is additional input eps for dfun= 2
C (eps-regression). cf: ../R/loss2_ccsvm function
      subroutine loss2_ccsvm(n, y, f, weights, cfun, dfun, s, eps,
     +      delta, los)
       implicit none
       integer cfun, dfun, i, n
       double precision y(n),f(n),u(n),z(n),gval(n),s,eps,los,delta,
     +   weights(n)
       external compute_g, compute_z, compute_u 

      call compute_u(dfun, n, y, f, u)
      call compute_z(dfun, n, u, z, eps)
      call compute_g(cfun, n, z, s, delta, gval)
      los = 0.d0
      do 20 i=1, n
      los = los + weights(i)*gval(i)
20    continue

      return
      end

### penalized nonconvex loss based linear models (NCL) for regression and classification
nclreg <- function(x, ...) UseMethod("nclreg")

nclreg.default <- function(x, ...) {
    if (extends(class(x), "Matrix"))
        return(nclreg.matrix(x = x, ...))
    stop("no method for objects of class ", sQuote(class(x)),
         " implemented")
}

nclreg.formula <- function(formula, data, weights, offset=NULL, contrasts=NULL, ...){
    ## extract x, y, etc from the model formula and frame
    if(!attr(terms(formula, data=data), "intercept"))
        stop("non-intercept model is not implemented")
    if(missing(data)) data <- environment(formula)
    mf <- match.call(expand.dots = FALSE)
    m <- match(c("formula", "data", "weights",
                 "offset"), names(mf), 0L)
    mf <- mf[c(1L, m)]
    mf$drop.unused.levels <- TRUE
    mf[[1L]] <- as.name("model.frame")
    mf <- eval(mf, parent.frame())

    mt <- attr(mf, "terms") # allow model.frame to have updated it

    Y <- model.response(mf, "any") # e.g. factors are allowed
    ## avoid problems with 1D arrays, but keep names
    if(length(dim(Y)) == 1L) {
        nm <- rownames(Y)
        dim(Y) <- NULL
        if(!is.null(nm)) names(Y) <- nm
    }
    ## null model support
    X <- if (!is.empty.model(mt)) model.matrix(mt, mf, contrasts) else matrix(,NROW(Y), 0L)
    ## avoid any problems with 1D or nx1 arrays by as.vector.
    weights <- as.vector(model.weights(mf))
    if(!length(weights)) weights <- rep(1, nrow(mf))
    else if(any(weights < 0)) stop("negative weights not allowed")
    if(!is.null(weights) && !is.numeric(weights))
        stop("'weights' must be a numeric vector")
    if(length(weights) != length(Y))
        stop("'weights' must be the same length as response variable")

    offset <- as.vector(model.offset(mf))
    if(!is.null(offset)) {
        if(length(offset) != NROW(Y))
            stop(gettextf("number of offsets is %d should equal %d (number of observations)", length(offset), NROW(Y)), domain = NA)
    }

    RET <- nclreg_fit(X[,-1], Y, weights=weights, offset=offset, ...)
    RET$call <- match.call()
    RET <- c(RET, list(formula=formula, terms = mt, data=data,
                       contrasts = attr(X, "contrasts"),
                       xlevels = .getXlevels(mt, mf)))
    class(RET) <- "nclreg"
    RET
}
nclreg.matrix <- function(x, y, weights, offset=NULL, ...){
    RET <- nclreg_fit(x, y, weights, offset=offset, ...)
    RET$call <- match.call()
    return(RET)
}

### compute second derivative of robust loss, cf: Appendix in Wang (2016), Quadratic majorization for nonconvex loss with applications to the boosting algorithm
compute.2d <- function(y, f, s, family=c("clossR", "closs", "gloss", "qloss")){
    family <- match.arg(family)
    if(family=="clossR") u <- y-f
    else u <- y*f
    if(family %in% c("clossR", "closs")){
        b <- -(u^2-2*u-s^2+1)/(s^4*exp((1-u)^2/(2*s^2)))
        cval <- 1
        if(family=="closs")
            cval <- 1/(1 - exp(-1/(2*s^2)))
        cval*b
    }
    else if(family=="gloss"){
        2^s*s*exp(u)*(1+exp(u))^(-s-2)*(s*exp(u)-1)
    }
    else if(family=="qloss")
        sqrt(2/pi)*u/s^3*exp(-u^2/(2*s^2))
}

nclreg_fit <- function(x,y, weights, offset, rfamily=c("clossR", "closs", "gloss", "qloss"), s=NULL, fk=NULL, iter=10, reltol=1e-5, penalty=c("enet","mnet","snet"), nlambda=100, lambda=NULL, type.path=c("active", "nonactive", "onestep"), decreasing=FALSE, lambda.min.ratio=ifelse(nobs<nvars,.05, .001),alpha=1, gamma=3, standardize=TRUE, intercept=TRUE, penalty.factor = NULL, maxit=1000, type.init=c("bst", "ncl", "heu"), mstop.init=10, nu.init=0.1, eps=.Machine$double.eps, epscycle=10, thresh=1e-6, trace=FALSE){
### compute h value
    compute.h <- function(rfamily, y, fk_old, s, B){
        if(rfamily=="clossR")
            h <- gradient(family=rfamily, u=y-fk_old, s=s)/B+fk_old
        else if(rfamily %in% c("closs", "gloss", "qloss")) 
            h <- -y*gradient(family=rfamily, u=y*fk_old, s=s)/B+fk_old
        h
    }
    call <- match.call()
    rfamily <- match.arg(rfamily)
    penalty <- match.arg(penalty)
    type.path <- match.arg(type.path)
    type.init <- match.arg(type.init)
    if(type.path=="active")
	    active <- 1
    else active <- 0
    if (!is.null(lambda) && length(lambda) > 1 && all(lambda == cummin(lambda))){
	    decreasing <- TRUE
#	    if(type.path=="active"){
#	    decreasing <- FALSE
#	      warnings("choose type.path='nonactive' with increasing sequence of lambda, or let lambda=rev(lambda) as computed below\n")
#	    lambda <- rev(lambda) 
#       }
    }
    else if(!is.null(lambda) && length(lambda) > 1 && all(lambda == cummax(lambda)))
	    decreasing <- FALSE
#    else if(is.null(lambda) && decreasing && type.path=="active")
#	    stop("set decreasing=FALSE or type.path='nonactive'")
    if(rfamily %in% c("closs", "gloss", "qloss"))
        if(!all(names(table(y)) %in% c(1, -1)))
            stop("response variable must be 1/-1 for family ", rfamily, "\n")
    nm <- dim(x)
    nobs <- n <- nm[1]
    if(length(y)!=n) stop("length of y is different from row of x\n")
    nvars <- m <- nm[2]
    B <- bfunc(family=rfamily, s=s)
    if(missing(weights)) weights=rep(1,nobs)
    weights <- as.vector(weights)
    w <- weights/sum(weights)
    if(!is.null(weights) && !is.numeric(weights))
        stop("'weights' must be a numeric vector")
    ## check weights and offset
    if( !is.null(weights) && any(weights < 0) ){
        stop("negative weights not allowed")
    }
    if (is.null(offset)){
         is.offset <- FALSE
         offset <- rep.int(0, nobs)
     }else is.offset <- TRUE
    pentype <- switch(penalty,
                      "enet"=1,
                      "mnet"=2,
                      "snet"=3)

    if(is.null(penalty.factor))
        penalty.factor <- rep(1, nvars)
    if(all(penalty.factor==0)){	
        lambda <- rep(0, nvars) 
        penalty.factor <- rep(1, nvars)
    }
    xold <- x
    if(standardize){
         tmp <- stan(x, weights)
         x <- tmp$x
         meanx <- tmp$meanx
         normx <- tmp$normx
    }
    if(is.null(s)){
        warning("missing nonconvex loss tuning parameter s value, and a default value is provided\n")
        s <- switch(rfamily,       
                    "closs"=1,
                    "gloss"=1,
                    "qloss"=2,
                    "clossR"=1
                    )
    }
    penfac <- penalty.factor/sum(penalty.factor) * nvars
    zscore <- function(rfamily, RET, s){
        if(rfamily=="clossR"){
            z <- gradient(family=rfamily, u=y-RET$fitted.values, s=s)
            scores <- abs(crossprod(x, w*z))/(penfac*alpha)
        } 
        else if(rfamily %in% c("closs", "gloss", "qloss")){ 
            z <- gradient(family=rfamily, u=y*RET$fitted.values, s=s)
            scores <- abs(crossprod(x, w*(y*z)))/(penfac*alpha) ### note y=1/-1, thus can be further simplified without y
        }
    }
### initiate predictive value
    start <- NULL
    if(is.null(fk) || is.null(lambda)){
        if(type.init %in% c("ncl", "heu")){ ### use ncl function to generate intercept-only model
            RET <- ncl_fit(x=matrix(1, ncol=1, nrow=length(y)), y=y, iter=10000, reltol=1e-20, weights=weights, s=s, rfamily=rfamily, trace=FALSE)
            if(type.init=="ncl") start <- c(coef(RET), rep(0, nvars))
### it is similar to the following optimization results
                                        #fn <- function(b) sum(loss(y, f=b, cost, family = rfamily, s=s))
                                        #res <- optim(0, fn, method="SANN")
                                        #RET$fitted.values <- res$par
                                        #RET$h <- compute.h(rfamily, y, RET$fitted.values, s, B) 
### end of intercept computing
### check KKT condition for intercept only model, the following value should be close to 0
### if(rfamily=="gloss")
### mean(w*s*2^s*y*exp(y*coef(RET))*(exp(y*coef(RET))+1)^(-s-1)
            else if(type.init=="heu"){ ### heuristic for choosing starting values
                v <- zscore(rfamily, RET, s)
                ix <- which(v >= quantile(v, 0.9))
                b0.1 <- coef(RET)
                beta.1 <- rep(0, nvars)
                beta.1[ix] <- 1
                start <- c(b0.1, beta.1)
                RET$fitted.values <- x %*% beta.1 + b0.1
                RET$h <- compute.h(rfamily, y, RET$fitted.values, s, B) 
            }
        }
        else if(type.init=="bst") ### use bst function to generate results
        {
	    RET <- bst(x, y, family=rfamily, ctrl = bst_control(mstop=mstop.init, nu=nu.init, s=s, intercept=TRUE))
            RET$fitted.values <- RET$yhat
            RET$h <- compute.h(rfamily, y, RET$fitted.values, s, B) 
	    start <- c(attributes(coef(RET))$intercept, coef(RET))
        }
    }
    else {
        RET <- NULL
        RET$fitted.values <- fk
    }
    cost <- 0.5 # the optimization problem doesn't use cost argument, but loss function is from bst package
    los <- 2*loss(y, f=RET$fitted.values, cost, family = rfamily, s=s, fk=RET$fitted.values)
    if(is.null(lambda)){
        h <- RET$h                               
### If standardize=TRUE, x has already been standardized. Therefore, in the sequel, we don't standardize x again
### method A, to obtain lambda values from fitting the penalized regression. 
        lambda <- glmreg_fit(x=x*sqrt(B), y=h*sqrt(B), weights=weights, offset=offset, lambda.min.ratio=lambda.min.ratio, nlambda=nlambda, alpha=alpha,gamma=gamma, rescale=FALSE, standardize=FALSE, intercept=intercept, penalty.factor = penalty.factor, maxit=1, eps=eps, family="gaussian", penalty=penalty)$lambda
### with type.init, add two different lambda sequences and make lambda values flexible to have different solution paths
        #if(type.init %in% c("bst", "heu")){
            if(!decreasing)#{### solution path backward direction
                lambda <- rev(lambda)
        #}
                                        #cat("lambda in method A", lambda[1], "\n")
### method B is the same as method A to obtain lmax 
                                        #w <- weights/sum(weights)
                                        #        h1 <- h*sqrt(B)
                                        #        x1 <- x*sqrt(B)
                                        #	lmax <- max(abs(crossprod(x1,w*(h1-mean(h1))))/(penfac*alpha))
                                        #        cat("lambda in method B", lmax, "\n")
### method C considers the KKT conditions for the original nonconvex loss
                                        #assuming    RET <- ncl(y~1, data=data.frame(y, 1), iter=10000, del=1e-20, weights=weights, s=s, rfamily=rfamily, trace=FALSE)
					#if(rfamily=="clossR"){
                                        #z <- gradient(family=rfamily, u=y-RET$fitted.values, s=s)
                                        #lmax <- max(abs(crossprod(x, w*z))/(penfac*alpha))
                                        # } 
                                        #else if(rfamily %in% c("closs", "gloss", "qloss")){ 
                                        #    z <- gradient(family=rfamily, u=y*RET$fitted.values, s=s)
                                        #    lmax <- max(abs(crossprod(x, w*(y*z)))/(penfac*alpha)) ### note y=1/-1, thus can be further simplified without y
                                        #}
					#cat("lambda in method C", lmax, "\n")
### method D is the same as method C, but with the expressed gradients formula
                                        #assuming    RET <- ncl(y~1, data=data.frame(y, 1), iter=10000, del=1e-20, weights=weights, s=s, rfamily=rfamily, trace=FALSE)
					#b0 <- coef(RET)
					#if(rfamily=="clossR")
					#lmax <- max(abs(crossprod(x, w/s^2*(y-b0)*exp(-(y-b0)^2/(2*s^2)))/(penfac*alpha)))
                                        # if(rfamily=="closs"){
                                        #    cval <- 1/(1 - exp(-1/(2*s^2)))
                                        #    lmax <- max(abs(crossprod(x, w*cval/s^2*y*(y*coef(RET)-1)*exp(-(1-y*coef(RET))^2/(2*s^2)))/(penfac*alpha)))
                                        #}
                                        # else if(rfamily=="gloss")
                                        #    lmax <- max(abs(crossprod(x, w*s*2^s*y*exp(y*coef(RET))*(exp(y*coef(RET))+1)^(-s-1))/(penfac*alpha)))
                                        # else if(rfamily=="qloss")
                                        #       lmax <- max(abs(crossprod(x, w*sqrt(2/pi)/s*y*exp(-coef(RET)^2/(2*s^2)))/(penfac*alpha)))
					#cat("lambda in method D", lmax, "\n")
                                        #        lpath <- seq(log(lmax), log(lambda.min.ratio * lmax), length.out=nlambda)
                                        #        lambda <- exp(lpath)
    }
    else
        nlambda <- length(lambda)
    beta <- matrix(0, ncol=nlambda, nrow=m)
    b0 <- rep(0, nlambda)
    ### initialize start values -begin
    fk_old <- RET$fitted.values
    h <- compute.h(rfamily, y, fk_old, s, B)
    if(any(is.nan(h))){
      cat('h has NaN, fk_old used to compute h at beginning', fk_old)
      h <- y
    }
    tmp <- init(w, h, offset, family="gaussian")
    mustart <- tmp$mu
    etastart <- tmp$eta
    ### initialize the start values -end
    stopit <- FALSE
### for each element of the lambda sequence, iterate until convergency
    typeA <- function(beta, b0){
        i <- 1
        los <- pll <- matrix(NA, nrow=iter, ncol=nlambda)
        while(i <= nlambda){
            if(trace) message("loop in lambda:", i, "\n")
            if(trace) {
                cat("Quadratic majorization iterations ...\n")
            }
            k <- 1
            d1 <- 10
            while(d1 > reltol && k <= iter){
                fk_old <- RET$fitted.values
                h <- compute.h(rfamily, y, fk_old, s, B)
                if(any(is.nan(h))){ # exit loop
                    stopit <- TRUE
                    break
                }
		RET <- glmreg_fit(x=x*sqrt(B), y=h*sqrt(B), weights=weights, offset=offset, lambda=lambda[i],alpha=alpha,gamma=gamma, rescale=FALSE, standardize=FALSE, intercept=intercept, penalty.factor = penalty.factor, maxit=maxit, eps=eps, family="gaussian", penalty=penalty, start=start)
		RET$b0 <- RET$b0/sqrt(B)
### for LASSO, the above two lines are equivalent to the next line
                                        #RET <- glmreg_fit(x=x, y=h, weights=weights, lambda=lambda[i]/B,alpha=alpha,gamma=gamma, rescale=FALSE, standardize=FALSE, penalty.factor = penalty.factor, maxit=maxit, eps=eps, family="gaussian", penalty=penalty)
                fk <- RET$fitted.values <- predict(RET, newx=x)
                start <- coef(RET)
                los[k, i] <- 2*mean(loss(y, f=fk, cost, family = rfamily, s=s, fk=NULL))
###penalized loss value for beta
                penval <- .Fortran("penGLM",
                                   start=as.double(RET$beta),
                                   m=as.integer(m),
                                   lambda=as.double(lambda[i]*penalty.factor),
                                   alpha=as.double(alpha),
                                   gam=as.double(gamma),
                                   penalty=as.integer(pentype),
                                   pen=as.double(0.0),
                                   PACKAGE="mpath")$pen
                if(standardize)  ### lambda value, hence penval, depends on whether standardize is TRUE/FALSE
                    pll[k, i] <- los[k, i] + n*penval
                else pll[k, i] <- los[k, i] + penval
                d1 <- sum((fk_old - fk)^2)
                                        #d1 <- sum((fk_old - fk)^2)/sum(fk_old^2) ### this can cause a problem if fk_old is zero
                if(trace) cat("\n  iteration", k, ": relative change of fk", d1, ", robust loss value", los[k, i], ", penalized loss value", pll[k, i], "\n")
                if(trace) cat("  d1=", d1, ", k=", k, ", d1 > reltol && k <= iter: ", (d1 > reltol && k <= iter), "\n")
                k <- k + 1
            }
            if(!stopit){
                beta[,i] <- as.vector(RET$beta)
                b0[i] <- RET$b0
                i <- i + 1
            }
            else i <- nlambda + 1
        }
        list(beta=beta, b0=b0, RET=RET, risk=los, pll=pll)
    }
### only for direction="bwd", cycle through only on active set. for each element of the lambda sequence, iterate until convergency
    typeB <- function(beta, b0){
	i <- 1
	x.act <- x ### current active set
	start.act <- start
	m.act <- dim(x.act)[2]
	penalty.factor.act <- penalty.factor
        los <- pll <- matrix(NA, nrow=iter, ncol=nlambda)
        while(i <= nlambda){
            if(trace) message("loop in lambda:", i, ", lambda=", lambda[i], "\n")
            if(trace) {
                cat("Quadratic majorization iterations ...\n")
            }
            k <- 1
            d1 <- 10
            while(d1 > reltol && k <= iter){
                fk_old <- RET$fitted.values
                h <- compute.h(rfamily, y, fk_old, s, B)
	    	if(any(is.nan(h))){ # exit loop 
                   cat('h has NaN, in typeB fk_old used to compute h', fk_old)
	    		stopit <- TRUE
                    break
                }
               RET <- .Fortran("glmreg_fit_fortran",
	                        x=as.double(x.act), 
				y=as.double(h), 
				weights=as.double(weights),
			        n=as.integer(n),
			        m=as.integer(m.act),	
				start=as.double(start.act), 
				etastart=as.double(etastart),
	                        mustart=as.double(mustart),
			       	offset=as.double(offset),
			       	nlambda=as.integer(1),
			       	lambda=as.double(lambda[i]/B), 
				alpha=as.double(alpha),        
			        gam=as.double(gamma), 
				rescale=as.integer(0),
			       	standardize=as.integer(0),
			       	intercept=as.integer(intercept),
				penaltyfactor=as.double(penalty.factor.act),
				thresh=as.double(thresh), 
				epsbino=as.double(0), 
				maxit=as.integer(maxit),
			        eps=as.double(eps), 
				theta=as.double(0),
			       	family=as.integer(1), 
				penalty=as.integer(pentype), 
				trace=as.integer(trace),
				beta=as.double(matrix(0, ncol=1, nrow=m)),
				b0=as.double(0),
				yhat=as.double(rep(0, n)),
                satu=as.integer(0),
				PACKAGE="mpath")
     # epsbino, theta are not used in the above Fortran call with family="gaussian"
#		RET <- glmreg_fit(x=x.act*sqrt(B), y=h*sqrt(B), weights=weights, offset=offset, lambda=lambda[i],alpha=alpha,gamma=gamma, rescale=FALSE, standardize=FALSE, penalty.factor = penalty.factor.act, maxit=maxit, eps=eps, family="gaussian", penalty=penalty, start=start.act)
#		RET$b0 <- RET$b0/sqrt(B)  ### this line should be removed after calling glmreg_fit_fortran
### for LASSO, the above two lines are equivalent to the next line if and only if standardize=FALSE
                                        #RET <- glmreg_fit(x=x, y=h, weights=weights, lambda=lambda[i]/B,alpha=alpha,gamma=gamma, rescale=FALSE, standardize=FALSE, penalty.factor = penalty.factor, maxit=maxit, eps=eps, family="gaussian", penalty=penalty)
#                fk <- RET$fitted.values <- predETt(RET, newx=x.act)
#                start.act <- coef(RET)
                fk <- RET$fitted.values <- RET$yhat
                start.act <- c(RET$b0, RET$beta)
                etastart <- RET$yhat
		mustart <- RET$mustart
	  	los[k, i] <- 2*mean(loss(y, f=fk, cost, family = rfamily, s=s, fk=NULL))
###penalized loss value for beta
if(any(is.na(RET$beta))){
                cat('rfamily=', rfamily, 's=', s, 'B=', B, "\n")
		cat('fk_old', fk_old, "\n")
	        cat("h=", h, "\n")
	        cat("start.act=", start.act, "\n")
	        cat("mustart=", mustart, "\n")
	        cat("etastart=", etastart, "\n")
}
                penval <- .Fortran("penGLM",
                                   start=as.double(RET$beta),
                                   m=as.integer(m.act),
                                   lambda=as.double(lambda[i]/B*penalty.factor.act),
                                   alpha=as.double(alpha),
                                   gam=as.double(gamma),
                                   penalty=as.integer(pentype),
                                   pen=as.double(0.0),
                                   PACKAGE="mpath")$pen
     		if(standardize)  ### lambda value, hence penval, depends on whether standardize is TRUE/FALSE
                    pll[k, i] <- los[k, i] + n*penval
                else pll[k, i] <- los[k, i] + penval
                d1 <- sum((fk_old - fk)^2)
                                        #d1 <- sum((fk_old - fk)^2)/sum(fk_old^2) ### this can cause a problem if fk_old is zero
                if(trace) cat("\n  iteration", k, ": relative change of fk", d1, ", robust loss value", los[k, i], ", penalized loss value", pll[k, i], "\n")
                if(trace) cat("  d1=", d1, ", k=", k, ", d1 > reltol && k <= iter: ", (d1 > reltol && k <= iter), "\n")
                k <- k + 1
            }
            if(!stopit){
                tmp <- as.vector(RET$beta)
                activeset <- which(abs(tmp) > 0)
                if(length(activeset)==0)    break
                beta[activeset,i] <- tmp[activeset]
		start.act <- start.act[which(abs(start.act) > 0)]
		x.act <- x[, activeset] #update active set
		if(length(activeset)==1)
                    x.act <- matrix(x.act, ncol=1)
		m.act <- length(activeset) #update active set
	        penalty.factor.act <- penalty.factor[activeset]
                                        # beta[,i] <- as.vector(RET$beta)
                b0[i] <- RET$b0
      		i <- i + 1
            }
            else i <- nlambda + 1
        }
        list(beta=beta, b0=b0, RET=RET, risk=los, pll=pll)
    }
#typeBB was converted from typeB, works for both "bwd" and "fwd".
### these number conversion should be consistent to those in Fortran subroutines src/compute_h and loss
    rfamilytype <- switch(rfamily,       
                    "clossR"=11,
                    "closs"=12,
                    "gloss"=13,
                    "qloss"=14
                    )
    typeBB <- function(beta, b0){
        if(type.path=="active" && decreasing)
	    RET <- .Fortran("nclreg_ad",
			    x=as.double(x), 
			    y=as.double(y),
			    weights=as.double(weights),
			    n=as.integer(n),
			    m=as.integer(m),
			    start=as.double(start), 
			    etastart=as.double(etastart),
			    mustart=as.double(mustart),
			    offset=as.double(offset),
			    iter=as.integer(iter),
			    nlambda=as.integer(nlambda), 
			    lambda=as.double(lambda), 
			    alpha=as.double(alpha),
		        gam=as.double(gamma), 
			    standardize=as.integer(0), 
			    intercept=as.integer(intercept), 
			    penaltyfactor=as.double(penalty.factor),
			    maxit=as.integer(maxit), 
			    eps=as.double(eps), 
			    epscycle=as.double(epscycle), 
			    penalty=as.integer(pentype), 
			    trace=as.integer(trace), 
			    del=as.double(reltol), 
			    rfamily=as.integer(rfamilytype),
			    B=as.double(B), 
			    s=as.double(s),
			    thresh=as.double(thresh),
			    #cost=as.double(cost),
			    decreasing=as.integer(decreasing),
			    beta=as.double(beta),
		        b0=as.double(b0), 
			    yhat=as.double(RET$fitted.values), 
			    los=as.double(rep(0, nlambda)),
			    pll=as.double(rep(0, nlambda)),
			    nlambdacal=as.integer(0),
			    PACKAGE="mpath")
    else
	    RET <- .Fortran("nclreg_fortran",
			    x=as.double(x), 
			    y=as.double(y),
			    weights=as.double(weights),
			    n=as.integer(n),
			    m=as.integer(m),
			    start=as.double(start), 
			    etastart=as.double(etastart),
			    mustart=as.double(mustart),
			    offset=as.double(offset),
			    iter=as.integer(iter),
			    nlambda=as.integer(nlambda), 
			    lambda=as.double(lambda), 
			    alpha=as.double(alpha),
		        gam=as.double(gamma), 
			    standardize=as.integer(0), 
			    intercept=as.integer(intercept), 
			    penaltyfactor=as.double(penalty.factor),
			    maxit=as.integer(maxit), 
			    eps=as.double(eps), 
			    epscycle=as.double(epscycle), 
			    penalty=as.integer(pentype), 
			    trace=as.integer(trace), 
			    del=as.double(reltol), 
			    rfamily=as.integer(rfamilytype),
			    B=as.double(B), 
			    s=as.double(s),
			    thresh=as.double(thresh),
			    #cost=as.double(cost),
			    decreasing=as.integer(decreasing),
			    active=as.integer(active),
			    beta=as.double(beta),
		        b0=as.double(b0), 
			    yhat=as.double(RET$fitted.values), 
			    los=as.double(rep(0, nlambda)),
			    pll=as.double(rep(0, nlambda)),
			    nlambdacal=as.integer(0),
			    PACKAGE="mpath")
       	    list(beta=matrix(RET$beta, ncol=nlambda), b0=RET$b0, RET=RET, risk=RET$los, pll=RET$pll, nlambdacal=RET$nlambdacal)
    }
### update for one element of lambda depending on increasing sequence of lambda (last element of lambda) or decreasing sequence of lambda (then first element of lambda) in each MM iteration, and iterate until convergency of prediction. Then fit a solution path based on the sequence of lambda. Experiment code. Argument direction has been removed.
    typeC <- function(beta, b0){
        if(trace) {
            cat("Quadratic majorization iterations ...\n")
        }
        los <- pll <- rep(NA, nlambda)
        k <- 1
        fk <- RET$fitted.values
        d1 <- 10
        lam <- lambda[ifelse(decreasing, nlambda, 1)]
	while(d1 > reltol && k <= iter){
	    fk_old <- fk
            h <- compute.h(rfamily, y, fk_old, s, B)
	    RET <- glmreg_fit(x=x*sqrt(B), y=h*sqrt(B), weights=weights, offset=offset, lambda=lam, alpha=alpha,gamma=gamma, rescale=FALSE, standardize=FALSE, intercept=intercept, penalty.factor = penalty.factor, maxit=maxit, eps=eps, family="gaussian", penalty=penalty, start=start)
            RET$b0 <- RET$b0/sqrt(B)
	    fk <- predict(RET, newx=x)
            start <- coef(RET)
            d1 <- sum((fk_old - fk)^2)
            if(trace) cat("\n  iteration", k, ": relative change of fk", d1, "\n")
            if(trace) cat("  d1=", d1, ", k=", k, ", d1 > reltol && k <= iter: ", (d1 > reltol && k <= iter), "\n")
            k <- k + 1
        }
### fit a solution path    
        RET <- glmreg_fit(x=x*sqrt(B), y=h*sqrt(B), weights=weights, offset=offset, lambda=lambda, alpha=alpha,gamma=gamma, rescale=FALSE, standardize=FALSE, intercept=intercept, penalty.factor = penalty.factor, maxit=maxit, eps=eps, family="gaussian", penalty=penalty, start=start)
        RET$b0 <- RET$b0/sqrt(B)
        for(i in 1:nlambda){
            los[i] <- 2*weighted.mean(loss(y, f=RET$fitted.values[,i], cost, family = rfamily, s=s, fk=NULL), w=w)
###penalized loss value for beta
            penval <- .Fortran("penGLM",
                               start=as.double(RET$beta),
                               m=as.integer(m),
                               lambda=as.double(lambda[i]*penalty.factor),
                               alpha=as.double(alpha),
                               gam=as.double(gamma),
                               penalty=as.integer(pentype),
                               pen=as.double(0.0),
                               PACKAGE="mpath")$pen
            if(standardize)  ### lambda value, hence penval, depends on whether standardize is TRUE/FALSE
                pll[i] <- los[i] + n*penval
            else pll[i] <- los[i] + penval
	}
        beta <- RET$beta
        b0 <- RET$b0
        list(beta=beta, b0=b0, RET=RET, risk=los, pll=pll)
    }
#    if(type.path=="active") tmp <- typeB(beta, b0)
# typeA and typeB are combined into typeBB
    if(type.path=="active" || type.path=="nonactive") tmp <- typeBB(beta, b0)
#    if(type.path=="active") tmp <- typeBB(beta, b0)
#    else if(type.path=="nonactive") tmp <- typeA(beta, b0)
    else if(type.path=="onestep") tmp <- typeC(beta, b0)
    beta <- tmp$beta
    b0 <- tmp$b0
    RET <- tmp$RET
    RET$risk <- tmp$risk
    if(standardize) {
        beta <- beta/normx
        b0 <- b0 - crossprod(meanx, beta)
    }
    if(is.null(colnames(x))) varnames <- paste("V", 1:ncol(x), sep="")
    else varnames <- colnames(x)
    dimnames(beta) <- list(varnames, round(lambda, digits=4))
    RET$beta <- beta
    RET$b0 <- matrix(b0, nrow=1)
    RET$x <- xold
    RET$y <- y
    RET$call <- call
    RET$lambda <- lambda
    RET$nlambda <- nlambda
    RET$penalty <- penalty
    RET$s <- s
    RET$risk <- tmp$risk
    RET$pll <- tmp$pll
    RET$nlambdacal <- tmp$nlambdacal
    RET$rfamily <- RET$family <- rfamily
    RET$type.init <- type.init
    RET$mstop.init <- mstop.init
    RET$nu.init <- nu.init
    RET$decreasing <- decreasing
    RET$type.path <- type.path
    RET$is.offset <- is.offset
    class(RET) <- "nclreg"
    RET
}

predict.nclreg <- function(object, newdata=NULL, newy=NULL, newoffset=NULL, which=1:object$nlambda, type=c("response","class","loss", "error", "coefficients", "nonzero"), na.action=na.pass, ...){
    type=match.arg(type)
    if(is.null(newdata)){
        if(!match(type,c("coefficients", "nonzero"),FALSE))stop("You need to supply a value for 'newdata'")
        ynow <- object$y
    }
    else{
        if(!is.null(object$terms)){
            mf <- model.frame(delete.response(object$terms), newdata, na.action = na.action, xlev = object$xlevels)
            ynow <- model.frame(object$terms, newdata)[,1] ### extract response variable
            newdata <- model.matrix(delete.response(object$terms), mf, contrasts = object$contrasts)
            if(!is.null(ynow) && !is.null(newy))
                warnings("response y is used from newdata, but newy is also provided. Check if newdata contains the same y as newy\n")
        }
        else ynow <- newy
    }
    if(type=="coefficients") return(coef.glmreg(object)[,which])
    if(type=="nonzero"){
        nbeta <- object$beta[,which]
        if(length(which)>1) return(eval(parse(text="glmnet:::nonzeroCoef(nbeta[,,drop=FALSE],bystep=TRUE)")))
        else return(which(abs(nbeta) > 0))
    }
    if(is.null(newdata))
        newx <- as.data.frame(object$x)
    else newx <- as.data.frame(newdata)
    if(dim(newx)[2]==dim(object$beta)[1]) ### add intercept
        newx <- cbind(1, newx)
    if(object$is.offset)
     if(is.null(newoffset))
       stop("offset is used in the estimation but not provided in prediction\n")
     else offset <- newoffset
   else offset <- rep(0, length(ynow))
    res <- offset + as.matrix(newx) %*% rbind(object$b0, object$beta)
    if(type=="response") return(res[, which])
    if(type %in% c("loss", "error") && is.null(ynow)) stop("response variable y missing\n")
    if(type=="loss"){
        tmp <- rep(NA, length(which))
        for(i in 1:length(which))
            tmp[i] <- mean(loss(ynow, res[,which[i]], family = object$family, s=object$s))
        return(tmp)
    }
    if(type=="error"){
        tmp <- rep(NA, length(which))
        for(i in 1:length(which))
            tmp[i] <- evalerr(object$family, ynow, res[,which[i]])
        return(tmp)
    }
}


coef.nclreg <- function(object, ...)
    coef.glmreg(object)

### standardize predictor variables to have mean 0 and mean value of sum of squares = 1
stan <- function(x, weights){
    w <- weights/sum(weights)
    xx <- x
    meanx <- drop(w %*% x)
    x <- scale(x, meanx, FALSE) # centers x
    x <- sqrt(w) * x
    one <- rep(1, dim(x)[1])
    normx <- sqrt(drop(one %*% (x^2)))
    x <- scale(xx, meanx, normx)
    list(x=x, meanx=meanx, normx=normx) 
}

C     used in zipath.R
C     inputs: family: 3 (poisson), 4 (negbin)
C             theta
C     outputs: coefc, coefz, thetaout
      subroutine zipath_nonactive(x, z, y, y1, weights, n, kx, kz, 
     +     start_count, start_zero, mustart_count, mustart_zero, 
     +     offsetx, offsetz, intercept, nlambda, lambda_count,
     +     lambda_zero, alpha_count, alpha_zero,  
     +     gam_count, gam_zero, penaltyfactor_count, 
     +     penaltyfactor_zero, maxit, eps, family,
     +     penalty, trace, coefc, coefz, yhat, iter,
     +     del, rescale, thresh, epsbino, 
     +     theta_fixed, maxit_theta, theta, thetaout)
      implicit none
      integer n,i,ii,j,kx, kz, penalty,nlambda,family, 
     +     maxit, y1(n), trace, iter, intercept,
     +     rescale, jk_count, jk_zero, stopit,m_count_act, maxit_theta,
     +     m_zero_act, satu, varsel_count(kx), varsel_count_old(kx),
     +     varsel_zero(kz), varsel_zero_old(kz), theta_fixed
      double precision x(n, kx), z(n, kz), weights(n), 
     +     start_count(kx+1), dpois, dnbinom, start_zero(kz+1), 
     +     mustart_count(n), mustart_zero(n), offsetx(n), offsetz(n), 
     +     lambda_count(nlambda), thetaout(nlambda),
     +     lambda_zero(nlambda), alpha_count, alpha_zero, gam_count, 
     +     gam_zero, eps, penaltyfactor_count(kx), y(n),
     +     penaltyfactor_zero(kz), probi(n), thresh, epsbino, 
     +     theta, coefc(kx+1, nlambda), coefz(kz+1, nlambda), b0_x, b0z,
     +     yhat(n), del, start_count_act(kx+1), start_zero_act(kz+1), 
     +     x_act(n, kx), z_act(n, kz), betax(kx), betaz(kz),
     +     penaltyfactor_count_act(kx), penaltyfactor_zero_act(kz)
      external :: dpois, dnbinom, gfunc

      stopit = 0
      m_count_act = kx
      m_zero_act = kz
      jk_count = kx
      jk_zero = kz

      do ii=1, n
            if(y1(ii) .EQ. 1)then
               probi(ii)=0
            else
              probi(ii)=mustart_zero(ii) 
              if(family .EQ. 3)then
              probi(ii)=probi(ii)/(probi(ii)+(1-probi(ii))*dpois(0,
     +                  mustart_count(ii), 0))
              else if(family .EQ. 4)then
              probi(ii)=probi(ii)/(probi(ii)+(1-probi(ii))*dnbinom(0,
     +                  theta, mustart_count(ii), 0))
              endif
            endif
      enddo
      call copymatrix(n, kx, x, x_act)
      call copymatrix(n, kz, z, z_act)

      do 5 j=1, kx
         betax(j)=0
 5    continue
      do 105 j=1, kz
         betaz(j)=0
 105    continue
      call dcopy(kx+1, start_count, 1, start_count_act, 1)
      call dcopy(kz+1, start_zero, 1, start_zero_act, 1)
      call dcopy(kx, penaltyfactor_count, 1, penaltyfactor_count_act, 1)
      call dcopy(kz, penaltyfactor_zero, 1, penaltyfactor_zero_act, 1)
      do 101 j=1, kx
         varsel_count_old(j)=j
         varsel_count(j)=j
 101  continue
      do 102 j=1, kz
         varsel_zero_old(j)=j
         varsel_zero(j)=j
 102  continue

      i=1
 10   if(i .LE. nlambda)then
        if(trace .EQ. 1)then
            call intpr("Fortran lambda iteration i=", -1, i, 1)
        endif
        call zi_onelambda(x_act, z_act, y, y1, probi, weights, n, 
     +           m_count_act, m_zero_act, start_count_act, 
     +           start_zero_act, mustart_count, mustart_zero, offsetx, 
     +           offsetz, intercept, lambda_count(i), lambda_zero(i), 
     +           alpha_count, alpha_zero, gam_count, gam_zero, 
     +           penaltyfactor_count_act, penaltyfactor_zero_act,
     +           maxit, eps, family, penalty, trace, yhat, iter, del,
     +           rescale, thresh, epsbino, theta_fixed,
     +           maxit_theta, theta, betax, b0_x, betaz, b0z, satu)
         coefc(1, i) = b0_x
         if(jk_count .GT. 0)then
            do 200 ii = 1, m_count_act
               coefc(1+varsel_count(ii), i) = betax(ii)
 200        continue
         endif
         thetaout(i)=theta
         coefz(1, i) = b0z
         if(satu==1 .AND. i==1)then
             do ii=1, m_zero_act
             start_zero_act(ii)=0
             enddo
         else
         if(jk_zero .GT. 0)then
            do 210 ii = 1, m_zero_act
               coefz(1+varsel_zero(ii), i) = betaz(ii)
 210        continue
         endif
         endif
         i = i + 1
         goto 10
      endif

      return
      end

gfunc <- function(mu, family, epsbino){
    n <- length(mu)
     .Fortran("gfunc",
              mu=as.double(mu),
              n=as.integer(n),
              family=as.integer(family),
              epsbino=as.double(epsbino),
              eta=as.double(rep(0, n)),
              PACKAGE="mpath")$eta
}

C     used in zipath.R
C     inputs: family: 3 (poisson), 4 (negbin)
C     theta
C     kx: number of variables of x having no intercept column
C     kz: number of variables of z having no intercept column
C     outputs: coefc, coefz, theta, thetaout
      subroutine zipath_active(x, z, y, y1, weights, n, kx, kz, 
     +     start_count, start_zero, mustart_count, mustart_zero, 
     +     offsetx, offsetz, intercept, nlambda, lambda_count,
     +     lambda_zero, alpha_count, alpha_zero,  
     +     gam_count, gam_zero, penaltyfactor_count, 
     +     penaltyfactor_zero, maxit, eps, family,
     +     penalty, trace, coefc, coefz, yhat, iter,
     +     del, rescale, thresh, epsbino, 
     +     theta_fixed, maxit_theta, theta, thetaout)
      implicit none
      integer n,i,ii,j,jj,kx, kz, penalty,nlambda,family, 
     +     maxit, y1(n), trace, iter, intercept, 
     +     rescale, jk_count, jk_zero, activeset_count(kx), 
     +     activeset_count_old(kx), activeset_zero(kz),
     +     activeset_zero_old(kz), m_count_act, maxit_theta,
     +     m_zero_act, AllocateStatus, jc, jz, 
     +     nact, conv, theta_fixed, fakec, fakez, satu
      double precision x(n, kx), z(n, kz), weights(n), 
     +     start_count(kx+1), dpois, dnbinom, b0_xall, b0zall, 
     +     start_zero(kz+1),
     +     mustart_count(n), mustart_zero(n), offsetx(n), offsetz(n), 
     +     lambda_count(nlambda), thetaout(nlambda),
     +     lambda_zero(nlambda), alpha_count, alpha_zero, gam_count, 
     +     gam_zero, eps, penaltyfactor_count(kx), y(n),
     +     penaltyfactor_zero(kz), probi(n), thresh, epsbino, 
     +     theta, thetaall, coefc(kx+1, nlambda), coefz(kz+1, nlambda),
     +     b0_x, b0z, yhat(n), del, betaxall(kx), betazall(kz)
      double precision, dimension(:, :), allocatable :: x_act, z_act
      double precision, dimension(:), allocatable :: start_count_act,
     +     start_zero_act, betax, betaz,
     +     penaltyfactor_count_act, penaltyfactor_zero_act
      external :: dpois, dnbinom, gfunc
      
      if(kx==0 .OR. kz==0)then
         return
      endif
      do ii=1, kx
         betaxall(ii)=0
         activeset_count(ii)=ii
         activeset_count_old(ii)=ii
      enddo
      do ii=1, kz
         betazall(ii)=0
         activeset_zero(ii)=ii
         activeset_zero_old(ii)=ii
      enddo
C     find current active set
      call find_activeset(kx, start_count(2: (kx+1)), eps, 
     +     activeset_count, jk_count)
      call find_activeset(kz, start_zero(2: (kz+1)), eps, 
     +     activeset_zero, jk_zero)
C     When all coef are zero except intercept, choose a predictor
      fakec=0
      fakez=0
      jz=0
      jc=0
      if(jk_count==0)then
         jk_count = 1
         activeset_count(1)=kx
         fakec=1
      endif
      if(jk_zero==0)then
         jk_zero = 1
         activeset_zero(1)=kz
         fakez=1
      endif
      do ii=1, jk_count
         activeset_count_old(ii)=activeset_count(ii)
      enddo
      do ii=1, jk_zero
         activeset_zero_old(ii)=activeset_zero(ii)
      enddo
      
      m_count_act = jk_count
      m_zero_act = jk_zero

      do ii=1, n
         if(y1(ii) .EQ. 1)then
            probi(ii)=0
         else
            probi(ii)=mustart_zero(ii) 
            if(family .EQ. 3)then
               probi(ii)=probi(ii)/(probi(ii)+(1-probi(ii))*dpois(0,
     +              mustart_count(ii), 0))
            else if(family .EQ. 4)then
               probi(ii)=probi(ii)/(probi(ii)+(1-probi(ii))*dnbinom(0,
     +              theta, mustart_count(ii), 0))
            endif
         endif
      enddo
      allocate(start_count_act(jk_count+1), stat=AllocateStatus)
      allocate(penaltyfactor_count_act(jk_count),stat=AllocateStatus)
      allocate(betax(jk_count), stat=AllocateStatus)
      allocate(x_act(n, jk_count), stat=AllocateStatus)
      start_count_act(1)=start_count(1)
      do jj=1, n
         do ii=1, jk_count
            x_act(jj, ii)=x(jj, activeset_count(ii))
         enddo
      enddo
      do 5 j=1, jk_count
         betax(j)=0
         start_count_act(j+1)=start_count(1+activeset_count(j))
         penaltyfactor_count_act(j)=
     +        penaltyfactor_count(activeset_count(j))
 5    continue
      
      allocate(start_zero_act(jk_zero+1), stat=AllocateStatus)
      allocate(penaltyfactor_zero_act(jk_zero), stat=AllocateStatus)
      allocate(betaz(jk_zero), stat=AllocateStatus)
      allocate(z_act(n, jk_zero), stat=AllocateStatus)
      start_zero_act(1)=start_zero(1)
      do jj=1, n
         do ii=1, jk_zero
            z_act(jj, ii)=z(jj, activeset_zero(ii))
         enddo
      enddo
      do 105 j=1, jk_zero
         betaz(j)=0
         start_zero_act(j+1)=start_zero(1+activeset_zero(j))
         penaltyfactor_zero_act(j)=
     +        penaltyfactor_zero(activeset_zero(j))
 105  continue
      nact = 2
      i=1
 10   if(i .LE. nlambda)then
         if(trace .EQ. 1)then
C            i is not array
C            call intpr("Fortran lambda iteration i=", -1, i, 1), 
            call intpr("kx=", -1, kx, 1)
            call intpr("kz=", -1, kz, 1)
            call intpr("m_count_act", -1, m_count_act, 1)
            call intpr("m_zero_act", -1, m_zero_act, 1)
            call dblepr("start_count_act", -1, start_count_act,
     +           m_count_act+1)
            call dblepr("start_zero_act", -1, start_zero_act,
     +           m_zero_act+1)
            call dblepr("betax", -1, betax, m_count_act)
            call dblepr("betaz", -1, betaz, m_zero_act)
         endif
         j = 1
         conv=0
13000    if(j <= nact .AND. conv==0)then
            if(trace==1)then
               call intpr("active set iteration, j=", -1, j, 1)
               call intpr("cycling through only active sets", -1, 1, 1)
            endif
            call zi_onelambda(x_act, z_act, y, y1, probi, weights, n, 
     +           m_count_act, m_zero_act, start_count_act, 
     +           start_zero_act, mustart_count, mustart_zero, offsetx, 
     +           offsetz, intercept, lambda_count(i), lambda_zero(i), 
     +           alpha_count, alpha_zero, gam_count, gam_zero, 
     +           penaltyfactor_count_act, penaltyfactor_zero_act,
     +           maxit, eps, family, penalty, trace, yhat, iter, del,
     +           rescale, thresh, epsbino, theta_fixed, maxit_theta, 
     +           theta, betax, b0_x, betaz, b0z, satu)
C     update start_count with start_count_act, start_zero with
C     start_zero_act
            start_count(1)=b0_x
            do ii=1, jk_count
               start_count(activeset_count(ii)+1)=betax(ii)
            enddo
            start_zero(1)=b0z
            if(satu==1 .AND. i==1)then
               do ii=1, jk_zero
                  start_zero_act(1+ii)=0
                  betaz(ii)=0
               enddo
            endif
            do ii=1, jk_zero
               start_zero(activeset_zero(ii)+1)=betaz(ii)
            enddo
            if(j .NE. nact)then
               thetaall = theta
               call zi_onelambda(x, z, y, y1, probi, weights, n, kx,
     +              kz, start_count, start_zero, mustart_count, 
     +              mustart_zero, offsetx, offsetz, intercept, 
     +              lambda_count(i), lambda_zero(i), alpha_count, 
     +              alpha_zero, gam_count,gam_zero,penaltyfactor_count, 
     +              penaltyfactor_zero, maxit, eps, family, penalty, 
     +              trace, yhat, 2, del, rescale, thresh, epsbino,
     +              theta_fixed, maxit_theta, thetaall, betaxall, 
     +              b0_xall, betazall, b0zall, satu)
               if(satu==1 .AND. i==1)then
                  do ii=1, kz
                     start_zero(1+ii)=0
                     betazall(ii)=0
                  enddo
               endif
               call find_activeset(kx, betaxall, eps, activeset_count
     +              , jk_count)
               if(jk_count==0)then
                  jk_count = 1
                  activeset_count(1)=kx
                  fakec=1
               endif
               call find_activeset(kz, betazall, eps, activeset_zero
     +              , jk_zero)
               if(jk_zero==0)then
                  jk_zero = 1
                  activeset_zero(1)=kz
                  fakez=1
               endif
            endif
C     check if converged here!
C            if(trace==1)then
C               call intpr("activeset_count",-1,activeset_count,jk_count)
C               call intpr("activeset_count_old",
C     +              -1,activeset_count_old,m_count_act)
C               call intpr("activeset_zero", -1, activeset_zero, jk_zero)
C               call intpr("activeset_zero_old", -1,activeset_zero_old,
C     +              m_zero_act)
C               call intpr("jk_count=", -1, jk_count, 1)
C               call intpr("m_count_act=", -1, m_count_act, 1)
C               call intpr("jk_zero=", -1, jk_zero, 1)
C               call intpr("m_zero_act=", -1, m_zero_act, 1)
C            endif
            if(jk_count .NE. m_count_act)then
               conv=0
            else if(jk_zero .NE. m_zero_act)then
               conv=0
            else
               jc=0
               do ii=1, max(jk_count, m_count_act)
                  if(activeset_count(ii)==activeset_count_old(ii))then
                     jc=jc+1
                  endif
               enddo
               jz=0
C     what if activeset_zero==activeset_zero_old after only one iteration as
C     in the above? Say, we only have betaz=0.2 with one element, but
C     jk_zero=kz now. Then we have a problem here to update the values as
C     stated below! Because in this case, the iteration stops.
               do ii=1, max(jk_zero, m_zero_act)
                  if(activeset_zero(ii)==activeset_zero_old(ii))then
                     jz=jz+1
                  endif
               enddo
               if(jk_count==jc .AND. jk_zero==jz)then
                  conv=1
               endif
            endif
            if(jk_count .NE. m_count_act .OR. jk_count .NE. jc)then
               theta = thetaall
               deallocate(betax, start_count_act, 
     +              penaltyfactor_count_act, x_act)
               allocate(betax(jk_count), stat=AllocateStatus)
               allocate(start_count_act(jk_count+1),stat=AllocateStatus)
               allocate(penaltyfactor_count_act(jk_count),stat=
     +              AllocateStatus)
               start_count_act(1) = b0_xall
               do 11135 ii=1, jk_count
                  betax(ii)=0
                  start_count_act(ii+1)=betaxall(activeset_count(ii))
                  activeset_count_old(ii)=activeset_count(ii)
                  penaltyfactor_count_act(ii)=
     +                 penaltyfactor_count(activeset_count(ii))
11135          continue
               allocate(x_act(n, jk_count), stat=AllocateStatus)
C     update x_act matrix
               do 11155 jj=1, n
                  do 11145 ii=1, jk_count
                     x_act(jj, ii) = x(jj, activeset_count(ii))
11145             continue
11155          continue
               m_count_act = jk_count
            else
               if(fakec==1)then
                  start_count_act(1)=b0_xall
                  start_count_act(2)=betaxall(1)
               endif
            endif
            if(jk_zero .NE. m_zero_act .OR. jk_zero .NE. jz)then
               deallocate(betaz, start_zero_act, 
     +              penaltyfactor_zero_act, z_act)
               allocate(betaz(jk_zero), stat=AllocateStatus)
               allocate(start_zero_act(jk_zero+1),stat=AllocateStatus)
               allocate(penaltyfactor_zero_act(jk_zero),stat=
     +              AllocateStatus)
               start_zero_act(1) = b0zall
               do 12135 ii=1, jk_zero
                  betaz(ii)=0
                  start_zero_act(ii+1)=betazall(activeset_zero(ii))
                  activeset_zero_old(ii)=activeset_zero(ii)
                  penaltyfactor_zero_act(ii)=
     +                 penaltyfactor_zero(activeset_zero(ii))
12135          continue
               allocate(z_act(n, jk_zero), stat=AllocateStatus)
               do 12155 jj=1, n
                  do 12145 ii=1, jk_zero
                     z_act(jj, ii) = z(jj, activeset_zero(ii))
12145             continue
12155          continue
               m_zero_act = jk_zero
            else
               if(fakez==1)then
                  start_zero_act(1)=b0zall
                  start_zero_act(2)=betazall(1)
               endif
            endif
            j=j+1
            goto 13000
         endif
         coefc(1, i) = b0_x
         if(jk_count .GT. 0)then
            do 200 ii = 1, jk_count
               coefc(1+activeset_count(ii), i) = betax(ii)
 200        continue
         endif
         thetaout(i)=theta
         coefz(1, i) = b0z
         if(jk_zero .GT. 0)then
            do 210 ii = 1, jk_zero
               coefz(1+activeset_zero(ii), i) = betaz(ii)
 210        continue
         endif
         i = i + 1
         goto 10
      endif
      deallocate(betax, start_count_act, x_act, 
     +     penaltyfactor_count_act)
      deallocate(betaz, start_zero_act, z_act, 
     +     penaltyfactor_zero_act)

      return
      end

### Obtained from the now orphaned R package pamr version 1.56.2
### Author: T. Hastie, R. Tibshirani, Balasubramanian Narasimhan, Gil Chu

balanced.folds <- function(y, nfolds = min(min(table(y)), 10)) {
   totals <- table(y)
   fmax <- max(totals)
   nfolds <- min(nfolds, fmax)     
   nfolds= max(nfolds, 2)
         # makes no sense to have more folds than the max class size
   folds <- as.list(seq(nfolds))
   yids <- split(seq(y), y) 
         # nice we to get the ids in a list, split by class
###Make a big matrix, with enough rows to get in all the folds per class
   bigmat <- matrix(NA, ceiling(fmax/nfolds) * nfolds, length(totals))
   for(i in seq(totals)) {
cat(i)
     if(length(yids[[i]])>1){bigmat[seq(totals[i]), i] <- sample(yids[[i]])}
     if(length(yids[[i]])==1){bigmat[seq(totals[i]), i] <- yids[[i]]}

   }
   smallmat <- matrix(bigmat, nrow = nfolds)# reshape the matrix
### Now do a clever sort to mix up the NAs
   smallmat <- permute.rows(t(smallmat))   ### Now a clever unlisting
         # the "clever" unlist doesn't work when there are no NAs
         #       apply(smallmat, 2, function(x)
         #        x[!is.na(x)])
   res <-vector("list", nfolds)
   for(j in 1:nfolds) {
     jj <- !is.na(smallmat[, j])
     res[[j]] <- smallmat[jj, j]
   }
   return(res)
 }

C ###compute updated weights for composite loss function with dfun=poisson and negbin
C update weights. cf subroutine update_wt for other dfun
C output: weights_update
      subroutine compute_wt3(n,y,mu,weights,theta,cfun,family,s,delta,
     +       weights_update)
       implicit none
       integer cfun, family, n, i
       double precision weights(n),weights_update(n),y(n),mu(n),z(n),
     + v(n), theta,s,delta
       external compute_v, loglikFor
 
       do i=1, n
       call loglikFor(1,y(i),mu(i),theta,1.0D0,family,z(i))
C convert MLE of log-likelihood to minimization of negative log-likelihood
       z(i)=-z(i)
       enddo
C     compute derivative of -g(z)
             call compute_v(cfun, n, z, s, delta, v)
C     update weights
             do 10 i=1, n
              weights_update(i)=-weights(i)*v(i)
10         continue
      return
      end

C ###compute composite loss values for dfun=poisson and negbin
C input: family=3,4. May work for 1 but not for 2 since y in irglmreg is +1/-1 not
C 1/0)
C ### output: z is negative log-likelihood value at the current
C parameter
C , i.e., s(u) value 
C ### output: los is the sum of g(s(u)) values
      subroutine loss3(n,y,mu,theta,weights,cfun,family,s,delta,los)
      implicit none
        integer cfun, family, n, i
        double precision weights(n),y(n),mu(n),z,gval,theta,s,delta,los
        double precision z_saturate, dfun_val
        external compute_g, loglikFor
 
       los=0.0D0
       do i=1, n
C ### z is log-likelihood value at the current
        call loglikFor(1,y(i),mu(i),theta,1.0D0,family,z)
C ### z_saturate is log-likelihood value in the saturate model
        call loglikFor(1,y(i),y(i),theta,1.0D0,family,z_saturate)
C  minimization problem, dfun s(x) must be convex, thus take negative value
C  of z, but -z can be negative, thus add z_saturate such that dfun_val
C  >=0
        dfun_val = z_saturate - z
        call compute_g(cfun,1,dfun_val,s,delta,gval)
        los=los+weights(i)*gval
       enddo

       return
       end

      subroutine preprocess(x,y,n,m,weights,family,standardize,normx,xd,
     +     mu)
      implicit none
      integer i,j,n,m, standardize,family
      double precision x(n,m),y(n),weights(n),normx(m), xd(m), 
     +     wtnew(n),meanx(m),mu,wsum,ddot,xtmp(n,m)

      do i=1, n
         do j=1,m
            xtmp(i,j) = x(i,j)
         enddo
      enddo
      wsum=0      
C     compute weighted means sum(weights_i*y_i)
      mu = ddot(n, y, 1, weights, 1)
      do 20 i=1, n
         wsum = wsum + weights(i)
 20   continue
      mu = mu/wsum
      
C###  Center x and y, and scale x, and save the means and sds 
C     for now, except for gaussian family, ignore weights
      do 30 i=1, n
         wtnew(i) = weights(i)/wsum
 30   continue
C     compute weighted column averages meanx = x^(transpose) * wtnew

      call DGEMV('T',n, m, 1.0D0, x, n, wtnew, 1, 0.0D0, meanx, 1)

      if(standardize.EQ.1)then
         do 70 j=1, m
            do 60 i=1, n
               x(i,j) = x(i,j) - meanx(j)
C     deal the weights later
               x(i,j) = dsqrt(wtnew(i)) * x(i,j)
 60         continue
 70      continue
         if(family .EQ. 1)then
            do 80 i=1, n
               y(i) = y(i) - mu
               y(i)= dsqrt(wtnew(i)) * y(i)
 80         continue
         endif
         do 110 j=1, m
            normx(j)=0
            xd(j) =1
            do 100 i=1, n
               normx(j) = normx(j) + x(i,j) * x(i,j)
 100        continue
            normx(j) = dsqrt(normx(j))
 110     continue
      else
         do 120 j=1, m
            xd(j) = 0
            normx(j) = 1
            do 125 i=1, n
               xd(j) = xd(j) + weights(i) * x(i,j) * x(i,j)
 125        continue
 120     continue
      endif
      if(standardize.EQ.1 .AND. family .EQ. 1)then
         do 140 j=1, m
            do 130 i=1, n
               x(i,j) = x(i,j)/normx(j)
 130        continue
 140     continue
      endif
      if(standardize .EQ. 1 .AND. family .NE. 1)then
         do 150 j=1, m
            do 160 i=1, n
               x(i,j) = (xtmp(i,j)-meanx(j))/normx(j)
 160        continue
 150     continue
      endif
      
      return
      end

C     #coordinate descent algorithm
C     #ref: Regularization paths for generalized linear models via coordinate descent, Friedman et al., JSS, 2010, 33(1)
C     output:
C     beta
C     b0
C     mu
C     yhat
C     jj
C     input: others except the above mentioned output
      subroutine lmnetGaus(x, y, n, m, weights, lambda, alpha, gam, 
     + thresh, maxit, eps, standardize, intercept, penalty, 
     + xd,beta, b0, avg, jj, rescale, converged, activeset, jk, fullset)

      implicit none
      integer maxit, standardize, intercept, penalty, n,m,i,j,jj,
     +     converged,jkold,
     +     rescale,fullset(m),k,activeset(m),activesetold(m),convact,jk 
      double precision x(n, m), y(n), weights(n), lambda(m),
     +     meanx(m), xd(m), resid(n),
     +     alpha, gam, thresh, eps, avg, wsum, mu(n),
     +     wtnew(n), b0, beta(m), ddot

      do i=1, n
         resid(i) = y(i)
      enddo
      if(standardize .EQ. 1)then
         b0 = 0
      else 
         b0 = avg
      endif
      if(intercept .EQ. 0)then
          b0 = 0
      endif
      wsum=0
C     compute weighted means sum(weights_i*y_i)
      mu = ddot(n, y, 1, weights, 1)
      do 20 i=1, n
         wsum = wsum + weights(i)
 20   continue
      do 30 i=1, n
         wtnew(i) = weights(i)/wsum
 30   continue
C     compute weighted column averages meanx = x^(transpose) * wtnew
      call DGEMV('T',n, m, 1.0D0, x, n, wtnew, 1, 0.0D0, meanx, 1)
      convact=0
C     Active Set Convergence:After a cycle through all the variables, we can restrict 
C     further iterations to the active set till convergence + one more cycle through all
C     the variables to check if active set has changed. If no change, we are done, 
C     otherwise, the process if repeated.
C     some loop, if no change to the active set, stop
      k = 1
C     max.cycle: in case of nonconvergence the program looks for a cycle
C     that repeats itself, default is 2 (k <= 2)      
      do 2000 while (k <= 2 .AND. convact == 0)
C     set maxit=1, and have a complete cycle through all the variables
         call loop_gaussian(x,y,n,m,penalty,thresh,eps,1,
     +        standardize, intercept,
     +        beta,b0,resid,xd,lambda,alpha,gam,weights,avg,meanx, 
     +        jj,rescale, converged, fullset, m)
         call find_activeset(m, beta, eps, activesetold, jkold)
C     it is possible, jk=0 if beta=0, like intercept-only model
         if(jkold .EQ. 0)then
            convact=1
            exit
         endif
C     restrict further iterations to the active set till convergence
         call loop_gaussian(x,y,n,m,penalty,thresh,eps,maxit,
     +        standardize, intercept,
     +        beta,b0,resid,xd,lambda,alpha,gam,weights,avg,meanx, 
     +        jj,rescale, converged, activesetold, jkold)
C     set maxit=1, and have a complete cycle through all the variables
         call loop_gaussian(x,y,n,m,penalty,thresh,eps,1,
     +        standardize, intercept,
     +        beta,b0,resid,xd,lambda,alpha,gam,weights,avg,meanx, 
     +        jj,rescale, converged, fullset, m)
         call find_activeset(m, beta, eps, activeset, jk)
C     check if the active set was changed--begin
C     it is possible, jk=0 if beta=0, like intercept-only model
         if(jk .EQ. 0)then
            convact=1
            exit
         endif
         if(jkold .NE. jk)then
            convact=0
         else
            do 90 j=1, jk
               if(activesetold(j) .NE. activeset(j))then
                  convact=0
                  exit
               else 
                  if(j .EQ. jk)then
                     convact = 1
                     exit
                  endif
               endif 
 90         continue
         endif
C     check if the active set was changed--end
         k = k + 1
 2000 continue
C     used to match outside loop in subroutine midloop
      jj = jj - 1

      return
      end

!     coordinate descent algorithm

      subroutine enet(z, t, lone, ltwo, res)
      implicit none
      double precision z, t, lone, ltwo, res

      call soth(z, lone, res)
      res= res/(t + ltwo)
      return
      end
      
      subroutine mcp(z, t, lone, ltwo, gam, rescale, res)
      implicit none
      integer rescale
      double precision z, t, v, lone, ltwo, gam, res

      if(rescale .EQ. 1) then
         v = 1
      else
         v = t
      endif
      if(dabs(z) .LE. v*gam * lone * (1+ltwo)) then
         call soth(z, lone, res)                
         if(rescale .EQ. 1) then
            res =res/(t*(1 + ltwo - 1/gam))
         else 
            res =res/(t + ltwo - 1/gam)
         endif
      else 
         if(rescale .EQ. 1) then
            res = z/(t*(1 + ltwo))
         else 
            res = z/(t + ltwo)
         endif
      endif
      return
      end

      subroutine scad(z, t, lone, ltwo, gam, rescale, res)
      implicit none
      integer rescale
      double precision z, t, lone, ltwo, gam, res
      
      if(rescale .EQ. 1) then
         if(dabs(z) .LE. lone   + lone * (1+ltwo)) then
            call soth(z, lone, res)                
C     if(rescale .EQ. 1) then
            res =res/(t*(1 + ltwo))
C     else 
C     res =res/(t + ltwo)
C     endif
         else if(dabs(z) .LE. gam * lone * (1+ltwo)) then
            call soth(z, gam*lone/(gam-1), res)
            res=res/(t*(1-1/(gam-1)+ltwo))
         else 
            res=z/(t*(1+ltwo))
         endif     
      else
         if(dabs(z) .LE. lone   + lone * (t+ltwo)) then
            call soth(z, lone, res)                
            res =res/(t + ltwo)
         else if(dabs(z) .LE. gam * lone * (t+ltwo)) then
            call soth(z, gam*lone/(gam-1), res)
            res=res/(t-1/(gam-1)+ltwo)
         else 
            res=z/(t+ltwo)
         endif
      endif     
      return
      end   

!     input variables
!     x
!     y
!     n
!     m
!     eps
!     maxit
!     threshold
!     standardize
!     beta
!     b0
!     xd
!     lambda
!     alpha
!     gam
!     jk
!     
!     output variables
!     beta
!     jj       


      subroutine loop_gaussian(x, y, n, m, penalty, thresh, eps, maxit, 
     +    standardize, intercept, beta, b0, resid,xd, lambda, alpha, 
     +    gam, wtold, avg, meanx, jj, rescale, converged, activeset, jk)
      implicit none
      integer n, m, maxit,standardize,intercept,jj,i,j,penalty,converged
      integer rescale, activeset(m), jk, ii
      double precision x(n, m), y(n), thresh, eps,beta(m),beta_old(m) 
      double precision lambda(m), alpha, gam, wtold(n), avg, meanx(m)
      double precision z, b0, xd(m), yhat(n), resid(n),b0_old
      
      jj = 1
      converged = 0
 1000 if(jj .LE. maxit .AND. converged .EQ.0)then
         do 100 ii = 1, jk
            j = activeset(ii)
            beta_old(j) = beta(j)
 100     continue
         b0_old = b0
         do 10 i = 1, n
            yhat(i) = b0
            do 20 ii = 1, jk
               j = activeset(ii)
               yhat(i) = yhat(i) + x(i,j) * beta(j)
 20         continue         
 10      continue
         do 30 i = 1, n
            resid(i) = y(i) - yhat(i)
 30      continue
!     When cycling through the variables, we could restrict ourselves to the current 
!     active set and visit all the variables e.g. every 10th iteration to update 
!     the active set.
         do 40 ii = 1, jk
            j = activeset(ii)
            z = 0.0D0
            if(standardize.EQ.1)then
               do 50 i = 1, n
                  z = z + x(i,j) * resid(i)
 50            continue 
               z = z + beta(j)    
            else 
               do 60 i = 1, n
                  z= z + wtold(i)*x(i,j) * (resid(i) + x(i,j)*beta(j)) 
 60            continue
            endif
            if(penalty.EQ.1) then
               call enet(z, xd(j), lambda(j)*alpha,lambda(j)*(1-alpha),
     +              beta(j))
            else if(penalty.EQ.2) then
               call mcp(z, xd(j), lambda(j)*alpha, lambda(j)*(1-alpha),
     +              gam, rescale, beta(j))
            else if(penalty.EQ.3) then
               call scad(z, xd(j), lambda(j)*alpha, 
     +              lambda(j)*(1-alpha), gam, rescale, beta(j))
            endif
            if(dabs(beta(j) - beta_old(j)) .GT. eps)then
               do 70 i = 1, n
                  resid(i) = resid(i) - x(i, j) * (beta(j)-beta_old(j))
 70            continue
            endif
 40      continue
         if(intercept == 1)then
          if(standardize.EQ.0)then
            b0 = 0.0D0
            do 80 ii = 1, jk
               j = activeset(ii)
               b0 = b0 + meanx(j) * beta(j)
 80         continue
            b0 = avg - b0
          endif
         endif
         call checkConvergence(m, beta, beta_old, eps, thresh, 
     +        converged, activeset, jk)
         jj = jj + 1
         goto 1000
      endif

      return
      end

!     soft-threshold=g operator
      subroutine soth(z, g, res)
      implicit none
      double precision res, z, g
      
      if(z .GT. g)then
         res=z-g
      else if (dabs(z) .LE. g) then
         res=0
      else if (z .LT. -g) then
         res=z+g
      endif
      return
      end

      subroutine checkConvergence(m, beta, betaold,eps,thresh,converged,
     +     activeset, jk)
      implicit none
      integer m, j, jk, ii, converged, activeset(m)
      double precision beta(m), betaold(m), eps, thresh

      converged=1
      do 10 ii=1, jk
         j = activeset(ii)
C     20: only check active set      
         if(dabs(beta(j)) .GT. eps .AND. dabs(betaold(j)) .GT. eps) then
            if(dabs((beta(j)-betaold(j))/betaold(j)) .GT. thresh) then
               converged=0
               exit
            endif
         else if(dabs(beta(j)) .LE. eps .AND. dabs(betaold(j)) .GT. eps)
     +           then
            converged=0
            exit
         else if(dabs(beta(j)) .GT. eps .AND. dabs(betaold(j)) .LE. eps)
     +           then
            converged=0
            exit
         endif
C     20: only check active set      
 10   continue

      return
      end

convexMin <- function(beta,X,penalty,gamma,l2,family, theta)
{
  n <- nrow(X)
  p <- ncol(X)
  l <- ncol(beta)
  
  if (penalty=="mnet") k <- 1/gamma
  else if (penalty=="snet") k <- 1/(gamma-1)
  else if (penalty=="enet") return(NULL)
  
  val <- NULL
  for (i in 1:l) {
    if (i==1) A1 <- rep(1,p)
    else A1 <- beta[-1,i]==0
    if (i==l) {
      L2 <- l2[i]
      U <- A1
    } else {
      if (is.na(beta[1,i+1])) break
      A2 <- beta[-1,i+1]==0
      U <- A1&A2
      L2 <- l2[i+1]
    }
    if (sum(!U)==0) next
    Xu <- X[,!U]
    if (family=="gaussian") {
      if (any(A1!=A2)) {
        cmin <- min(eigen(crossprod(Xu)/n)$values)
      }
      eigen.min <- cmin - k + L2
    }
    if (family=="binomial") {
      if (i==l) eta <- beta[1,i] + X%*%beta[-1,i]
      else eta <- beta[1,i+1] + X%*%beta[-1,i+1]
      pi. <- exp(eta)/(1+exp(eta))
      w <- as.numeric(pi.*(1-pi.))
      w[eta > log(.9999/.0001)] <- .0001
      w[eta < log(.0001/.9999)] <- .0001
      Xu <- sqrt(w) * cbind(1,Xu)
      xwxn <- crossprod(Xu)/n
      eigen.min <- min(eigen(xwxn-diag(c(0,diag(xwxn)[-1]*(k-L2))))$values)
    }
    if (family=="poisson" || family=="negbin") {
      if (i==l) eta <- beta[1,i] + X%*%beta[-1,i]
      else eta <- beta[1,i+1] + X%*%beta[-1,i+1]
      mu<- exp(eta)
      if(family=="poisson")
       w <- mu
      else w <- mu/(1+1/theta*mu)
      w <- as.numeric(w)
      Xu <- sqrt(w) * cbind(1,Xu)
      xwxn <- crossprod(Xu)/n
      eigen.min <- min(eigen(xwxn-diag(c(0,diag(xwxn)[-1]*(k-L2))))$values)
    }

    if (eigen.min < 0) {
      val <- i
      break
    }
  }
  val
}

C     used in irglmreg_fortran subroutine for one single lambda penalty
C     parameter (adapted from nclreg_onelambda.f)
C     input: start_act, etastart, mustart
C     output: start_act, etastart, mustart, beta_1, b0_1, fk
      subroutine irglmreg_onelambda(x_act,y,weights,n,m_act,start_act, 
     +     etastart, mustart, yhat, offset, lambda_i,alpha,gam,rescale,
     +     standardize,intercept,penaltyfactor_act,maxit,eps,theta,
     +   penalty,trace,iter,del,cfun,dfun,s,thresh,beta_1,b0_1,fk,delta,
     +   weights_update)
      implicit none
      integer n,k,j,intercept,penalty,maxit,trace,iter,m_act,
     +        satu, cfun, dfun, dfunnew, i, ii, standardize, rescale
      double precision y(n), weights(n),etastart(n), mustart(n), 
     +     offset(n),lambda_i,alpha,gam,eps,
     +     thresh, b0_1, yhat(n), d, d1, del, fk_old(n), s, 
     +     fk(n), x_act(n, m_act), start_act(m_act+1),
     +     beta_1(m_act), penaltyfactor_act(m_act),
     +     weights_update(n), delta, ytmp(n), theta
      external update_wt, loss2, penGLM

      if(dfun==5)then
         do i=1, n
           ytmp(i)=(y(i)+1.0D0)/2.0D0
         enddo
       else 
         do i=1, n
           ytmp(i)=y(i)
         enddo
      endif
        k = 1
         d = 10
            do 10 ii=1, n
            weights_update(ii)=weights(ii)
 10         continue
 500     if(d .GT. del .AND. k .LE. iter)then
            if(trace .EQ. 1)then
               call intpr("  irglmreg_onelambda iteration k=", -1, k, 1)
               call dblepr("     start_act", -1, start_act, m_act+1)
            endif
            call dcopy(n, yhat, 1, fk_old, 1)
CCCCCCCCCCCCCCCCCCC BEGIN
C instead of updating h values in nclreg_onelambda, update
C weights: call compute_h(rfamily, n, y, fk_old, s, B, h)
C     compute u
C            call compute_u(dfun, n, y, fk_old, u)
C     compute z=s(u)
C            call compute_z(dfun, n, u, z, s)
C     compute derivative of -g(z)
C            call compute_v(cfun, n, z, s, delta, v)
C     update weights
C            do 10 ii=1, n
C            weights_update(ii)=weights(ii)*v(ii)
C 10         continue
CCCCCCCCCCCCCCCCCCC END
C     check if h has NAN value, can be useful to debug
C            if(trace .EQ. 1)then
C            do 30 ii=1, n
C               if(h(ii) .NE. h(ii))then
C                  call intpr("  irglmreg_onelambda iteration k=",-1,k,1)
C                  call intpr("    ii=", -1, ii, 1)
C                  call dblepr("     h(ii)", -1, h(ii), 1)
C                  call dblepr("     fk_old(ii)", -1, fk_old(ii), 1)
C                  exit
C               endif
C 30         continue
C            endif
C    unlike nclreg_onelambda, we update weights not h (here is y); also we may have different family depending on the dfun function
            if(dfun .EQ. 1 .OR. dfun .EQ. 4)then
                dfunnew = 1
            else if(dfun .EQ. 5)then 
                dfunnew= 2
              else if(dfun .EQ. 8)then 
                dfunnew= 3
              else if(dfun .EQ. 9)then 
                dfunnew= 4
              else
                  call rexit("not implemented yet")
            endif
            call glmreg_fit_fortran(x_act, ytmp,weights_update,n,m_act,
     +           start_act,etastart,mustart, offset, 1, lambda_i, alpha,
     +           gam,rescale,standardize,intercept, penaltyfactor_act, 
     +           thresh, 0.0D0, maxit, eps, theta, dfunnew,
     +           penalty, trace, beta_1, b0_1, yhat, satu)
            if(dfun .EQ. 1 .OR. dfun .EQ. 4 .OR. dfun .EQ. 5)then
                call update_wt(n,weights,y,etastart,cfun,dfun,s,delta,
     +          weights_update)
            else if(dfun .EQ. 8 .OR. dfun .EQ. 9)then
             call compute_wt3(n,y,mustart,weights,theta,cfun,dfunnew,s,
     +        delta,weights_update)
            endif
            call dcopy(n, yhat, 1, fk, 1)
            call dcopy(n, yhat, 1, mustart, 1)
            if(dfun .NE. 1 .OR. dfun .NE. 4)then
              call dcopy(n, yhat, 1, etastart, 1)
            endif
C     this is not needed for
C     the above glmreg_fit call (that calls zeval) if family=1.
        if(dfun .EQ. 1 .OR. dfun .EQ. 4)then
            start_act(1) = b0_1
            if(m_act .GT. 0)then
               do 100 j=1, m_act
                  start_act(j+1)=beta_1(j)
 100           continue
            endif
        endif
           d = 0.0D0
            do 120 ii=1, n
               d=d+(fk_old(ii) - fk(ii))**2.0D0
  120       continue
           d1 = 0.0D0
            do 130 ii=1, n
               d1=d1+fk(ii)**2.0D0
  130       continue
               d=d/d1
            if(trace .EQ. 1)then
               call dblepr("beta_1", -1, beta_1, m_act)
            endif
            k = k + 1
            goto 500
         endif

      return
      end

### this is different from src/update_wt in that the inputs are different 
update_wt <- function(y, ypre, weights, cfun, s, dfun, delta=0.0001){
    if(dfun %in% 4:7 && !setequal(unique(y), c(-1, 1))) stop("y must be -1/1 for binary classification")
    n <- length(y)
    if(missing(weights)) weights <- rep(1, n)
    if(cfun==6)
        if(s > 1) delta <- (s-1)/2 else
                                       if(s==1) delta <- 0 else{
                                                               if(is.null(delta)) stop("delta must be provided")
                                                               if(delta <= 0) stop("delta must be positive")
                                                           }
    .Fortran("update_wt",
             n=as.integer(n),
             weights=as.double(weights),
             y=as.double(y),
             f=as.double(ypre),
             cfun=as.integer(cfun),
             dfun=as.integer(dfun),
             s=as.double(s),
             delta=as.double(delta),
             weights_update=as.double(rep(0, n)),
             PACKAGE="mpath")$weights_update
}

! Lemmon and Schafer: Developing Statistical Software in Fortran 95,
! Springer 2005, page 39. No argument checking or overflow checking
!-----Factorial------------------------------------------------------
!
!  Function to calculate factorials resursively
!
!---------------------------------------------------------------------
      RECURSIVE INTEGER FUNCTION Factorial(n)  RESULT(Fact)

      IMPLICIT NONE
      INTEGER, INTENT(IN) :: n

      IF (n == 0) THEN
         Fact = 1
      ELSE
         Fact = n * Factorial(n-1)
      END IF

      END FUNCTION Factorial

C### eta is the estimated beta_0 in the intercept-only model, derived
Cfrom ../R/init function
      subroutine init(n, wt, y, offset, family, mu, eta)
      implicit none
      integer i, n, family
      double precision wt(n), y(n), offset(n), mu1, mu(n), eta(n), ddot

C compute weighted means sum(wt_i * y_i)
      mu1 = ddot(n, y, 1, wt, 1)
      do 20 i= 1, n
           mu(i) = mu1 + offset(i)
 20   continue
      do 30 i= 1, n
      if(family.EQ.1)then
              eta(i)=mu(i)
        else if(family.EQ.2)then
              eta(i)=dlog(mu(i)/(1.0D0-mu(i)))
        else if(family.EQ.3 .OR. family.EQ.4)then
              eta(i)=dlog(DMAX1(1.0D0, mu(i)))
      endif
 30   continue
 
      return
      end

C     fit a zero inflated penalized regression for a single penalty
C     parameter. A building block used in zipath_fortran.f
C     inputs: family: 3 (poisson), 4 (negbin)
C     theta, start_count_act, start_zero_act
C     m_count_act: number of count model variables of x having no intercept column
C     m_zero_act: number of zero model variables of z having no intercept column
C     outputs: betax, b0_x, betaz, b0z, theta, start_count_act, start_zero_act
      subroutine zi_onelambda(x_act, z_act, y, y1, probi, weights, n, 
     +     m_count_act, m_zero_act, start_count_act, start_zero_act,
     +     mustart_count, mustart_zero, offsetx, offsetz,
     +     intercept,lambda_count,lambda_zero, alpha_count,alpha_zero,  
     +     gam_count, gam_zero, penaltyfactor_count_act, 
     +     penaltyfactor_zero_act, maxit, eps, family,
     +     penalty, trace, yhat, iter, del, rescale, thresh, 
     +     epsbino, theta_fixed, maxit_theta, theta, 
     +     betax, b0_x, betaz, b0z, satu)
      implicit none
      integer n,ii,k,j,penalty, family, maxit, y1(n), trace, iter, 
     +     rescale, stopit,m_count_act, maxit_theta, intercept,
     +     m_zero_act, theta_fixed, satu
      double precision weights(n), dpois, dnbinom, 
     +     etastart_count(n), etastart_zero(n),
     +     mustart_count(n), mustart_zero(n), offsetx(n), offsetz(n), 
     +     lambda_count, thetastart, lambda_zero, alpha_count, 
     +     alpha_zero, gam_count,  gam_zero, eps,
     +     penaltyfactor_count_act(m_count_act), y(n),
     +     penaltyfactor_zero_act(m_zero_act), wt(n), probi(n), thresh, 
     +     epsbino, theta, b0_x, b0z,yhat(n), d, del, theta0, penval,
     +     x_act(n, m_count_act), z_act(n, m_zero_act), pll_old, pll,
     +     start_count_act(m_count_act+1), start_zero_act(m_zero_act+1),
     +     betax(m_count_act), betaz(m_zero_act), los_old,los
      external :: dpois, dnbinom, gfunc, ziloss

      stopit = 0
      k = 1
      d = 10
      call gfunc(mustart_count, n, family,epsbino,etastart_count)
      call gfunc(mustart_zero, n, 2, 0.0D0, etastart_zero)
      call ziloss(n, y, offsetx, offsetz, weights, etastart_count,
     +     etastart_zero, family, theta, los_old)
      call penGLM(betax, m_count_act, 
     +     lambda_count*penaltyfactor_count_act, 
     +     alpha_count, gam_count, penalty, penval)
      pll_old=los_old - penval
      call penGLM(betaz, m_zero_act, 
     +     lambda_zero*penaltyfactor_zero_act, 
     +     alpha_zero, gam_zero, penalty, penval)
      pll_old=pll_old - penval
 500  if(d .GT. del .AND. k .LE. iter)then
         if(trace .EQ. 1)then
            call intpr("  EM algorithm iteration k=", -1, k, 1)
            call dblepr("     d=", -1, d, 1)
         endif
         do 30 ii=1, n
            wt(ii)=weights(ii)*(1-probi(ii)) 
 30      continue
         if(family .NE. 4 .OR. theta_fixed .EQ. 1)then
            call glmreg_fit_fortran(x_act,y,wt,n,m_count_act,
     +           start_count_act,etastart_count,mustart_count,
     +           offsetx,1, lambda_count, alpha_count, gam_count,
     +           rescale,0, intercept, penaltyfactor_count_act, thresh,
     +           epsbino, maxit, eps, theta, family,  
     +           penalty, 0, betax, b0_x, yhat, satu)
         else
            thetastart = theta 
            call glmregnb_fortran(x_act,y,wt,n,m_count_act,offsetx,
     +           1, lambda_count, penalty,alpha_count, gam_count, 
     +           rescale, 0, intercept, penaltyfactor_count_act, thresh,
     +           maxit_theta, maxit, eps, epsbino, start_count_act, 
     +           etastart_count, mustart_count, thetastart, 0, 
     +           theta0, trace, betax, b0_x, theta, yhat)
         endif
C     yhat: the fitted mean values, obtained by transforming the
C     linear predictors by the inverse of the link function.
C         call dcopy(n, yhat, 1, mustart_count, 1)
C     etastart_count: linear predictors, obtained by transforming the
C     mean values by the link function.
C         call gfunc(mustart_count, n, family,epsbino,etastart_count)
         start_count_act(1) = b0_x
         if(m_count_act .GT. 0)then
            do 100 j=1, m_count_act
               start_count_act(j+1)=betax(j)
 100        continue
         endif
         do ii=1, n
            yhat(ii)=0
         enddo
         call glmreg_fit_fortran(z_act,probi,weights,n,m_zero_act,
     +        start_zero_act, etastart_zero, mustart_zero,offsetz,
     +        1, lambda_zero, alpha_zero, gam_zero, rescale,
     +        0, intercept, penaltyfactor_zero_act, thresh,
     +        epsbino, maxit, eps, theta, 2,  
     +        penalty, 0, betaz, b0z, yhat, satu)
C         call dcopy(n, yhat, 1, mustart_zero, 1)
C         call gfunc(mustart_zero, n, 2, 0.0D0, etastart_zero)
         do ii=1, n
            if(y1(ii) .EQ. 0)then
               probi(ii)=mustart_zero(ii) 
               if(family .EQ. 3)then
                  probi(ii)=probi(ii)/(probi(ii)+(1-probi(ii))*dpois(0,
     +                 mustart_count(ii), 0))
               else if(family .EQ. 4)then
                  probi(ii)=probi(ii)/(probi(ii)+(1-probi(ii))*
     +                 dnbinom(0, theta, mustart_count(ii), 0))
               endif
            endif
         enddo
         
         start_zero_act(1) = b0z
         if(m_zero_act .GT. 0)then
            do 1100 j=1, m_zero_act
               start_zero_act(j+1)=betaz(j)
 1100       continue
         endif
         call ziloss(n, y, offsetx, offsetz, weights, etastart_count,
     +        etastart_zero, family, theta, los)
         call penGLM(betax, m_count_act, 
     +        lambda_count*penaltyfactor_count_act, 
     +        alpha_count, gam_count, penalty, penval)
         pll=los - penval
         call penGLM(betaz, m_zero_act, 
     +        lambda_zero*penaltyfactor_zero_act, 
     +        alpha_zero, gam_zero, penalty, penval)
         pll=pll - penval
         d=abs((pll-pll_old)/pll_old)
         pll_old = pll
         k = k + 1
         goto 500
      endif

      return
      end

C generate outer product between two vectors, similar to %o% in R
C input: m, x, n, y
C output: z
      subroutine outprod(m, x, n, y, z)
              implicit none
              integer m, n, i, j
              double precision x(m), y(n), z(m, n)

      do 10 i=1, m
         do 20 j=1, n
              z(i, j) = x(i)*y(j)
  20  continue
  10  continue
      
      return
      end

C compute value of g(z) as a function of z
C input: z, cfun, s
C output: gval
      subroutine compute_g(cfun, n, z, s, delta, gval)
              implicit none
              integer cfun, i, n
              double precision PI, s, s1, s2, z(n), delta, gval(n)

      if(cfun .EQ. 1 .OR. cfun .EQ. 3)then
          s1 = s**2/2
      else if(cfun .EQ. 2)then
          PI=4.D0*DATAN(1.D0)
          s1 = s**2*PI**2/2
      else if(cfun .EQ. 4)then
          s1 = s**2
      else if(cfun .EQ. 5)then
          s1 = dexp(-s)
      else if(cfun .EQ. 6)then
          s1 = delta**(s-1)/(1+delta)**(s+1)
          s2 = -1.0D0/s*(delta/(1+delta))**s + delta**s/(1+delta)**(s+1)
      else if(cfun .EQ. 8)then
          PI=4.D0*DATAN(1.D0)
          s1 = 2*dexp(-delta/s)/dsqrt(PI*s*delta)
          s2 = s1*delta - erf(dsqrt(delta/s))
      endif
     
      do 10 i=1, n
      if(cfun .EQ. 1)then
        if(z(i) .LE. s1)then
          gval(i)=z(i)
        else 
          gval(i)=s*(2*z(i))**0.5-s1
        endif
      else if(cfun .EQ. 2)then
        if(z(i) .LE. s1)then
          gval(i)=s**2*(1-dcos((2*z(i))**0.5/s))
        else 
          gval(i)=2*s**2
        endif
      else if(cfun .EQ. 3)then
           if(z(i) .LE. s1)then
               gval(i)=1-(1-2*z(i)/s**2)**3
               gval(i)=gval(i)*s**2/6.0
           else
               gval(i)=s**2/6.0
           endif
      else if(cfun .EQ. 4)then
        gval(i)=1-dexp(-z(i)/s1)
        gval(i)=gval(i)*s1
      else if(cfun .EQ. 5)then
        gval(i)=dlog((1+z(i))/(1+z(i)*s1))
        gval(i)=gval(i)/(1-s1)
      else if(cfun .EQ. 6)then
          if(z(i) > delta)then
            gval(i)=(z(i)/(1+z(i)))**s/s + s2
          else
            gval(i)=s1*z(i)
          endif
      else if(cfun .EQ. 7)then
            if(z(i) .LE. s)then
                gval(i)=z(i)
            else
                gval(i)=s
            endif
      else if(cfun .EQ. 8)then
          if(z(i) > delta)then
            gval(i)=erf(dsqrt(z(i)/s)) + s2
          else
            gval(i)=s1*z(i)
          endif
      endif
 10   continue
             
            return
            end 

error.bars <-
function(x, up, lo, width = 0.02, ...)
{
	xlim <- range(x)
	ci <- diff(xlim) * width
	segments(x, up, x, lo, ...)
	segments(x - ci, up, x + ci, up, ...)
	segments(x - ci, lo, x + ci, lo, ...)
	range(up, lo)
}

### log-likelihood function, except for gaussian family
logLik.glmreg <- function(object, newx, y, weights, offset, na.action=na.pass, ...){
if(missing(newx) || missing(y)) return(object$twologlik/2)
if(!is.null(object$terms)){
 mf <- model.frame(delete.response(object$terms), newx, na.action = na.action, xlev = object$xlevels)
 newx <- model.matrix(delete.response(object$terms), mf, contrasts = object$contrasts)
 newx <- newx[,-1] ### remove the intercept
}
family <- object$family
nlambda <- object$nlambda
beta <- object$beta
b0 <- object$b0
famtype <- switch(family,
      "gaussian"=1,
      "binomial"=2,
      "poisson"=3,
      "negbin"=4)
 nm <- dim(newx)
 nobs <- nm[1]
 nvars <- nm[2]
 if(is.matrix(beta)){
 if(dim(beta)[1] != nvars)
 stop("the dimension of coefficients beta and that of newx do not match\n")
}
if(missing(weights)) weights <- rep(1, nobs)
w <- weights
if(object$is.offset){
     if(is.null(offset))
       stop("offset is used in the estimation but not provided in prediction\n")
}else offset <- rep(0, nobs)
#if(missing(offset))
# offset <- object$offset
 res <- .Fortran("pred",
 n=as.integer(nobs),
 m=as.integer(nvars),
 nlambda = as.integer(nlambda),
 x=as.double(newx),
 b=as.double(beta),
 a0=as.double(b0),
 offset=as.double(offset),
 family=as.integer(famtype),
 eta=as.double(matrix(0, nobs, nlambda)),
 mu=as.double(matrix(0, nobs, nlambda)),
 PACKAGE="mpath")
 mu <- matrix(res$mu, ncol=nlambda) 
 th <- object$theta
 if(nlambda > 1 && length(object$theta) ==1)
 th <- rep(th, nlambda)
# res <- llfun(y, mu, w, th=th, family, nlambda)
 res <- rep(NA, nlambda)
 for(k in 1:nlambda) 
 res[k] <- .Fortran("loglikFor",
  n=as.integer(length(y)), 
  y=as.double(y), 
  mu=as.double(mu[,k]), 
  theta=as.double(th[k]), 
  w=as.double(w), 
  family=as.integer(famtype),
  ll=as.double(0),
  PACKAGE="mpath")$ll
 return(res)
}

llfun <- function(y, mu, w, th, family, nlambda){
 ll <- rep(NA, nlambda)
switch(family,
  "gaussian"={
    for(k in 1:nlambda)
    ll[k] <- -sum(w*(y-mu[,k])^2)
},
  "negbin"={
    for(k in 1:nlambda)
    ll[k] <- sum(w*(lgamma(th[k] + y) - lgamma(th[k]) - lgamma(y + 1) + th[k] * log(th[k]) +
           y * log(mu[,k] + (y == 0)) - (th[k] + y) * log(th[k] + mu[,k])))
},
  "binomial"={
    for(k in 1:nlambda){
     tmp <- rep(0, length(y))
    for(i in 1:length(y))
     if(is.na(mu[i,k])) cat("mu[i,k]=", mu[i,k], "\n")
     if(mu[i,k]>0 && mu[i,k] <1)
      tmp[i] <- w[i]*(y[i]*log(mu[i,k]) + (1-y[i])*log(1-mu[i,k]))
      ll[k] <- sum(tmp)
}
},
  "poisson"={
    for(k in 1:nlambda)
    ll[k] <- sum(w*(-mu[,k] + y*log(mu[,k])-lgamma(y+1)))
}
)
return(ll)
}

AIC.glmreg <- function(object, ..., k)
object$aic
BIC.glmreg <- function(object, ...)
object$bic
AIC.zipath <- function(object, ..., k)
object$aic
BIC.zipath <- function(object, ...)
object$bic

logLik.zipath <- function(object, newdata, X, Z, y, offsetx, offsetz, weights, na.action=na.pass, 
                   link = c("logit", "probit", "cloglog", "cauchit", "log"),
...){
if(missing(y) || missing(X) || missing(Z)) return(object$loglik)
linkstr <- match.arg(link)
  linkobj <- make.link(linkstr)
  linkinv <- linkobj$linkinv
Y <- y
## set up likelihood
  ziPoisson <- function(parms) {
  kx <- NCOL(X)
  kz <- NCOL(Z)
    Y0 <- Y <= 0
    Y1 <- Y > 0
#    offsetx <- offsetz <- 0
    ## count mean
    mu <- as.vector(exp(X %*% parms[1:kx] + offsetx))
    ## binary mean
    phi <- as.vector(linkinv(Z %*% parms[(kx+1):(kx+kz)] + offsetz))
    ## log-likelihood for y = 0 and y >= 1
    loglik0 <- log( phi + exp( log(1-phi) - mu ) ) ## -mu = dpois(0, lambda = mu, log = TRUE)
    loglik1 <- log(1-phi) + dpois(Y, lambda = mu, log = TRUE)
    ## collect and return
    loglik <- sum(weights[Y0] * loglik0[Y0]) + sum(weights[Y1] * loglik1[Y1])
    loglik
  }

  ziNegBin <- function(parms) {
  kx <- NCOL(X)
  kz <- NCOL(Z)
    Y0 <- Y <= 0
    Y1 <- Y > 0
#    offsetx <- offsetz <- 0
    ## count mean
    mu <- as.vector(exp(X %*% parms[1:kx] + offsetx))
    ## binary mean
    phi <- as.vector(linkinv(Z %*% parms[(kx+1):(kx+kz)] + offsetz))
    ## negbin size
    theta <- exp(parms[(kx+kz)+1])
    ## log-likelihood for y = 0 and y >= 1
    loglik0 <- log( phi + exp( log(1-phi) + suppressWarnings(dnbinom(0, size = theta, mu = mu, log = TRUE)) ) )
    loglik1 <- log(1-phi) + suppressWarnings(dnbinom(Y, size = theta, mu = mu, log = TRUE))

    ## collect and return
    loglik <- sum(weights[Y0] * loglik0[Y0]) + sum(weights[Y1] * loglik1[Y1])
    loglik
  }
  ziGeom <- function(parms) 
  ziNegBin(c(parms, 0))

family <- object$family
nlambda <- object$nlambda
  loglikfun <- switch(family,
                      "poisson" = ziPoisson,
                      "geometric" = ziGeom,
                      "negbin" = ziNegBin)

ll <- rep(NA, nlambda)
if(!is.null(object$terms)){
mf <- model.frame(delete.response(object$terms$full), newdata, na.action = na.action, xlev = object$levels)
    X <- model.matrix(delete.response(object$terms$count), mf, contrasts = object$contrasts$count)
    Z <- model.matrix(delete.response(object$terms$zero),  mf, contrasts = object$contrasts$zero)
}
else
    if(is.null(X) || is.null(Z))
        stop("X and Z must be provided\n")
    if(missing(weights)) weights <- rep(1, length(y))
w <- weights
if(family == "negbin")
parms <- rbind(object$coefficients$count, object$coefficients$zero, log(object$theta))
else parms <- rbind(object$coefficients$count, object$coefficients$zero)
 ll <- rep(NA, nlambda)
for(k in 1:nlambda)
ll[k] <- loglikfun(parms[,k])
ll
}

cv.glmregNB <- function(formula, data, weights, offset=NULL, 
lambda=NULL, nfolds=10, foldid, plot.it=TRUE, se=TRUE, n.cores=2, trace=FALSE, parallel=FALSE, 
...){
    call <- match.call()
 mf <- Call <- match.call()
                                     m <- match(c("formula", "data", "subset", "weights", "na.action",
                                                "etastart", "mustart", "offset"), names(mf), 0)
                                            mf <- mf[c(1, m)]
                                            mf$drop.unused.levels <- TRUE
                                            mf[[1L]] <- as.name("model.frame")
                                            mf <- eval.parent(mf)
                                            Terms <- attr(mf, "terms")
                                            Y <- model.response(mf, "numeric")
  ## null model support
                                            X <- if (!is.empty.model(Terms)) model.matrix(Terms, mf, contrasts) else matrix(,NROW(Y),0)
  nobs <- n <- length(Y)
  nvars <- m <- dim(X) - 1
 weights <- model.weights(mf)
 offset <- as.vector(model.offset(mf))
 if(!length(weights)) weights <- rep(1, nrow(mf))
  if(any(weights < 0)) stop("negative weights not allowed")

  K <- nfolds
      glmregNB.obj <- do.call("glmregNB", list(formula, data, weights, offset=offset, lambda=lambda, ...))
    lambda <- glmregNB.obj$lambda
    nlambda <- length(lambda)
    if(is.null(offset)) offset <- rep(0, nobs)
    if(missing(foldid))
    all.folds <- cv.folds(n, K)
    else all.folds <- foldid
    if(parallel){
    cl <- eval(parse(text="parallel:::makeCluster(n.cores)"))
    registerDoParallel(cl)
    i <- 1  ###needed to pass R CMD check with parallel code below
    residmat <- foreach(i=seq(K), .combine=cbind) %dopar% {
      omit <- all.folds[[i]]
### changed 5/20/2013 fixed theta
      if(!is.null(offset)){
         offsetnow <- offset[- omit]
         newoffset <- offset[omit]
         }else offsetnow <- newoffset <- NULL
      fitcv <- do.call("glmregNB", list(formula, data[-omit,], weights[-omit], offset=offsetnow, nlambda=nlambda, lambda=lambda, theta.est=FALSE, theta0=glmregNB.obj$theta, ...))
### remove the first column, which is for intercept
      fitcv$terms <- NULL ### logLik requires data frame if terms is not NULL
      logLik(fitcv, newx=X[omit,-1, drop=FALSE], Y[omit], weights=weights[omit], offset=newoffset)
   }
   eval(parse(text="parallel:::stopCluster(cl)"))
    }
    else{
          residmat <- matrix(NA, nlambda, K)
     for(i in seq(K)) {
       if(trace)
         cat("\n CV Fold", i, "\n\n")
       omit <- all.folds[[i]]
 ### changed 5/20/2013 fixed theta
      if(!is.null(offset)){
         offsetnow <- offset[- omit]
         newoffset <- offset[omit]
         }else offsetnow <- newoffset <- NULL
       fitcv <- do.call("glmregNB", list(formula, data[-omit,], weights[-omit], offset=offsetnow, nlambda=nlambda,  lambda=lambda, theta.est=FALSE, theta0=glmregNB.obj$theta, trace=trace, ...))
 ### remove the first column, which is for intercept
       fitcv$terms <- NULL ### logLik requires data frame if terms is not NULL
       residmat[, i] <- logLik(fitcv, newx=X[omit,-1, drop=FALSE], Y[omit], weights=weights[omit], offset=newoffset)
     }
    }
   cv <- apply(residmat, 1, mean)
    cv.error <- sqrt(apply(residmat, 1, var)/K)
    lambda.which <- which.max(cv)
    obj<-list(fit=glmregNB.obj, residmat=residmat, lambda = lambda, cv = cv, cv.error = cv.error, foldid=all.folds, lambda.which= lambda.which, lambda.optim = lambda[lambda.which])
    class(obj) <- "cv.glmreg"
    if(plot.it) plot(obj,se=se)
    obj
  }

plot.zipath=function(x, xvar=c("norm","lambda"),type=c("count", "zero"), label=FALSE,...){
  xvar=match.arg(xvar)
  type=match.arg(type)
 if(type=="count"){
 b <- x$coefficients$count[-1,]
 df <- apply(abs(b)>0, 2, sum)
 #plotCoef(b,lambda=x$lambda.count,df=df,label=label,xvar=xvar,...)
 eval(parse(text="glmnet:::plotCoef(b,lambda=x$lambda.count,df=df,label=label,xvar=xvar,...)"))
 } else{
 b <- x$coefficients$zero[-1,]
 df <- apply(abs(b)>0, 2, sum)
 #plotCoef(b,lambda=x$lambda.zero,df=df,label=label,xvar=xvar,...)
 eval(parse(text="glmnet:::plotCoef(b,lambda=x$lambda.zero,df=df,label=label,xvar=xvar,...)"))
}
}

cv.nclreg <- function(x, ...) UseMethod("cv.nclreg", x)

cv.nclreg.default <- function(x, ...) {
    if (extends(class(x), "Matrix"))
        return(cv.nclreg.matrix(x = x, ...))
    stop("no method for objects of class ", sQuote(class(x)),
         " implemented")
}

cv.nclreg.formula <- function(formula, data, weights, offset=NULL, ...){
    ## extract x, y, etc from the model formula and frame
    if(!attr(terms(formula, data=data), "intercept"))
        stop("non-intercept model is not implemented")
    if(missing(data)) data <- environment(formula)
    mf <- match.call(expand.dots = FALSE)
    m <- match(c("formula", "data", "weights",
                 "offset"), names(mf), 0L)
    mf <- mf[c(1L, m)]
    mf$drop.unused.levels <- TRUE
    mf[[1L]] <- as.name("model.frame")
    mf <- eval(mf, parent.frame())

    mt <- attr(mf, "terms") # allow model.frame to have updated it

    Y <- model.response(mf, "any") # e.g. factors are allowed
    ## avoid problems with 1D arrays, but keep names
    if(length(dim(Y)) == 1L) {
        nm <- rownames(Y)
        dim(Y) <- NULL
        if(!is.null(nm)) names(Y) <- nm
        if(!is.null(nm)) names(Y) <- nm
    }
    ## null model support
    X <- if (!is.empty.model(mt)) model.matrix(mt, mf, contrasts) else matrix(,NROW(Y), 0L)
    ## avoid any problems with 1D or nx1 arrays by as.vector.
    weights <- as.vector(model.weights(mf))
    if(!length(weights)) weights <- rep(1, nrow(mf))
    else if(any(weights < 0)) stop("negative weights not allowed")
    if(!is.null(weights) && !is.numeric(weights))
        stop("'weights' must be a numeric vector")
    if(length(weights) != length(Y))
        stop("'weights' must be the same length as response variable")

    offset <- as.vector(model.offset(mf))
    if(!is.null(offset)) {
        if(length(offset) != NROW(Y))
            stop(gettextf("number of offsets is %d should equal %d (number of observations)", length(offset), NROW(Y)), domain = NA)
    }
### End of addition 08/07/2012 

    RET <- cv.nclreg_fit(X[,-1], Y, weights,...)
    RET$call <- match.call()
    return(RET)
}
cv.nclreg.matrix <- function(x, y, weights, offset=NULL, ...){
    RET <- cv.nclreg_fit(x, y, weights, offset=offset, ...)
    RET$call <- match.call()
    return(RET)
}

cv.nclreg_fit <- function(x, y, weights, offset, lambda=NULL, balance=TRUE, 
                          rfamily=c("clossR", "closs", "gloss", "qloss"), s=1.5,  
                          nfolds=10, foldid, type = c("loss", "error"), plot.it=TRUE, se=TRUE, n.cores=2, trace=FALSE, parallel=FALSE,  
                          ...){
    call <- match.call()
    type <- match.arg(type)
    if(missing(foldid) && nfolds < 3)
        stop("smallest nfolds should be 3\n")
    rfamily <- match.arg(rfamily)
    nm <- dim(x)
    nobs <- n <- nm[1]
    nvars <- m <- nm[2]
    if(missing(weights)) weights <- rep(1, nobs)
    K <- nfolds
    nclreg.obj <- nclreg_fit(x, y, weights, offset=offset, lambda=lambda, rfamily=rfamily, s=s, ...)
    lambda <- nclreg.obj$lambda
    nlambda <- length(lambda)
    if(missing(foldid)){
        if(rfamily %in% c("closs", "gloss", "qloss") && balance)  
            invisible(capture.output(all.folds <- eval(parse(text="balanced.folds(y, K)"))))
        else all.folds <- cv.folds(length(y), K)
    }
    else all.folds <- foldid
    if(parallel){
    cl <- eval(parse(text="parallel:::makeCluster(n.cores)"))
    registerDoParallel(cl)
    i <- 1  ###needed to pass R CMD check with parallel code below
    residmat <- foreach(i=seq(K), .combine=cbind) %dopar% {
        omit <- all.folds[[i]]
        if(!is.null(offset)){
          offsetnow <- offset[- omit]
          newoffset <- offset[omit]
          }else offsetnow <- newoffset <- NULL
        fitcv <- nclreg_fit(x[ - omit,,drop=FALSE ], y[ -omit], weights=weights[- omit], offset=offsetnow, s=s, lambda=lambda, rfamily=rfamily, ...)
	predict(fitcv, newdata = x[omit,  ,drop=FALSE], newy=y[omit], weights=weights[omit], newoffset=newoffset, type=type)
    }
    eval(parse(text="parallel:::stopCluster(cl)"))
    }
    else{
        residmat <- matrix(NA, nlambda, K)
        for(i in seq(K)){
        if(trace)
          cat("\n CV Fold", i, "\n\n")
        omit <- all.folds[[i]]
        if(!is.null(offset)){
          offsetnow <- offset[- omit]
          newoffset <- offset[omit]
          }else offsetnow <- newoffset <- NULL
        fitcv <- nclreg_fit(x[ - omit,,drop=FALSE ], y[ -omit], weights=weights[- omit], offset=offsetnow, s=s, lambda=lambda, rfamily=rfamily, ...)
	    residmat[,i] <- predict(fitcv, newdata = x[omit,  ,drop=FALSE], newy=y[omit], weights=weights[omit], newoffset=newoffset, type=type)
        }
    }
    cv <- apply(residmat, 1, mean)
    cv.error <- sqrt(apply(residmat, 1, var)/K)
    lambda.which <- which.min(cv)
    obj<-list(fit=nclreg.obj, residmat=residmat, lambda=lambda, cv = cv, cv.error = cv.error, foldid=all.folds, lambda.which= lambda.which, lambda.optim = lambda[lambda.which])
    if(plot.it) plot.cv.nclreg(obj,se=se, ylab=type)
    class(obj) <- "cv.nclreg"
    obj
}

"plot.cv.nclreg" <-
    function(x,se=TRUE,ylab=NULL, main=NULL, width=0.02, col="darkgrey", ...){
        lambda <- x$lambda
	cv <- x$cv
        cv.error <- x$cv.error
	plot(log(lambda), cv, type = "b", xlab = expression(log(lambda)), ylab= ylab, ylim = range(cv, cv + cv.error, cv - cv.error), main=main)
        if(se)
            error.bars(log(lambda), cv + cv.error, cv - cv.error,
                       width = width, col=col)

        invisible()
    }

coef.cv.nclreg=function(object,which=object$lambda.which,...){
    coef(object$fit,which=which,...)
}

C     loss subroutine converted from R functions ziPoisson and ziNegBin
C     in package mpath/zipath.R
C     family=3 - ziPoisson; 4 - ziNegBin
C     output: los is the average loss

      subroutine ziloss(n, y, offsetx, offsetz, weights, fc, fz, 
     +           family, theta, los)
      implicit none
      integer family, i, n
      double precision y(n), offsetx(n), offsetz(n), weights(n), 
     +fc(n), fz(n), mu, phi, theta, los, dpois, dnbinom
      external dpois, dnbinom

      los = 0.d0
      do 20 i=1, n
         mu=dexp(fc(i)+offsetx(i))
         call linkinv(1, fz(i)+offsetz(i), 2, phi)
         if(family .EQ. 3)then
            if(y(i) < 1)then
               los=los+weights(i)*dlog(phi+dexp(dlog(1.0D0-phi)-mu))
            else
               los=los+weights(i)*(dlog(1.0D0-phi)
     +          +dpois(int(y(i)), mu, 1))
            endif
         else if(family .EQ. 4)then
            if(y(i) < 1)then
               los=los+weights(i)*dlog(phi+dexp(dlog(1.0D0-phi)+
     +                 dnbinom(0, theta, mu, 1)))
            else
               los=los+weights(i)*(dlog(1.0D0-phi)+
     +          dnbinom(int(y(i)),theta,mu,1))
            endif
         endif
 20   continue

      return
      end

C### compute gradient w.r.t. u=margin y*f or u=y-f for nonconvex loss
C converted from R package bst R/gradient function
C input: family, u, s, no change
C output: g
C family: 
C    clossR: 11, closs: 12, gloss: 13, qloss: 14
C
      subroutine gradient(family, n, u, s, g)
      implicit none
      integer family, n, i
      double precision u(n), s, g(n), cval, PI

      if(family .EQ. 11)then
              do 10 i=1, n
              g(i) = u(i)/(s**2*dexp(u(i)**2/(2*s**2)))
10    continue 
      else if(family .EQ. 12)then
              cval = 1/(1 - dexp(-1/(2*s**2)))
              do 20 i=1, n
              g(i)=cval*(u(i)-1)/(s**2*dexp((1-u(i))**2/(2*s**2)))
20    continue 
      else if(family .EQ. 13)then
              do 30 i=1, n
              g(i)=-2**s*s*dexp(u(i))*(dexp(u(i))+1)**(-s-1)
30    continue 
      else if(family .EQ. 14)then
              PI=4.D0*DATAN(1.D0)
              do 40 i=1, n
              g(i) = -sqrt(2.0)/(dsqrt(PI)*s)*dexp(-u(i)**2/(2*s**2))
40    continue 
      endif

      return
      end

#.First.lib <- function(lib, pkg)
.onLoad <- function(lib, pkg)
{
    library.dynam("mpath", pkg, lib)
}
#coordinate descent algorithm
#ref: Regularization paths for generalized linear models via coordinate descent, Friedman et al., JSS, 2010, 33(1)
glmreg <- function(x, ...) UseMethod("glmreg")

glmreg.default <- function(x, ...) {
    if (extends(class(x), "Matrix"))
        return(glmreg.matrix(x = x, ...))
    stop("no method for objects of class ", sQuote(class(x)),
         " implemented")
}

glmreg.formula <- function(formula, data, weights, offset=NULL, contrasts=NULL, 
                           x.keep=FALSE, y.keep=TRUE, ...){
    ## extract x, y, etc from the model formula and frame
    #if(!attr(terms(formula, data=data), "intercept"))
    #    stop("non-intercept model is not implemented")
    if(missing(data)) data <- environment(formula)
    mf <- match.call(expand.dots = FALSE)
    m <- match(c("formula", "data", "weights",
                 "offset"), names(mf), 0L)
    mf <- mf[c(1L, m)]
    mf$drop.unused.levels <- TRUE
    mf[[1L]] <- as.name("model.frame")
    mf <- eval(mf, parent.frame())

    mt <- attr(mf, "terms") # allow model.frame to have updated it

    Y <- model.response(mf, "any") # e.g. factors are allowed
    ## avoid problems with 1D arrays, but keep names
    if(length(dim(Y)) == 1L) {
        nm <- rownames(Y)
        dim(Y) <- NULL
        if(!is.null(nm)) names(Y) <- nm
    }
    ## null model support
    X <- if (!is.empty.model(mt)) model.matrix(mt, mf, contrasts) else matrix(,NROW(Y), 0L)
    ## avoid any problems with 1D or nx1 arrays by as.vector.
    weights <- as.vector(model.weights(mf))
    if(!length(weights)) weights <- rep(1, nrow(mf))
    else if(any(weights < 0)) stop("negative weights not allowed")
    if(!is.null(weights) && !is.numeric(weights))
        stop("'weights' must be a numeric vector")
    if(length(weights) != length(Y))
        stop("'weights' must be the same length as response variable")

    offset <- as.vector(model.offset(mf))
    if(!is.null(offset)) {
        if(length(offset) != NROW(Y))
            stop(gettextf("number of offsets is %d should equal %d (number of observations)", length(offset), NROW(Y)), domain = NA)
    }
    ### End of addition 08/07/2012 

    RET <- glmreg_fit(X[,-1], Y, weights=weights, offset=offset,...)
    RET$call <- match.call()
    RET <- c(RET, list(formula=formula, terms = mt, data=data,
                       contrasts = attr(X, "contrasts"),
                       xlevels = .getXlevels(mt, mf)))
    if(x.keep) RET$x <- data[,colnames(data)%in%attr(mt, "term.labels")]
    if(y.keep) RET$y <- Y
    class(RET) <- "glmreg"
    RET
}
glmreg.matrix <- function(x, y, weights, offset=NULL, ...){
    RET <- glmreg_fit(x, y, weights=weights, offset=offset,...)
    RET$call <- match.call()
    return(RET)
}

glmreg_fit <- function(x, y, weights, start=NULL, etastart=NULL, mustart=NULL, offset=NULL, nlambda=100, lambda=NULL, lambda.min.ratio=ifelse(nobs<nvars,.05, .001),alpha=1, gamma=3, rescale=TRUE, standardize=TRUE, intercept=TRUE, penalty.factor = rep(1, nvars),thresh=1e-6, eps.bino=1e-5, maxit=1000, eps=.Machine$double.eps, theta, family=c("gaussian", "binomial", "poisson", "negbin"), penalty=c("enet","mnet","snet"), convex=FALSE, x.keep=FALSE, y.keep=TRUE, trace=FALSE){
    #if(!is.null(start) && !is.null(etastart) && !is.null(mustart))
    #stop("start, etastart and mustart is for testing only\n")
    family <- match.arg(family)
    penalty <- match.arg(penalty)
    if(!family %in% c("gaussian", "binomial", "poisson", "negbin")){
        print(family)
        stop("'family' not recognizied\n")
    }
    if(family == "gaussian") rescale <- FALSE
    #if(family!="gaussian" && y < 0)
    #  stop("except for gaussian family, response should be nonnegative")
    if (gamma <= 1 && penalty=="mnet") stop("gamma must be greater than 1 for the mnet penalty")
    if (gamma <= 2 && penalty=="snet") stop("gamma must be greater than 2 for the snet penalty")
    if (alpha < 0 || alpha > 1){
        cat("alpha=", alpha)
        stop("alpha must be equal or greater than 0; and less than or equal to 1")
    }
    if(!is.null(etastart) && !is.null(mustart)){
        if((length(etastart) != length(mustart)) || length(etastart) != length(y))
            stop("length of etastart and mustart should be the same as y\n") 
    }
    ## avoid any problems with 1D or nx1 arrays by as.vector.
    nm <- dim(x)
    nobs <- n <- nm[1]
    nvars <- m <- nm[2]
    if (is.null(lambda) && is.null(lambda.min.ratio)){
        stop("lambda or lambda.min.ratio must be provided\n")
    }
    if(missing(weights)) weights=rep(1,nobs)
    weights <- as.vector(weights)
    if(!is.null(weights) && !is.numeric(weights))
        stop("'weights' must be a numeric vector")
    ## check weights and offset
    if( !is.null(weights) && any(weights < 0) ){
        stop("negative weights not allowed")
    }
    if (is.null(offset)){
        is.offset <- FALSE
        offset <- rep.int(0, nobs)
    }else is.offset <- TRUE
    if(family=="binomial"){
        if(is.factor(y))
            y <- as.integer(y) - 1
        if(any(y > 1) || any(y<0))
            stop("response should be between 0 and 1 for family=binomial")
    }
    if(family=="negbin"){
        if(missing(theta))
            stop("theta has to be provided for family='negbin'\n")
        else if(length(theta) > 1)
            stop("theta has to be one scale parameter for family='negbin'\n")
        else if(theta <= 0){
            cat("theta=", theta, "\n")
            stop("theta has to be positive for family='negbin'\n")
        }
    }
    if(family != "negbin") theta <- 1 ### this theta is not useful but required as input in Fortran glmlink subroutine
    tracetype <- 0
    if(trace)
        tracetype <- 1
    famtype <- switch(family,
                      "gaussian"=1,
                      "binomial"=2,
                      "poisson"=3,
                      "negbin"=4)
    pentype <- switch(penalty,
                      "enet"=1,
                      "mnet"=2,
                      "snet"=3)
    stantype <- as.numeric(standardize)
    #  warning("alp = 1/theta in glm negative.binomial function\n")
    if(length(penalty.factor) != nvars) stop("number of penalty.factor should be the same as nvars")
    if(all(penalty.factor==0)){ 
        lambda <- rep(0, nvars)
        penalty.factor <- rep(1, nvars)
    }
    ### ref: McCullagh and Nelder, 2nd edition, 1989, page 121
    ### compute intercept-only model
    if(family!="negbin") tmpres <- glm(y~rep(1, nobs)-1, weights=weights, offset=offset, family=family)
    else tmpres <- glm(y~rep(1, nobs)-1, weights=weights, offset=offset, family=negative.binomial(theta))
    eta0 <- predict(tmpres, type="link")
    mu0 <- predict(tmpres, type="response")
    nulldev <- .Fortran("deveval",
                        n=as.integer(n),
                        y=as.double(y),
                        mu=as.double(rep(mu0, n)),
                        theta=as.double(theta),
                        weights=as.double(weights),
                        family=as.integer(famtype),
                        dev=as.double(0),
                        PACKAGE="mpath")$dev
    ### compute the pathwise coordinate descent, cf: section 2.5 in Friedman et al., JSS 2010, 33(1)
    if(is.null(weights)) weights <- rep(1, n)
    wt <- weights/sum(weights)
    if(is.null(mustart) || is.null(etastart) || is.null(lambda)){
        eta <- eta0
        mu <- mu0
    }
    else{
        mu <- mustart
        eta <- etastart
    }
    if(is.null(lambda)){
        w <- .Fortran("glmlink",
                      n=as.integer(1),
                      mu=as.double(mu),
                      family=as.integer(famtype),
                      theta=as.double(theta),
                      w = as.double(0),
                      ep = as.double(eps.bino),
                      PACKAGE="mpath")$w
        z <- .Fortran("zeval",
                      n=as.integer(n),
                      y=as.double(y),
                      eta=as.double(eta),
                      mu=as.double(rep(mu,n)),
                      w=as.double(rep(w,n)),
                      family=as.integer(famtype),
                      z=as.double(rep(0,n)),
                      PACKAGE="mpath")$z
        z <- z - offset
        lmax <- findlam(x=x, y=y, weights=weights, family=family, theta=theta, w=w, z=z, alpha=ifelse(alpha > 0, alpha, 1e-3), penalty.factor=penalty.factor, standardize=standardize) 
        #    if(penalty %in% c("mnet", "snet") && !rescale) lmax <- 0.5 * sqrt(lmax)
        lpath <- seq(log(lmax), log(lambda.min.ratio * lmax), length.out=nlambda)
        lambda <- exp(lpath)
    }
    else nlambda <- length(lambda)
    beta <- matrix(0, ncol=nlambda, nrow=m)
    b0 <- rep(0, nlambda)
    if(is.null(start))
        startv <- 0
    if(!is.null(start)){
        if(length(start) != (m+1))
            stop("length of start doesn't match x dimension\n")
        else{
            startv <- 1
            #if(length(start) > 1){
            #    for(j in 1: nlambda)
            #        beta[,j] <- start[-1]
            #}
            #b0 <- rep(start[1], nlambda)
        }
    }
    else{
        b0 <- rep(0, nlambda)
        start <- rep(0, dim(x)[2]+1)
    }
    resdev <- rep(0, nlambda)
    yhat <- matrix(0, nobs, nlambda)
    penfac <- penalty.factor/sum(penalty.factor) * nvars
    lam <- penfac %o% lambda
    if(family=="gaussian") {
        innermaxit <- maxit
        maxit <- 1
    }
    else innermaxit <- 1
    wtnew <- weights/sum(weights)
    meanx <- drop(wtnew %*% x)
    if(standardize){
        xx <- scale(x, meanx, FALSE) # centers x
        xx <- sqrt(wtnew) * xx
        one <- rep(1,n)  
        normx <- sqrt(one %*% xx^2)
        xx <- scale(x, meanx, normx)
    }
    tmp <- .Fortran("outloop",
                    x=as.double(x),
                    y=as.double(y),
                    weights=as.double(weights),
                    wt=as.double(wt),
                    n=as.integer(n),
                    m=as.integer(m),
                    penalty=as.integer(pentype),
                    nlambda=as.integer(nlambda),
                    lam=as.double(lam),
                    alpha=as.double(alpha),
                    gam=as.double(gamma),
                    theta=as.double(theta),
                    rescale=as.integer(rescale),
                    mu=as.double(mu),
                    eta=as.double(eta),
                    offset=as.double(offset),
                    family=as.integer(famtype),
                    standardize=as.integer(stantype),
                    intercept=as.integer(intercept),
                    nulldev=as.double(nulldev),
                    thresh=as.double(thresh),
                    maxit=as.integer(maxit),
                    innermaxit=as.integer(innermaxit),
                    eps=as.double(eps),
                    trace=as.integer(tracetype),
                    start=as.double(start),
                    startv=as.integer(startv),
                    b=as.double(beta),
                    bz=as.double(b0),
                    resdev=as.double(resdev),
                    ypre=as.double(yhat),
                    convout=as.integer(rep(0,nlambda)), 
                    satu=as.integer(0),
                    good=as.integer(nlambda),
                    ep = as.double(eps.bino),
                    outpll = as.double(matrix(0, maxit, nlambda)),
                    PACKAGE="mpath") 
    if(nlambda > 1 || tmp$satu==0)
        good <- 1:tmp$good # only those non-saturated models
    else if(tmp$satu==1)
        return(RET <- list(satu=1))
    ### Names
    if(is.null(colnames(x))) varnames <- paste("V", 1:ncol(x), sep="")
    else varnames <- colnames(x)
    beta <- matrix(tmp$b, ncol=nlambda)[,good]
    beta <- as.matrix(beta)
    b0 <- tmp$bz[good]
    ### note: pll was from standardized beta values if standardize=TRUE
    if(trace)
        pll <- matrix(tmp$outpll, ncol=nlambda)
    else pll <- NULL
    convex.min <- if (convex && all(weights==1)) convexMin(rbind(b0, beta), xx, penalty, gamma, lambda*(1-alpha), family, theta=theta) else NULL
    if(convex && any(weights!=1)) warnings("code for convex region implemented for weights=1 only\n")
    if (standardize){ 
        #if (family != "gaussian" && standardize){ 
        beta <- as.matrix(beta/as.vector(normx))
        if(intercept){
            b0 <- b0 - crossprod(meanx,beta)
            if (family == "gaussian"){
                b0 <- mean(y) + b0    ### changed 4/22/2015
                b0 <- b0 - mean(offset)
            }
        }
    }
    else normx <- NULL
    resdev <- tmp$resdev[good]
    yhat <- matrix(tmp$ypre, ncol=nlambda)[,good] 
    theta <- rep(theta, nlambda)
    if(is.null(dim(beta)))
        names(beta) <- colnames(x)
    else{
        rownames(beta) <- colnames(x)
        colnames(beta) <- round(lambda,digits=4)[good]
    }
    lambda <- lambda[good]
    nlambda <- length(lambda[good])
    b0 <- matrix(b0, ncol=nlambda)
    RET <- list(family=family,standardize=standardize, satu=tmp$satu, lambda=lambda, nlambda=nlambda, beta=beta, b0=b0, meanx=meanx, normx=normx, theta=theta[good], nulldev=nulldev, resdev=resdev, pll = pll, fitted.values=yhat, converged=tmp$convout[good], convex.min=convex.min, penalty.factor=penalty.factor, gamma=gamma, alpha=alpha, is.offset=is.offset, offset=offset)
    if(x.keep) RET$x <- x
    if(y.keep) RET$y <- y
    class(RET) <- "glmreg"
    RET$twologlik <- try(2*logLik(RET, newx=x, y=y, weights=weights, offset=offset))
    ###penalized log-likelihood function value for rescaled beta
    penval <- rep(NA, nlambda)
    for(j in (1:nlambda)){
        penval[j] <- .Fortran("penGLM", 
                              start=as.double(beta[,j]),
                              m=as.integer(m),
                              lambda=as.double(RET$lambda[j]*penalty.factor),
                              alpha=as.double(alpha),
                              gam=as.double(gamma),
                              penalty=as.integer(pentype),
                              pen=as.double(0.0),
                              PACKAGE="mpath")$pen
    }
    RET$penval <- penval
    if(standardize)
        RET$pllres <- RET$twologlik/2 - n*penval
    else RET$pllres <- RET$twologlik/2 - penval
    #if(intercept) npar <- 1+RET$df else npar <- RET$df
    if(nlambda == 1){
        RET$df <- sum(abs(beta) > 0) ### df= number of nonzero coefficients (intercept excluded)
        #RET$aic <- -RET$twologlik + 2*npar
        #RET$bic <- -RET$twologlik + log(n)*npar
    }
    else{
        RET$df <- apply(abs(beta) > 0, 2, sum) ##number of nonzero coefficients for each lambda
        #RET$aic <- -RET$twologlik + 2*npar
        #RET$bic <- -RET$twologlik + log(n)*npar
    }
    if(intercept) npar <- 1+RET$df else npar <- RET$df
    RET$aic <- -RET$twologlik + 2*npar
    RET$bic <- -RET$twologlik + log(n)*npar
    RET  
}


g <- function(mu, family, eps.bino=1e-5){
    switch(family,
           "gaussian"={
               eta <- mu
           },
           "binomial"={
               ep <- eps.bino
               n <- length(mu)
               eta <- rep(0, n)
               for(i in 1:n){
                   if(1-mu[i] > ep && mu[i] > ep)
                       eta[i] <- log(mu[i]/(1-mu[i]))
               }           
           },
           "poisson"={
               eta <- log(mu)
           },
           "negbin"={
               eta <- log(mu)
           }
    )
    return(eta)
}

### eta is the estimated beta_0 in the intercept-only model, need to check 2/29/2020
init <- function(wt, y, offset, family){
    mu <- as.vector(wt %*% y) + offset
    ### is the above mu wrong such that mu < 0?
    switch(family,
           "gaussian"={
               eta <- mu
           },
           "binomial"={
               eta <- log(mu/(1-mu))
           },
           "poisson"={
               eta <- log(pmax(1, mu))
           },
           "negbin"={ ###Negative binomial regression by Hilbe, page 52, Table 4.1. Note, Table 8.3 has errors.
               eta <- log(pmax(1, mu))
           }
    )
    list(mu=mu, eta=eta)
}

findlam <- function(x, y, weights, family, theta=NULL, w, z, alpha, penalty.factor, eps.bino=1e-5, standardize=TRUE){ 
    if(all(penalty.factor==0)) return(0)
    if (alpha <= 0 || alpha > 1){
        stop("alpha must be greater than 0 and less than or equal to 1, but alpha=", alpha)
    }

    # Check a matrix for constant 
    # Columns that are constant (e.g., an intercept column of all 1s).
    # It will stop with an informative error message if either
    # condition is met.

    # --- 0. Input Validation ---

    # Ensure input is a matrix or data.frame
    if (is.null(x) || (!is.data.frame(x) && !is.matrix(x))) {
        stop("Input 'x' must be a data frame or matrix.")
    }

    # Nothing to compare if there's only 0 or 1 column
    if (ncol(x) < 2) {
        return(invisible(TRUE)) 
    }

    # Convert to data.frame for consistent `sapply` behavior
    # and store original column names
    x_df <- as.data.frame(x)
    orig_colnames <- colnames(x_df)

    # Assign temporary names if none exist, for error reporting
    if (is.null(orig_colnames)) {
        orig_colnames <- paste0("Col", seq_len(ncol(x_df)))
    }


    # --- 1. Check for Constant (Intercept) Columns ---

    # We check this first, as it's a common mistake (e.g., adding a manual
    # intercept to a model that adds one automatically).

    # Get the number of unique, non-NA values in each column
    num_unique_vals <- sapply(x_df, function(col) {
                                  length(unique(na.omit(col)))
    })

    constant_col_indices <- which(num_unique_vals == 1)

    if (length(constant_col_indices) > 0) {
        # Get the names of the constant columns
        constant_col_names <- orig_colnames[constant_col_indices]

        stop(paste0(
                    "Error: Constant columns found. This is not allowed.\n",
                    "An intercept column (all 1s) may have been mistakenly included.\n",
                    "Constant columns: ", paste(constant_col_names, collapse = ", ")
                    ))
    }


    if(family=="gaussian" || standardize)  #changed 8/12/2016
        weights <- weights/sum(weights)
    penfac <- penalty.factor/sum(penalty.factor) * dim(x)[2]
    acvar <- which(penfac > 0)
    ### regression from unregularied variables
    if(length(acvar) < length(penalty.factor)){
        x <- x[, acvar]
        penfac <- penfac[acvar]
    }

    if(family == "negbin" && is.null(theta)){
        fit <- glm.nb(y ~ 1, weights = weights)
        mu <- fit$fitted.values
        w <- .Fortran("glmlink",
                      n = as.integer(length(y)),
                      mu = as.double(mu),
                      family = as.integer(4),
                      theta = as.double(fit$theta),
                      w = as.double(rep(0, length(y))),
                      ep = as.double(eps.bino),
                      PACKAGE="mpath")$w
        resid <- .Fortran("zeval",
                          n = as.integer(length(y)),
                          y = as.double(y),
                          eta = as.double(rep(0, length(y))),
                          mu = as.double(mu),
                          w = as.double(w),
                          family = as.integer(4),
                          z = as.double(rep(0, length(y))),
                          PACKAGE="mpath")$z
    }
    else{
        if(length(acvar) < length(penalty.factor))
            resid <- lm(z ~ x[,-acvar], weights=weights)$resid
        else resid <- z-weighted.mean(z, weights)
    }
    if(standardize){
        xx <- x
        meanx <- drop(weights %*% x)
        x <- scale(x, meanx, FALSE) # centers x
        x <- sqrt(weights) * x
        one <- rep(1, length(y))
        normx <- sqrt(drop(one %*% (x^2)))
        x <- scale(xx, meanx, normx)
        lmax <- max(abs(crossprod(x,weights*w*resid))/(penfac*alpha))
    }
    else
        lmax <- max(abs(crossprod(x,weights*w*resid))/(penfac*alpha))
    lmax
}

deviance.glmreg <- function(object, newx, y, weights, na.action=na.pass, ...){
    if(missing(newx) && missing(y)) return(object$resdev)
    family <- object$family
    if(missing(weights)) weights <- rep(1, length(y))
    if(!is.null(object$terms)){
        mf <- model.frame(delete.response(object$terms), newx, na.action = na.action, xlev = object$xlevels)
        newx <- model.matrix(delete.response(object$terms), mf, contrasts = object$contrasts)
        newx <- newx[,-1] ### remove the intercept
    }
    object$terms <- NULL ### now newx is matrix, but predict function requires a dataframe for newx unless terms is NULL
    mu <- predict(object, newx, type="response")
    famtype <- switch(family,
                      "gaussian"=1,
                      "binomial"=2,
                      "poisson"=3,
                      "negbin"=4)
    dev <- rep(NA, object$nlambda)
    for(j in 1:object$nlambda)
        dev[j] <- .Fortran("deveval",
                           n=as.integer(length(y)),
                           y=as.double(y),
                           mu=as.double(mu[,j]),
                           theta=as.double(object$theta[j]),
                           weights=as.double(weights),
                           family= as.integer(famtype),
                           dev=as.double(0),
                           PACKAGE="mpath")$dev
    return(dev)
}

### convert glm object to class glmreg, thus can be used to predict newdata
conv2glmreg <- function(object, family=c("poisson", "negbin")){
    family <- match.arg(family)
    if(inherits(object, "glm")) class(object) <- "glmreg"
    object$family <- family
    object$nlambda <- 1
    namecount <- names(object$coefficients)
    object$coefficients <- matrix(object$coefficients, ncol=1)
    rownames(object$coefficients) <- namecount
    object
}

### derived from predprob.glm in pscl
predprob.glmreg <- function(obj, which, newdata = NULL, at = NULL, ...){
    newx <- newdata
    isNegBin <- obj$family=="negbin"
    isPoisson <- obj$family=="poisson"
    isBinomial <- obj$family=="binomial"
    if(!isNegBin & !isPoisson & !isBinomial)
        stop(paste("your object of class",class(obj),"is unsupported by predprob.glmreg"))
    if(is.null(newdata))
        yhat <- predict(obj,
                        type="response", which=which)
    else
        yhat <- predict(obj,
                        newx=newx,
                        type="response", which=which)

    y <- obj$y
    yUnique <- if(is.null(at)) 0:max(y) else at
    nUnique <- length(yUnique)
    p <- matrix(NA,length(yhat),nUnique)
    dimnames(p) <- list(NULL,yUnique)

    if(isNegBin){
        for(i in 1:nUnique){
            p[,i] <- dnbinom(mu=yhat,
                             size=obj$theta[which],
                             x=yUnique[i])
        }
    }

    if(isPoisson){
        for(i in 1:nUnique){
            p[,i] <- dpois(lambda=yhat,
                           x=yUnique[i])
        }
    }

    if(isBinomial){
        if(is.null(newdata))
            p <- predict(obj,
                            type="response", which)
        else
            p <- predict(obj,
                            newx=newx,
                            type="response", which)
        p <- cbind(1-p,p)
        dimnames(p) <- list(NULL,c("0","1"))
    }

    p
    
}

\documentclass[a4paper]{article}
\usepackage[round]{natbib}
\usepackage{hyperref}
%\usepackage{natbib}
%\VignetteIndexEntry{Classification of Cancer Patients with Penalized Robust Nonconvex Loss Functions (without Results)}
%\VignetteDepends{knitr, openxlsx}
%\VignetteEngine{knitr::knitr}
\newcommand{\R}[1]{\texttt{#1}}
\hypersetup{%
  pdftitle = {Classification of Cancer Patients with Penalized Robust Nonconvex Loss Functions},
  pdfsubject = {package vignette},
  pdfauthor = {Zhu Wang},
  %% change colorlinks to false for pretty printing
  colorlinks = {true},
  linkcolor = {blue},
  citecolor = {blue},
  urlcolor = {red},
  hyperindex = {true},
  linktocpage = {true},
}

\author{Zhu Wang \\
  University of Tennessee Health Science Center\\
  zwang145@uthsc.edu}
\title{Classification of Cancer Patients with Penalized Robust Nonconvex Loss Functions}
\begin{document}
\sloppy
\setkeys{Gin}{width=0.6\textwidth, keepaspectratio}
\maketitle

This document presents analysis for the MAQC-II project, human breast cancer data set with penalized classification algorithms developed in \cite{wang2019mm} and implemented in R package \R{mpath}. 

Dataset comes from the MicroArray Quality Control (MAQC) II project and includes 278 breast cancer samples with 164 estrogen receptor (ER) positive cases. The data files \verb|GSE20194_series_matrix.txt.gz| and \verb|GSE20194_MDACC_Sample_Info.xls| can be downloaded from \url{http://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?token=rhojvaiwkcsaihq&acc=GSE20194}. After reading the data, some unused variables are removed. From 22283 genes, the dataset is pre-screened to obtain 3000 genes with the largest absolute values of the two-sample t-statistics. The 3000 genes are standardized. 
<<echo=FALSE,results='hide'>>=
library("knitr")
options(prompt = "R> ", continue = " ", width = 60, useFancyQuotes = FALSE)
opts_chunk$set(tidy=TRUE, highlight=FALSE, background="white")
opts_chunk$set(dev=c('pdf','postscript'), fig.path='cancerfigure/')
@
<<message=FALSE, eval=FALSE>>=
#The data files below were downloaded on June 1, 2016
require("openxlsx")
library("mpath")
bc <- t(read.delim("GSE20194_series_matrix.txt.gz",sep="",header=FALSE,skip=80))
colnames(bc) <- bc[1,]
bc <- bc[-1, -c(1, 2)]
###The last column is empty with variable name !series_matrix_table_end, thus omitted
bc <- bc[, -22284]
mode(bc) <- "numeric" ### convert character to numeric
dat1 <- openxlsx::openxlsx("GSE20194_MDACC_Sample_Info.xls", sheet=1, header=TRUE)
y <- dat1$characteristics..ER_status
y <- ifelse(y=="P", 1, -1)
table(y)
res <- rep(NA, dim(bc)[2])
for(i in 1:dim(bc)[2])
res[i] <- abs(t.test(bc[,i]~y)$statistic)
### find 3000 largest absolute value of t-statistic
tmp <- order(res, decreasing=TRUE)[1:3000]
dat <- bc[, tmp]
### standardize variables
dat <- scale(dat)
@
Set up configuration parameters. 
<<message=FALSE>>=
### number of replicates
nrun <- 100
### penalty type
penalty <- c("enet", "snet", "mnet")
### Smallest value for lambda, as a fraction of lambda.max, the smallest value for which all coefficients are zero except the intercept
ratio <- 0.25
type.path <- "nonactive"
nlam <- ifelse(type.path!="onestep", 30, 100)
### The training data is contaminated by randomly switching response variable labels at varying pre-specified proportions
per <- c(0, 0.05, 0.10, 0.15)
### what quantity is minimized for tuning parameter selection
tuning <- "error"
n.cores <- 5
### robust nonconvex loss function, rfamily type and logistic
type <- c("closs", "gloss", "qloss", "binomial")
### and corresponding labels
type1 <- c("Closs", "Gloss", "Qloss", "Logistic")
### and corresponding tuning parameter 
s <- c(0.9, 1.01, 0.5)
mstop <- 50
plot.it <- TRUE
@
The training data contains randomly selected 50 samples with positive estrogen receptor status and 50 samples with negative estrogen receptor status, and the rest were designated as the test data. The training data is contaminated by randomly switching response variable labels at varying pre-specified proportions \R{per}=\Sexpr{per}. This process is repeated \R{nrun}=\Sexpr{nrun} times. Robust non-convex loss functions include C-loss, G-loss and Q-loss, each with penalty LASSO, SCAD and MCP. The
initial values are derived using the boosting package \R{bst} with \R{mstop}=\Sexpr{mstop} and \R{nu} provided below depending on loss function \R{type}. For SCAD and MCP penalty, a penalty tuning parameter \R{gam} is provided below. To select optimal penalization tuning parameters, we run five-fold cross-validation averaging classification errors. The classification errors and number of selected variables are tabularized and plotted with \R{plot.it=TRUE}. %With \R{type.path="nonactive"}, the prediction accuracy can be slightly improved in some scenarios, for instance, with contamination rate 15\%. However, the computing times can be doubled. 
Finally, this script also contains results with penalized logistic regression using \R{glmreg}. 
<<brcancer, fig.show='hold', eval=FALSE>>=
summary7 <- function(x) c(summary(x), sd=sd(x))
ptm <- proc.time()
for(k in (1:4)){   ### k controls family argument rfamily type (see above)
  if(type[k]=="gloss")
  nu <- 0.1
  else 
  nu <- 0.01
  for(j in (1:3)){    ### j controls argument penalty type (see above)
    gam <- ifelse(penalty[j]=="snet", 3.7, 12)
    err.m1 <- nvar.m1 <- errbest.m1 <- lambest.m1 <- matrix(NA, ncol=4, nrow=nrun)
    nvarbest.m1 <- mstopcv.m1 <- matrix(NA, ncol=4, nrow=nrun)
    colnames(err.m1) <- c("cont-0%", "cont-5%", "cont-10%", "cont-15%")
    colnames(mstopcv.m1) <- colnames(nvarbest.m1) <- colnames(err.m1)
    colnames(nvar.m1) <- colnames(err.m1)
    colnames(errbest.m1) <- colnames(err.m1)
    colnames(lambest.m1) <- colnames(err.m1)
    for (ii in 1:nrun){
      set.seed(1000+ii)
      trid <- c(sample(which(y==1))[1:50], sample(which(y==-1))[1:50])
      dtr <- dat[trid, ]
      dte <- dat[-trid, ]
      ytrold <- y[trid]
      yte <- y[-trid]
      ### number of patients/no. variables in training and test data
      dim(dtr)
      dim(dte) 
      ### randomly contaminate data
      ntr <- length(trid)
      set.seed(1000+ii)
      con <- sample(ntr)
      for(i in (1:4)){    ### i controls how many percentage of data contaminated, see argument per above 
        ytr <- ytrold
        percon <- per[i]
        ### randomly flip labels of the samples in training set according to pre-defined contamination level
        if(percon > 0){
          ji <- con[1:(percon*ntr)]
          ytr[ji] <- -ytrold[ji]
        }
        ### fit a model with nclreg for nonconvex loss or glmreg for logistic loss, and use cross-validation to select best penalization parameter
	if(type[k] %in% c("closs", "gloss", "qloss")){
        dat.m1 <- nclreg(x=dtr, y=ytr, s=s[k], iter=100, rfamily = type[k], penalty=penalty[j], lambda.min.ratio=ratio, gamma=gam, mstop.init=mstop, nu.init=nu, type.path=type.path, decreasing=FALSE, type.init="bst")
        lambda <- dat.m1$lambda[1:nlam]
        set.seed(1000+ii)
	cvm1 <- cv.nclreg(x=dtr, y=ytr, nfolds=5, n.cores=n.cores, parallel=TRUE, s=s[k], lambda=lambda, rfamily = type[k], penalty=penalty[j], gamma=gam, type=tuning, plot.it=FALSE, type.init=dat.m1$type.init, mstop.init=dat.m1$mstop.init, nu.init=dat.m1$nu.init, type.path=type.path, decreasing=dat.m1$decreasing)
        err1 <- predict(dat.m1, newdata=dte, newy=yte, type="error")
      }
      else{
        dat.m1 <- glmreg(x=dtr, y=(ytr+1)/2, family = type[k], penalty=penalty[j], lambda.min.ratio=ratio, gamma=gam)
        set.seed(1000+ii)
	cvm1 <- cv.glmreg(x=dtr, y=(ytr+1)/2, nfolds=5, n.cores=n.cores, parallel=TRUE, lambda=dat.m1$lambda, 
        family = type[k], penalty=penalty[j], gamma=gam, plot.it=FALSE)
        err1 <- apply((yte > -1)!=predict(dat.m1, newx=dte, type="class"), 2, mean)          
      }
      optmstop <- cvm1$lambda.which
      err.m1[ii, i] <- err1[optmstop]
      if(ii==1) varid <- names(predict(dat.m1, which=optmstop, type="nonzero")) else
       varid <- intersect(varid, names(predict(dat.m1, which=optmstop, type="nonzero")))
      nvar.m1[ii, i] <- length(predict(dat.m1, which=optmstop, type="nonzero"))
      errbest.m1[ii, i] <- min(err1, na.rm=TRUE)     
      lambest.m1[ii, i] <- which.min(err1)
      nvarbest.m1[ii, i] <- length(predict(dat.m1, which=which.min(err1), type="nonzero"))
    }
    if(ii %% nrun ==0){
    if(type[k]%in% c("closs", "gloss", "qloss"))
    cat(paste("\nfamily ", type1[k], ", s=", s[k], sep=""), "\n")
    else cat(paste("\nfamily ", type1[k], sep=""), "\n")
    pentype <- switch(penalty[j],
    "enet"="LASSO",
    "mnet"="MCP",
    "snet"="SCAD")
    cat("penalty=", pentype, "\n")
    if(penalty[j] %in% c("snet", "mnet")) cat("gamma=", gam, "\n")
    cat("common variables selected:", varid, "\n")
    cat("best misclassification error\n")
    print(round(apply(errbest.m1, 2, summary7),4))
    cat("which lambda has best error\n")
    print(round(apply(lambest.m1, 2, summary7),1))
    cat("number of variables selected with best error\n")
    print(round(apply(nvarbest.m1, 2, summary7),1))
    cat("CV based misclassification error\n")
    print(round(apply(err.m1, 2, summary7),4))
    cat("number of variables selected by CV\n")
    print(round(apply(nvar.m1, 2, summary7),1))
    if(plot.it){
      par(mfrow=c(2, 1))        
      boxplot(err.m1, main="Misclassification error", subset="", sub=paste(type1[k], "-", pentype, sep=""))
      boxplot(nvar.m1, main="No. variables", subset="", sub=paste(type1[k], "-", pentype, sep=""))
    }
  }
}
}
  }
    print(proc.time()-ptm)
    @
    
    <<sessionInfo>>=
    sessionInfo();
    @
    \bibliographystyle{plainnat}
    \bibliography{mpath}

\end{document}

\name{be.zeroinfl}
\alias{be.zeroinfl}

\title{conduct backward stepwise variable elimination for zero inflated count regression}

\description{
 conduct backward stepwise variable elimination for zero inflated count regression from zeroinfl function}

\usage{
be.zeroinfl(object, data, dist=c("poisson", "negbin", "geometric"), alpha=0.05, 
            trace=FALSE)
}

\arguments{
  \item{object}{an object from function zeroinfl}
  \item{data}{argument controlling formula processing
    via \code{\link[stats]{model.frame}}.}
  \item{dist}{one of the distributions in zeroinfl function}
  \item{alpha}{significance level of variable elimination}
  \item{trace}{logical value, if TRUE, print detailed calculation results}
}
\details{
 conduct backward stepwise variable elimination for zero inflated count regression from zeroinfl function}
\value{an object of zeroinfl with all variables having p-values less than the significance level \code{alpha}}
\references{
Zhu Wang, Shuangge Ma, Ching-Yun Wang, Michael Zappitelli, Prasad Devarajan and Chirag R. Parikh (2014)
\emph{EM for Regularized Zero Inflated Regression Models with Applications to Postoperative Morbidity after Cardiac Surgery in Children}, \emph{Statistics in Medicine}. 33(29):5192-208.

Zhu Wang, Shuangge Ma and Ching-Yun Wang (2015) \emph{Variable selection for zero-inflated and overdispersed data with application to health care demand in Germany}, \emph{Biometrical Journal}. 57(5):867-84.
}

\author{Zhu Wang <zwang145@uthsc.edu>}

\keyword{models}
\keyword{regression}

 

\name{cv.zipath}
\alias{cv.zipath}
\alias{cv.zipath.matrix}
\alias{cv.zipath.formula}
\alias{cv.zipath.default}
\alias{predict.cv.zipath}
\alias{coef.cv.zipath}
\title{Cross-validation for zipath}
\description{Does k-fold cross-validation for zipath, produces a plot,
  and returns cross-validated log-likelihood values for \code{lambda}}
\usage{
\method{cv.zipath}{formula}(formula, data, weights, offset=NULL, contrasts=NULL, ...)
\method{cv.zipath}{matrix}(X, Z, Y, weights, offsetx=NULL, offsetz=NULL, ...)
\method{cv.zipath}{default}(X, ...)
\method{predict}{cv.zipath}(object, newdata, \dots)
\method{coef}{cv.zipath}(object, which=object$lambda.which, model = c("full", "count", "zero"), \dots)
}
\arguments{
  \item{formula}{symbolic description of the model with an optional numeric vector \code{offset} with an a priori known component to be included in the linear predictor of the count model or zero model. Offset must be a variable in \code{data} if used, while this is optional in \code{zipath}. See an example below.}
  \item{data}{arguments controlling formula processing
    via \code{\link[stats]{model.frame}}.}
  \item{weights}{Observation weights; defaults to 1 per observation}
   \item{offset}{optional numeric vector with an a priori known component to be
     included in the linear predictor of the count model or zero model. See below   for an example.}
   \item{X}{predictor matrix of the count model}
    \item{Z}{predictor matrix of the zero model}
    \item{Y}{response variable}
   \item{offsetx, offsetz}{optional numeric vector with an a priori known component to be
     included in the linear predictor of the count model (offsetx)or zero model (offsetz).}
  \item{contrasts}{a list with elements \code{"count"} and \code{"zero"}
      containing the contrasts corresponding to \code{levels} from the
      respective models} 
 \item{object}{object of class \code{cv.zipath}.}
 \item{newdata}{optionally, a data frame in which to look for variables with
    which to predict. If omitted, the original observations are used.}
 \item{which}{Indices of the pair of penalty parameters \code{lambda.count} and \code{lambda.zero} at which
    estimates are extracted.  By default, the one which generates the optimal cross-validation value.}
 \item{model}{character specifying for which component of the model the
    estimated coefficients should be extracted.}
\item{\dots}{Other arguments that can be passed to \code{zipath}.}
}
\details{The function runs \code{zipath} \code{nfolds}+1 times; the
  first to compute the \code{(lambda.count, lambda.zero)} sequence, and then to
  compute the fit with each of the folds omitted. 
  The model is fitted to the training data and then given the fitted model the log-likelihood is evaluated at the observations left out, i.e., the test data.
  The average value of log-likelihood and standard deviation over the
  folds is computed. Note that \code{cv.zipath} can be used to search for
  values for \code{count.alpha} or \code{zero.alpha}: it is required to call \code{cv.zipath} with a fixed vector \code{foldid} for different values of \code{count.alpha} or \code{zero.alpha}.
  
The methods for \code{\link[stats]{coef}} and \code{\link{predict}} were deprecated since version 0.3-25. In fact, the \code{fit} object was removed in the output of cv.zipath so that predict an object of cv.zipath is not feasible, and should be via zipath. See examples below. The reason for such a change is that cv.zipath can take both formula and matrix, hence \code{predict} on cv. zipath object can easily lead to problems in codes.

When \code{family="negbin"}, it can be slow because there is a repeated search for the \code{theta} values by default. One may change the default values from \code{init.theta=NULL, theta.fixed=FALSE} to \code{init.theta=MLE, theta.fixed=TRUE}, where MLE is a number from glm.nb in the R package MASS or something desired. 
}
\value{an object of class \code{"cv.zipath"} is returned, which is a
  list with the components of the cross-validation fit.
 \item{fit}{a fitted zipath object for the full data.}
 \item{residmat}{matrix for cross-validated log-likelihood at each \code{(count.lambda, zero.lambda)} sequence}
  \item{bic}{matrix of BIC values with row values for \code{lambda} and column values for \code{k}th cross-validation}
  \item{cv}{The mean cross-validated log-likelihood - a vector of length
    \code{length(count.lambda)}.}
  \item{cv.error}{estimate of standard error of \code{cv}.}
  \item{foldid}{an optional vector of values between 1 and \code{nfold}
    identifying what fold each observation is in.} 
 \item{lambda.which}{index of \code{(count.lambda, zero.lambda)} that gives maximum \code{cv}.}
 \item{lambda.optim}{value of \code{(count.lambda, zero.lambda)} that gives maximum \code{cv}.}
}
\references{
Zhu Wang, Shuangge Ma, Michael Zappitelli, Chirag Parikh, Ching-Yun Wang and Prasad Devarajan (2014)      \emph{Penalized Count Data Regression with Application to Hospital Stay after Pediatric Cardiac Surgery}, \emph{Statistical Methods in Medical Research}. 2014 Apr 17. [Epub ahead of print]

Zhu Wang, Shuangge Ma, Ching-Yun Wang, Michael Zappitelli, Prasad Devarajan and Chirag R. Parikh (2014)
\emph{EM for Regularized Zero Inflated Regression Models with Applications to Postoperative Morbidity after Cardiac Surgery in Children}, \emph{Statistics in Medicine}. 33(29):5192-208.

Zhu Wang, Shuangge Ma and Ching-Yun Wang (2015) \emph{Variable selection for zero-inflated and overdispersed data with application to health care demand in Germany}, \emph{Biometrical Journal}. 57(5):867-84.
}

\author{Zhu Wang <zwang145@uthsc.edu>}

\seealso{\code{\link{zipath}} and \code{\link{plot}}, \code{\link{predict}}, methods for \code{"cv.zipath"} object.}
\examples{
\dontrun{
data("bioChemists", package = "pscl")
fm_zip <- zipath(art ~ . | ., data = bioChemists, family = "poisson", nlambda=10)
fm_cvzip <- cv.zipath(art ~ . | ., data = bioChemists, family = "poisson", nlambda=10)
### prediction from the best model
pred <- predict(fm_zip, newdata=bioChemists, which=fm_cvzip$lambda.which)
coef(fm_zip, which=fm_cvzip$lambda.which)
fm_znb <- zipath(art ~ . | ., data = bioChemists, family = "negbin", nlambda=10)
fm_cvznb <- cv.zipath(art ~ . | ., data = bioChemists, family = "negbin", nlambda=10)
pred <- predict(fm_znb, which=fm_cvznb$lambda.which)
coef(fm_znb, which=fm_cvznb$lambda.which)
fm_zinb2 <- zipath(art ~ . +offset(log(phd))| ., data = bioChemists, 
		      family = "poisson", nlambda=10)
fm_cvzinb2 <- cv.zipath(art ~ . +offset(log(phd))| ., data = bioChemists, 
		      family = "poisson", nlambda=10)
pred <- predict(fm_zinb2, which=fm_cvzinb2$lambda.which)
coef(fm_zinb2, which=fm_cvzinb2$lambda.which)
}
}
\keyword{models}
\keyword{regression}

\name{glmregNB}
\alias{glmregNB}
\alias{glmregNegbin}
\title{fit a negative binomial model with lasso (or elastic net), snet and mnet regularization}
\description{
    Fit a negative binomial linear model via penalized maximum likelihood. The regularization path is computed for the lasso (or elastic net penalty), snet and mnet penalty, at a grid of values for the regularization parameter lambda. }
\usage{
glmregNB(formula, data, weights, offset=NULL, nlambda = 100, lambda=NULL, 
         lambda.min.ratio = ifelse(nobs<nvars,0.05,0.001), alpha=1, gamma=3, 
         rescale=TRUE, standardize = TRUE, penalty.factor = rep(1, nvars), 
         thresh = 0.001, maxit.theta = 10, maxit=1000, eps=.Machine$double.eps,
         trace=FALSE, start = NULL, etastart = NULL, mustart = NULL, 
         theta.fixed=FALSE, theta0=NULL, init.theta=NULL, link=log, 
         penalty=c("enet","mnet","snet"), method="glmreg_fit", model=TRUE, 
         x.keep=FALSE, y.keep=TRUE, contrasts=NULL, convex=FALSE, 
         parallel=TRUE, n.cores=2, ...)
}

\arguments{
  \item{formula}{formula used to describe a model.}
 \item{data}{argument controlling formula processing
    via \code{\link[stats]{model.frame}}.} 
  \item{weights}{an optional vector of `prior weights' to be used in the fitting process. Should be \code{NULL} or a numeric vector. Default is a vector of 1s with equal weight for each observation.}
   \item{offset}{optional numeric vector with an a priori known component to be
    included in the linear predictor of the model.}
  \item{nlambda}{The number of \code{lambda} values - default is 100.}
 \item{lambda}{A user supplied \code{lambda} sequence}
  \item{lambda.min.ratio}{Smallest value for \code{lambda}, as a fraction of
    \code{lambda.max}, the (data derived) entry value (i.e. the smallest
  value for which all coefficients are zero). The default depends on the
  sample size \code{nobs} relative to the number of variables
  \code{nvars}. If \code{nobs > nvars}, the default is \code{0.001},
  close to zero.  If \code{nobs < nvars}, the default is \code{0.05}.
}
 \item{alpha}{The L2 penalty mixing parameter, with
    \eqn{0\le\alpha\le 1}. \code{alpha=1} is lasso (mcp, scad) penalty; and \code{alpha=0} the ridge penalty.}
\item{gamma}{The tuning parameter of the \code{snet} or \code{mnet} penalty.}
 \item{rescale}{logical value, if TRUE, adaptive rescaling of the penalty parameter for \code{penalty="mnet"} or \code{penalty="snet"} with \code{family} other than "gaussian". See reference}
  \item{standardize}{Logical flag for x variable standardization, prior to
    fitting the model sequence. The coefficients are always returned on
    the original scale. Default is \code{standardize=TRUE}.
  If variables are in the same units already, you might not wish to
  standardize.}
 \item{penalty.factor}{This is a number that multiplies \code{lambda} to allow
    differential shrinkage of coefficients. Can be 0 for some variables, which implies
    no shrinkage, and that variable is always included in the
    model. Default is same shrinkage for all variables.}
  \item{thresh}{Convergence threshold for coordinate descent. Defaults value is \code{1e-6}.}
  \item{maxit.theta}{Maximum number of iterations for estimating \code{theta} scaling parameter}
\item{maxit}{Maximum number of coordinate descent iterations for each \code{lambda} value; default is 1000.}
  \item{eps}{If a number is less than \code{eps} in magnitude, then this number is considered as 0}
  \item{trace}{If \code{TRUE}, fitting progress is reported}
 \item{start, etastart, mustart, \dots}{ arguments for the \code{link{glmreg}} function}
 \item{init.theta}{ initial scaling parameter \code{theta}}
 \item{theta.fixed}{Estimate scale parameter theta? Default is FALSE. Note, the algorithm may become slow. In this case, one may use \code{glmreg} function with \code{family="negbin"}, and a fixed \code{theta}}.
 \item{theta0}{initial scale parameter vector theta, with length \code{nlambda} if \code{theta.fixed=TRUE}. Default is NULL}
\item{convex}{Calculate index for which objective function ceases to
    be locally convex? Default is FALSE and only useful if \code{penalty="mnet" or "snet"}.}
 \item{link}{link function, default is \code{log}}
 \item{penalty}{Type of regularization}
\item{method}{estimation method}
 \item{model, x.keep, y.keep}{logicals. If \code{TRUE} the corresponding components
    of the fit (model frame, response, model matrix) are returned.}
\item{contrasts}{the contrasts corresponding to \code{levels} from the
    respective models}
\item{parallel, n.cores}{a logical value, parallel computing or not for sequence of \code{lambda} with the number of CPU cores to use. The \code{lambda} loop will attempt to send different \code{lambda} off to different cores.}
}
\details{
 The sequence of models implied by \code{lambda} is fit by coordinate
  descent. This is a lasso (mcp, scad) or elastic net (mnet, snet) regularization path
  for fitting the negative binomial linear regression
  paths, by maximizing the penalized log-likelihood.
  Note that the objective function is 
  \deqn{-\sum (weights * loglik) + \lambda*penalty} if \code{standardize=FALSE} and \deqn{-\frac{weights}{\sum(weights)} * loglik + \lambda*penalty} if \code{standardize=TRUE}.
  }
\value{
An object with S3 class \code{"glmreg", "glmregNB"} for the various types of models.
  \item{call}{the call that produced the model fit}
  \item{b0}{Intercept sequence of length \code{length(lambda)}}
  \item{beta}{A \code{nvars x
      length(lambda)} matrix of coefficients.}
  \item{lambda}{The actual sequence of \code{lambda} values used}
  \item{resdev}{The computed deviance. The deviance calculations incorporate weights if
  present in the model. The deviance is defined to be 2*(loglike_sat -
  loglike), where loglike_sat is the log-likelihood for the saturated
  model (a model with a free parameter per observation). }
    \item{nulldev}{Null deviance (per observation). This is defined to
  be  2*(loglike_sat -loglike(Null)); The NULL model refers to the
  intercept model.}
  \item{nobs}{number of observations}
}
\references{                                                                          Zhu Wang, Shuangge Ma, Michael Zappitelli, Chirag Parikh, Ching-Yun Wang and Prasad Devarajan (2014)      \emph{Penalized Count Data Regression with Application to Hospital Stay after Pediatric Cardiac Surgery}, \emph{Statistical Methods in Medical Research}. 2014 Apr 17. [Epub ahead of print]
}

\author{Zhu Wang <zwang145@uthsc.edu>}

\seealso{\code{\link{print}}, \code{\link{predict}}, \code{\link{coef}} and \code{\link{plot}} methods, and the \code{\link{cv.glmregNB}} function.}
\examples{
\dontrun{
data("bioChemists", package = "pscl")
system.time(fm_nb1 <- glmregNB(art ~ ., data = bioChemists, parallel=FALSE))
system.time(fm_nb2 <- glmregNB(art ~ ., data = bioChemists, parallel=TRUE, n.cores=2))
coef(fm_nb1)
### ridge regression
fm <- glmregNB(art ~ ., alpha=0, data = bioChemists, lambda=seq(0.001, 1, by=0.01))
fm <- cv.glmregNB(art ~ ., alpha=0, data = bioChemists, lambda=seq(0.001, 1, by=0.01))
}
}
\keyword{models}
\keyword{regression}

\name{predict.zipath}
\alias{predict.zipath}
\alias{residuals.zipath}
\alias{terms.zipath}
\alias{model.matrix.zipath}
\alias{coef.zipath}
\alias{summary.zipath}
\alias{print.summary.zipath}
\alias{logLik.zipath}
\alias{fitted.zipath}
\alias{predprob.zipath}

\title{Methods for zipath Objects}
\description{
  Methods for extracting information from fitted penalized zero-inflated
  regression model objects of class \code{"zipath"}.
}

\usage{
\method{predict}{zipath}(object, newdata,  which = 1:object$nlambda, 
  type = c("response", "prob", "count", "zero", "nonzero"), na.action = na.pass,
  at = NULL, \dots)
\method{residuals}{zipath}(object, type = c("pearson", "response"), \dots)

\method{coef}{zipath}(object, which=1:object$nlambda, model = c("full", "count", "zero"), \dots)

\method{terms}{zipath}(x, model = c("count", "zero"), \dots)
\method{model.matrix}{zipath}(object, model = c("count", "zero"), \dots)
}

\arguments{
  \item{object, x}{an object of class \code{"zipath"} as returned by
    \code{\link{zipath}}.}
  \item{newdata}{optionally, a data frame in which to look for variables with
    which to predict. If omitted, the original observations are used.}
  \item{which}{Indices of the penalty parameters \code{lambda} at which
    predictions are required.  By default, all indices are returned.}
  \item{type}{character specifying the type of predictions or residuals,
    respectively. For details see below.}
  \item{na.action}{function determining what should be done with missing values
    in \code{newdata}. The default is to predict \code{NA}.}
  \item{at}{optionally, if \code{type = "prob"}, a numeric vector at which
    the probabilities are evaluated. By default \code{0:max(y)} is used
    where \code{y} is the original observed response.}
  \item{model}{character specifying for which component of the model the
    terms or model matrix should be extracted.}
  \item{\dots}{currently not used.}
}

\details{
  Re-uses the design of function zeroinfl in package pscl (see reference). A set of standard extractor functions for fitted model objects is available for
  objects of class \code{"zipath"}, including methods to the generic functions
  \code{\link[base]{print}} and \code{\link[base]{summary}} which print the estimated
  coefficients along with some further information. 
  As usual, the \code{summary} method returns an object of class \code{"summary.zipath"}
  containing the relevant summary statistics which can subsequently be printed
  using the associated \code{print} method.
  
  The methods for \code{\link[stats]{coef}} by default
  return a single vector of coefficients and their associated covariance matrix,
  respectively, i.e., all coefficients are concatenated. By setting the \code{model}
  argument, the estimates for the corresponding model components can be extracted.
  
  Both the \code{\link[stats]{fitted}} and \code{\link[stats]{predict}} methods can
  compute fitted responses. The latter additionally provides the predicted density
  (i.e., probabilities for the observed counts), the predicted mean from the count
  component (without zero inflation) and the predicted probability for the zero
  component. The \code{\link[stats]{residuals}} method can compute
  raw residuals (observed - fitted) and Pearson residuals (raw residuals scaled by
  square root of variance function).
}

\references{
Zhu Wang, Shuangge Ma, Michael Zappitelli, Chirag Parikh, Ching-Yun Wang and Prasad Devarajan (2014)      \emph{Penalized Count Data Regression with Application to Hospital Stay after Pediatric Cardiac Surgery}, \emph{Statistical Methods in Medical Research}. 2014 Apr 17. [Epub ahead of print]

Zhu Wang, Shuangge Ma, Ching-Yun Wang, Michael Zappitelli, Prasad Devarajan and Chirag R. Parikh (2014)
\emph{EM for Regularized Zero Inflated Regression Models with Applications to Postoperative Morbidity after Cardiac Surgery in Children}, \emph{Statistics in Medicine}. 33(29):5192-208.

Zhu Wang, Shuangge Ma and Ching-Yun Wang (2015) \emph{Variable selection for zero-inflated and overdispersed data with application to health care demand in Germany}, \emph{Biometrical Journal}. 57(5):867-84.
}
\author{Zhu Wang <zwang145@uthsc.edu>}

\seealso{\code{\link{zipath}}}

\examples{
\dontrun{
data("bioChemists", package = "pscl")
fm_zip <- zipath(art ~ . | ., data = bioChemists, nlambda=10)
plot(residuals(fm_zip) ~ fitted(fm_zip))
coef(fm_zip, model = "count")
coef(fm_zip, model = "zero")
summary(fm_zip)
logLik(fm_zip)
}
}

\keyword{regression}

\name{plot.glmreg}
\alias{plot.glmreg}
\title{plot coefficients from a "glmreg" object}
\description{
Produces a coefficient profile plot of the coefficient paths for a
fitted \code{"glmreg"} object.
}
\usage{
\method{plot}{glmreg}(x, xvar = c("norm", "lambda", "dev"), label = FALSE, shade=TRUE, ...)
}
\arguments{
  \item{x}{fitted \code{"glmreg"} model}
  \item{xvar}{What is on the X-axis. \code{"norm"} plots against the
    L1-norm of the coefficients, \code{"lambda"} against the log-lambda
    sequence, and \code{"dev"} against the percent deviance explained.}
  \item{label}{If \code{TRUE}, label the curves with variable sequence
    numbers.}
  \item{shade}{Should nonconvex region be shaded?  Default is TRUE. Code developed for all \code{weights=1} only}
  \item{\dots}{Other graphical parameters to plot}
}
\details{
A coefficient profile plot is produced. 
}
\author{Zhu Wang zwang145@uthsc.edu}
\seealso{\code{glmreg}, and \code{print}, \code{predict} and \code{coef} methods.}
\examples{
x=matrix(rnorm(100*20),100,20)
y=rnorm(100)
fit1=glmreg(x,y)
plot(fit1)
plot(fit1,xvar="lambda",label=TRUE)
}
\keyword{models}
\keyword{regression}

\name{gfunc}
\alias{gfunc}
\title{Convert response value to raw prediction in GLM}
\description{
 Compute response value to raw prediction such as linear predictor in GLM
}
\usage{
gfunc(mu, family, epsbino)
}
\arguments{
 \item{mu}{vector of numbers as response value in GLM, for instance, probability estimation if \code{family=2}}
 \item{family}{integer from 1-4, corresponding to "gaussian", "binomial", "poisson", "negbin", respectively}
 \item{epsbino}{a small positive value for \code{family=2} to avoid numeric unstability}
}

\value{linear predictor f=x'b for predictor x and coefficient b if the model is linear}

\keyword{regression}
\keyword{classification}

\name{cv.irsvm_fit}
\alias{cv.irsvm_fit}
\title{Internal function of cross-validation for irsvm}
\description{Internal function to conduct k-fold cross-validation for irsvm}
\usage{
cv.irsvm_fit(x, y, weights, cfun="ccave", s=c(1, 5), type=NULL, 
             kernel="radial", gamma=2^(-4:10), cost=2^(-4:4), 
             epsilon=0.1, balance=TRUE, nfolds=10, foldid, 
             trim_ratio=0.9, n.cores=2, ...)
}
\arguments{
 \item{x}{a data matrix, a vector, or a sparse '\emph{design} matrix' (object  of class
     \code{\link[Matrix]{Matrix}} provided by the \pkg{Matrix} package,
     or of class \code{matrix.csr}
     provided by the \pkg{SparseM} package, or of class
     \code{simple_triplet_matrix} provided by the \pkg{slam}
     package).}
   \item{y}{a response vector with one label for each row/component of
     \code{x}. Can be either a factor (for classification tasks)
     or a numeric vector (for regression).}
   \item{weights}{the weight of each subject. It should be in the same length    of \code{y}.}
    \item{cfun}{ character, type of convex cap (concave) function.\cr
    Valid options are:
     \itemize{
          \item \code{"hcave"}
         \item \code{"acave"}
         \item \code{"bcave"}
         \item \code{"ccave"}
         \item \code{"dcave"}
         \item \code{"ecave"}
         \item \code{"gcave"}
         \item \code{"tcave"}
      }
    }
\item{s}{ tuning parameter of \code{cfun}. \code{s > 0} and can be equal       to  0 for \code{cfun="tcave"}. If \code{s} is too close to 0 for                \code{cfun="acave", "bcave", "ccave"}, the calculated weights can become 0 for  all observations, thus crash the program.}
    \item{type}{\code{irsvm} can be used as a classification
     machine, or as a regression machine.
     Depending of whether \code{y} is
     a factor or not, the default setting for \code{type} is \code{C-classification} or \code{eps-regression}, respectively, but may be overwritten  by setting an explicit value.\cr
     Valid options are:
     \itemize{
       \item \code{C-classification}
       \item \code{nu-classification}
       \item \code{eps-regression}
       \item \code{nu-regression}
     }
   }
   \item{kernel, gamma}{the kernel used in training and predicting. You
      might consider changing some of the following parameters, depending
      on the kernel type.\cr
      \describe{
        \item{linear:}{\eqn{u'v}{u'*v}}
        \item{polynomial:}{\eqn{(\gamma u'v + coef0)^{degree}}{(gamma*u'*        v +      coef0)^degree}}
        \item{radial basis:}{\eqn{e^(-\gamma |u-v|^2)}{exp(-gamma*|u-v|^2)}}
        \item{sigmoid:}{\eqn{tanh(\gamma u'v + coef0)}{tanh(gamma*u'*v +         coef0)}}
      }
   }
   \item{cost}{cost of constraints violation (default: 1)---it is the
      \sQuote{C}-constant of the regularization term in the Lagrange formulation. This is proportional to the inverse of \code{lambda} in \code{irglmreg}.}
   \item{epsilon}{epsilon in the insensitive-loss function (default: 0.1)}
   \item{balance}{for \code{type="C-classification", "nu-classification"} only}
  \item{nfolds}{number of folds >=3, default is 10}
  \item{foldid}{an optional vector of values between 1 and \code{nfold}
    identifying what fold each observation is in. If supplied,
    \code{nfold} can be missing and will be ignored.}
 \item{trim_ratio}{a number between 0 and 1 for trimmed least squares, useful if \code{type="eps-regression"} or \code{"nu-regression"}.} 
 \item{n.cores}{The number of CPU cores to use. The cross-validation loop
    will attempt to send different CV folds off to different cores.}
  \item{\dots}{Other arguments that can be passed to \code{irsvm}.}
}
\details{
This function is the driving force behind \code{cv.irsvm}. Does a K-fold cross-validation to determine optimal tuning parameters  in SVM: \code{cost} and \code{gamma} if \code{kernel} is nonlinear. It can also choose \code{s} used in \code{cfun}.    
}

\value{an object of class \code{"cv.irsvm"} is returned, which is a
  list with the ingredients of the cross-validation fit.
\item{residmat}{matrix with row values for \code{kernel="linear"} are       \code{s, cost, error, k}, where \code{k} is the number of cross-validation fold. For nonlinear kernels, row values are \code{s, gamma, cost, error, k}. }
   \item{cost}{a value of \code{cost} that gives minimum cross-validated value in \code{irsvm}.}
   \item{gamma}{a value of \code{gamma} that gives minimum cross-validated value in \code{irsvm}}
   \item{s}{value of \code{s} for \code{cfun} that gives minimum cross-validated value in \code{irsvm}.}
}
\references{
 Zhu Wang (2024)
     \emph{Unified Robust Estimation}, \emph{Australian & New Zealand Journal         of      Statistics}. 66(1):77-102.
}

\author{Zhu Wang <zwang145@uthsc.edu>}
\seealso{\code{\link{cv.irsvm}} and \code{\link{irsvm}}}
\keyword{models}
\keyword{regression}

\name{ncl_fit}
\alias{ncl_fit}
\title{Internal function to fit a nonconvex loss based robust linear model}
\description{
    Fit a linear model via penalized nonconvex loss function.  
  }
\usage{
ncl_fit(x,y, weights, offset=NULL, 
        rfamily=c("clossR", "closs", "gloss", "qloss"), 
        s=NULL, fk=NULL, iter=10, reltol=1e-5, trace=FALSE)
}

\arguments{
  \item{x}{input matrix, of dimension nobs x nvars; each row is an
  observation vector.} 
  \item{y}{response variable. Quantitative for \code{rfamily="clossR"} and -1/1 for classifications.}
  \item{weights}{observation weights. Can be total counts if responses are proportion matrices. Default is 1 for each observation}
  \item{offset}{this can be used to specify an a priori known component to be included in the linear predictor during fitting. This should be NULL or a numeric vector of length equal to the number of cases. Currently only one offset term can be included in the formula.}
  \item{rfamily}{Response type and relevant loss functions (see above)}
  \item{s}{ nonconvex loss tuning parameter for robust regression and classification. }
  \item{fk}{ predicted values at an iteration in the MM algorithm }
 \item{iter}{number of iteration in the MM algorithm}
 \item{reltol}{convergency criteria} 
 \item{trace}{If \code{TRUE}, fitting progress is reported}
}
\details{
  The robust linear model is fit by majorization-minimization along with least squares.
  Note that the objective function is \deqn{weights*loss}.
  }
\value{
An object with S3 class \code{"ncl"} for the various types of models.
  \item{call}{the call that produced the model fit}
  \item{fitted.values}{predicted values}  
\item{h}{ pseudo response values in the MM algorithm }
}

\references{
   Zhu Wang (2021),
   \emph{MM for Penalized Estimation},
   \emph{TEST},
   \doi{10.1007/s11749-021-00770-2}
}

\author{Zhu Wang <zwang145@uthsc.edu>}

\seealso{\code{\link{ncl}}}
\keyword{models}
\keyword{regression}

 

\name{cv.glmregNB}
\alias{cv.glmregNB}
\title{Cross-validation for glmregNB}
\description{Does k-fold cross-validation for glmregNB, produces a plot,
  and returns cross-validated log-likelihood values for \code{lambda}}
\usage{
cv.glmregNB(formula, data, weights, offset=NULL, lambda=NULL, nfolds=10, 
            foldid, plot.it=TRUE, se=TRUE, n.cores=2, trace=FALSE, 
            parallel=FALSE, ...)
}
\arguments{
  \item{formula}{symbolic description of the model}
  \item{data}{arguments controlling formula processing
    via \code{\link[stats]{model.frame}}.}
  \item{weights}{Observation weights; defaults to 1 per observation}
  \item{offset}{this can be used to specify an a priori known component to be included in the linear predictor during fitting. This should be NULL or a numeric vector of length equal to the number of cases. Currently only one offset term can be included in the formula.}
  \item{lambda}{Optional user-supplied lambda sequence; default is
    \code{NULL}, and \code{glmregNB} chooses its own sequence}
  \item{nfolds}{number of folds - default is 10. Although \code{nfolds}
    can be as large as the sample size (leave-one-out CV), it is not
    recommended for large datasets. Smallest value allowable is \code{nfolds=3}}
  \item{foldid}{an optional vector of values between 1 and \code{nfold}
    identifying what fold each observation is in. If supplied,
    \code{nfold} can be missing.}
 \item{plot.it}{ a logical value, to plot the estimated log-likelihood values if \code{TRUE}. }
  \item{se}{ a logical value, to plot with standard errors. }
 \item{n.cores}{The number of CPU cores to use. The cross-validation loop
  will attempt to send different CV folds off to different cores.}
 \item{trace}{a logical value, print progress of cross-validation or not}
 \item{parallel}{a logical value, parallel computing or not}
\item{\dots}{Other arguments that can be passed to \code{glmregNB}.}
}
\details{The function runs \code{glmregNB} \code{nfolds}+1 times; the
  first to get the \code{lambda} sequence, and then the remainder to
  compute the fit with each of the folds omitted. The error is
  accumulated, and the average error and standard deviation over the
  folds is computed. 
  Note that \code{cv.glmregNB} does NOT search for
  values for \code{alpha}. A specific value should be supplied, else
  \code{alpha=1} is assumed by default. If users would like to
  cross-validate \code{alpha} as well, they should call \code{cv.glmregNB}
  with a pre-computed vector \code{foldid}, and then use this same fold vector
  in separate calls to \code{cv.glmregNB} with different values of
  \code{alpha}. 
}
\value{an object of class \code{"cv.glmregNB"} is returned, which is a
  list with the ingredients of the cross-validation fit.
 \item{fit}{a fitted glmregNB object for the full data.}
  \item{residmat}{matrix of log-likelihood values with row values for \code{lambda} and column values for \code{k}th cross-validation}
  \item{cv}{The mean cross-validated log-likelihood values - a vector of length
    \code{length(lambda)}.}
  \item{cv.error}{The standard error of cross-validated log-likelihood values - a vector of length
    \code{length(lambda)}.}
\item{lambda}{a vector of \code{lambda} values} 
  \item{foldid}{indicators of data used in each cross-validation, for reproductive purposes}
  \item{lambda.which}{index of \code{lambda} that gives maximum \code{cv} value.}
  \item{lambda.optim}{value of \code{lambda} that gives maximum \code{cv} value.}
}
\references{
Zhu Wang, Shuangge Ma, Michael Zappitelli, Chirag Parikh, Ching-Yun Wang and Prasad Devarajan (2014)
\emph{Penalized Count Data Regression with Application to Hospital Stay after Pediatric Cardiac Surgery}, \emph{Statistical Methods in Medical Research}. 2014 Apr 17. [Epub ahead of print] 
}
\author{Zhu Wang <zwang145@uthsc.edu>}
\seealso{\code{\link{glmregNB}} and \code{\link{plot}}, \code{\link{predict}}, and \code{\link{coef}} methods for \code{"cv.glmregNB"} object.}
\examples{
\dontrun{
data("bioChemists", package = "pscl")
fm_nb <- cv.glmregNB(art ~ ., data = bioChemists)
plot(fm_nb)
}
}
\keyword{models}
\keyword{regression}

\name{pval.zipath}
\alias{pval.zipath}

\title{compute p-values from penalized zero-inflated model with multi-split data}

\description{
  compute p-values from penalized zero-inflated Poisson, negative binomial and geometric model with multi-split data
  }
\usage{
pval.zipath(formula, data, weights, subset, na.action, offset, standardize=TRUE,
            family = c("poisson", "negbin", "geometric"),
            penalty = c("enet", "mnet", "snet"), gamma.count = 3, 
            gamma.zero = 3, prop=0.5, trace=TRUE, B=10, ...)
}

\arguments{
  \item{formula}{symbolic description of the model, see details.}
  \item{data}{argument controlling formula processing
    via \code{\link[stats]{model.frame}}.}
  \item{weights}{optional numeric vector of weights. If \code{standardize=TRUE}, weights are renormalized to weights/sum(weights). If \code{standardize=FALSE}, weights are kept as original input}
  \item{subset}{subset of data}
  \item{na.action}{how to deal with missing data}
  \item{offset}{Not implemented yet}
  \item{standardize}{logical value, should variables be standardized?}
  \item{family}{family to fit \code{zipath}}
  \item{penalty}{penalty considered as one of \code{enet, mnet, snet}.}
 \item{gamma.count}{The tuning parameter of the \code{snet} or \code{mnet} penalty for the count part of model.}
  \item{gamma.zero}{The tuning parameter of the \code{snet} or \code{mnet} penalty for the zero part of model.}  
  \item{prop}{proportion of data split, default is 50/50 split}
  \item{trace}{logical value, if TRUE, print detailed calculation results}
  \item{B}{number of repeated multi-split replications} 
  \item{...}{Other arguments passing to \code{glmreg_fit}}
}
\details{
  compute p-values from penalized zero-inflated Poisson, negative binomial and geometric model with multi-split data
  }
\value{
  \item{count.pval}{raw p-values in the count component}
  \item{zero.pval}{raw p-values in the zero component}
  \item{count.pval.q}{Q value for the count component}
  \item{zero.pval.q}{Q value for the zero component}
}
\references{
Nicolai Meinshausen, Lukas Meier and Peter Buehlmann (2013)
\emph{p-Values for High-Dimensional Regression}, \emph{Journal of the American Statistical Association}, 104(488), 1671--1681.

Zhu Wang, Shuangge Ma, Ching-Yun Wang, Michael Zappitelli, Prasad Devarajan and Chirag R. Parikh (2014)
\emph{EM for Regularized Zero Inflated Regression Models with Applications to Postoperative Morbidity after Cardiac Surgery in Children}, \emph{Statistics in Medicine}. 33(29):5192-208.

Zhu Wang, Shuangge Ma and Ching-Yun Wang (2015) \emph{Variable selection for zero-inflated and overdispersed data with application to health care demand in Germany}, \emph{Biometrical Journal}. 57(5):867-84.
}

\author{Zhu Wang <zwang145@uthsc.edu>}

\keyword{models}
\keyword{regression}

 

\name{breastfeed}
\alias{breastfeed}
\docType{data}
\title{ Breast feeding decision}
\description{
  In a UK hospital, 135 expectant mothers were surveyed on the decision of breastfeeding their
babies or not, along with two-level predictive factors 
}
\usage{data(breastfeed)}

\source{
Stephane Heritier, Eva Cantoni, Samuel Copt and Maria-Pia Victoria-Fese (2009). \emph{Robust Methods in Biostatistics}, John Wiley & Sons}

\examples{
data(breastfeed)
str(breastfeed)
}
\keyword{datasets}

\name{irglmreg_fit}
\alias{irglmreg_fit}
\title{Internal function for robust penalized generalized linear models}
\description{
    Fit a robust penalized GLM where the loss function is a composite function \code{cfun}o\code{dfun} + penalty. This does computing for \code{irglmreg}.
  }
\usage{
irglmreg_fit(x, y, weights, offset, cfun="ccave", dfun="gaussian", s=NULL, 
             delta=0.1, fk=NULL, iter=10, reltol=1e-5, 
             penalty=c("enet","mnet","snet"), nlambda=100, lambda=NULL, 
             type.path=c("active", "nonactive"), decreasing=TRUE, 
             lambda.min.ratio=ifelse(nobs<nvars,.05, .001), alpha=1, gamma=3,
             rescale=TRUE, standardize=TRUE, intercept=TRUE, 
             penalty.factor= NULL, maxit=1000, type.init=c("bst", "co", "heu"), 
             init.family=NULL, mstop.init=10, nu.init=0.1, 
             eps=.Machine$double.eps, epscycle=10, thresh=1e-6, parallel=FALSE,
             n.cores=2, theta, trace=FALSE, tracelevel=1)
}

\arguments{
  \item{x}{input matrix, of dimension nobs x nvars; each row is an
  observation vector.} 
  \item{y}{response variable. Quantitative for \code{dfun=1} and -1/1 otherwise for classifications.}
  \item{weights}{observation weights. Can be total counts if responses are proportion matrices. Default is 1 for each observation}
  \item{offset}{this can be used to specify an a priori known component to be included in the linear predictor during fitting. This should be NULL or a numeric vector of length equal to the number of cases. Currently only one offset term can be included in the formula.}
  \item{cfun}{ character, type of convex cap (concave) function.\cr
  Valid options are:
   \itemize{
        \item \code{"hcave"}
        \item \code{"acave"}
        \item \code{"bcave"}
        \item \code{"ccave"}
        \item \code{"dcave"}
        \item \code{"ecave"}
        \item \code{"gcave"}
        \item \code{"tcave"}
    }
  }
  \item{dfun}{ character, type of convex downward function. \cr
    Valid options are:
    \itemize{
        \item \code{"gaussian"}
        \item \code{"gaussianC"}
        \item \code{"binomial"}
        }
    }
\item{s}{ tuning parameter of \code{cfun}. \code{s > 0} and can be equal    to 0 for \code{cfun="tcave"}. If \code{s} is too close to 0 for               \code{cfun="acave", "bcave", "ccave"}, the calculated weights can become 0    for all observations, thus crash the program.}
  \item{delta}{a small positive number provided by user only if                \code{cfun="gcave"} and \code{0 < s <1}}
  \item{fk}{ predicted values at an iteration in the IRCO algorithm }
  \item{nlambda}{The number of \code{lambda} values - default is 100. The sequence may be truncated before \code{nlambda} is reached if a close to saturated model is fitted. See also \code{satu}.}
  \item{lambda}{ by default, the algorithm provides a sequence of regularization values, or a user supplied \code{lambda} sequence }
  \item{type.path}{solution path for \code{parallel=FALSE}. If \code{type.path="active"}, then cycle through only the active set in the next increasing \code{lambda} sequence. If \code{type.path="nonactive"}, no active set for each element of the lambda sequence and cycle through all the predictor variables.} 
  \item{lambda.min.ratio}{Smallest value for \code{lambda}, as a fraction of
    \code{lambda.max}, the (data derived) entry value (i.e. the smallest
  value for which all coefficients are zero except the intercept). Note, there is no closed formula for \code{lambda.max}. The default of \code{lambda.min.ratio} depends on the
  sample size \code{nobs} relative to the number of variables
  \code{nvars}. If \code{nobs > nvars}, the default is \code{0.001},
  close to zero.  If \code{nobs < nvars}, the default is \code{0.05}.}
  \item{alpha}{The \eqn{L_2} penalty mixing parameter, with
    \eqn{0 \le alpha\le 1}. \code{alpha=1} is lasso (mcp, scad) penalty; and \code{alpha=0} the ridge penalty. However, if \code{alpha=0}, one must provide \code{lambda} values.}
  \item{gamma}{The tuning parameter of the \code{snet} or \code{mnet} penalty.}
  \item{rescale}{logical value, if TRUE, adaptive rescaling of the penalty       parameter for \code{penalty="mnet"} or \code{penalty="snet"} with               \code{dfun="binomial"}. See \code{glmreg_fit}}
  \item{standardize}{logical value for x variable standardization, prior to
    fitting the model sequence. The coefficients are always returned on
    the original scale. Default is \code{standardize=TRUE}.}
  \item{intercept}{logical value: if TRUE (default), intercept(s) are fitted; otherwise, intercept(s)  are set to zero}
  \item{penalty.factor}{This is a number that multiplies \code{lambda} to allow
    differential shrinkage of coefficients. Can be 0 for some variables, which implies
    no shrinkage, and that variable is always included in the
    model. Default is same shrinkage for all variables.}
 \item{type.init}{a method to determine the initial values. If \code{type.init="ncl"}, an intercept-only model as initial parameter and run \code{irglmreg} regularization path forward from lambda_max to lambda_min. If \code{type.init="heu"}, heuristic initial parameters and run \code{irglmreg} path backward or forward depending on \code{decreasing}, between lambda_min and lambda_max. If \code{type.init="bst"}, run a boosting model with \code{bst} in package bst, depending on \code{mstop.init, nu.init} and run \code{irglmreg} backward or forward depending on \code{decreasing}.}
 \item{init.family}{ character value for initial family, one of "clossR",     "closs","gloss","qloss", which can be used to derive an initial estimator, if   the selection is different from the default value }
 \item{mstop.init}{ an integer giving the number of boosting iterations when \code{type.init="bst"} }
 \item{nu.init}{ a small number (between 0 and 1) defining the step size or shrinkage parameter when \code{type.init="bst"}. }
 \item{decreasing}{ only used if \code{lambda=NULL}, a logical value used to determine regularization path direction either from lambda_max to a potentially modified lambda_min or vice versa if \code{type.init="bst", "heu"}. Since this is a nonconvex optimization, it is possible to generate different estimates for the same \code{lambda} depending on \code{decreasing}. The choice of \code{decreasing} picks different starting values.}
 \item{iter}{number of iteration in the IRCO algorithm}
 \item{maxit}{Within each IRCO algorithm iteration, maximum number of coordinate descent iterations for each \code{lambda} value; default is 1000.}
 \item{reltol}{convergency criteria in the IRCO algorithm} 
 \item{eps}{If a coefficient is less than \code{eps} in magnitude, then it is reported to be 0}
 \item{epscycle}{ If \code{nlambda} > 1 and the relative loss values from two consecutive \code{lambda} values change > \code{epscycle}, then re-estimate parameters in an effort to avoid trap of local optimization.}
 \item{thresh}{Convergence threshold for coordinate descent. Defaults value is \code{1e-6}.}
 \item{penalty}{Type of regularization}
 \item{theta}{ an overdispersion scaling parameter for \code{family="negbin"}}
 \item{parallel, n.cores}{If \code{TRUE}, to compute solution of \code{lambda} with parallel computing in number of \code{n.cores}. If \code{FALSE}, sequential computing. If \code{NULL}, still sequential computing with a different convergence criteria based on penalized loss values}
 \item{trace, tracelevel}{If \code{TRUE}, fitting progress is reported. If \code{tracelevel=2}, deeper level of fitting progress is reported.}
}

\details{
A case weighted penalized least squares or GLM is fit by the iteratively reweighted convex optimization (IRCO), where the loss function is a composite function \code{cfun}o\code{dfun} + penalty. Here convex is the loss function     induced by \code{dfun}, not the penalty function.
  The sequence of robust models implied by \code{lambda} is fit by IRCO along with coordinate
  descent. Note that the objective function is \deqn{weights*loss + \lambda*penalty,} if \code{standardize=FALSE} and \deqn{
  \frac{weights}{\sum(weights)}*loss + \lambda*penalty,} if \code{standardize=TRUE}. 
  }
\value{
An object with S3 class \code{"irglmreg"} for the various types of models.
  \item{call}{the call that produced the model fit}
  \item{b0}{Intercept sequence of length \code{length(lambda)}}
  \item{beta}{A \code{nvars x length(lambda)} matrix of coefficients.}
  \item{lambda}{The actual sequence of \code{lambda} values used}
  \item{weights_update}{A \code{nobs x length(lambda)} matrix of weights computed by the IRCO algorithm. The entry of i-th row and j-th column is the weight for the i-th observation and j-th \code{lambda} value.}
  \item{decreasing}{ if \code{lambda} is an increasing sequence or not, used to determine regularization path direction either from lambda_max to a potentially modified lambda_min or vice versa if \code{type.init="bst", "heu"}. }
}

\references{
   Zhu Wang (2024)
    \emph{Unified Robust Estimation}, \emph{Australian & New Zealand Journal of      Statistics}. 66(1):77-102.  
}

\author{Zhu Wang <zwang145@uthsc.edu>}

\seealso{\code{\link{irglmreg}}}
\keyword{models}
\keyword{regression}

 

\name{compute_wt}
\alias{compute_wt}
\title{Weight value from concave function}
\description{
 Weight value from concave function
}
\usage{
compute_wt(z, weights, cfun, s, delta=0.0001)
}
\arguments{
 \item{z}{vector nonnegative values from \code{dfun}, e.g., u^2/2}
 \item{weights}{optional numeric vector of weights.}
 \item{cfun}{integer from 1-8, concave function as in \code{irglm_fit}}
 \item{s}{a numeric value, see details in \code{irglm_fit}}
 \item{delta}{a positive small value, see details in \code{irglm_fit}}
}

\value{Weight value from concave function}

\seealso{\code{\link{irglmreg}}}
\examples{
u <- seq(0, 4, by=0.01)
z <- u^2/2 ### this is dfun
res <- compute_wt(z, cfun=1, s=1)
plot(z, res, ylab="Weight", type="l", lwd=2, 
     main=expression(paste("hcave", "   (", sigma, "=1)", )))
}
\references{
Zhu Wang (2024)
      \emph{Unified Robust Estimation}, \emph{Australian & New Zealand           Journal         of      Statistics}. 66(1):77-102.
}
\author{Zhu Wang <zwang145@uthsc.edu>}

\keyword{regression}
\keyword{classification}

\name{estfunReg}
\alias{estfunReg}
\alias{estfunReg.zipath}
\title{Extract Empirical First Derivative of Log-likelihood Function}
\description{
 Generic function for extracting the empirical first derivative of log-likelihood function of a fitted regularized model.
}
\usage{
estfunReg(x, \dots)
}
\arguments{
 \item{x}{a fitted model object.}
 \item{\dots}{arguments passed to methods.}
}

\value{A matrix containing the empirical first derivative of log-likelihood functions.
Typically, this should be an \eqn{n \times k}{n x k} matrix corresponding
to \eqn{n} observations and \eqn{k} parameters. The columns should be named
as in \code{\link{coef}} or \code{\link{terms}}, respectively.
}

\seealso{\code{\link{zipath}}}
\references{
Zhu Wang, Shuangge Ma and Ching-Yun Wang (2015) \emph{Variable selection for zero-inflated and overdispersed data with application to health care demand in Germany}, \emph{Biometrical Journal}. 57(5):867-84.
}
\author{Zhu Wang <zwang145@uthsc.edu>}

\examples{
data("bioChemists", package = "pscl")
fm_zinb <- zipath(art ~ . | ., data = bioChemists, family = "negbin", nlambda=10, maxit.em=1)
res <- estfunReg(fm_zinb, which=which.min(fm_zinb$bic))
}

\keyword{regression}

\name{mpath-internal}
\title{Internal mpath functions}
\alias{balanced.folds}
\alias{cv.folds}
\alias{findlam}
\alias{g}
\alias{init}
\alias{llfun}
\alias{pen_sum}
\alias{predictzeroinfl1}
\alias{cfun2num}
\alias{y2num}
\alias{y2num4glm}
\alias{check_s}
\description{Internal mpath functions}
\author{Zhu Wang, author of ncvreg, author of pamr:balanced.folds (see COPYRIGHTS)}
\details{These are not intended for use by users. \code{findlam} is to compute the maximum value of \code{lambda} so that the coefficient estimation is zero. \code{g} is to compute a link function (not used at this moment), \code{init} is to compute eta, which is the estimated beta_0 in the intercept-only model. \code{pen_sum} is to compute the total penalty value from the estimated coefficients, \code{llfun} to compute the log-likelihood function. \code{convexMin} (from ncvreg) calculates the
    lowest index for which the penalized objective function is locally convex. 

Functions not used anymore: pen_eval, pen_sum, pen_zipath and zipath.control
}

\keyword{internal}

\name{irsvm}
\alias{irsvm}
\alias{irsvm.formula}
\alias{irsvm.matrix}
\alias{irsvm.default}
\alias{coef.irsvm}

\title{fit case weighted support vector machines with robust loss functions}
\description{
    Fit case weighted support vector machines with robust loss functions. This is the wrapper function of \code{irsvm_fit}, which does the computing.
  }
\usage{
\method{irsvm}{formula}(formula, data, weights, contrasts=NULL, ...)
\method{irsvm}{matrix}(x, y, weights, ...)
\method{irsvm}{default}(x,  ...)
}

\arguments{
  \item{formula}{symbolic description of the model, see details.}
  \item{data}{argument controlling formula processing
    via \code{\link[stats]{model.frame}}.}
  \item{weights}{optional numeric vector of weights}
  \item{x}{input matrix, of dimension nobs x nvars; each row is an
  observation vector}
  \item{y}{response variable. Quantitative for \code{type="eps-regression", "nu-regression"} and -1/1 for \code{type="C-classification", "nu-Classification"}.}
   \item{contrasts}{the contrasts corresponding to \code{levels} from the
     respective models}
  \item{...}{Other arguments passing to \code{irsvm_fit}}
}
\details{
  Fit a robust SVM where the loss function is a composite function \code{cfun}o\code{type} + penalty.
    The model is fit by the iteratively reweighted SVM, an application of the iteratively reweighted convex optimization (IRCO). Here convex is the loss function  induced by \code{type}. 

  For linear kernel, the coefficients of the regression/decision hyperplane
   can be extracted using the \code{coef} method.
}

\value{
An object with S3 class \code{"wsvm"} for various types of models.
\item{call}{the call that produced this object}
\item{weights_update}{weights in the final iteration of the IRCO algorithm}
\item{cfun, s}{original input arguments}
\item{delta}{delta value used for \code{cfun="gcave"}}
}

\references{
   Zhu Wang (2024)
    \emph{Unified Robust Estimation}, \emph{Australian & New Zealand Journal of      Statistics}. 66(1):77-102.
}

\author{Zhu Wang <zwang145@uthsc.edu>}
\seealso{\code{\link{irsvm_fit}}, \code{\link{print}}, \code{\link{predict}}, \code{\link{coef}}.}
\examples{
#binomial
x=matrix(rnorm(100*20),100,20)
g2=sample(c(-1,1),100,replace=TRUE)
fit=irsvm(x,g2,s=1,cfun="ccave",type="C-classification")
}
\keyword{models}
\keyword{regression}

 

\name{zipath}
\alias{zipath}
\alias{zipath.formula}
\alias{zipath.matrix}
\alias{zipath.default}

\title{Fit zero-inflated count data linear model with lasso (or elastic net), snet or mnet regularization}
\description{
  Fit zero-inflated regression models for count data via penalized maximum likelihood.
}

\usage{
\method{zipath}{formula}(formula, data, weights, offset=NULL, contrasts=NULL, ... )
\method{zipath}{matrix}(X, Z, Y, weights, offsetx=NULL, offsetz=NULL, ...)
\method{zipath}{default}(X, ...)
}
\arguments{
  \item{formula}{symbolic description of the model, see details.}
  \item{data}{argument controlling formula processing
    via \code{\link[stats]{model.frame}}.}
  \item{weights}{optional numeric vector of weights.}
  \item{offset}{optional numeric vector with an a priori known component to be
    included in the linear predictor of the count model or zero model. See below for an example.}
  \item{contrasts}{a list with elements \code{"count"} and \code{"zero"}
     containing the contrasts corresponding to \code{levels} from the
     respective models}
\item{X}{predictor matrix of the count model}
   \item{Z}{predictor matrix of the zero model}
   \item{Y}{response variable}
  \item{offsetx, offsetz}{optional numeric vector with an a priori known component to be
    included in the linear predictor of the count model (offsetx)or zero model (offsetz).}
\item{...}{Other arguments which can be passed to \code{glmreg} or \code{glmregNB}}
}

\value{
  An object of class \code{"zipath"}, i.e., a list with components including
  \item{coefficients}{a list with elements \code{"count"} and \code{"zero"}
    containing the coefficients from the respective models,}
  \item{residuals}{a vector of raw residuals (observed - fitted),}
  \item{fitted.values}{a vector of fitted means,}
  \item{weights}{the case weights used,}
  \item{terms}{a list with elements \code{"count"}, \code{"zero"} and
    \code{"full"} containing the terms objects for the respective models,}
  \item{theta}{estimate of the additional \eqn{\theta}{theta} parameter of the
    negative binomial model (if a negative binomial regression is used),}
  \item{loglik}{log-likelihood of the fitted model,}
  \item{family}{character string describing the count distribution used,}
  \item{link}{character string describing the link of the zero-inflation model,}
  \item{linkinv}{the inverse link function corresponding to \code{link},}
  \item{converged}{logical value, TRUE indicating successful convergence of \code{zipath}, FALSE indicating otherwise}
  \item{call}{the original function call}
  \item{formula}{the original formula}
  \item{levels}{levels of the categorical regressors}
  \item{contrasts}{a list with elements \code{"count"} and \code{"zero"}
    containing the contrasts corresponding to \code{levels} from the
    respective models,}
  \item{model}{the full model frame (if \code{model = TRUE}),}
  \item{y}{the response count vector (if \code{y = TRUE}),}
  \item{x}{a list with elements \code{"count"} and \code{"zero"}
    containing the model matrices from the respective models
    (if \code{x = TRUE}),}
}

\references{     
Zhu Wang, Shuangge Ma, Michael Zappitelli, Chirag Parikh, Ching-Yun Wang and Prasad Devarajan (2014)      \emph{Penalized Count Data Regression with Application to Hospital Stay after Pediatric Cardiac Surgery}, \emph{Statistical Methods in Medical Research}. 2014 Apr 17. [Epub ahead of print]

Zhu Wang, Shuangge Ma, Ching-Yun Wang, Michael Zappitelli, Prasad Devarajan and Chirag R. Parikh (2014)
\emph{EM for Regularized Zero Inflated Regression Models with Applications to Postoperative Morbidity after Cardiac Surgery in Children}, \emph{Statistics in Medicine}. 33(29):5192-208.

Zhu Wang, Shuangge Ma and Ching-Yun Wang (2015) \emph{Variable selection for zero-inflated and overdispersed data with application to health care demand in Germany}, \emph{Biometrical Journal}. 57(5):867-84.
}

\author{Zhu Wang <zwang145@uthsc.edu>}

\seealso{\code{\link{zipath_fit}},
  \code{\link{glmreg}}, \code{\link{glmregNB}}
}

\examples{
## data
data("bioChemists", package = "pscl")
## with simple inflation (no regressors for zero component)
fm_zip <- zipath(art ~ 1 | ., data = bioChemists, nlambda=10)
summary(fm_zip)
fm_zip <- zipath(art ~ . | 1, data = bioChemists, nlambda=10)
summary(fm_zip)
\dontrun{
fm_zip <- zipath(art ~ . | 1, data = bioChemists, nlambda=10)
summary(fm_zip)
fm_zinb <- zipath(art ~ . | 1, data = bioChemists, family = "negbin", nlambda=10)
summary(fm_zinb)
## inflation with regressors
## ("art ~ . | ." is "art ~ fem + mar + kid5 + phd + ment | fem + mar + kid5 + phd + ment")
fm_zip2 <- zipath(art ~ . | ., data = bioChemists, nlambda=10)
summary(fm_zip2)
fm_zinb2 <- zipath(art ~ . | ., data = bioChemists, family = "negbin", nlambda=10)
summary(fm_zinb2)
### non-penalized regression, compare with zeroinfl
fm_zinb3 <- zipath(art ~ . | ., data = bioChemists, family = "negbin", 
lambda.count=0, lambda.zero=0, reltol=1e-12)
summary(fm_zinb3)
library("pscl")
fm_zinb4 <- zeroinfl(art ~ . | ., data = bioChemists, dist = "negbin")
summary(fm_zinb4)
### offset
exposure <- rep(0.5, dim(bioChemists)[1])
fm_zinb <- zipath(art ~ . +offset(log(exposure))| ., data = bioChemists, 
		  family = "poisson", nlambda=10)
coef <- coef(fm_zinb)
### offset can't be specified in predict function as it has been contained
pred <- predict(fm_zinb)
## without inflation
## ("art ~ ." is "art ~ fem + mar + kid5 + phd + ment")
fm_pois <- glmreg(art ~ ., data = bioChemists, family = "poisson")
coef <- coef(fm_pois)
fm_nb <- glmregNB(art ~ ., data = bioChemists)
coef <- coef(fm_nb)
### high-dimensional
#R CMD check --use-valgrind can be too time extensive for the following model
#bioChemists <- cbind(matrix(rnorm(915*100), nrow=915), bioChemists)
#fm_zinb <- zipath(art ~ . | ., data = bioChemists, family = "negbin", nlambda=10)
}
}

\keyword{regression}

\name{methods}

\alias{AIC.glmreg}
\alias{AIC.zipath}
\alias{BIC.glmreg}
\alias{BIC.zipath}

\title{ Methods for mpath Objects }
\description{
  Methods for models fitted by coordinate descent algorithms.
}
\usage{
\method{AIC}{glmreg}(object, ..., k)
\method{AIC}{zipath}(object, ..., k)
\method{BIC}{glmreg}(object, ...)
\method{BIC}{zipath}(object, ...)

}
\arguments{
  \item{object}{ objects of class \code{glmreg} or \code{zipath}.}
 \item{\dots}{ additional arguments passed to calls. }
  \item{k}{  numeric, the \emph{penalty} per parameter to be used; the default
             \code{k = 2} is the classical AIC. \code{k} has been hard coded in the function and there is no impact to the value of \code{AIC} if \code{k} is changed }
}
\references{
Zhu Wang, Shuangge Ma, Michael Zappitelli, Chirag Parikh, Ching-Yun Wang and Prasad Devarajan (2014)      \emph{Penalized Count Data Regression with Application to Hospital Stay after Pediatric Cardiac Surgery}, \emph{Statistical Methods in Medical Research}. 2014 Apr 17. [Epub ahead of print]

Zhu Wang, Shuangge Ma, Ching-Yun Wang, Michael Zappitelli, Prasad Devarajan and Chirag R. Parikh (2014)
\emph{EM for Regularized Zero Inflated Regression Models with Applications to Postoperative Morbidity after Cardiac Surgery in Children}, \emph{Statistics in Medicine}. 33(29):5192-208.

Zhu Wang, Shuangge Ma and Ching-Yun Wang (2015) \emph{Variable selection for zero-inflated and overdispersed data with application to health care demand in Germany}, \emph{Biometrical Journal}. 57(5):867-84.
}
\author{Zhu Wang <zwang145@uthsc.edu>}

\keyword{methods}

\name{cv.irsvm}
\alias{cv.irsvm}
\alias{cv.irsvm.formula}
\alias{cv.irsvm.matrix}
\alias{cv.irsvm.default}
\title{Cross-validation for irsvm}
\description{Does k-fold cross-validation for irsvm}
\usage{
\method{cv.irsvm}{formula}(formula, data, weights, contrasts=NULL, ...)
\method{cv.irsvm}{matrix}(x, y, weights, ...)
\method{cv.irsvm}{default}(x,  ...)
}
\arguments{
  \item{formula}{symbolic description of the model, see details.}
  \item{data}{argument controlling formula processing
    via \code{\link[stats]{model.frame}}.}
  \item{x}{\code{x} matrix as in \code{irsvm}.}
  \item{y}{response \code{y} as in \code{irsvm}.}
  \item{weights}{Observation weights; defaults to 1 per observation}
  \item{contrasts}{the contrasts corresponding to \code{levels} from the respective models.}
  \item{\dots}{Other arguments that can be passed to \code{irsvm}.}
}
\details{Does a K-fold cross-validation to determine optimal tuning parameters in SVM: \code{cost} and \code{gamma} if \code{kernel} is nonlinear. It can also choose \code{s} used in \code{cfun}. 
}
\value{An object contains a list of ingredients of cross-validation including optimal tuning parameters.
  \item{residmat}{matrix with row values for \code{kernel="linear"} are \code{s, cost, error, k}, where \code{k} is the number of cross-validation fold. For nonlinear kernels, row values are \code{s, gamma, cost, error, k}.}
  \item{cost}{a value of \code{cost} that gives minimum cross-validated value in \code{irsvm}.} 
  \item{gamma}{a value of \code{gamma} that gives minimum cross-validated value in \code{irsvm}}
  \item{s}{value of \code{s} for \code{cfun} that gives minimum cross-validated value in \code{irsvm}.}
}

\examples{
\dontrun{
x <- matrix(rnorm(40*2), ncol=2)
y <- c(rep(-1, 20), rep(1, 20))
x[y==1,] <- x[y==1, ] + 1
irsvm.opt <- cv.irsvm(x, y, type="C-classification", s=1, kernel="linear", cfun="acave")
irsvm.opt$cost
irsvm.opt$gamma
irsvm.opt$s
}
}
\references{
 Zhu Wang (2024)
     \emph{Unified Robust Estimation}, \emph{Australian & New Zealand Journal         of      Statistics}. 66(1):77-102.
}
\author{Zhu Wang <zwang145@uthsc.edu>}

\seealso{\code{\link{irsvm}}}
\keyword{models}
\keyword{regression}

\name{cv.zipath_fit}
\alias{cv.zipath_fit}
\title{Cross-validation for zipath}
\description{Internal function k-fold cross-validation for zipath, produces a plot, and returns cross-validated log-likelihood values for \code{lambda}}
\usage{
cv.zipath_fit(X, Z, Y, weights, offsetx, offsetz, nlambda=100, lambda.count=NULL,
              lambda.zero=NULL, nfolds=10, foldid, plot.it=TRUE, se=TRUE, 
              n.cores=2, trace=FALSE, parallel=FALSE, ...)
}
\arguments{
   \item{X}{predictor matrix of the count model}
   \item{Z}{predictor matrix of the zero model}
   \item{Y}{response variable}
   \item{weights}{optional numeric vector of weights.}
   \item{offsetx}{optional numeric vector with an a priori known component to be included in the linear predictor of the count model.}
   \item{offsetz}{optional numeric vector with an a priori known component to be included in the linear predictor of the zero model.}
  \item{nlambda}{number of \code{lambda} value, default value is 10.}
  \item{lambda.count}{Optional user-supplied lambda.count sequence; default is
    \code{NULL}}
  \item{lambda.zero}{Optional user-supplied lambda.zero sequence; default is
    \code{NULL}}
  \item{nfolds}{number of folds >=3, default is 10}
  \item{foldid}{an optional vector of values between 1 and \code{nfold}
    identifying what fold each observation is in. If supplied,
    \code{nfold} can be missing and will be ignored.}
 \item{plot.it}{ a logical value, to plot the estimated log-likelihood values if \code{TRUE}. }
  \item{se}{ a logical value, to plot with standard errors. }
 \item{n.cores}{The number of CPU cores to use. The cross-validation loop
  will attempt to send different CV folds off to different cores.}
\item{trace}{ a logical value, print progress of cross-validation or not }
 \item{parallel}{ a logical value, parallel computing or not}
  \item{\dots}{Other arguments that can be passed to \code{zipath}.}
}
\details{The function runs \code{zipath} \code{nfolds}+1 times; the
  first to compute the \code{(lambda.count, lambda.zero)} sequence, and then to
  compute the fit with each of the folds omitted. The log-likelihood value is
  accumulated, and the average value and standard deviation over the
  folds is computed. Note that \code{cv.zipath} can be used to search for
  values for \code{count.alpha} or \code{zero.alpha}: it is required to call \code{cv.zipath} with a fixed vector \code{foldid} for different values of \code{count.alpha} or \code{zero.alpha}.

The method for \code{\link[stats]{coef}} by default
  return a single vector of coefficients, i.e., all coefficients are concatenated. By setting the \code{model}
  argument, the estimates for the corresponding model components can be extracted.
}
\value{an object of class \code{"cv.zipath"} is returned, which is a
  list with the components of the cross-validation fit.
 \item{fit}{a fitted zipath object for the full data.}
 \item{residmat}{matrix for cross-validated log-likelihood at each \code{(count.lambda, zero.lambda)} sequence}
  \item{bic}{matrix of BIC values with row values for \code{lambda} and column values for \code{k}th cross-validation}
  \item{cv}{The mean cross-validated log-likelihood - a vector of length
    \code{length(count.lambda)}.}
  \item{cv.error}{estimate of standard error of \code{cv}.}
  \item{foldid}{an optional vector of values between 1 and \code{nfold}
    identifying what fold each observation is in.} 
 \item{lambda.which}{index of \code{(count.lambda, zero.lambda)} that gives maximum \code{cv}.}
 \item{lambda.optim}{value of \code{(count.lambda, zero.lambda)} that gives maximum \code{cv}.}
}
\references{
Zhu Wang, Shuangge Ma, Michael Zappitelli, Chirag Parikh, Ching-Yun Wang and Prasad Devarajan (2014)      \emph{Penalized Count Data Regression with Application to Hospital Stay after Pediatric Cardiac Surgery}, \emph{Statistical Methods in Medical Research}. 2014 Apr 17. [Epub ahead of print]

Zhu Wang, Shuangge Ma, Ching-Yun Wang, Michael Zappitelli, Prasad Devarajan and Chirag R. Parikh (2014)
\emph{EM for Regularized Zero Inflated Regression Models with Applications to Postoperative Morbidity after Cardiac Surgery in Children}, \emph{Statistics in Medicine}. 33(29):5192-208.

Zhu Wang, Shuangge Ma and Ching-Yun Wang (2015) \emph{Variable selection for zero-inflated and overdispersed data with application to health care demand in Germany}, \emph{Biometrical Journal}. 57(5):867-84.
}

\author{Zhu Wang <zwang145@uthsc.edu>}

\seealso{\code{\link{zipath}} and \code{\link{plot}}, \code{\link{predict}}, and \code{\link{coef}} methods for \code{"cv.zipath"} object.}
\keyword{models}
\keyword{regression}

\name{meatReg}
\alias{meatReg}
\title{Meat Matrix Estimator}
\description{
 Estimating the variance of the first derivative of log-likelihood function}
 
\usage{
meatReg(x, which, \dots)
}
\arguments{
 \item{x}{a fitted model object. Currently only implemented for \code{zipath} object with \code{family="negbin"}}
 \item{which}{which penalty parameter(s)?}
 \item{\dots}{arguments passed to the \code{\link{estfunReg}} function.}
}

\details{
See reference below
}

\value{A \deqn{k \times k}{k x k} covariance matrix of first derivative of log-likelihood function}

\seealso{\code{\link{sandwichReg}}, \code{\link{breadReg}}, \code{\link{estfunReg}}}

\references{
Zhu Wang, Shuangge Ma and Ching-Yun Wang (2015) \emph{Variable selection for zero-inflated and overdispersed data with application to health care demand in Germany}, \emph{Biometrical Journal}. 57(5):867-84.
}
\author{Zhu Wang <zwang145@uthsc.edu>}

\examples{
data("bioChemists", package = "pscl")
fm_zinb <- zipath(art ~ . | ., data = bioChemists, family = "negbin", nlambda=10, maxit.em=1)
meatReg(fm_zinb, which=which.min(fm_zinb$bic))
}

\keyword{regression}

\name{loss3}
\alias{loss3}
\title{Composite Loss Value for GLM}
\description{
 Compute composite loss value
}
\usage{
loss3(y, mu, theta, weights, cfun, family, s, delta)
}
\arguments{
 \item{y}{response variable values, 0/1 if \code{family=2}, or binomial}
 \item{mu}{response prediction of \code{y}. If \code{mu} is linear predictor, use function \code{loss2} instead}
 \item{theta}{scale parameter for \code{family=4}, negative binomial}
 \item{weights}{observation weights, same length as \code{y}}
 \item{cfun}{integer from 1-8, concave function as in \code{irglm_fit}}
 \item{family}{integer 2, 3 or 4, convex function binomial, Poisson or negative binomial, respectively}
\item{s}{ tuning parameter of \code{cfun}. \code{s > 0} and can be equal to 0 for \code{cfun="tcave"}.} 
   \item{delta}{a small positive number provided by user only if \code{cfun="gcave"} and \code{0 < s <1}}
}

\value{Weighted loss values}
\details{For large \code{s} values, the loss can be 0 with \code{cfun=2,3,4}, or "acave", "bcave", "ccave".}
\seealso{
    \code{\link{loss2}}
    \code{\link{irglm}}
    \code{\link{irglmreg}}
    \code{\link{loss2_irsvm}}
}
\references{
      Zhu Wang (2024)
    \emph{Unified Robust Estimation}, \emph{Australian & New Zealand Journal of      Statistics}. 66(1):77-102.
  }
\author{Zhu Wang <zwang145@uthsc.edu>}

\keyword{regression}
\keyword{classification}

\name{loss2_irsvm}
\alias{loss2_irsvm}
\title{Composite Loss Value for epsilon-insensitive Type}
\description{
 Compute composite loss value for epsilon-insensitive type function
}
\usage{
loss2_irsvm(y, f, weights, cfun, dfun, s, eps, delta=0.0001)
}
\arguments{
 \item{y}{response variable values}
 \item{f}{fitted values of \code{y}}
 \item{weights}{observation weights, same length as \code{y}}
 \item{cfun}{integer from 1-8, concave function as in \code{irsvm_fit}}
 \item{dfun}{integer value, only \code{dfun=2} is implemented for now. Convex function as in \code{irsvm_fit}}
  \item{s}{ tuning parameter of \code{cfun}. \code{s > 0} and can be equal to 0 for \code{cfun="tcave"}.}
    \item{delta}{a small positive number provided by user only if  \code{cfun="gcave"} and \code{0 < s <1}}
 \item{eps}{non-negative parameter for epsilon-insensitive loss}
}

\value{Weighted loss values}
\details{For large \code{s} values, the loss can be 0 with \code{cfun=2,3,4},   or "acave", "bcave", "ccave".}
\seealso{
    \code{\link{irglmreg}},
    \code{\link{loss2}}
}
\references{
      Zhu Wang (2024)
    \emph{Unified Robust Estimation}, \emph{Australian & New Zealand Journal of      Statistics}. 66(1):77-102.
  }
\author{Zhu Wang <zwang145@uthsc.edu>}

\keyword{regression}
\keyword{classification}

\name{conv2zipath}
\alias{conv2zipath}

\title{convert zeroinfl object to class zipath}
\description{convert zeroinfl object to class zipath, which then can be used to predict new data}

\usage{
conv2zipath(object, family=c("poisson", "negbin", "geometric"))
}
\arguments{
  \item{object}{an object of class zeroinfl}
  \item{family}{one of families in zeroinfl class}
}
\value{an object of class zipath}
\author{Zhu Wang <zwang145@uthsc.edu>}

\name{conv2glmreg}
\alias{conv2glmreg}

\title{convert glm object to class glmreg}
\description{convert glm object to class glmreg, which then can be used for other purposes}

\usage{
conv2glmreg(object, family=c("poisson", "negbin"))
}
\arguments{
  \item{object}{an object of class glm}
  \item{family}{one of families in glm class}
}
\value{an object of class glmreg}
\author{Zhu Wang <zwang145@uthsc.edu>}

\name{rzi}
\alias{rzi}

\title{random number generation of zero-inflated count response}

\description{
random number generation of zero-inflated count response
}
\usage{
rzi(n, x, z, a, b, theta=1, family=c("poisson", "negbin", "geometric"), infl=TRUE)
}

\arguments{
  \item{n}{sample size of random number generation}
  \item{x}{design matrix of count model}
  \item{z}{design matrix of zero model}
  \item{a}{coefficient vector for \code{x}, length must be the same as column size of \code{x}}
  \item{b}{coefficient vector for \code{z}, length must be the same as column size of \code{z}}
  \item{theta}{dispersion parameter for \code{family="negbin"}}
  \item{family}{distribution of count model}
  \item{infl}{logical value, if TRUE, zero-inflated count response}
}
\details{random number generation of zero-inflated count response} 
\value{numeric vector of zero-inflated count response}
\references{
Zhu Wang, Shuangge Ma, Ching-Yun Wang, Michael Zappitelli, Prasad Devarajan and Chirag R. Parikh (2014)
\emph{EM for Regularized Zero Inflated Regression Models with Applications to Postoperative Morbidity after Cardiac Surgery in Children}, \emph{Statistics in Medicine}. 33(29):5192-208.

Zhu Wang, Shuangge Ma and Ching-Yun Wang (2015) \emph{Variable selection for zero-inflated and overdispersed data with application to health care demand in Germany}, \emph{Biometrical Journal}. 57(5):867-84.
}

\author{Zhu Wang <zwang145@uthsc.edu>}

\keyword{models}
\keyword{regression}

 

\name{ncl}
\alias{ncl}
\alias{ncl.formula}
\alias{ncl.matrix}
\alias{ncl.default}

\title{fit a nonconvex loss based robust linear model}
\description{
    Fit a linear model via penalized nonconvex loss function.  
  }
\usage{
\method{ncl}{formula}(formula, data, weights, offset=NULL, contrasts=NULL, 
x.keep=FALSE, y.keep=TRUE, ...)
\method{ncl}{matrix}(x, y, weights, offset=NULL, ...)
\method{ncl}{default}(x,  ...)
}

\arguments{
  \item{formula}{symbolic description of the model, see details.}
  \item{data}{argument controlling formula processing
    via \code{\link[stats]{model.frame}}.}
  \item{weights}{optional numeric vector of weights. If \code{standardize=TRUE}, weights are renormalized to weights/sum(weights). If \code{standardize=FALSE}, weights are kept as original input}
  \item{x}{input matrix, of dimension nobs x nvars; each row is an
  observation vector}
  \item{y}{response variable. Quantitative for \code{rfamily="clossR"} and -1/1 for classification. }
  \item{offset}{Not implemented yet}
  \item{contrasts}{the contrasts corresponding to \code{levels} from the
    respective models}
  \item{x.keep, y.keep}{ For glmreg: logical values indicating whether the response
          vector and model matrix used in the fitting process should be
          returned as components of the returned value.
          For ncl_fit: x is a design matrix of dimension n * p,
          and x is a vector of observations of length n.}
  \item{...}{Other arguments passing to \code{ncl_fit}}
}
\details{
  The robust linear model is fit by majorization-minimization along with linear regression. Note that the objective function is \deqn{weights*loss}.
  }

\value{
An object with S3 class \code{"ncl"} for the various types of models.
  \item{call}{the call that produced this object}
  \item{fitted.values}{predicted values}
  \item{h}{ pseudo response values in the MM algorithm }
}

\references{
   Zhu Wang (2021),
   \emph{MM for Penalized Estimation},
   \emph{TEST},
   \doi{10.1007/s11749-021-00770-2}
}

\author{Zhu Wang <zwang145@uthsc.edu>}
\seealso{\code{\link{print}}, \code{\link{predict}}, \code{\link{coef}}.}
\examples{
#binomial
x=matrix(rnorm(100*20),100,20)
g2=sample(c(-1,1),100,replace=TRUE)
fit=ncl(x,g2,s=1,rfamily="closs")
}
\keyword{models}
\keyword{regression}

 

\name{stan}
\alias{stan}

\title{standardize variables}
\description{
 Standardize variables. For each column, return mean 0 and mean value of sum of squares = 1.
  }
\usage{stan(x, weights)}

\arguments{
  \item{x}{numeric variables, can be a matrix or vector}
  \item{weights}{numeric positive vector of weights}
}
\value{A list with the following items.
\item{x}{standardized variables with each column: mean value 0 and mean value of sum of squares = 1.}
\item{meanx}{a vector of means for each column in the original \code{x}}
\item{normx}{a vector of scales for each column in the original \code{x}}
}
\author{Zhu Wang <zwang145@uthsc.edu>}

\name{cv.glmreg}
\alias{cv.glmreg}
\alias{cv.glmreg.formula}
\alias{cv.glmreg.matrix}
\alias{cv.glmreg.default}
\alias{plot.cv.glmreg}
\alias{predict.cv.glmreg}
\alias{coef.cv.glmreg}
\title{Cross-validation for glmreg}
\description{Does k-fold cross-validation for glmreg, produces a plot,
  and returns cross-validated log-likelihood values for \code{lambda}}
\usage{
\method{cv.glmreg}{formula}(formula, data, weights, offset=NULL, contrasts=NULL, ...)
\method{cv.glmreg}{matrix}(x, y, weights, offset=NULL, ...)
\method{cv.glmreg}{default}(x,  ...)
\method{plot}{cv.glmreg}(x,se=TRUE,ylab=NULL, main=NULL, width=0.02, col="darkgrey", ...)
\method{predict}{cv.glmreg}(object, newx, ...)
\method{coef}{cv.glmreg}(object,which=object$lambda.which, ...)
}
\arguments{
  \item{formula}{symbolic description of the model, see details.}
  \item{data}{argument controlling formula processing
    via \code{\link[stats]{model.frame}}.}
  \item{x}{\code{x} matrix as in \code{glmreg}. It could be object of \code{cv.glmreg}.}
  \item{y}{response \code{y} as in \code{glmreg}.}
  \item{weights}{Observation weights; defaults to 1 per observation}
  \item{offset}{this can be used to specify an a priori known component to be included in the linear predictor during fitting. This should be NULL or a numeric vector of length equal to the number of cases. Currently only one offset term can be included in the formula.}
   \item{contrasts}{the contrasts corresponding to \code{levels} from the
     respective models}
  \item{object}{object of \code{cv.glmreg}}
  \item{newx}{Matrix of values at which predictions are to be made.  Not
    used for \code{type="coefficients"}}
  \item{which}{Indices of the penalty parameter \code{lambda} at which
    estimates are extracted. By default, the one which generates the optimal cross-validation value.}
  \item{se}{logical value, if \code{TRUE}, standard error curve is also plotted} 
  \item{ylab}{ylab on y-axis}
  \item{main}{title of plot}
  \item{width}{width of lines}
  \item{col}{color of standard error curve}
  \item{\dots}{Other arguments that can be passed to \code{glmreg}.}
}
\details{The function runs \code{glmreg} \code{nfolds}+1 times; the
  first to compute the \code{lambda} sequence, and then to
  compute the fit with each of the folds omitted. The error or the log-likelihood value is
  accumulated, and the average value and standard deviation over the
  folds is computed. Note that \code{cv.glmreg} can be used to search for
  values for \code{alpha}: it is required to call \code{cv.glmreg} with a fixed vector \code{foldid} for different values of \code{alpha}. 
}
\value{an object of class \code{"cv.glmreg"} is returned, which is a
  list with the ingredients of the cross-validation fit.
 \item{fit}{a fitted glmreg object for the full data.}
  \item{residmat}{matrix of log-likelihood values with row values for \code{lambda} and column values for \code{k}th cross-validation}
  \item{bic}{matrix of BIC values with row values for \code{lambda} and column values for \code{k}th cross-validation}
  \item{cv}{The mean cross-validated log-likelihood values - a vector of length
    \code{length(lambda)}.}
 \item{cv.error}{estimate of standard error of \code{cv}.}
  \item{foldid}{an optional vector of values between 1 and \code{nfold}
    identifying what fold each observation is in.}
 \item{lambda}{a vector of \code{lambda} values} 
  \item{lambda.which}{index of \code{lambda} that gives maximum \code{cv} value.}
  \item{lambda.optim}{value of \code{lambda} that gives maximum \code{cv} value.}
}
\references{Zhu Wang, Shuangge Ma, Michael Zappitelli, Chirag Parikh, Ching-Yun Wang and Prasad Devarajan (2014)      \emph{Penalized Count Data Regression with Application to Hospital Stay after Pediatric Cardiac Surgery}, \emph{Statistical Methods in Medical Research}. 2014 Apr 17. [Epub ahead of print]
}
\author{Zhu Wang <zwang145@uthsc.edu>}

\seealso{\code{\link{glmreg}} and \code{\link{plot}}, \code{\link{predict}}, and \code{\link{coef}} methods for \code{"cv.glmreg"} object.}
\examples{
data("bioChemists", package = "pscl")
fm_pois <- cv.glmreg(art ~ ., data = bioChemists, family = "poisson")
title("Poisson Family",line=2.5)
predict(fm_pois, newx=bioChemists[,-1])[1:4]
coef(fm_pois)
}
\keyword{models}
\keyword{regression}

\name{cv.nclreg_fit}
\alias{cv.nclreg_fit}
\title{Internal function of cross-validation for nclreg}
\description{Internal function to conduct k-fold cross-validation for nclreg, produces a plot,
  and returns cross-validated loss values for \code{lambda}}
\usage{
cv.nclreg_fit(x, y, weights, offset, lambda=NULL, balance=TRUE, 
              rfamily=c("clossR", "closs", "gloss", "qloss"), s=1.5, 
              nfolds=10, foldid, type = c("loss", "error"), 
              plot.it=TRUE, se=TRUE, n.cores=2, trace=FALSE, 
              parallel=FALSE, ...)
}
\arguments{
  \item{x}{\code{x} matrix as in \code{nclreg}.}
  \item{y}{response \code{y} as in \code{nclreg}.}
  \item{weights}{Observation weights; defaults to 1 per observation}
 \item{offset}{this can be used to specify an a priori known component to be   included in the linear predictor during fitting. This should be NULL or a numeric vector of length equal to the number of cases. Currently only one offset term can be included in the formula.}
  \item{lambda}{Optional user-supplied lambda sequence; default is
    \code{NULL}, and \code{nclreg} chooses its own sequence}
  \item{balance}{for \code{rfamily="closs", "gloss", "qloss"} only}
  \item{rfamily}{response variable distribution and nonconvex loss function}
  \item{s}{ nonconvex loss tuning parameter for robust regression and classification. }
  \item{nfolds}{number of folds >=3, default is 10}
  \item{foldid}{an optional vector of values between 1 and \code{nfold}
    identifying what fold each observation is in. If supplied,
    \code{nfold} can be missing and will be ignored.}
  \item{type}{ cross-validation criteria. For \code{type="loss"}, loss function values and \code{type="error"} is misclassification error. }
  \item{plot.it}{ a logical value, to plot the estimated loss values if \code{TRUE}. }
  \item{se}{ a logical value, to plot with standard errors. }
  \item{n.cores}{The number of CPU cores to use. The cross-validation loop
    will attempt to send different CV folds off to different cores.}
  \item{trace}{ a logical value, print progress of cross validation or not}
  \item{parallel}{ a logical value, parallel computing or not}
  \item{\dots}{Other arguments that can be passed to \code{nclreg}.}
}
\details{The function runs \code{nclreg} \code{nfolds}+1 times; the
  first to compute the \code{lambda} sequence, and then to
  compute the fit with each of the folds omitted. The error or the loss value is
  accumulated, and the average value and standard deviation over the
  folds is computed. Note that \code{cv.nclreg} can be used to search for
  values for \code{alpha}: it is required to call \code{cv.nclreg} with a fixed vector \code{foldid} for different values of \code{alpha}.
}

\value{an object of class \code{"cv.nclreg"} is returned, which is a
  list with the ingredients of the cross-validation fit.
  \item{fit}{a fitted nclreg object for the full data.}
  \item{residmat}{matrix of loss values or errors with row values for \code{lambda} and column values for \code{k}th cross-validation}
  \item{cv}{The mean cross-validated loss values or errors - a vector of length
    \code{length(lambda)}.}
  \item{cv.error}{estimate of standard error of \code{cv}.}
  \item{foldid}{an optional vector of values between 1 and \code{nfold}
    identifying what fold each observation is in.}
  \item{lambda}{a vector of \code{lambda} values} 
  \item{lambda.which}{index of \code{lambda} that gives minimum \code{cv} value.}
  \item{lambda.optim}{value of \code{lambda} that gives minimum \code{cv} value.}
}
\references{
  Zhu Wang (2021), 
  \emph{MM for Penalized Estimation}, 
  \emph{TEST},
  \doi{10.1007/s11749-021-00770-2}
 }

\author{Zhu Wang <zwang145@uthsc.edu>}
\seealso{\code{\link{nclreg}} and \code{\link{plot}}, \code{\link{predict}}, and \code{\link{coef}} methods for \code{"cv.nclreg"} object.}
\keyword{models}
\keyword{regression}

\name{hessianReg}
\alias{hessianReg}
\title{Hessian Matrix of Regularized Estimators}
\description{
 Constructing Hessian matrix for regularized regression parameters.
}
\usage{
hessianReg(x, which, \dots)
}
\arguments{
 \item{x}{a fitted model object.}
 \item{which}{which penalty parameter(s)?}
 \item{\dots}{arguments passed to the \code{meatReg} function.}
}

\details{\code{hessianReg} is a function to compute the Hessian matrix estimate of non-zero regularized estimators. Implemented only for \code{zipath} object with \code{family="negbin"} in the current version. 
}

\value{A matrix containing the Hessian matrix estimate for the non-zero parameters.}

\seealso{\code{\link{breadReg}}, \code{\link{meatReg}}}

\references{
Zhu Wang, Shuangge Ma and Ching-Yun Wang (2015) \emph{Variable selection for zero-inflated and overdispersed data with application to health care demand in Germany}, \emph{Biometrical Journal}. 57(5):867-84.
}
\author{Zhu Wang <zwang145@uthsc.edu>}

\examples{
data("bioChemists", package = "pscl")
fm_zinb <- zipath(art ~ . | ., data = bioChemists, family = "negbin", nlambda=10, maxit.em=1)
hessianReg(fm_zinb, which=which.min(fm_zinb$bic))
}

\keyword{regression}

\name{summary.glmregNB}
\alias{summary.glmregNB}
\alias{print.summary.glmregNB}
\title{
Summary Method Function for Objects of Class 'glmregNB'
}
\description{
Summary results of fitted penalized negative binomial regression model
}
\usage{
\method{summary}{glmregNB}(object, \dots)
}
\arguments{
\item{object}{
fitted model object of class \code{glmregNB}.
}
\item{\dots}{
arguments passed to or from other methods.
}}
\value{
Summary of fitted penalized negative binomial model
}
\details{
This function is a method for the generic function
\code{summary()} for class \code{"glmregNB"}.
It can be invoked by calling \code{summary(x)} for an
object \code{x} of the appropriate class, or directly by
calling \code{summary.glmregNB(x)} regardless of the
class of the object.
}
\references{
Zhu Wang, Shuangge Ma, Michael Zappitelli, Chirag Parikh, Ching-Yun Wang and Prasad Devarajan (2014)
\emph{Penalized Count Data Regression with Application to Hospital Stay after Pediatric Cardiac Surgery}, \emph{Statistical Methods in Medical Research}. 2014 Apr 17. [Epub ahead of print]
}

\author{Zhu Wang <zwang145@uthsc.edu>}
\seealso{
\code{\link{summary}}, \code{\link[MASS]{glm.nb}}}
\keyword{models}

\name{glmreg}
\alias{glmreg}
\alias{glmreg.formula}
\alias{glmreg.matrix}
\alias{glmreg.default}
\alias{logLik.glmreg}
\alias{deviance.glmreg}

\title{fit a GLM with lasso (or elastic net), snet or mnet regularization}
\description{
    Fit a generalized linear model via penalized maximum likelihood.  The
  regularization path is computed for the lasso (or elastic net penalty), scad (or snet) and mcp (or mnet penalty), at a grid
  of values for the regularization parameter lambda. Fits
  linear, logistic, Poisson and negative binomial (fixed scale parameter) regression models.
  }
\usage{
\method{glmreg}{formula}(formula, data, weights, offset=NULL, contrasts=NULL, 
x.keep=FALSE, y.keep=TRUE, ...)
\method{glmreg}{matrix}(x, y, weights, offset=NULL, ...)
\method{glmreg}{default}(x,  ...)
}

\arguments{
  \item{formula}{symbolic description of the model, see details.}
  \item{data}{argument controlling formula processing
    via \code{\link[stats]{model.frame}}.}
  \item{weights}{optional numeric vector of weights. If \code{standardize=TRUE}, weights are renormalized to weights/sum(weights). If \code{standardize=FALSE}, weights are kept as original input}
  \item{offset}{this can be used to specify an a priori known component to be included in the linear predictor during fitting. This should be NULL or a numeric vector of length equal to the number of cases. Currently only one offset term can be included in the formula.}
  \item{x}{input matrix, of dimension nobs x nvars; each row is an
  observation vector}
  \item{y}{response variable. Quantitative for \code{family="gaussian"}.
  Non-negative counts for \code{family="poisson"} or \code{family="negbin"}. For
  \code{family="binomial"} should be either a factor with two levels or
  a vector of proportions.}
  \item{x.keep, y.keep}{logical values: keep response variables or keep response variable?} 
  \item{contrasts}{the contrasts corresponding to \code{levels} from the
    respective models}
  \item{...}{Other arguments passing to \code{glmreg_fit}}
}
\details{
  The sequence of models implied by \code{lambda} is fit by coordinate
  descent. For \code{family="gaussian"} this is the lasso, mcp or scad sequence if
  \code{alpha=1}, else it is the enet, mnet or snet sequence.
 For the other families, this is a lasso (mcp, scad) or elastic net (mnet, snet) regularization path
  for fitting the generalized linear regression
  paths, by maximizing the appropriate penalized log-likelihood.
  Note that the objective function for \code{"gaussian"} is \deqn{1/2*
  weights*RSS + \lambda*penalty,} if \code{standardize=FALSE} and \deqn{1/2*
  \frac{weights}{\sum(weights)}*RSS + \lambda*penalty,} if \code{standardize=TRUE}. For the other models it is
  \deqn{-\sum (weights * loglik) + \lambda*penalty} if \code{standardize=FALSE} and \deqn{-\frac{weights}{\sum(weights)} * loglik + \lambda*penalty} if \code{standardize=TRUE}.
  }

\value{
An object with S3 class \code{"glmreg"} for the various types of models.
  \item{call}{the call that produced this object}
  \item{b0}{Intercept sequence of length \code{length(lambda)}}
  \item{beta}{A \code{nvars x
      length(lambda)} matrix of coefficients.}
  \item{lambda}{The actual sequence of \code{lambda} values used}
  \item{offset}{the offset vector used.}
  \item{resdev}{The computed deviance (for \code{"gaussian"}, this
      is the R-square). The deviance calculations incorporate weights if
  present in the model. The deviance is defined to be 2*(loglike_sat -
  loglike), where loglike_sat is the log-likelihood for the saturated
  model (a model with a free parameter per observation). }
    \item{nulldev}{Null deviance (per observation). This is defined to
  be  2*(loglike_sat -loglike(Null)); The NULL model refers to the
  intercept model.}
  \item{nobs}{number of observations}
  \item{pll}{penalized log-likelihood values for standardized coefficients in the IRLS iterations. For \code{family="gaussian"}, not implemented yet.}
  \item{pllres}{penalized log-likelihood value for the estimated model on the original scale of coefficients}
  \item{fitted.values}{the fitted mean values, obtained by transforming the linear predictors by the inverse of the link function. }
}
\references{
Breheny, P. and Huang, J. (2011) \emph{Coordinate descent
  algorithms for nonconvex penalized regression, with applications to
  biological feature selection}. \emph{Ann. Appl. Statist.}, \bold{5}: 232-253.

Zhu Wang, Shuangge Ma, Michael Zappitelli, Chirag Parikh, Ching-Yun Wang and Prasad Devarajan (2014) 
\emph{Penalized Count Data Regression with Application to Hospital Stay after Pediatric Cardiac Surgery}, \emph{Statistical Methods in Medical Research}.
2014 Apr 17. [Epub ahead of print]
}

\author{Zhu Wang <zwang145@uthsc.edu>}
\seealso{\code{\link{print}}, \code{\link{predict}}, \code{\link{coef}} and \code{\link{plot}} methods, and the \code{\link{cv.glmreg}} function.}
\examples{
#binomial
x=matrix(rnorm(100*20),100,20)
g2=sample(0:1,100,replace=TRUE)
fit2=glmreg(x,g2,family="binomial")
#poisson and negative binomial
data("bioChemists", package = "pscl")
fm_pois <- glmreg(art ~ ., data = bioChemists, family = "poisson")
coef(fm_pois)
fm_nb1 <- glmreg(art ~ ., data = bioChemists, family = "negbin", theta=1)
coef(fm_nb1)
#offset
x <- matrix(rnorm(100*20),100,20)
y <- rpois(100, lambda=1)
exposure <- rep(0.5, length(y))
fit2 <- glmreg(x,y, lambda=NULL, nlambda=10, lambda.min.ratio=1e-4, 
	       offset=log(exposure), family="poisson")
predict(fit2, newx=x, newoffset=log(exposure))
\dontrun{
fm_nb2 <- glmregNB(art ~ ., data = bioChemists)
coef(fm_nb2)
}
}
\keyword{models}
\keyword{regression}

 

\name{cv.glmreg_fit}
\alias{cv.glmreg_fit}
\title{Internal function of cross-validation for glmreg}
\description{Internal function to conduct k-fold cross-validation for glmreg, produces a plot,
  and returns cross-validated log-likelihood values for \code{lambda}}
\usage{
cv.glmreg_fit(x, y, weights, offset, lambda=NULL, balance=TRUE, 
              family=c("gaussian", "binomial", "poisson", "negbin"), 
              type=c("loss", "error"), nfolds=10, foldid, plot.it=TRUE,
              se=TRUE, n.cores=2, trace=FALSE, parallel=FALSE, ...)
}
\arguments{
  \item{x}{\code{x} matrix as in \code{glmreg}.}
  \item{y}{response \code{y} as in \code{glmreg}.}
  \item{weights}{Observation weights; defaults to 1 per observation}
  \item{offset}{this can be used to specify an a priori known component to be included in the linear predictor during fitting. This should be NULL or a numeric vector of length equal to the number of cases. Currently only one offset term can be included in the formula.}
  \item{lambda}{Optional user-supplied lambda sequence; default is
    \code{NULL}, and \code{glmreg} chooses its own sequence}
 \item{balance}{for \code{family="binomial"} only}
 \item{family}{response variable distribution}
\item{type}{ cross-validation criteria. For \code{type="loss"}, loss function (log-negative-likelihood) values and \code{type="error"} is misclassification error if \code{family="binomial"}. } 
 \item{nfolds}{number of folds >=3, default is 10}
  \item{foldid}{an optional vector of values between 1 and \code{nfold}
    identifying what fold each observation is in. If supplied,
    \code{nfold} can be missing and will be ignored.}
 \item{plot.it}{ a logical value, to plot the estimated log-likelihood values if \code{TRUE}. }
  \item{se}{ a logical value, to plot with standard errors. }
\item{parallel, n.cores}{a logical value, parallel computing or not with the number of CPU cores to use. The cross-validation loop will attempt to send different CV folds off to different cores.}
 \item{trace}{ a logical value, print progress of cross validation or not} 
\item{\dots}{Other arguments that can be passed to \code{glmreg}.}
}
\details{The function runs \code{glmreg} \code{nfolds}+1 times; the
  first to compute the \code{lambda} sequence, and then to
  compute the fit with each of the folds omitted. The error or the log-likelihood value is
  accumulated, and the average value and standard deviation over the
  folds is computed. Note that \code{cv.glmreg} can be used to search for
  values for \code{alpha}: it is required to call \code{cv.glmreg} with a fixed vector \code{foldid} for different values of \code{alpha}.
}

\value{an object of class \code{"cv.glmreg"} is returned, which is a
  list with the ingredients of the cross-validation fit.
 \item{fit}{a fitted glmreg object for the full data.}
  \item{residmat}{matrix of log-likelihood values with row values for \code{lambda} and column values for \code{k}th cross-validation}
  \item{cv}{The mean cross-validated log-likelihood values - a vector of
    \code{length(lambda)}.}
 \item{cv.error}{estimate of standard error of \code{cv}.}
  \item{foldid}{an optional vector of values between 1 and \code{nfold}
    identifying what fold each observation is in.}
  \item{lambda}{a vector of \code{lambda} values}
  \item{lambda.which}{index of \code{lambda} that gives maximum \code{cv} value.}
  \item{lambda.optim}{value of \code{lambda} that gives maximum \code{cv} value.}
}
\references{
Zhu Wang, Shuangge Ma, Michael Zappitelli, Chirag Parikh, Ching-Yun Wang and Prasad Devarajan (2014)
\emph{Penalized Count Data Regression with Application to Hospital Stay after Pediatric Cardiac Surgery}, \emph{Statistical Methods in Medical Research}. 2014 Apr 17. [Epub ahead of print]
}

\author{Zhu Wang <zwang145@uthsc.edu>}
\seealso{\code{\link{glmreg}} and \code{\link{plot}}, \code{\link{predict}}, and \code{\link{coef}} methods for \code{"cv.glmreg"} object.}
\keyword{models}
\keyword{regression}

\name{irglm}
\alias{irglm}
\alias{irglm.formula}

\title{fit a robust generalized linear models}
\description{
    Fit a robust GLM where the loss function is a composite function \code{cfun}o\code{dfun}.
  }
\usage{
\method{irglm}{formula}(formula, data, weights, offset=NULL, contrasts=NULL,
 cfun="ccave", dfun=gaussian(), s=NULL, delta=0.1, fk=NULL, init.family=NULL,
 iter=10, reltol=1e-5, theta, x.keep=FALSE, y.keep=TRUE, trace=FALSE, ...)
}

\arguments{
  \item{formula}{symbolic description of the model, see details.}
  \item{data}{argument controlling formula processing
    via \code{\link[stats]{model.frame}}.}
  \item{weights}{optional numeric vector of weights.}
  \item{x}{input matrix, of dimension nobs x nvars; each row is an
  observation vector}
  \item{y}{response variable. Quantitative for \code{dfun=1} and -1/1 for classification. }
  \item{contrasts}{the contrasts corresponding to \code{levels} from the
    respective models}
  \item{offset}{this can be used to specify an a priori known component to be included in the linear predictor during fitting. This should be NULL or a numeric vector of length equal to the number of cases. Currently only one offset term can be included in the formula.}
   \item{cfun}{ character, type of concave function, which has a cap shape.\cr
   Valid options are:
   \itemize{
       \item \code{"hcave"}
       \item \code{"acave"}
       \item \code{"bcave"}
       \item \code{"ccave"}
       \item \code{"dcave"}
       \item \code{"ecave"}
       \item \code{"gcave"}
       \item \code{"tcave"}
   }
   }
   \item{dfun}{ character, type of convex component. Convex is often referred to as convex down: if you connect any two points on the curve by a line segment then the segment is always below the curve. \cr
   Valid options are:
   \itemize{
       \item \code{gaussian()}
       \item \code{binomial()}
       \item \code{poisson()}
       }
   }
   \item{init.family}{ character value for initial family, one of "clossR","closs","gloss","qloss", which can be used to derive an initial estimator, if the selection is different from the default value }
  \item{s}{ tuning parameter of \code{cfun}. \code{s > 0} and can be equal to 0 for \code{cfun="tcave"}. If \code{s} is too close to 0 for  \code{cfun="acave", "bcave", "ccave"}, the calculated weights can become 0 for all observations, thus crash the program.} 
 \item{delta}{a small positive number provided by user only if \code{cfun="gcave"} and \code{0 < s <1}}
  \item{fk}{ predicted values at an iteration in the IRGLM algorithm }
 \item{iter}{number of iteration in the IRGLM algorithm}
 \item{reltol}{convergency criteria in the IRGLM algorithm}
\item{theta}{ an overdispersion scaling parameter for \code{family=negbin()}} 
  \item{x.keep, y.keep}{ logical values indicating whether the response
          vector and model matrix used in the fitting process should be
          returned as components of the returned value, x is a design matrix of dimension n * p,
          and x is a vector of observations of length n.}
 \item{trace}{if \code{TRUE}, fitting progress is reported}
  \item{...}{other arguments passing to \code{irglm}}
}
\details{
  A robust linear, logistic or Poisson regression model is fit by the iteratively reweighted GLM (IRGLM). The output \code{weights_update} is a useful diagnostic to the outlier status of the observations.
  }
\value{
An object with S3 class \code{"irglm", "glm"} for various types of models.
  \item{call}{the call that produced the model fit}
  \item{weights}{original weights used in the model}
  \item{weights_update}{weights in the final iteration of the IRGLM algorithm}
  \item{cfun, s, dfun}{original input arguments}
  \item{is.offset}{is offset used?}
}

\references{
 Zhu Wang (2024)
     \emph{Unified Robust Estimation}, \emph{Australian & New Zealand Journal         of      Statistics}. 66(1):77-102.
}

\author{Zhu Wang <zwang145@uthsc.edu>}
\seealso{\code{\link{print}}, \code{\link{predict}}, \code{\link{coef}}.}
\examples{
x=matrix(rnorm(100*20),100,20)
g2=sample(c(-1,1),100,replace=TRUE)
fit=irglm(g2~x,data=data.frame(cbind(x, g2)), s=1, cfun="ccave", dfun=gaussian())
fit$weights_update
}
 
\keyword{models}
\keyword{regression}

 

\name{tuning.zipath}
\alias{tuning.zipath}

\title{find optimal path for penalized zero-inflated model}

\description{
Fit penalized zero-inflated models, generate multiple paths with varying penalty parameters, therefore determine optimal path with respect to a particular penalty parameter
  }
\usage{
tuning.zipath(formula, data, weights, subset, na.action, offset, standardize=TRUE,
              family = c("poisson", "negbin", "geometric"), 
              penalty = c("enet", "mnet", "snet"), lambdaCountRatio = .0001, 
              lambdaZeroRatio = c(.1, .01, .001), maxit.theta=1, gamma.count=3, 
              gamma.zero=3, ...)
}

\arguments{
  \item{formula}{symbolic description of the model, see details.}
  \item{data}{argument controlling formula processing
    via \code{\link[stats]{model.frame}}.}
  \item{weights}{optional numeric vector of weights. If \code{standardize=TRUE}, weights are renormalized to weights/sum(weights). If \code{standardize=FALSE}, weights are kept as original input}
  \item{subset}{subset of data}
  \item{na.action}{how to deal with missing data}
  \item{offset}{Not implemented yet}
  \item{standardize}{logical value, should variables be standardized?}
  \item{family}{family to fit}
  \item{penalty}{penalty considered as one of \code{enet, mnet, snet}.}
 \item{lambdaCountRatio, lambdaZeroRatio}{Smallest value for \code{lambda.count}
       and \code{lambda.zero}, respectively, as a fraction of
    \code{lambda.max}, the (data derived) entry value (i.e. the smallest
  value for which all coefficients are zero except the intercepts). This lambda.max can be a surrogate value for \code{penalty="mnet"} or \code{"snet"}}
 \item{maxit.theta}{For family="negbin", the maximum iteration allowed for estimating scale parameter theta. Note, the default value 1 is for computing speed purposes, and is typically too small and less desirable in real data analysis}
  \item{gamma.count}{The tuning parameter of the \code{snet} or \code{mnet} penalty for the count part of model.}
  \item{gamma.zero}{The tuning parameter of the \code{snet} or \code{mnet} penalty for the zero part of model.}
  \item{...}{Other arguments passing to \code{zipath}}
}
\details{
From the default \code{lambdaZeroRatio = c(.1, .01, .001)} values, 
find optimal lambdaZeroRatio for penalized zero-inflated Poisson, negative binomial and geometric model 
  }
\value{
  An object of class zipath with the optimal lambdaZeroRatio
}
\references{
Zhu Wang, Shuangge Ma, Michael Zappitelli, Chirag Parikh, Ching-Yun Wang and Prasad Devarajan (2014)      \emph{Penalized Count Data Regression with Application to Hospital Stay after Pediatric Cardiac Surgery}, \emph{Statistical Methods in Medical Research}. 2014 Apr 17. [Epub ahead of print]

Zhu Wang, Shuangge Ma, Ching-Yun Wang, Michael Zappitelli, Prasad Devarajan and Chirag R. Parikh (2014)
\emph{EM for Regularized Zero Inflated Regression Models with Applications to Postoperative Morbidity after Cardiac Surgery in Children}, \emph{Statistics in Medicine}. 33(29):5192-208.

Zhu Wang, Shuangge Ma and Ching-Yun Wang (2015) \emph{Variable selection for zero-inflated and overdispersed data with application to health care demand in Germany}, \emph{Biometrical Journal}. 57(5):867-84.
}

\author{Zhu Wang <zwang145@uthsc.edu>}
\seealso{\code{\link{zipath}}}
\examples{
\dontrun{
## data
data("bioChemists", package = "pscl")

## inflation with regressors
## ("art ~ . | ." is "art ~ fem + mar + kid5 + phd + ment | fem + mar + kid5 + phd + ment")
fm_zip2 <- tuning.zipath(art ~ . | ., data = bioChemists, nlambda=10)
summary(fm_zip2)
fm_zinb2 <- tuning.zipath(art ~ . | ., data = bioChemists, family = "negbin", nlambda=10)
summary(fm_zinb2)
}
}


\keyword{models}
\keyword{regression}

 

\name{glmreg_fit}
\alias{glmreg_fit}
\title{Internal function to fit a GLM with lasso (or elastic net), snet and mnet regularization}
\description{
    Fit a generalized linear model via penalized maximum likelihood.  The
  regularization path is computed for the lasso (or elastic net penalty), snet and mnet penalty, at a grid
  of values for the regularization parameter lambda. Fits
  linear, logistic, Poisson and negative binomial (fixed scale parameter) regression models.
  }
\usage{
glmreg_fit(x, y, weights, start=NULL, etastart=NULL, mustart=NULL, offset = NULL, 
           nlambda=100, lambda=NULL, lambda.min.ratio=ifelse(nobs<nvars,.05, .001), 
           alpha=1, gamma=3, rescale=TRUE, standardize=TRUE, intercept=TRUE, 
           penalty.factor = rep(1, nvars), thresh=1e-6, eps.bino=1e-5, maxit=1000, 
           eps=.Machine$double.eps, theta, 
           family=c("gaussian", "binomial", "poisson", "negbin"),
           penalty=c("enet","mnet","snet"), convex=FALSE, x.keep=FALSE, y.keep=TRUE,
           trace=FALSE)
}

\arguments{
  \item{x}{input matrix, of dimension nobs x nvars; each row is an
  observation vector.} 
  \item{y}{response variable. Quantitative for \code{family="gaussian"}.
  Non-negative counts for \code{family="poisson"} or \code{family="negbin"}. For
  \code{family="binomial"} should be either a factor with two levels or
  a vector of proportions.} 
  \item{weights}{observation weights. Can be total counts if responses are proportion matrices. Default is 1 for each observation}
  \item{start}{starting values for the parameters in the linear predictor.}
  \item{etastart}{starting values for the linear predictor.}
  \item{mustart}{starting values for the vector of means.}
  \item{offset}{this can be used to specify an a priori known component to be included in the linear predictor during fitting. This should be NULL or a numeric vector of length equal to the number of cases. Currently only one offset term can be included in the formula.}
  \item{nlambda}{The number of \code{lambda} values - default is 100. The sequence may be truncated before \code{nlambda} is reached if a close to saturated model is fitted. See also \code{satu}.}
  \item{lambda}{ by default, the algorithm provides a sequence of regularization values, or a user supplied \code{lambda} sequence. When \code{alpha=0}, the largest lambda value is not defined (infinity). Thus, the largest \code{lambda} for \code{alpha=0.001} is computed, and the sequence of \code{lambda} values is calculated afterwards. }
  \item{lambda.min.ratio}{Smallest value for \code{lambda}, as a fraction of
    \code{lambda.max}, the (data derived) entry value (i.e. the smallest
  value for which all coefficients are zero except the intercept). Note, there is no closed formula for \code{lambda.max} in general. If \code{rescale=TRUE}, \code{lambda.max} is the same for \code{penalty="mnet" or "snet"}. Otherwise, some modifications are required. For instance, for small \code{gamma} value, half of the square root (if \code{lambda.max} is too small) of the computed \code{lambda.max} can be used when \code{penalty="mnet" or "snet"}. The default of \code{lambda.min.ratio} depends on the
  sample size \code{nobs} relative to the number of variables
  \code{nvars}. If \code{nobs > nvars}, the default is \code{0.001},
  close to zero.  If \code{nobs < nvars}, the default is \code{0.05}.}
  \item{alpha}{The \eqn{L_2} penalty mixing parameter, with
    \eqn{0 \le alpha\le 1}. \code{alpha=1} is lasso (mcp, scad) penalty; and \code{alpha=0} the ridge penalty. However, if \code{alpha=0}, one must provide \code{lambda} values.}
 \item{gamma}{The tuning parameter of the \code{snet} or \code{mnet} penalty.}
 \item{rescale}{logical value, if TRUE, adaptive rescaling of the penalty parameter for \code{penalty="mnet"} or \code{penalty="snet"} with \code{family} other than "gaussian". See reference}
 \item{standardize}{logical value for x variable standardization, prior to
    fitting the model sequence. The coefficients are always returned on
    the original scale. Default is \code{standardize=TRUE}.}
 \item{intercept}{logical value: if TRUE (default), intercept(s) are fitted; otherwise, intercept(s) are set to zero}
 \item{penalty.factor}{This is a number that multiplies \code{lambda} to allow
    differential shrinkage of coefficients. Can be 0 for some variables, which implies
    no shrinkage, and that variable is always included in the
    model. Default is same shrinkage for all variables.}
 \item{thresh}{Convergence threshold for coordinate descent. Defaults value is \code{1e-6}.}
 \item{eps.bino}{a lower bound of probabilities to be truncated, for computing weights and related values when \code{family="binomial"}. It is also used when \code{family="negbin"}.}
 \item{maxit}{Maximum number of coordinate descent iterations for each \code{lambda} value; default is 1000.}
 \item{eps}{If a coefficient is less than \code{eps} in magnitude, then it is reported to be 0}
 \item{convex}{Calculate index for which objective function ceases to
    be locally convex? Default is FALSE and only useful if \code{penalty="mnet" or "snet"}.}
 \item{theta}{ an overdispersion scaling parameter for \code{family="negbin"}}
 \item{family}{Response type (see above)}
 \item{penalty}{Type of regularization}
 \item{x.keep, y.keep}{ For glmreg: logical values indicating whether the response
          vector and model matrix used in the fitting process should be
          returned as components of the returned value.
          For glmreg_fit: x is a design matrix of dimension n * p,
          and x is a vector of observations of length n.}
 \item{trace}{If \code{TRUE}, fitting progress is reported}
}
\details{
  The sequence of models implied by \code{lambda} is fit by coordinate
  descent. For \code{family="gaussian"} this is the lasso, mcp or scad sequence if
  \code{alpha=1}, else it is the enet, mnet or snet sequence.
 For the other families, this is a lasso (mcp, scad) or elastic net (mnet, snet) regularization path
  for fitting the generalized linear regression
  paths, by maximizing the appropriate penalized log-likelihood.
  Note that the objective function for \code{"gaussian"} is \deqn{1/2*
  weights*RSS + \lambda*penalty,} if \code{standardize=FALSE} and \deqn{1/2*
  \frac{weights}{\sum(weights)}*RSS + \lambda*penalty,} if \code{standardize=TRUE}. For the other models it is
  \deqn{-\sum (weights * loglik) + \lambda*penalty} if \code{standardize=FALSE} and \deqn{-\frac{weights}{\sum(weights)} * loglik + \lambda*penalty} if \code{standardize=TRUE}. 
  }
\value{
An object with S3 class \code{"glmreg"} for the various types of models.
  \item{call}{the call that produced the model fit}
  \item{b0}{Intercept sequence of length \code{length(lambda)}}
  \item{beta}{A \code{nvars x length(lambda)} matrix of coefficients.}
  \item{lambda}{The actual sequence of \code{lambda} values used}
  \item{satu}{satu=1 if a saturated model (deviance/null deviance < 0.05) is fit. Otherwise satu=0. The number of \code{nlambda} sequence may be truncated before \code{nlambda} is reached if \code{satu=1}.}
  \item{dev}{The computed deviance (for \code{"gaussian"}, this
      is the R-square). The deviance calculations incorporate weights if
  present in the model. The deviance is defined to be 2*(loglike_sat -
  loglike), where loglike_sat is the log-likelihood for the saturated
  model (a model with a free parameter per observation). }
    \item{nulldev}{Null deviance (per observation). This is defined to
  be  2*(loglike_sat -loglike(Null)); The NULL model refers to the
  intercept model.}
  \item{nobs}{number of observations}
}
\references{
Breheny, P. and Huang, J. (2011) \emph{Coordinate descent
  algorithms for nonconvex penalized regression, with applications to
  biological feature selection}. \emph{Ann. Appl. Statist.}, \bold{5}: 232-253.

Zhu Wang, Shuangge Ma, Michael Zappitelli, Chirag Parikh, Ching-Yun Wang and Prasad Devarajan (2014)
\emph{Penalized Count Data Regression with Application to Hospital Stay after Pediatric Cardiac Surgery}, \emph{Statistical Methods in Medical Research}. 2014 Apr 17. [Epub ahead of print]
}

\author{Zhu Wang <zwang145@uthsc.edu>}

\seealso{\code{\link{glmreg}}}
\keyword{models}
\keyword{regression}

 

\name{zipath_fit}
\alias{zipath_fit}

\title{Internal function to fit zero-inflated count data linear model with lasso (or elastic net), snet or mnet regularization}
\description{
  Fit zero-inflated regression models for count data via penalized maximum likelihood.
}

\usage{
zipath_fit(X, Z, Y, weights, offsetx, offsetz, standardize=TRUE, 
           intercept = TRUE, family = c("poisson", "negbin", "geometric"), 
           link = c("logit", "probit", "cloglog", "cauchit", "log"), 
           penalty = c("enet", "mnet",  "snet"), start = NULL, y = TRUE, 
           x = FALSE, nlambda=100, lambda.count=NULL, lambda.zero=NULL, 
           type.path=c("active", "nonactive"), penalty.factor.count=NULL, 
           penalty.factor.zero=NULL, lambda.count.min.ratio=.0001, 
           lambda.zero.min.ratio=.1, alpha.count=1, alpha.zero=alpha.count, 
           gamma.count=3, gamma.zero=gamma.count, rescale=FALSE, 
           init.theta=NULL, theta.fixed=FALSE, EM=TRUE, maxit.em=200, 
           convtype=c("count", "both"), maxit= 1000, maxit.theta =10, 
           reltol = 1e-5, thresh=1e-6, eps.bino=1e-5, shortlist=FALSE, 
           trace=FALSE, ...)
}
\arguments{
  \item{X}{predictor matrix of the count model}
  \item{Z}{predictor matrix of the zero model}
  \item{Y}{response variable}
  \item{weights}{optional numeric vector of weights.}
  \item{offsetx}{optional numeric vector with an a priori known component to be
    included in the linear predictor of the count model.}
  \item{offsetz}{optional numeric vector with an a priori known component to be
    included in the linear predictor of the zero model.}
  \item{intercept}{Should intercept(s) be fitted (default=TRUE) or set to
   zero (FALSE)}
  \item{standardize}{Logical flag for x variable standardization, prior to
    fitting the model sequence. The coefficients are always returned on
    the original scale. 
    Default is \code{standardize=TRUE}.}
  \item{family}{character specification of count model family (a log link is 
    always used).}
  \item{link}{character specification of link function in the binary
    zero-inflation model (a binomial family is always used).}
  \item{y, x}{logicals. If \code{TRUE} the corresponding response and model matrix are returned.}
  \item{penalty}{penalty considered as one of \code{enet, mnet, snet}.}
  \item{start}{starting values for the parameters in the linear predictor.}
  \item{nlambda}{number of \code{lambda} value, default value is 100. The sequence may be truncated before \code{nlambda} is reached if a close to saturated model for the zero component is fitted.}
  \item{lambda.count}{A user supplied \code{lambda.count} sequence. Typical usage is to have the program compute its own \code{lambda.count} and \code{lambda.zero} sequence based on
    \code{nlambda} and \code{lambda.min.ratio}.} 
  \item{lambda.zero}{A user supplied \code{lambda.zero} sequence.}
\item{type.path}{solution path with default value \code{"active"}, which is less time computing than \code{"nonactive"}. If \code{type.path="nonactive"}, no active set for each element of the lambda sequence and cycle through all the predictor variables. 
    If \code{type.path="active"}, then cycle through only the active set, then cycle through all the variables for the same penalty parameter. See details below.} 
  \item{penalty.factor.count, penalty.factor.zero}{These are numeric vectors with the same length as predictor variables. that multiply \code{lambda.count, lambda.zero}, respectively, to allow differential shrinkage of coefficients. Can be 0 for some variables, which implies
    no shrinkage, and that variable is always included in the
    model. Default is same shrinkage for all variables.}
 \item{lambda.count.min.ratio, lambda.zero.min.ratio}{Smallest value for \code{lambda.count} 
       and \code{lambda.zero}, respectively, as a fraction of
    \code{lambda.max}, the (data derived) entry value (i.e. the smallest
  value for which all coefficients are zero except the intercepts).
Note, there is a closed formula for \code{lambda.max} for \code{penalty="enet"}. If \code{rescale=TRUE}, \code{lambda.max} is the same for \code{penalty="mnet" or "snet"}. Otherwise, some modifications are required. In the current implementation, for small \code{gamma} value, the square root of the computed \code{lambda.zero[1]} is used when \code{penalty="mnet" or "snet"}. 
}
  \item{alpha.count}{The elastic net mixing parameter for the count part of model. The default value 1 implies no L_2 penalty, as in LASSO.}  
  \item{alpha.zero}{The elastic net mixing parameter for the zero part of model. The default value 1 implies no L_2 penalty, as in LASSO.}  
  \item{gamma.count}{The tuning parameter of the \code{snet} or \code{mnet} penalty for the count part of model.}
  \item{gamma.zero}{The tuning parameter of the \code{snet} or \code{mnet} penalty for the zero part of model.}
  \item{rescale}{logical value, if TRUE, adaptive rescaling}
  \item{init.theta}{The initial value of \code{theta} for \code{family="negbin"}. This is set to \code{NULL} since version 0.3-24.} 
  \item{theta.fixed}{Logical value only used for \code{family="negbin"}. If \code{TRUE} and \code{init.theta} is provided with a numeric value > 0, then \code{init.theta} is not updated. 
      If \code{theta.fixed=FALSE}, then \code{init.theta} will be updated. In this case, if \code{init.theta=NULL}, its initial value is computed with intercept-only zero-inflated negbin model. }
\item{EM}{Using \code{EM} algorithm. Not implemented otherwise}
\item{convtype}{convergency type, default is for count component only for speedy computation} 
\item{maxit.em}{Maximum number of EM algorithm}
\item{maxit}{Maximum number of coordinate descent algorithm}
 \item{maxit.theta}{Maximum number of iterations for estimating \code{theta} scaling parameter if family="negbin". Default value \code{maxit.theta} may be increased, yet may slow the algorithm}
 \item{eps.bino}{a lower bound of probabilities to be claimed as zero, for computing weights and related values when \code{family="binomial"}.}
\item{reltol}{Convergence criteria, default value 1e-5 may be reduced to make more accurate yet slow}
\item{thresh}{Convergence threshold for coordinate descent. Defaults value is \code{1e-6}.}
\item{shortlist}{logical value, if TRUE, limited results return}
\item{trace}{If \code{TRUE}, progress of algorithm is reported}
\item{...}{Other arguments which can be passed to \code{glmreg} or \code{glmregNB}}
}

\details{
  The algorithm fits penalized zero-inflated count data regression models using the coordinate descent algorithm within the EM algorithm.
  The returned fitted model object is of class \code{"zipath"} and is similar
  to fitted \code{"glm"} and \code{"zeroinfl"} objects. For elements such as \code{"coefficients"} a list is returned with elements for the zero and count component,
  respectively. 
 
  If \code{type.path="active"}, the algorithm iterates for a pair (lambda_count, lambda_zero) in a loop:\cr
  Step 1: For initial coefficients start_count of the count model and start_zero of the zero model, the EM algorithm is iterated until convergence for the active set with non-zero coefficients determined from start_count and start_zero, respectively.\cr
  Step 2: EM is iterated for all the predict variables once.\cr
  Step 3: If active set obtained from Step 2 is the same as in Step 1, stop; otherwise, repeat Step 1 and Step 2.\cr
  If \code{type.path="nonactive"}, the EM algorithm iterates for a pair (lambda_count, lambda_zero) with all the predict variables until convergence. 

  A set of standard extractor functions for fitted model objects is available for
  objects of class \code{"zipath"}, including methods to the generic functions
  \code{\link[base]{print}}, \code{\link[stats]{coef}}, 
  \code{\link[stats]{logLik}}, \code{\link[stats]{residuals}}, 
  \code{\link[stats]{predict}}. 
  See \code{\link{predict.zipath}} for more details on all methods.

  The program may terminate with the following message:

\code{Error in: while (j <= maxit.em && !converged)} \code{\{ :}\cr
\code{ Missing value, where TRUE/FALSE is necessary}\cr
\code{Calls: zipath}\cr
\code{Additionally: Warning:}\cr
\code{In glmreg_fit(Znew, probi, weights = weights, standardize = standardize,  :}\cr
\code{ saturated model, exiting ...}\cr
\code{Execution halted}\cr

  One possible reason is that the fitted model is too complex for the data. There are two suggestions to overcome the error. One is to reduce the number of variables. Second, find out what lambda values caused the problem and omit them. Try with other lambda values instead.

}

\value{
  An object of class \code{"zipath"}, i.e., a list with components including
  \item{coefficients}{a list with elements \code{"count"} and \code{"zero"}
    containing the coefficients from the respective models,}
  \item{residuals}{a vector of raw residuals (observed - fitted),}
  \item{fitted.values}{a vector of fitted means,}
  \item{weights}{the case weights used,}
  \item{terms}{a list with elements \code{"count"}, \code{"zero"} and
    \code{"full"} containing the terms objects for the respective models,}
  \item{theta}{estimate of the additional \eqn{\theta}{theta} parameter of the
    negative binomial model (if a negative binomial regression is used),}
  \item{loglik}{log-likelihood of the fitted model,}
  \item{family}{character string describing the count distribution used,}
  \item{link}{character string describing the link of the zero-inflation model,}
  \item{linkinv}{the inverse link function corresponding to \code{link},}
  \item{converged}{logical value, TRUE indicating successful convergence of \code{zipath}, FALSE indicating otherwise}
  \item{call}{the original function call}
  \item{formula}{the original formula}
  \item{levels}{levels of the categorical regressors}
  \item{model}{the full model frame (if \code{model = TRUE}),}
  \item{y}{the response count vector (if \code{y = TRUE}),}
  \item{x}{a list with elements \code{"count"} and \code{"zero"}
    containing the model matrices from the respective models
    (if \code{x = TRUE}),}
}

\references{     
Zhu Wang, Shuangge Ma, Michael Zappitelli, Chirag Parikh, Ching-Yun Wang and Prasad Devarajan (2014)      \emph{Penalized Count Data Regression with Application to Hospital Stay after Pediatric Cardiac Surgery}, \emph{Statistical Methods in Medical Research}. 2014 Apr 17. [Epub ahead of print]

Zhu Wang, Shuangge Ma, Ching-Yun Wang, Michael Zappitelli, Prasad Devarajan and Chirag R. Parikh (2014)
\emph{EM for Regularized Zero Inflated Regression Models with Applications to Postoperative Morbidity after Cardiac Surgery in Children}, \emph{Statistics in Medicine}. 33(29):5192-208.

Zhu Wang, Shuangge Ma and Ching-Yun Wang (2015) \emph{Variable selection for zero-inflated and overdispersed data with application to health care demand in Germany}, \emph{Biometrical Journal}. 57(5):867-84.
}

\author{Zhu Wang <zwang145@uthsc.edu>}

\seealso{\code{\link{zipath}},
  \code{\link{glmreg}}, \code{\link{glmregNB}}
}

\keyword{regression}

\name{irglmreg}
\alias{irglmreg}
\alias{irglmreg.formula}
\alias{irglmreg.matrix}
\alias{irglmreg.default}

\title{Fit a robust penalized generalized linear models}
\description{
    Fit a robust penalized GLM where the loss function is a composite function \code{cfun}o\code{dfun} + penalty. This is the wrapper function of \code{irglmreg_fit} 
  }
\usage{
\method{irglmreg}{formula}(formula, data, weights, offset=NULL, contrasts=NULL, ...)
\method{irglmreg}{matrix}(x, y, weights, offset=NULL, ...)
\method{irglmreg}{default}(x,  ...)
}

\arguments{
  \item{formula}{symbolic description of the model, see details.}
  \item{data}{argument controlling formula processing
    via \code{\link[stats]{model.frame}}.}
  \item{weights}{optional numeric vector of weights. If \code{standardize=TRUE}, weights are renormalized to weights/sum(weights). If \code{standardize=FALSE}, weights are kept as original input}
  \item{x}{input matrix, of dimension nobs x nvars; each row is an
  observation vector}
  \item{y}{response variable. Quantitative for \code{rfamily="clossR"} and -1/1 for classification. }
  \item{offset}{Not implemented yet}
  \item{contrasts}{the contrasts corresponding to \code{levels} from the
    respective models}
  \item{...}{Other arguments passing to \code{irglmreg_fit}}
}
\details{
    The computing is done by the iteratively reweighted penalized GLM, an application of the iteratively reweighted convex optimization (IRCO). Here convex is the loss function induced by \code{dfun}, not the penalty function. The output \code{weights_update} is a useful diagnostic to the outlier status of     the observations.
The regularization path is computed for the lasso (or elastic net penalty), scad (or snet) and mcp (or mnet penalty), at a grid of values for the regularization parameter lambda. 
  The sequence of robust models implied by \code{lambda} is fit by the IRCO along with coordinate
  descent. Note that the objective function is \deqn{weights*loss + \lambda*penalty,} if \code{standardize=FALSE} and \deqn{
  \frac{weights}{\sum(weights)}*loss + \lambda*penalty,} if \code{standardize=TRUE}. 
  }

\value{
An object with S3 class \code{"irglmreg"} for the various types of models.
  \item{call}{the call that produced this object}
  \item{b0}{Intercept sequence of length \code{length(lambda)}}
  \item{beta}{A \code{nvars x
      length(lambda)} matrix of coefficients.}
  \item{lambda}{The actual sequence of \code{lambda} values used}
  \item{nobs}{number of observations}
  \item{risk}{if \code{type.path="nonactive"}, a matrix with number of rows \code{iter} and number of columns \code{nlambda}, loss values along the regularization path. If \code{type.path="fast"}, a vector of length \code{nlambda}, loss values along the regularization path}
  \item{pll}{if \code{type.path="nonactive"}, a matrix with number of rows \code{iter} and number of columns \code{nlambda}, penalized loss values along the regularization path. If \code{type.path="fast"}, a vector of length \code{nlambda}, penalized loss values along the regularization path}
  \item{fitted.values}{predicted values depending on \code{standardize}, internal use only}
}

\references{
   Zhu Wang (2024)
    \emph{Unified Robust Estimation}, \emph{Australian & New Zealand Journal of      Statistics}. 66(1):77-102.
}

\author{Zhu Wang <zwang145@uthsc.edu>}
\seealso{\code{\link{print}}, \code{\link{predict}}, \code{\link{coef}} and \code{\link{plot}} methods, and the \code{\link{cv.irglmreg}} function.}
\examples{
#binomial
x=matrix(rnorm(100*20),100,20)
g2=sample(c(-1,1),100,replace=TRUE)
fit1=irglmreg(x,g2,s=1,cfun="ccave",dfun="gaussian",type.path="active",
              decreasing=TRUE,type.init="bst")
#fit1$risk
\dontrun{
### different solution paths via a combination of type.path, decreasing and type.init
fit1=irglmreg(x,g2,s=1,cfun="ccave",dfun="gaussian",type.path="active",
           decreasing=TRUE,type.init="bst")
fit2=irglmreg(x,g2,s=1,cfun="ccave",dfun="gaussian",type.path="active",
           decreasing=FALSE,type.init="bst")
fit3=irglmreg(x,g2,s=1,cfun="ccave",dfun="gaussian",type.path="nonactive",
           decreasing=TRUE,type.init="bst")
fit4=irglmreg(x,g2,s=1,cfun="ccave",dfun="gaussian",type.path="nonactive",
           decreasing=FALSE,type.init="bst")
fit5=irglmreg(x,g2,s=1,cfun="ccave",dfun="gaussian",type.path="active",
           decreasing=TRUE,type.init="co")
fit6=irglmreg(x,g2,s=1,cfun="ccave",dfun="gaussian",type.path="active",
           decreasing=FALSE,type.init="co")
fit7=irglmreg(x,g2,s=1,cfun="ccave",dfun="gaussian",type.path="nonactive",
           decreasing=TRUE,type.init="co")
fit8=irglmreg(x,g2,s=1,cfun="ccave",dfun="gaussian",type.path="nonactive",
           decreasing=FALSE,type.init="co")
}
}
\keyword{models}
\keyword{regression}

 

\name{breadReg}
\alias{breadReg}
\alias{breadReg.zipath}
\title{Bread for Sandwiches in Regularized Estimators}
\description{
 Generic function for extracting an estimator for the bread of
 sandwiches.
}
\usage{
breadReg(x, which, \dots)
}
\arguments{
 \item{x}{a fitted model object.}
 \item{which}{which penalty parameter(s)?}
 \item{\dots}{arguments passed to methods.}
}

\value{A matrix containing an estimator for the penalized second derivative of log-likelihood function.
Typically, this should be an \eqn{k \times k}{k x k} matrix corresponding
to \eqn{k} parameters. The rows and columns should be named
as in \code{\link{coef}} or \code{\link{terms}}, respectively.}

\seealso{\code{\link{meatReg}}, \code{\link{sandwichReg}}}

\references{

Zhu Wang, Shuangge Ma and Ching-Yun Wang (2015) \emph{Variable selection for zero-inflated and overdispersed data with application to health care demand in Germany}, \emph{Biometrical Journal}. 57(5):867-84.

}
\author{Zhu Wang <zwang145@uthsc.edu>}

\examples{
data("bioChemists", package = "pscl")
fm_zinb <- zipath(art ~ . | ., data = bioChemists, family = "negbin", nlambda=10, maxit.em=1)
breadReg(fm_zinb, which=which.min(fm_zinb$bic))
}
\keyword{regression}

\name{se}
\alias{se}
\alias{se.zipath}
\title{Standard Error of Regularized Estimators}
\description{
 Generic function for computing standard errors of non-zero regularized estimators
}
\usage{
se(x, which, log=TRUE, \dots)
}
\arguments{
 \item{x}{a fitted model object.}
 \item{which}{which penalty parameter(s)?}
 \item{log}{if TRUE, the computed standard error is for log(theta) for negative binomial regression, otherwise, for theta.}
 \item{\dots}{arguments passed to methods.}
}

\value{A vector containing standard errors of non-zero regularized estimators.}

\seealso{\code{\link{zipath}}}
\references{
Zhu Wang, Shuangge Ma and Ching-Yun Wang (2015) \emph{Variable selection for zero-inflated and overdispersed data with application to health care demand in Germany}, \emph{Biometrical Journal}. 57(5):867-84.
}
\author{Zhu Wang <zwang145@uthsc.edu>}

\examples{
data("bioChemists", package = "pscl")
fm_zinb <- zipath(art ~ . | ., data = bioChemists, family = "negbin", nlambda=10, maxit.em=1)
res <- se(fm_zinb, which=which.min(fm_zinb$bic))
}

\keyword{regression}

\name{sandwichReg}
\alias{sandwichReg}
\title{Making Sandwiches with Bread and Meat for Regularized Estimators}
\description{
 Constructing sandwich covariance matrix estimators by
 multiplying bread and meat matrices for regularized regression parameters.
}
\usage{
sandwichReg(x, breadreg.=breadReg, meatreg.=meatReg, which, log=FALSE, \dots)
}
\arguments{
 \item{x}{a fitted model object.}
 \item{breadreg.}{either a breadReg matrix or a function for computing
   this via \code{breadreg.(x)}.}
 \item{meatreg.}{either a breadReg matrix or a function for computing
   this via \code{meatreg.(x, \dots)}.}
 \item{which}{which penalty parameters(s) to compute?}
 \item{log}{if TRUE, the corresponding element is with respect to log(theta) in negative binomial regression. Otherwise, for theta} 
 \item{\dots}{arguments passed to the \code{meatReg} function.}
}

\details{\code{sandwichReg} is a function to compute an estimator for the covariance of the non-zero parameters. It takes a breadReg matrix (i.e., estimator of the expectation of the negative
derivative of the penalized estimating functions) and a meatReg matrix (i.e.,
estimator of the variance of the log-likelihood function) and multiplies
them to a sandwich with meat between two slices of bread. By default
\code{\link{breadReg}} and \code{\link{meatReg}} are called. Implemented only for \code{zipath} object with \code{family="negbin"} in the current version. 
}

\value{A matrix containing the sandwich covariance matrix estimate for the non-zero parameters.}

\seealso{\code{\link{breadReg}}, \code{\link{meatReg}}}

\references{
Zhu Wang, Shuangge Ma and Ching-Yun Wang (2015) \emph{Variable selection for zero-inflated and overdispersed data with application to health care demand in Germany}, \emph{Biometrical Journal}. 57(5):867-84.
}
\author{Zhu Wang <zwang145@uthsc.edu>}

\examples{
data("bioChemists", package = "pscl")
fm_zinb <- zipath(art ~ . | ., data = bioChemists, family = "negbin", nlambda=10, maxit.em=1)
sandwichReg(fm_zinb, which=which.min(fm_zinb$bic))
}

\keyword{regression}

\name{irsvm_fit}
\alias{irsvm_fit}
\title{Fit iteratively reweighted support vector machines for robust loss functions}
\description{
\code{irsvm_fit} is used to train a subject weighted support vector machine where the weights are provided iteratively from robust loss function with the iteratively reweighted convex optimization (IRCO). It can be used to carry out robust regression and binary classification. This does computing for the wrapper function \code{irsvm}.
}
\usage{
irsvm_fit(x, y, weights, cfun="ccave", s=NULL, delta=0.0001, type = NULL, 
          kernel="radial", cost=1, epsilon = 0.1, iter=10, reltol=1e-5, 
          trace=FALSE, ...)
}
\arguments{
  \item{x}{a data matrix, a vector, or a sparse '\emph{design} matrix' (object of class
    \code{\link[Matrix]{Matrix}} provided by the \pkg{Matrix} package,
    or of class \code{matrix.csr}
    provided by the \pkg{SparseM} package, or of class
    \code{simple_triplet_matrix} provided by the \pkg{slam}
    package).}
  \item{y}{a response vector with one label for each row/component of
    \code{x}. Can be either a factor (for classification tasks)
    or a numeric vector (for regression).}
  \item{weights}{the weight of each subject. It should be in the same length of \code{y}.}
   \item{cfun}{ character, type of convex cap (concave) function.\cr
   Valid options are:
    \itemize{
         \item \code{"hcave"}
        \item \code{"acave"}
        \item \code{"bcave"}
        \item \code{"ccave"}
        \item \code{"dcave"}
        \item \code{"ecave"}
        \item \code{"gcave"}
        \item \code{"tcave"}
     }
   }
 \item{s}{ tuning parameter of \code{cfun}. \code{s > 0} and can be equal    to  0 for \code{cfun="tcave"}. If \code{s} is too close to 0 for \code{cfun="acave", "bcave", "ccave"}, the calculated weights can become 0 for all observations, thus crash the program.}
   \item{delta}{a small positive number provided by user only if                 \code{cfun="gcave"} and \code{0 < s <1}}
   \item{type}{\code{irsvm_fit} can be used as a classification
    machine, or as a regression machine.
    Depending of whether \code{y} is
    a factor or not, the default setting for \code{type} is \code{C-classification} or \code{eps-regression}, respectively, but may be overwritten by setting an explicit value.\cr
    Valid options are:
    \itemize{
      \item \code{C-classification}
      \item \code{nu-classification}
      \item \code{eps-regression}
      \item \code{nu-regression}
    }
  }
  \item{kernel}{the kernel used in training and predicting. You
     might consider changing some of the following parameters, depending
     on the kernel type.\cr
     \describe{
       \item{linear:}{\eqn{u'v}{u'*v}}
       \item{polynomial:}{\eqn{(\gamma u'v + coef0)^{degree}}{(gamma*u'*v +      coef0)^degree}}
       \item{radial basis:}{\eqn{e^(-\gamma |u-v|^2)}{exp(-gamma*|u-v|^2)}}
       \item{sigmoid:}{\eqn{tanh(\gamma u'v + coef0)}{tanh(gamma*u'*v + coef0)}}
       }
     }
  \item{cost}{cost of constraints violation (default: 1)---it is the
     \sQuote{C}-constant of the regularization term in the Lagrange formulation. This is proportional to the inverse of \code{lambda} in \code{irglmreg}.} 
  \item{epsilon}{epsilon in the insensitive-loss function (default: 0.1)}
\item{iter}{number of iteration in the IRCO algorithm}
  \item{reltol}{convergency criteria in the IRCO algorithm}
  \item{trace}{If \code{TRUE}, fitting progress is reported}
\item{\dots}{additional parameters for function \code{wsvm} in package \pkg{WeightSVM}}
}

\value{
  An object of class \code{"wsvm"} (see package \pkg{WeightSVM}) containing the fitted model, including:
  \item{SV}{The resulting support vectors (possibly scaled).}
  \item{index}{The index of the resulting support vectors in the data
    matrix. Note that this index refers to the preprocessed data (after
    the possible effect of \code{na.omit} and \code{subset})}
  \item{coefs}{The corresponding coefficients times the training labels.}
  \item{rho}{The negative intercept.}
  \item{sigma}{In case of a probabilistic regression model, the scale
    parameter of the hypothesized (zero-mean) laplace distribution estimated by
    maximum likelihood.}
  \item{probA, probB}{numeric vectors of length 2, number of
    classes, containing the parameters of the logistic distributions fitted to
    the decision values of the binary classifiers (1 / (1 + exp(a x + b))).}
}
\details{
    A case weighted SVM is fit by the IRCO algorithm, where the loss function is a composite function of \code{cfun}o\code{type}, plus a \eqn{L\_2} penalty.
 Additional arguments include \code{degree, gamma, coef0}, 
 \code{class.weights, cachesize, tolerance, shrinking, propbability, fitted}, the same as \code{"wsvm"} in package \pkg{WeightSVM}.
}
\references{
   Zhu Wang (2024)
   \emph{Unified Robust Estimation}, \emph{Australian & New Zealand Journal of Statistics}. 66(1):77-102.
  }
\author{
  Zhu Wang \email{zwang145@uthsc.edu}
}
\seealso{\code{\link{irsvm}}, \code{\link{print}}, \code{\link{predict}}, \code{\link{coef}} and     \code{\link{plot}}.}
\examples{
data(iris)
 iris <- subset(iris, Species \%in\% c("setosa", "versicolor"))
 # default with factor response:
  model <- irsvm(Species ~ ., data = iris, kernel="linear", trace=TRUE)
  model <- irsvm(Species ~ ., data = iris)
 # alternatively the traditional interface:
  x <- subset(iris, select = -Species)
  y <- iris$Species
model <- irsvm(x, y)
  # test with train data
  pred <- predict(model, x)
  # (same as:)
  pred <- fitted(model)
 
  # Check accuracy:
  table(pred, y)
 # compute decision values and probabilities:
  pred <- predict(model, x, decision.values = TRUE)
  attr(pred, "decision.values")
 
  # visualize (classes by color, SV by crosses):
  plot(cmdscale(dist(iris[,-5])),
       col = as.integer(iris[,5]),
       pch = c("o","+")[1:100 \%in\% model$index + 1])
 
  ## try regression mode on two dimensions
 
  # create data
  x <- seq(0.1, 5, by = 0.05)
  y <- log(x) + rnorm(x, sd = 0.2)
 
  # estimate model and predict input values
  m   <- irsvm(x, y)
  new <- predict(m, x)
 
  # visualize
 plot(x, y)
  points(x, log(x), col = 2)
  points(x, new, col = 4) 
}
 \keyword{neural}
 \keyword{nonlinear}
 \keyword{classif}

\name{nclreg_fit}
\alias{nclreg_fit}
\title{Internal function to fitting a nonconvex loss based robust linear model with regularization}
\description{
    Fit a linear model via penalized nonconvex loss function.  The
  regularization path is computed for the lasso (or elastic net penalty), scad (or snet) and mcp (or mnet penalty), at a grid
  of values for the regularization parameter lambda.
  }
\usage{
nclreg_fit(x, y, weights, offset, rfamily=c("clossR", "closs", "gloss", "qloss"), 
           s=NULL, fk=NULL, iter=10, reltol=1e-5, 
           penalty=c("enet","mnet","snet"), nlambda=100,lambda=NULL, 
           type.path=c("active", "nonactive", "onestep"), decreasing=FALSE, 
           lambda.min.ratio=ifelse(nobs<nvars,.05, .001), alpha=1, gamma=3, 
           standardize=TRUE, intercept=TRUE, penalty.factor=NULL, maxit=1000, 
           type.init=c("bst", "ncl", "heu"), mstop.init=10, nu.init=0.1, 
           eps=.Machine$double.eps, epscycle=10, thresh=1e-6, trace=FALSE)
}

\arguments{
  \item{x}{input matrix, of dimension nobs x nvars; each row is an
  observation vector.} 
  \item{y}{response variable. Quantitative for \code{rfamily="clossR"} and -1/1 for classifications.}
  \item{weights}{observation weights. Can be total counts if responses are proportion matrices. Default is 1 for each observation}
  \item{offset}{this can be used to specify an a priori known component to be included in the linear predictor during fitting. This should be NULL or a numeric vector of length equal to the number of cases. Currently only one offset term can be included in the formula.}
  \item{rfamily}{Response type and relevant loss functions (see above)}
  \item{s}{ nonconvex loss tuning parameter for robust regression and classification. The \code{s} value is for robust nonconvex loss where smaller \code{s} value is more robust to outliers with \code{rfamily="closs"}, and larger \code{s} value more robust with\cr \code{rfamily="clossR", "gloss", "qloss"}.}
  \item{fk}{ predicted values at an iteration in the MM algorithm }
  \item{nlambda}{The number of \code{lambda} values - default is 100. The sequence may be truncated before \code{nlambda} is reached if a close to saturated model is fitted. See also \code{satu}.}
  \item{lambda}{ by default, the algorithm provides a sequence of regularization values, or a user supplied \code{lambda} sequence }
  \item{type.path}{solution path. If \code{type.path="active"}, then cycle through only the active set in the next increasing \code{lambda} sequence. If \code{type.path="nonactive"}, no active set for each element of the lambda sequence and cycle through all the predictor variables. If \code{type.path="onestep"}, update for one element of lambda depending on \code{decreasing=FALSE} (last element of lambda) or \code{decreasing=TRUE} (then first element of lambda) in each MM iteration, and iterate until convergency of prediction. Then fit a solution path based on the sequence of lambda.}
  \item{lambda.min.ratio}{Smallest value for \code{lambda}, as a fraction of
    \code{lambda.max}, the (data derived) entry value (i.e. the smallest
  value for which all coefficients are zero except the intercept). Note, there is no closed formula for \code{lambda.max}. The default of \code{lambda.min.ratio} depends on the
  sample size \code{nobs} relative to the number of variables
  \code{nvars}. If \code{nobs > nvars}, the default is \code{0.001},
  close to zero.  If \code{nobs < nvars}, the default is \code{0.05}.}
  \item{alpha}{The \eqn{L_2} penalty mixing parameter, with
    \eqn{0 \le alpha\le 1}. \code{alpha=1} is lasso (mcp, scad) penalty; and \code{alpha=0} the ridge penalty. However, if \code{alpha=0}, one must provide \code{lambda} values.}
  \item{gamma}{The tuning parameter of the \code{snet} or \code{mnet} penalty.}
  \item{standardize}{logical value for x variable standardization, prior to
    fitting the model sequence. The coefficients are always returned on
    the original scale. Default is \code{TRUE}.}
  \item{intercept}{logical value: if TRUE (default), intercept(s) are fitted; otherwise, intercept(s)  are set to zero}
  \item{penalty.factor}{This is a number that multiplies \code{lambda} to allow
    differential shrinkage of coefficients. Can be 0 for some variables, which implies
    no shrinkage, and that variable is always included in the
    model. Default is same shrinkage for all variables.}
 \item{type.init}{a method to determine the initial values. If \code{type.init="ncl"}, an intercept-only model as initial parameter and run \code{nclreg} regularization path forward from lambda_max to lambda_min. If \code{type.init="heu"}, heuristic initial parameters and run \code{nclreg} path backward or forward depending on \code{decreasing}, between lambda_min and lambda_max. If \code{type.init="bst"}, run a boosting model with \code{bst} in package bst, depending on \code{mstop.init, nu.init} and run \code{nclreg} backward or forward depending on \code{decreasing}.}
 \item{mstop.init}{ an integer giving the number of boosting iterations when \code{type.init="bst"} }
 \item{nu.init}{ a small number (between 0 and 1) defining the step size or shrinkage parameter when \code{type.init="bst"}. }
 \item{decreasing}{ only used if \code{lambda=NULL}, a logical value used to    determine regularization path direction either from lambda_max to a potentially modified lambda_min or vice versa if \code{type.init="bst","heu"}. Since this is a nonconvex optimization, it is possible to generate different estimates for the same \code{lambda} depending on \code{decreasing}.  The choice of \code{decreasing} picks different starting values.}
 \item{iter}{number of iteration in the MM algorithm}
 \item{maxit}{Within each MM algorithm iteration, maximum number of coordinate  descent iterations for each \code{lambda} value; default is 1000.}
 \item{reltol}{convergency criteria} 
 \item{eps}{If a coefficient is less than \code{eps} in magnitude, then it is reported to be 0}
 \item{epscycle}{ If \code{nlambda} > 1 and the relative loss values from two consecutive \code{lambda} values change > \code{epscycle}, then re-estimate parameters in an effort to avoid trap of local optimization.}
 \item{thresh}{Convergence threshold for coordinate descent. Defaults value is \code{1e-6}.}
 \item{penalty}{Type of regularization}
 \item{trace}{If \code{TRUE}, fitting progress is reported}
}
\details{
  The sequence of robust models implied by \code{lambda} is fit by majorization-minimization along with coordinate
  descent. Note that the objective function is \deqn{weights*loss + \lambda*penalty,} if \code{standardize=FALSE} and \deqn{\frac{weights}{\sum(weights)}*loss + \lambda*penalty,} if \code{standardize=TRUE}.
  }
\value{
An object with S3 class \code{"nclreg"} for the various types of models.
  \item{call}{the call that produced the model fit}
  \item{b0}{Intercept sequence of length \code{length(lambda)}}
  \item{beta}{A \code{nvars x length(lambda)} matrix of coefficients.}
  \item{lambda}{The actual sequence of \code{lambda} values used}
  \item{decreasing}{ if \code{lambda} is an increasing sequence or not, used to determine regularization path direction either from lambda_max to a potentially modified lambda_min or vice versa if \code{type.init="bst", "heu"}. }
}

\references{
   Zhu Wang (2021),
   \emph{MM for Penalized Estimation},
   \emph{TEST},
   \doi{10.1007/s11749-021-00770-2}
 }

\author{Zhu Wang <zwang145@uthsc.edu>}

\seealso{\code{\link{nclreg}}}
\keyword{models}
\keyword{regression}

 

\name{docvisits}
\alias{docvisits}
\docType{data}
\title{ Doctor visits}
\description{
  A cohort of 3066 Americans over the age of 50 were studied on health care utilization, doctor
office visits.
}
\usage{data(docvisits)}

\source{
Stephane Heritier, Eva Cantoni, Samuel Copt and Maria-Pia Victoria-Fese (2009). \emph{Robust Methods in Biostatistics}, John Wiley & Sons}

\examples{
data(docvisits)
str(docvisits)
}
\keyword{datasets}

\name{loss2}
\alias{loss2}
\title{Composite Loss Value}
\description{
 Compute composite loss value
}
\usage{
loss2(y, f, weights, cfun, dfun, s, delta=0.0001)
}
\arguments{
 \item{y}{response variable values}
 \item{f}{linear predictor values of \code{y}. If \code{f} is predicted \code{response} of model, use function \code{loss3} instead}
 \item{weights}{observation weights, same length as \code{y}}
 \item{cfun}{integer from 1-8, concave function as in \code{irglm_fit}}
 \item{dfun}{integer from 1-7, convex function as in \code{irglm_fit}}
\item{s}{ tuning parameter of \code{cfun}. \code{s > 0} and can be equal    to 0 for \code{cfun="tcave"}.} 
   \item{delta}{a small positive number provided by user only if \code{cfun="gcave"} and \code{0 < s <1}}
}

\value{Weighted loss values}
\details{An internal function. For large \code{s} values, the loss can be 0 with \code{cfun=2,3,4}, or "acave", "bcave", "ccave".}
\seealso{
    \code{\link{loss3}}
    \code{\link{irglm}}
    \code{\link{irglmreg}}
    \code{\link{loss2_irsvm}}
}
\references{
      Zhu Wang (2024)
    \emph{Unified Robust Estimation}, \emph{Australian & New Zealand Journal of      Statistics}. 66(1):77-102.
  }
\author{Zhu Wang <zwang145@uthsc.edu>}

\keyword{regression}
\keyword{classification}

\name{predict.glmreg}
\alias{predict.glmreg}
\alias{coef.glmreg}
\title{Model predictions based on a fitted "glmreg" object.}
\description{
This function returns predictions from
a fitted \code{"glmreg"} object.
}
\usage{
\method{predict}{glmreg}(object,newx,newoffset,which=1:length(object$lambda),
type=c("link","response","class","coefficients","nonzero"), na.action=na.pass, ...)
\method{coef}{glmreg}(object,which=1:length(object$lambda),...)
}
\arguments{
  \item{object}{Fitted \code{"glmreg"} model object.}
  \item{newx}{Matrix of values at which predictions are to be made.  Not
    used for \code{type="coefficients"}}
  \item{which}{Indices of the penalty parameter \code{lambda} at which
    predictions are required.  By default, all indices are returned.}
  \item{type}{Type of prediction: \code{"link"} returns the
    linear predictors; \code{"response"} gives the fitted values;
    \code{"class"} returns the binomial outcome with the highest
    probability; \code{"coefficients"} returns the coefficients.}
  \item{newoffset}{an offset term used in prediction}
  \item{na.action}{action for missing data value}
  \item{\dots}{ arguments for predict }}
\value{The returned object depends on \code{type}.}
\references{
Zhu Wang, Shuangge Ma, Michael Zappitelli, Chirag Parikh, Ching-Yun Wang and Prasad Devarajan (2014)
\emph{Penalized Count Data Regression with Application to Hospital Stay after Pediatric Cardiac Surgery}, \emph{Statistical Methods in Medical Research}. 2014 Apr 17. [Epub ahead of print]
}
\author{Zhu Wang <zwang145@uthsc.edu>}
\seealso{\code{\link{glmreg}}}
\examples{
## Dobson (1990) Page 93: Randomized Controlled Trial :
counts <- c(18,17,15,20,10,20,25,13,12)
outcome <- gl(3,1,9)
treatment <- gl(3,3)
print(d.AD <- data.frame(treatment, outcome, counts))
fit <- glmreg(counts ~ outcome + treatment, data=d.AD, family="poisson")
predict(fit, newx=d.AD[,1:2])
summary(fit)
coef(fit)
}
\keyword{models}
\keyword{regression}

\name{update_wt}
\alias{update_wt}
\title{Compute weight value}
\description{
 Compute weight value
}
\usage{
update_wt(y, ypre, weights, cfun, s, dfun, delta=0.0001)
}
\arguments{
 \item{y}{input value of response variable}
 \item{ypre}{predicted value of response variable}
 \item{weights}{optional numeric vector of weights.}
 \item{cfun}{integer from 1-8, concave function as in \code{irglm_fit}}
 \item{dfun}{integer value, convex function as in \code{irglm_fit}}
 \item{s}{a numeric value, see details in \code{irglm_fit}}
 \item{delta}{a positive small value, see details in \code{irglm_fit}}
}

\value{Weight value}

\seealso{\code{\link{compute_wt}}}
\references{
     Zhu Wang (2024)
    \emph{Unified Robust Estimation}, \emph{Australian & New Zealand Journal of      Statistics}. 66(1):77-102.
}
\author{Zhu Wang <zwang145@uthsc.edu>}

\keyword{regression}
\keyword{classification}

\name{cv.nclreg}
\alias{cv.nclreg}
\alias{cv.nclreg.formula}
\alias{cv.nclreg.matrix}
\alias{cv.nclreg.default}
\alias{plot.cv.nclreg}
\alias{coef.cv.nclreg}
\title{Cross-validation for nclreg}
\description{Does k-fold cross-validation for nclreg, produces a plot,
  and returns cross-validated loss values for \code{lambda}}
\usage{
\method{cv.nclreg}{formula}(formula, data, weights, offset=NULL, ...)
\method{cv.nclreg}{matrix}(x, y, weights, offset=NULL, ...)
\method{cv.nclreg}{default}(x,  ...)
\method{plot}{cv.nclreg}(x,se=TRUE,ylab=NULL, main=NULL, width=0.02, col="darkgrey", ...)
\method{coef}{cv.nclreg}(object,which=object$lambda.which, ...)
}
\arguments{
  \item{formula}{symbolic description of the model, see details.}
  \item{data}{argument controlling formula processing
    via \code{\link[stats]{model.frame}}.}
  \item{x}{\code{x} matrix as in \code{nclreg}. It could be object of \code{cv.nclreg}.}
  \item{y}{response \code{y} as in \code{nclreg}.}
  \item{weights}{Observation weights; defaults to 1 per observation}
  \item{offset}{Not implemented yet}
  \item{object}{object of \code{cv.nclreg}}
  \item{which}{Indices of the penalty parameter \code{lambda} at which
    estimates are extracted. By default, the one which generates the optimal cross-validation value.}
  \item{se}{logical value, if \code{TRUE}, standard error curve is also plotted} 
  \item{ylab}{ylab on y-axis}
  \item{main}{title of plot}
  \item{width}{width of lines}
  \item{col}{color of standard error curve}
  \item{\dots}{Other arguments that can be passed to \code{nclreg}.}
}
\details{The function runs \code{nclreg} \code{nfolds}+1 times; the
  first to compute the \code{lambda} sequence, and then to
  compute the fit with each of the folds omitted. The error or the loss value is
  accumulated, and the average value and standard deviation over the
  folds is computed. Note that \code{cv.nclreg} can be used to search for
  values for \code{alpha}: it is required to call \code{cv.nclreg} with a fixed vector \code{foldid} for different values of \code{alpha}. 
}
\value{an object of class \code{"cv.nclreg"} is returned, which is a
  list with the ingredients of the cross-validation fit.
  \item{fit}{a fitted nclreg object for the full data.}
  \item{residmat}{matrix of loss values with row values for \code{lambda} and column values for \code{k}th cross-validation}
  \item{bic}{matrix of BIC values with row values for \code{lambda} and column values for \code{k}th cross-validation}
  \item{cv}{The mean cross-validated loss values - a vector of length
    \code{length(lambda)}.}
  \item{cv.error}{estimate of standard error of \code{cv}.}
  \item{foldid}{an optional vector of values between 1 and \code{nfold}
    identifying what fold each observation is in.}
  \item{lambda}{a vector of \code{lambda} values} 
  \item{lambda.which}{index of \code{lambda} that gives minimum \code{cv} value.}
  \item{lambda.optim}{value of \code{lambda} that gives minimum \code{cv} value.}
}

\references{
   Zhu Wang (2021),
   \emph{MM for Penalized Estimation},
   \emph{TEST},
   \doi{10.1007/s11749-021-00770-2}
 }
\author{Zhu Wang <zwang145@uthsc.edu>}

\seealso{\code{\link{nclreg}} and \code{\link{plot}}, \code{\link{predict}}, and \code{\link{coef}} methods for \code{"cv.nclreg"} object.}
\keyword{models}
\keyword{regression}

\name{cv.irglmreg_fit}
\alias{cv.irglmreg_fit}
\title{Internal function of cross-validation for irglmreg}
\description{Internal function to conduct k-fold cross-validation for irglmreg, produces a plot,
  and returns cross-validated loss values for \code{lambda}}
\usage{
cv.irglmreg_fit(x, y, weights, offset, lambda=NULL, balance=TRUE, cfun=4, dfun=1, 
                s=1.5, nfolds=10, foldid, type = c("loss", "error"), plot.it=TRUE,
                se=TRUE, n.cores=2, trace=FALSE, parallel=FALSE, ...)
}
\arguments{
  \item{x}{\code{x} matrix as in \code{irglmreg}.}
  \item{y}{response \code{y} as in \code{irglmreg}.}
  \item{weights}{Observation weights; defaults to 1 per observation}
  \item{offset}{this can be used to specify an a priori known component to be included in the linear predictor during fitting. This should be NULL or a numeric vector of length equal to the number of cases. Currently only one offset term can be included in the formula.}
  \item{lambda}{Optional user-supplied lambda sequence; default is
    \code{NULL}, and \code{irglmreg} chooses its own sequence}
  \item{balance}{for \code{dfun=4, 5, 6} only}
  \item{cfun}{ a number from 1 to 7, type of convex cap (concave) function }
  \item{dfun}{ a number from 1, 4-7, type of convex downward function}
  \item{s}{ nonconvex loss tuning parameter for robust regression and classification. }
  \item{nfolds}{number of folds >=3, default is 10}
  \item{foldid}{an optional vector of values between 1 and \code{nfold}
    identifying what fold each observation is in. If supplied,
    \code{nfold} can be missing and will be ignored.}
  \item{type}{ cross-validation criteria. For \code{type="loss"}, loss function values and \code{type="error"} is misclassification error. }
  \item{plot.it}{ a logical value, to plot the estimated log-likelihood values if \code{TRUE}. }
  \item{se}{ a logical value, to plot with standard errors. }
  \item{n.cores}{The number of CPU cores to use. The cross-validation loop
    will attempt to send different CV folds off to different cores.}
  \item{trace}{ a logical value, print progress of cross validation or not}
  \item{parallel}{ a logical value, parallel computing or not}
  \item{\dots}{Other arguments that can be passed to \code{irglmreg}.}
}
\details{The function runs \code{irglmreg} \code{nfolds}+1 times; the
  first to compute the \code{lambda} sequence, and then to
  compute the fit with each of the folds omitted. The error or the log-likelihood value is
  accumulated, and the average value and standard deviation over the
  folds is computed. Note that \code{cv.irglmreg} can be used to search for
  values for \code{alpha}: it is required to call \code{cv.irglmreg} with a fixed vector \code{foldid} for different values of \code{alpha}.
}

\value{an object of class \code{"cv.irglmreg"} is returned, which is a
  list with the ingredients of the cross-validation fit.
  \item{fit}{a fitted irglmreg object for the full data.}
  \item{residmat}{matrix of loss values or errors with row values for \code{lambda} and column values for \code{k}th cross-validation}
  \item{cv}{The mean cross-validated loss values or errors - a vector of length
    \code{length(lambda)}.}
  \item{cv.error}{estimate of standard error of \code{cv}.}
  \item{foldid}{an optional vector of values between 1 and \code{nfold}
    identifying what fold each observation is in.}
  \item{lambda}{a vector of \code{lambda} values} 
  \item{lambda.which}{index of \code{lambda} that gives minimum \code{cv} value.}
  \item{lambda.optim}{value of \code{lambda} that gives minimum \code{cv} value.}
}
\references{
 Zhu Wang (2024)
     \emph{Unified Robust Estimation}, \emph{Australian & New Zealand Journal         of      Statistics}. 66(1):77-102.
}

\author{Zhu Wang <zwang145@uthsc.edu>}
\seealso{\code{\link{irglmreg}} and \code{\link{plot}}, \code{\link{predict}}, and \code{\link{coef}} methods for \code{"cv.irglmreg"} object.}
\keyword{models}
\keyword{regression}
